#!/usr/bin/tclsh
# Part of MCU 8051 IDE ( http://mcu8051ide.sf.net )

############################################################################
#    Copyright (C) 2007-2009 by Martin Ošmera                              #
#    martin.osmera@gmail.com                                               #
#                                                                          #
#    This program is free software; you can redistribute it and#or modify  #
#    it under the terms of the GNU General Public License as published by  #
#    the Free Software Foundation; either version 2 of the License, or     #
#    (at your option) any later version.                                   #
#                                                                          #
#    This program is distributed in the hope that it will be useful,       #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of        #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
#    GNU General Public License for more details.                          #
#                                                                          #
#    You should have received a copy of the GNU General Public License     #
#    along with this program; if not, write to the                         #
#    Free Software Foundation, Inc.,                                       #
#    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             #
############################################################################

# --------------------------------------------------------------------------
# DESCRIPTION
# Implements source code editor with syntax highligh and
# lightweight syntax validation.
# This GUI component consist of line numbers border, icon border,
# editor, command line and status bar.
#
# Consist of:
#	* PROCEDURES RELATED TO EDITOR COMMAND LINE
#	* GENERAL PURPOSE PROCEDURES
#	* EXPORTS TO OTHER DATA FORMATS (XHTML && LATEX)
#	* KEY EVENT HANDLERS
# --------------------------------------------------------------------------

# Load syntax highlighting
source "${::LIB_DIRNAME}/ASMsyntaxhighlight.tcl"
source "${::LIB_DIRNAME}/R_ASMsyntaxhighlight.tcl"
source "${::LIB_DIRNAME}/Csyntaxhighlight.tcl"
source "${::LIB_DIRNAME}/LSTsyntaxhighlight.tcl"

# Initialize variable containing count of matched strings
set ::editor_search_count 0

class Editor {

	## COMMON
	 ## Editor to use
	  # 0 - Native editor
	  # 1 - Vim
	  # 2 - Emacs
	  # 3 - Nano
	  # 4 - dav
	  # 5 - le
	common editor_to_use	0
	common intentation_mode	{normal};# Editor indentation mode
	common spaces_no_tabs	0	;# Bool: Use spaces instead of tabs
	common number_of_spaces	8	;# Number of spaces to use instead of tab
	common auto_brackets	1	;# Automaticaly insert oposite brackets, quotes, etc.
	common auto_completion	1	;# Enable popup-base completion for code editor
	common cline_completion	1	;# Enable popup-base completion for command line
	common autosave		0	;# Int: 0 == Disable autosave; N > 0 == minutes
	common hg_trailing_sp	1	;# Bool: Highlight trailing spaces

	common finishigh_hg_dlg_max 	;# Int: Highlight dialog -- maximum value for progress bar
	common finishigh_hg_dlg_const	;# Int: Highlight dialog -- current value for progress bar

	common set_shortcuts	{}	;# Currently set shortcut bindigs
	common shortcuts_cat	{edit}	;# Key shortcut categories related to this segment
	common count		0	;# Counter of class instances
	common bookmark		0	;# Auxiliary variable for popup menu for Icon Border
	common breakpoint	0	;# Auxiliary variable for popup menu for Line Numbers
	common pmenu_cline	0	;# Auxiliary variable for popup menu for Icon Border and Line Numbers
	common wrap_char	"\uB7"	;# Character intended for marking wrapped lines

	# Commands supported by editor command line
	common editor_commands {
		animate		assemble	auto-indent	bookmark	breakpoint
		capitalize	clear		comment		copy		custom
		cut		date		exit		exit-program	find
		goto		help		char		indent		kill-line
		open		paste		redo		reload		replace
		run		save		set-icon-border	sim		set-line-numbers
		step		tolower		toupper		uncomment	undo
		unindent	d2h		d2o		d2b		h2d
		h2o		h2b		o2h		o2d		o2b
		b2h		b2d		b2o		hibernate	resume
		switch-mcu	set-xcode	set-xdata
	}
	# Editor commands wich can take options
	common commands_with_option {find replace}
	## Tags which defines background color for specific type of lines
	 # {{tagname	bg-color	bool-priority} ...}
	common line_markers {
		{sel			#AAAAFF}
		{tag_current_line	#FFFF88}
		{tag_bookmark		#DDDDFF}
		{tag_breakpoint		#FF0000}
		{tag_simulator_curr	#AAFFAA}
		{tag_error_line		#FFDDDD}
		{tag_trailing_space	#E8FFF0}
	}

	# Font for command line: Normal help window text
	common cl_hw_nrml_font	[font create	\
		-family {courier} -size -14	\
	]
	# Font for command line: Bold help window text
	common cl_hw_bold_font	[font create	\
		-family {courier} -size -14	\
		-weight bold			\
	]
	# Font for command line: Subheader in help window text
	common cl_hw_hdr_font	[font create	\
		-family {courier} -size -14	\
		-weight bold			\
	]
	# Font for command line: Main header in help window text
	common cmd_line_win_font [font create	\
		-size -17 -weight bold		\
		-family {helvetica}		\
	]
	common cmd_line_fontSize	14		;# Font size for command line
	common cmd_line_fontFamily	{courier}	;# Font family for command line
	# Font for editor command line
	common cmd_line_font	[font create	\
		-family $cmd_line_fontFamily	\
		-size -$cmd_line_fontSize	\
	]
	## Highlight tags for command line
	 # {
	 #	{tag_name ?foreground? ?overstrike? ?italic? ?bold?}
	 # }
	common cmd_line_highlighting	{
		{tag_cmd		#0000DD	0 0 1}
		{tag_argument		#00DD00	0 0 0}
		{tag_option		#DD0000	0 0 1}
	}

	common normal_text_bg	#FFFFFF	;# Default background color for editor
	common iconBorder_bg	#C8C5FF	;# Default background color for icon border
	common lineNumbers_bg	#9497D8	;# Default background color for line numbers
	common lineNumbers_fg	#FFFFFF	;# Default foreground color for line numbers
	# Items in editor menu, which should be disabled when editor goes to simulator mode
	common freezable_menu_items {
		Cut Paste Undo Redo Comment Uncomment Indent
		Unindent Uppercase Lowercase Capitalize
	}
	# Items in editor menu, which should be disabled when editor is in read only mode
	common read_na_only_menu_items {
		Cut Paste Undo Redo Comment Uncomment Indent
		Unindent Uppercase Lowercase Capitalize
	}

	common fontSize		14		;# Default font size
	common fontFamily	{courier}	;# Default font family

	# Default font for editor
	common defaultFont		\
		[font create		\
		-size -$fontSize	\
		-family $fontFamily	\
	]
	common defaultCharWidth		0	;# Width of one character of the default font
	common defaultCharHeight	0	;# Height of one character of the default font
	# Font for status bar (Normal)
	common statusBarFont		\
		[font create		\
		-size -12		\
		-family {courier}	\
	]
	# Font for status bar (Bold)
	common statusBarBoldFont	\
		[font create		\
		-size -12		\
		-weight bold		\
		-family {courier}	\
	]
	# Definition of editor popup menu
	common EDITORMENU { 
		{command	{LJMP this line} {$edit:jmp}	2	"ljmp_this_line {}"
			{exec}		"Program jump"}
		{command	{LCALL this line} {$edit:call}	4	"lcall_this_line {}"
			{exec}		"Call subprogram"}
		{separator}
		{command	{Breakpoint}	{$edit:breakpoint}	0	"Breakpoint {}"
			{flag}		"Add/Remove breakpoint to/from current line"}
		{command	{Bookmark}	{$edit:bookmark}	1	"Bookmark {}"
			{bookmark_add}	"Add/Remove bookmark to/from current line"}
		{separator}
		{command	{Undo}		{$edit:undo}	0		"undo {}"
			{undo}		"Take back last operation"}
		{command	{Redo}		{$edit:redo}	2		"redo {}"
			{redo}		"Take back last undo"}
		{separator}
		{command	{Cut}		{$edit:cut}		2	"cut {}"
			{editcopy}	"Move selected text into the clipboard"}
		{command	{Copy}		{$edit:copy}		0	"copy {}"
			{editcut}	"Copy selected text into the clipboard"}
		{command	{Paste}		{$edit:paste}		0	"paste {}"
			{editpaste}	"Paste text from clipboard"}
		{separator}
		{command	{Select all}	{$edit:select_all}	0	"select_all {}"
			{}		"Select all text in the editor"}
		{separator}
		{command	{Comment}	{$edit:comment}		1	"comment {}"
			{}		"Comment selected text"}
		{command	{Uncomment}	{$edit:uncomment}	4	"uncomment {}"
			{}		"Uncomment selected text"}
		{separator}
		{command	{Indent}	{$edit:indent}		2	"indent {}"
			{forward}	"Indent selected text"}
		{command	{Unindent}	{$edit:unindent}	1	"unindent {}"
			{back}		"Unindent selected text"}
		{separator}
		{command	{Uppercase}	{$edit:uppercase}	0	"lowercase {}"
			{up0}		"Indent selected text"}
		{command	{Lowercase}	{$edit:lowercase}	2	"uppercase {}"
			{down0}		"Unindent selected text"}
		{command	{Capitalize}	{$edit:capitalize}	4	"capitalize {}"
			{}		"Unindent selected text"}
		{separator}
		{command	{Save file}	{$edit:save}		0	"save {}"
			{filesave}	"Save this file"}
	}
	# Definition of popup menu for icon border
	common IBMENU {
		{checkbutton	"Bookmark"	{$edit:bookmark}	{::Editor::bookmark}	1 0 0
			{Bookmark ${::Editor::pmenu_cline}}}
		{separator}
		{command	"Configure panel"	""	0	{configDialogs_mkDialog Colors}
			{configure}	"Invoke editor configuration dialog"}
		{command	"Hide"			""	0	{show_hine_IconB}
			{2leftarrow}	"Hide this panel"}
	}
	# Definition of popup menu for line numbers
	common LNMENU {
		{checkbutton	"Breakpont"	{$edit:breakpoint}	{::Editor::breakpoint}	1 0 0
			{Breakpoint ${::Editor::pmenu_cline}}}
		{separator}
		{command	"Configure panel"	""	0	{configDialogs_mkDialog Colors}
			{configure}	"Invoke editor configuration dialog"}
		{command	"Hide panel" 		""	0	{show_hine_LineN}
			{2leftarrow}	"Hide this panel"}
	}
	# Definition of popup menu for editor statis bar
	common STATMENU {
		{command	"Split vertical" 	{$edit:split_v}		8	{__split_vertical}
			{view_left_right}	"Hide this panel"}
		{command	"Split horizontal" 	{$edit:split_h}		6	{__split_horizontal}
			{view_top_bottom}	"Hide this panel"}
		{separator}
		{command	"Close current view" 	{$edit:close_cv}	2	{__close_current_view_from_pmenu}
			{view_remove}		"Hide this panel"}
		{separator}
		{command	"Back" 			{$edit:prev}		0	{__prev_editor_from_pmenu}
			{left}			"Hide this panel"}
		{command	"Forward" 		{$edit:next}		0	{__next_editor_from_pmenu}
			{right}			"Hide this panel"}
	}

	## PUBLIC
	public variable editor			;# text widget identifier
	public variable ed_sc_frame		;# frame identifier (need packing)
	public variable show_iconBorder	1	;# on/off indicator for Icon Border (bool)
	public variable show_lineNum	1	;# on/off indicator for Line NUmbers (bool)
	public variable iconBorder		;# Identifier of Icon Border text widget
	public variable lineNumbers		;# Identifier of Line Numbers text widget
	public variable scrollbar		;# Identifier of scrollbar widget
	public variable lastEnd		1	;# Last end index of Edior text widget (for speed optimalization)
	public variable Sbar_lock_file		;# Identifier of image label widget at the left site of status bar
	public variable Sbar_sim_mode		;# Identifier of label widget at the left site of status bar
	public variable Sbar_ssim_mode
	public variable Sbar_dis_mode		;# Identifier of label widget at the left site of status bar
	public variable Sbar_CRT_frame		;# ID of frame on statusbar containing labels "Line: x Col: x Total: x"
	public variable Sbar_row		;# ID of label showing current line number
	public variable Sbar_col		;# ID of label showing current column
	public variable Sbar_total		;# ID of label showing total number of lines
	public variable Sbar_image		;# Identifier of floppy disk icon at the middle site of status bar
	public variable Sbar_fileName		;# Identifier of the text of filename at the right site of status bar
	public variable Sbar_prog_lang		;# ID of text specifing file type at the right site of status bar
	public variable fullFileName		;# Full file name of the current file ("" == untitled)
	public variable filename		;# Name of currently opened file or 'untitled'
	public variable modified	0	;# Boolean value indicating than the text has been modified since last save
	public variable encoding		;# Current character encoding (eg. 'utf-8')
	public variable eol			;# Current End of Line character (one of {lf crlf cr})
	public variable ro_mode		0	;# Bool: Read only mode

	## PRIVATE
	private variable finishigh_hg_dlg_wdg	{}	;# Widget: Finishing highlight dialog
	private variable finishigh_hg_dlg_tmr	{}	;# Timer: Finishing highlight dialog
	private variable object_initialized	0	;# Bool: Flag "Object initialized"
	private variable c_hg_tags_created	0	;# Bool: C language highlight tags created
	private variable lst_hg_tags_created	0	;# Bool: Code listing highlight tags created
	private variable cmd_prefix			;# Command prefix for popup menu
	private variable last_cur_line		1	;# Number of the last current line in the editor
	private variable last_sim_line		1	;# Number of the last simulator line in the editor
	private variable enable_parseAll	1	;# Enable reparese whole document (used by: parse_all)
	private variable bookmarks		{0}	;# List of boolean bookmarks, eg. {0 0 0 1 1 0 0}
	private variable breakpoints		{0}	;# List of boolean breakpoint, eg. {0 0 0 1 1 0 0}
	private variable menu				;# Identifier of popup menu for editor text widget
	private variable stat_menu			;# Identifier of popup menu for editor statusbar
	private variable IB_menu			;# Identifier of popup menu for icon border
	private variable ins_mode_lbl			;# Identifier of insertion mode label (on status bar)
	private variable sel_mode_lbl			;# Identifier of selection mode label (on status bar)
	private variable left_frame_L			;# ID of frame containing Line Numbers
	private variable left_frame_R			;# ID of frame containing Icon Border
	private variable LN_menu			;# Identifier of popup menu for line numbers
	private variable frozen			0	;# True if editor is in simulator mode
	private variable getDataAsXHTML_abort	0	;# Set this variable to 1 to immediate stop export to XHTML
	private variable getDataAsLaTeX_abort	0	;# Set this variable to 1 to immediate stop export to LaTeX
	private variable changeLCase_abort	0	;# Set this variable to 1 to immediate stop changing letter case
	private variable parentObject			;# Identifier parent GUI component (some frame widget)
	private variable lastUpDownIndex	0	;# Last column index (for Up and Down actions)
	private variable scroll_in_progress	0	;# Bool: scroll procedure in progress
	private variable highlighted_lines	{0}	;# String/array of highlighted lines (eg. 00011111110001111)
	private variable critical_edit_proc	0	;# Bool: Critical edit procedure in progess
	private variable map_of_wraped_lines	{}	;# Map of wrapped lines (eg. {0 0 5 0 0 2 0})
	private variable number_of_wraps	0	;# Number of line wraps
	private variable ins_ovr_mode		1	;# Current insertion mode (1 == INS; 0 == OVR)
	private variable editor_width		0	;# Width of active area of the editor widget
	private variable cmd_line			;# ID of command line entry widget
	private variable cmd_line_listbox	{}	;# Widget: ListBox of command line auto-completion window
	private variable completion_win_opened	0	;# Bool: Editor popup-based completion window opended
	private variable completion_listbox		;# Widget: ListBox of editor popup-based completion
	private variable do_not_hide_comp_win	0	;# Bool: Disable highing of editor completion win. on KeyRelease
	private variable autosave_timer		{}	;# ID of autosave timer (command "after")
	private variable key_handler_buffer	{}	;# List: Buffer for <Key> event handler
	private variable key_handler_in_progress 0	;# Bool: <Key> event handler in progress
	private variable statusbar_menu_config	{}	;# List: Statusbra menu configuration list
	private variable auto_switching_lock	0	;# Bool: Automatic file switching enabled
	private variable selection_in_progress	0	;# Bool: Procedure "editor_selection" in progress
	private variable selection_mode		0	;# Bool: Block selection mode flag
	private variable save_in_progress	0	;# Bool: Saving in progress
	 ## Programming language
	  # 0 - Assembly language
	  # 1 - C
	  # 2 - Code listing
	  # 3 - ASX8051
	private variable prog_language		0

	private variable top_frame			;# Widget: Container frame for embedded external editor
	private variable terminal_created	0	;# Bool: Terminal emulator to run embedded editor has been created
	private variable top_frame_idx		0	;# Int: Unique number of container frame for embedded editor
	private variable pid			{}	;# Int: Process indentifier of embedded external editor (e.g. Vim)

	## Array: Strings avaliable for autocompletion
	 # Index 0 - Labels in assembly
	 # Index 1 - Constants/variables
	 # Index 2 - Functions
	 # Index 3 - Macros
	 # Index 4 - SFR's
	 # Index 5 - Expression symbols
	 # Index 6 - Doxygen tags
	private variable autocompletion_list
	private variable invoke_com_win_in_p	0	;# Bool: invoke_completion_popup_window in progress
	private variable completion_win_str_i	1.0	;# TextIndex: String to complete - start index
	private variable completion_win_end_i	1.0	;# TextIndex: String to complete - end index
	private variable completion_win_mode	0	;# Int: Completion window mode
	private variable comp_win_loading_in_p	0	;# Bool: Completion window list loading is in progress
	private variable comp_win_loading_max	1	;# Int: Maximum for progressbar in the completion window
	private variable macl_invocations	0	;# Int: Number of invocations of "manage_autocompletion_list"
	private variable doxytag_fg	{#000000}	;# Color: Highlight color for doxygen tag
	private variable indirect_fg	{#000000}	;# Color: Highlight color for indirect address
	private variable symbol_fg	{#000000}	;# Color: Highlight color for asm. symbol
	private variable sfr_fg		{#000000}	;# Color: Highlight color for SFR
	private variable label_fg	{#000000}	;# Color: Highlight color for asm. label
	private variable macro_fg	{#000000}	;# Color: Highlight color for asm. macro
	private variable const_fg	{#000000}	;# Color: Highlight color for asm. const
	private variable dir_fg		{#000000}	;# Color: Highlight color for asm. directive
	private variable ins_fg		{#000000}	;# Color: Highlight color for instruction
	private variable doxytag_font	$defaultFont	;# Font: Font for doxygen tag
	private variable indirect_font	$defaultFont	;# Font: Font for indirect address
	private variable symbol_font	$defaultFont	;# Font: Font for asm. symbol
	private variable sfr_font	$defaultFont	;# Font: Font for SFR
	private variable label_font	$defaultFont	;# Font: Font for asm. label
	private variable macro_font	$defaultFont	;# Font: Font for asm. macro
	private variable const_font	$defaultFont	;# Font: Font for asm. const
	private variable dir_font	$defaultFont	;# Font: Font for asm. directive
	private variable ins_font	$defaultFont	;# Font: Font for instruction

	## Object constructor
	 # @parm Bool create_tags	- Create highlighting tags
	 # @parm String eol_char	- EOL (one of {lf cr crlf})
	 # @parm String enc		- Character encoding (some iso-8859-x or utf-8)
	 # @parm Bool read_only		- Read only flag
	 # @parm Bool switch_lock	- Automatic file switching enabled
	 # @parm widget parentobject	- Reference to parent object
	 # @parm String fileName	- filename to be showed in statusbar
	 # @parm String filepath	- location where to optionaly save the data, "" == untitled document
	 # @parm String cmd_prefix	- command prefix for popup menu
	 # @parm String data		- an input text data
	constructor {create_tags eol_char enc read_only switch_lock parentobject fileName filepath Cmd_prefix data} {
		# increment instance counter
		incr count

		# Set object variables
		array set autocompletion_list {0 {} 1 {} 2 {} 3 {} 4 {} 5 {} 6 {}}
		set autocompletion_list(5) ${::ASMsyntaxHighlight::expr_instructions}
		set autocompletion_list(6) [concat		\
			${::CsyntaxHighlight::doxy_tags_type2}	\
			${::CsyntaxHighlight::doxy_tags_type1}	\
			${::CsyntaxHighlight::doxy_tags_type0}	\
		]
		set auto_switching_lock [expr {!$switch_lock}]
		set ro_mode $read_only
		set encoding $enc
		set eol $eol_char
		set cmd_prefix $Cmd_prefix
		set parentObject $parentobject	;# Identifier parent GUI component (some frame widget)
		set fullFileName $filepath	;# Full file name (including path) of current file
		set filename $fileName		;# Name of currently opened file or 'untitled'
		refresh_avaliable_SFR

		determinate_prog_lang 0

		if {$editor_to_use} {
			set ed_sc_frame [frame .editor_frame$count -bd 2 -relief sunken]
			recreate_terminal $fullFileName
			set editor [text .editor_frame$count.dummy_editor]
			return
		}

		# Create frames
		set ed_sc_frame	[frame .editor_frame$count]
		set top_frame	[frame $ed_sc_frame.editor_top_frame -relief sunken -bd 1]
		set left_frame	[frame $top_frame.editor_left_frame]
		set left_frame_L [frame $left_frame.left -bg $iconBorder_bg]
		set left_frame_R [frame $left_frame.right -bg $lineNumbers_bg]
		set bottom_frame [frame $ed_sc_frame.bottom_frame]
		set statusbar	[frame $bottom_frame.editor_status -bg #DDDDDD]

		# Create command line
		set cmd_line [text $bottom_frame.cmd_line		\
			-bd 1 -bg #FFFFFF -highlightcolor #8888FF	\
			-highlightthickness 1 -height 1			\
			-font $cmd_line_font				\
		]
		setStatusTip -widget $cmd_line	\
			-text "Editor command line, type `help' for more"
		$cmd_line delete 1.0 end

		## Create "Icon border"
		set iconBorder [text $left_frame_L.editor_iconB	\
			-yscrollcommand "$this scrollSet"	\
			-font $defaultFont	\
			-cursor left_ptr	\
			-width 2		\
			-bd 0			\
			-highlightthickness 0	\
			-bg $iconBorder_bg	\
			-exportselection 0	\
			-state disabled		\
			-takefocus 0		\
			-cursor hand1		\
			-relief flat		\
		]
		$iconBorder tag configure center -justify center
		setStatusTip -widget $iconBorder \
			-text "Icon border - click to add/remove bookmark"
		# Create poup menu for "Icon border"
		set IB_menu $iconBorder.editor_iconB_menu

		## Create "Line numbers"
		set lineNumbers [text $left_frame_R.editor_lines	\
			-yscrollcommand "$this scrollSet"	\
			-font $defaultFont	\
			-cursor left_ptr	\
			-width 0		\
			-bd 0			\
			-highlightthickness 0	\
			-exportselection 0	\
			-bg $lineNumbers_bg	\
			-fg $lineNumbers_fg	\
			-state normal		\
			-takefocus 0		\
			-cursor hand1		\
			-relief flat		\
		]
		$lineNumbers delete 1.0 end
		$lineNumbers insert end {1}
		$lineNumbers configure -state disabled
		$lineNumbers tag configure right -justify right
		$lineNumbers tag configure center -justify center
		$lineNumbers tag raise center right
		setStatusTip -widget $lineNumbers \
			-text "Line numbers - click to add/remove breakpoint"

		# Create poup menu for "Line numbers"
		set LN_menu $lineNumbers.editor_lines_menu

		# Create "Editor"
		set editor [text $top_frame.editor		\
			-yscrollcommand "$this scrollSet"	\
			-bg $normal_text_bg	\
			-font $defaultFont	\
			-undo 1			\
			-wrap word		\
			-maxundo 0		\
			-selectborderwidth 1	\
			-bd 0 -relief flat	\
		]
		# Create scrollbar
		set scrollbar [scrollbar $top_frame.editor_scrollbar	\
			-orient vertical	\
			-command "$this scroll"	\
		]


		# Set new font attributes
		set defaultCharWidth [font measure $defaultFont -displayof $editor { }]
		set defaultCharHeight [font metrics $defaultFont -displayof $editor -linespace]

		## Pack that all into mainframe
		 # Parts of Left frame
		pack $lineNumbers -fill y -expand 1 -side right	;# -pady 3
		pack $iconBorder -fill y -expand 1 -side left	;#-pady 3
		pack [frame $left_frame.editor_redutant_frame] -side left
		pack $left_frame_L -side left -fill y
		pack $left_frame_R -side right -fill y
		 # Parts of Top frame
		pack $left_frame -side left -fill y -expand 0
		pack $scrollbar -fill y -expand 0 -side right
		pack $editor -fill both -expand 1 -side left
		 # Parts of Bottom frame
		pack $statusbar -side bottom -fill x
		 # Bottom and Top frame$ins_mode_lbl
		pack $bottom_frame -side bottom -fill x
		pack $top_frame -side top -fill both -expand 1

		## Create statusbar
		set stat_menu $statusbar.popup_menu
		set status_left		[frame $statusbar.editor_status_left -bg #DDDDDD]
		set status_middle	[frame $statusbar.editor_status_middle -width 16 -bg #DDDDDD -width 20]
		set status_right	[frame $statusbar.editor_status_right -bg #DDDDDD]
		set ins_mode_lbl	[Label $statusbar.ins_mode_lbl	\
			-text "INS"	-fg #000000 -pady 0	\
			-bg #DDDDDD	-cursor hand1		\
			-helptext "Insertion mode"		\
			-font $statusBarBoldFont		\
		]
		set sel_mode_lbl	[Label $statusbar.sel_mode_lbl	\
			-text "NORM"	-fg #000000 -pady 0	\
			-bg #DDDDDD	-cursor hand1		\
			-helptext "Selection mode"		\
			-font $statusBarBoldFont -width 7	\
		]
		setStatusTip -widget $sel_mode_lbl -text "Selection mode -- BLK == block; NORM == normal"
		bind $sel_mode_lbl <Button-1> "$this switch_sel_mode"

		pack $status_left	-side left -padx 10
		pack $status_middle	-side left
		pack $ins_mode_lbl	-side left -padx 5
		pack $sel_mode_lbl	-side left -padx 5
		pack $status_right	-side right -fill x -padx 10

		# Frame for "Line: x Col: x Total: x"
		set Sbar_CRT_frame [frame $status_left.sbar_crt_frame -bg #DDDDDD]
		# Labels "Line:"
		pack [label $Sbar_CRT_frame.sbar_row_lbl	\
			-text "Line:" -fg {#444444}		\
			-bg #DDDDDD 				\
			-font $statusBarFont -pady 0		\
		] -side left -pady 0
		set Sbar_row [label $Sbar_CRT_frame.sbar_row_val	\
			-fg {#0000AA} -font $statusBarBoldFont -pady 0	\
			-bg #DDDDDD -anchor e -bd 1			\
		]
		pack $Sbar_row -side left -pady 0
		# Labels "Column:"
		pack [label $Sbar_CRT_frame.sbar_col_lbl	\
			-text " Column:" -fg {#444444}		\
			-bg #DDDDDD				\
			-font $statusBarFont -pady 0		\
		] -side left -pady 0
		set Sbar_col	[label $Sbar_CRT_frame.sbar_col_val	\
			-fg {#0000AA} -font $statusBarBoldFont -pady 0	\
			-bg #DDDDDD -anchor e				\
		]
		pack $Sbar_col -side left -pady 0
		# Labels "Total:"
		pack [label $Sbar_CRT_frame.sbar_total_lbl	\
			-text " Total:" -fg {#444444}		\
			-bg #DDDDDD				\
			-font $statusBarFont -pady 0		\
		] -side left -pady 0
		set Sbar_total	[label $Sbar_CRT_frame.sbar_total_val	\
			-fg {#006600} -font $statusBarBoldFont -pady 0	\
			-bg #DDDDDD -anchor e				\
		]
		pack $Sbar_total -side left -pady 0
		# Image label: Lock/Unlock file
		set Sbar_lock_file [Button $status_left.editor_status_left_lock	\
			-padx 5 -pady 0 -bg #DDDDDD -bd 0 -relief flat		\
			-highlightthickness 0 -command "$this invert_lock"	\
			-cursor hand1 -activebackground {#DDDDDD}		\
		]
		set_lock $auto_switching_lock
		# Label: "Simulator mode"
		set Sbar_sim_mode [Label $status_left.editor_status_left_l0	\
			-font $statusBarBoldFont	\
			-fg #DD0000 -bg #DDDDDD		\
			-helptext "Editor status bar"	\
			-padx 5 -pady 0			\
			-text "Simulator mode    "	\
		]
		# Label: "Simulator mode"
		set Sbar_ssim_mode [Label $status_left.editor_status_left_l2	\
			-font $statusBarBoldFont	\
			-fg #555555 -bg #DDDDDD		\
			-helptext "Editor status bar"	\
			-padx 5 -pady 0			\
			-text "Starting simulator"	\
		]
		# Label: "Editor disabled"
		set Sbar_dis_mode [Label $status_left.editor_status_left_l1	\
			-font $statusBarBoldFont	\
			-fg #3333DD -bg #DDDDDD		\
			-helptext "Editor status bar"	\
			-padx 5 -pady 0			\
			-text "Editor disabled"		\
		]
		set Sbar_image [Label $status_middle.editor_status_middle_l	\
			-bg #DDDDDD	\
			-pady 0		\
			-cursor hand1	\
		]
		bind $Sbar_image <Button-1>	"$this save"
		bind $Sbar_image <Double-1>	"$this save"
		setStatusTip -widget $Sbar_image \
			-text "File has been modified, click to save"

		pack $Sbar_CRT_frame

		set Sbar_fileName [Label $status_right.editor_status_right_l	\
			-text $filename			\
			-helptext $filename		\
			-font $statusBarBoldFont	\
			-bg #DDDDDD			\
		]
		pack $Sbar_fileName -side left
		setStatusTip -widget $Sbar_fileName \
			-text "Name of the current file or \"untitled\" if the file has not been saved under any name"

		set Sbar_prog_lang [Label $status_right.sbar_prog_lang	\
			-helptext "File type\n  C/H\tC source / header\n  ASM\tAssembly language\n  LST\tCode listing\n  ASX\tASX8051 assembler"	\
			-font $statusBarBoldFont	\
			-bg #DDDDDD			\
		]
		adjust_sbar_to_prog_lang
		pack $Sbar_prog_lang -side right -padx 5
		setStatusTip -widget $Sbar_prog_lang -text "File type"

		# Set status bar event bindings
		bind $statusbar <ButtonRelease-3> "$this statusbar_popup_menu %X %Y; break"
		foreach widget [winfo children $statusbar] {
			bind $widget <ButtonRelease-3> "$this statusbar_popup_menu %X %Y; break"
			foreach wdg [winfo children $widget] {
				bind $wdg <ButtonRelease-3> "$this statusbar_popup_menu %X %Y; break"
				foreach w [winfo children $wdg] {
					bind $w <ButtonRelease-3> "$this statusbar_popup_menu %X %Y; break"
				}
			}
		}

		# Create text tags
		if {$create_tags} {
			create_highlighting_tags
		}
		define_line_markers
		$editor tag configure c_lang_var_or_func
		$editor tag raise sel tag_current_line
		$editor tag raise sel tag_bookmark
		$editor tag raise sel tag_simulator_curr
		$editor tag raise sel tag_trailing_space
		$editor tag raise tag_error_line	tag_bookmark
		$editor tag raise tag_error_line	tag_trailing_space
		$editor tag raise tag_current_line	tag_bookmark
		$editor tag raise tag_current_line	tag_error_line
		$editor tag raise tag_trailing_space	tag_current_line
		$editor tag raise tag_simulator_curr	tag_current_line
		$editor tag raise tag_simulator_curr	tag_error_line
		$editor tag raise tag_simulator_curr	tag_bookmark
		$editor tag raise tag_simulator_curr	tag_trailing_space

		# Insert the given data
		if {$data != {}} {
			$editor insert end $data
			$editor edit modified 0
			$editor edit reset
		}

		# Reset status modified
		set modified 0

		## Set unredefinable event bindings for editor
		 # Set priorities
		bindtags $editor [list $editor . all]
		 # Special keys
		for {set i 1} {$i < 21} {incr i} {
			bind $editor "<Key-F$i>" {continue}
		}
		bind $editor <Control-Key>	{continue}
		bind $editor <Alt-Key>		{continue}
		 # Keep default
		foreach key {
				<ButtonRelease-1>	<B1-Enter>	<B1-Leave>
				<B2-Motion>		<Button-5>	<Button-4>
				<MouseWheel>
			} {
				bind $editor $key [bind Text $key]
		}

		 # Other
		foreach key {
				<Double-Shift-Button-1>		<Shift-Button-1>
				<Triple-Shift-Button-1>		<Control-Button-1>
				<Control-Shift-Key-Right>	<Control-Shift-Key-Left>
				<Control-Key-Right>		<Control-Key-Left>
				<Shift-Key-Next>		<Shift-Key-Prior>
				<Shift-Key-Right>		<Shift-Key-Left>
				<Shift-Key-End>			<Key-End>
				<Control-Key-T>
			} {
				bind $editor $key "
					[bind Text $key]
					$this rightPanel_adjust \[expr {int(\[%W index insert\])}\]
					$this recalc_status_counter {}
					$this resetUpDownIndex
					break"
		}
		bind $editor <Shift-Button-5>	{
			%W yview scroll 1 page
			break
		}
		bind $editor <Shift-Button-4>	{
			%W yview scroll -1 page
			break
		}
		bind $editor <Double-Button-1> "
			if {\[string is alnum -strict \[%W get insert-1c insert\]\]} {
				[bind Text <Double-Button-1>]
			}

			$this rightPanel_adjust \[expr {int(\[%W index insert\])}\]
			$this recalc_status_counter {}
			$this resetUpDownIndex
			break"
		bind $editor <Shift-Key-Down>	"
			if {\[catch {$this shift_down}\]} {
				[bind Text <Shift-Key-Down>]
				$this rightPanel_adjust \[expr {int(\[%W index insert\])}\]
				$this recalc_status_counter {}
			}
			break"
		bind $editor <Shift-Key-Up>	"
			if {\[catch {$this shift_up}\]} {
				[bind Text <Shift-Key-Up>]
				$this rightPanel_adjust \[expr {int(\[%W index insert\])}\]
				$this recalc_status_counter {}
			}
			break"
		bind $editor <Control-Insert>	"$this copy; break"
		bind $editor <Shift-Insert>	"$this paste; break"
		bind $editor <Shift-Delete>	"$this cut; break"
		bind $editor <Control-Key-Down>	"$this control_down; break"
		bind $editor <Control-Key-Up>	"$this control_up; break"
		bind $editor <Key-Down>		"$this down; break"
		bind $editor <Key-Up>		"$this up; break"
		bind $editor <Key-Home>		"$this home_press; break"
		bind $editor <Shift-Key-Home>	"$this shift_home; break"
		bind $editor <Key-Insert>	"$this switch_ins_ovr; break"
		bind $editor <Key-Tab>		"$this tab_press; break"
		bind $editor <Key-ISO_Left_Tab>	"$this unindent; break"
		bind $editor <<Paste>>		"$this paste; break"
		bind $editor <<Cut>>		"$this cut; break"
		bind $editor <<Copy>>		"$this copy; break"
		bind $editor <Control-Key-V>	"$this paste; break"
		bind $editor <Control-Key-X>	"$this cut; break"
		bind $editor <Control-Key-C>	"$this copy; break"
		bind $editor <<Undo>>		"$this undo; break"
		bind $editor <<Redo>>		"$this redo; break"
		bind $editor <Shift-Return>	"$this shift_enter; break"
		bind $editor <Shift-KP_Enter>	"$this shift_enter; break"
		bind $editor <Return>		"$this enter; break"
		bind $editor <KP_Enter>		"$this enter; break"
		bind $editor <Key>		"$this Key %A; break"
		bind $editor <KeyRelease>	"$this KeyRelease %K; break"
		bind $editor <ButtonRelease-3>	"$this popupMenu %X %Y %x %y; break"
		bind $editor <Key-Menu>		"$this Key_Menu; break"
		bind $editor <Key-Escape>	"$this key_escape"
		bind $editor <Control-Key-y>	"$this delete_current_line; break"
		bind $editor <Key-BackSpace>	"$this key_backspace; break"
		bind $editor <Key-Delete>	"$this key_delete; break"
		bind $editor <Key-Prior> "
			[bind Text <Key-Prior>]
			$this resetUpDownIndex
			$this rightPanel_adjust \[expr {int(\[%W index insert\])}\]
			$this recalc_status_counter {}
			break"
		bind $editor <Key-Next> "
			[bind Text <Key-Next>]
			$this resetUpDownIndex
			$this rightPanel_adjust \[expr {int(\[%W index insert\])}\]
			$this recalc_status_counter {}
			break"
		bind $editor <Control-Key-Next> "
			%W xview scroll -3 page
			$this resetUpDownIndex
			$this rightPanel_adjust \[expr {int(\[%W index insert\])}\]
			$this recalc_status_counter {}
			break"
		bind $editor <Control-Key-Prior> "
			%W xview scroll 3 page
			$this resetUpDownIndex
			$this rightPanel_adjust \[expr {int(\[%W index insert\])}\]
			$this recalc_status_counter {}
			break"
		bind $editor <Key-Left> "
			[bind Text <Key-Left>]
			$this resetUpDownIndex
			$this recalc_status_counter {}
			$this rightPanel_adjust \[expr {int(\[%W index insert\])}\]
			break"
		bind $editor <Key-Right> "
			[bind Text <Key-Right>]
			$this resetUpDownIndex
			$this recalc_status_counter {}
			$this rightPanel_adjust \[expr {int(\[%W index insert\])}\]
			break"
		bind $editor <Control-Key-t> "
			[bind Text <Control-Key-t>]
			$this resetUpDownIndex
			$this parse \[expr {int(\[$editor index insert\])}\]
			update
			break"
		bind $editor <<PasteSelection>> "
			[bind Text <<PasteSelection>>]
			$this resetUpDownIndex
			$this recalc_left_frame
			$this parse \[expr {int(\[$editor index insert\])}\]
			catch {$editor tag remove sel sel.first sel.last}
			update
			break"
		bind $editor <Button-1> "
			[bind Text <Button-1>]
			$this rightPanel_adjust \[expr {int(\[%W index insert\])}\]
			$this resetUpDownIndex
			$this recalc_status_counter {%x %y}
			focus -force $editor
			break"
		bind $editor <B1-Motion> "
			[bind Text <B1-Motion>]
			$this rightPanel_adjust \[expr {int(\[%W index @%x,%y\])}\]
			$this resetUpDownIndex
			$this recalc_status_counter {%x %y}
			break"
		bind $editor <<Selection>> "$this editor_selection; break"


		# Set event bindings for editor command line
		bind $cmd_line <Key-Escape> "
			\${::X::actualProject} cmd_line_off
			pack forget $cmd_line
			catch {$this cmd_line_menu_close_now}
			focus $editor
			update
			break"
		for {set i 1} {$i < 21} {incr i} {
			bind $cmd_line <F$i>		{continue}
			bind $cmd_line <Control-F$i>	{continue}
		}
		bind $cmd_line <Control-Key>	{continue}
		bind $cmd_line <Alt-Key>	{continue}
		bind $cmd_line <Control-a>	{%W tag add sel {insert linestart} {insert lineend}; break}
		bind $cmd_line <Key-Return>	"$this cmd_line_enter;		break"
		bind $cmd_line <Key-KP_Enter>	"$this cmd_line_enter;		break"
		bind $cmd_line <KeyPress>	"$this cmd_line_key_press %A;	break"
		bind $cmd_line <Delete>		"$this cmd_line_key Delete;	break"
		bind $cmd_line <BackSpace>	"$this cmd_line_key BackSpace;	break"
		bind $cmd_line <Home>		"$this cmd_line_key Home;	break"
		bind $cmd_line <End>		"$this cmd_line_key End;	break"
		bind $cmd_line <Left>		"$this cmd_line_key Left;	break"
		bind $cmd_line <Right>		"$this cmd_line_key Right;	break"
		bind $cmd_line <Down>		"if \[$this cmd_line_down\] break"
		bind $cmd_line <Shift-Left>	"if {!\[$this cmd_line_key SLeft\]} {break}"
		bind $cmd_line <Shift-Right>	"if {!\[$this cmd_line_key SRight\]} {break}"
		foreach keysym {Shift-Home Shift-End Up} {
			bind $cmd_line <$keysym> "[bind Text <$keysym>];break"
		}
		foreach keysym {Undo Redo Cut Copy Paste} {
			bind $cmd_line <<$keysym>> "[bind Text <<$keysym>>];break"
		}

		# Create bindings for defined key shortcuts
		shortcuts_reevaluate

		# Create editor popup menu
		set menu $editor.editor_menu
		makePopupMenu

		bind $editor <Configure> "$this Configure"
		bind $editor <FocusIn> "$parentObject filelist_editor_selected $this"
		bind $cmd_line <FocusIn> "$parentObject filelist_editor_selected $this"
 

		# Set event bindings for "Line numbers"
		bind $lineNumbers <<Selection>> "false_selection $lineNumbers; break"
		bind $lineNumbers <Button-1> "
			$parentObject filelist_editor_selected $this
			focus -force $editor
			$this lineNumbers_click %x %y
			break"
		bind $lineNumbers <ButtonRelease-3> "
			$parentObject filelist_editor_selected $this
			focus -force $editor
			$this lineNumbers_popup_menu %X %Y %x %y
			break"

		# Set event bindings for "Icon border"
		bind $iconBorder <<Selection>> "false_selection $iconBorder; break"
		bind $iconBorder <Button-1> "
			$parentObject filelist_editor_selected $this
			focus -force $editor
			$this iconBorder_click %x %y
			break"
		bind $iconBorder <ButtonRelease-3> "
			$parentObject filelist_editor_selected $this
			focus -force $editor
			$this iconBorder_popup_menu %X %Y %x %y
			break"

		# Finalize initialization
		set object_initialized 1
		change_RO_MODE $ro_mode
	}

	## Object destructor
	destructor {
		if {$editor_to_use} {
			kill_childern
		} {
			# Stop autosave timer
			catch {
				after cancel $autosave_timer
			}
			# Cancel highlight dialog timer
			if {$finishigh_hg_dlg_tmr != {}} {
				after cancel $finishigh_hg_dlg_tmr
			}
			# Unregister statusbar tips
			menu_Sbar_remove $menu
			menu_Sbar_remove $IB_menu
			menu_Sbar_remove $LN_menu
		}

		# Destroy main frame
		destroy $ed_sc_frame
	}

	## Refresh list of avaliable SFR's and SFB's on the target uC
	 # @return void
	public method refresh_avaliable_SFR {} {
		set autocompletion_list(4) [$parentObject cget -avaliable_SFR]
	}


	# -------------------------------------------------------------------
	# PROCEDURES RELATED TO EDITOR COMMAND LINE
	# -------------------------------------------------------------------

	## Binding for command line event <KeyPress>
	 # @parm String key - text/binary code of pressed key
	 # @retrun void
	public method cmd_line_key_press {key} {
		if {![string is print -strict $key]} {
			return
		}
		catch {
			$cmd_line delete sel.first sel.last
		}
		$cmd_line insert insert $key
		cmd_line_highlight
	}

	## Binding for various KeyPress events on command line
	 # @parm key - ID of presses key
	 # @retrun Bool - event accepted
	public method cmd_line_key {key} {
		set result 0
		switch -- $key {
			{Delete} {
				if {[llength [$cmd_line tag nextrange sel 1.0]]} {
					$cmd_line delete sel.first sel.last
				} {
					if {[$cmd_line index insert] != [$cmd_line index {insert lineend}]} {
						$cmd_line delete insert {insert+1c}
					}
				}
			}
			{BackSpace} {
				if {[llength [$cmd_line tag nextrange sel 1.0]]} {
					$cmd_line delete sel.first sel.last
				} {
					if {[$cmd_line index insert] != [$cmd_line index {insert linestart}]} {
						$cmd_line delete {insert-1c} insert
					}
				}
			}
			{Home} {
				$cmd_line tag remove sel 1.0 end
				$cmd_line mark set insert {insert linestart}
			}
			{End} {
				$cmd_line tag remove sel 1.0 end
				$cmd_line mark set insert {insert lineend}
			}
			{Left} {
				$cmd_line tag remove sel 1.0 end
				if {[$cmd_line index insert] != [$cmd_line index {insert linestart}]} {
					$cmd_line mark set insert {insert-1c}
				}
			}
			{Right} {
				$cmd_line tag remove sel 1.0 end
				if {[$cmd_line index insert] != [$cmd_line index {insert lineend}]} {
					$cmd_line mark set insert {insert+1c}
				}
			}
			{SLeft} {
				if {[$cmd_line index insert] != [$cmd_line index {insert linestart}]} {
					set result 1
				}
			}
			{SRight} {
				if {[$cmd_line index insert] != [$cmd_line index {insert lineend}]} {
					set result 1
				}
			}
			default {
				error "Unrecognized key: $key"
				return 0
			}
		}
		cmd_line_highlight 1
		return $result
	}

	## Get list of commands which starts with the given string
	 # @parm String cmd - (in)complite command
	 # @retrun List - possible command
	private method cmd_line_get_possible_cmds {cmd} {
		set result {}
		set end [string length $cmd]
		incr end -1
		foreach command $editor_commands {
			set shortcmd [string range $command 0 $end]

			if {$command == $cmd} {
				return $cmd

			} elseif {$shortcmd == $cmd} {
				lappend result $command
			}
		}
		return $result
	}

	## Binding for command line event <Key-Return> and <Key-KP_Enter>
	 # @return void
	public method cmd_line_enter {} {
		if {[winfo exists .editor_cmd_help_widow]} {
			grab release .editor_cmd_help_widow
			destroy .editor_cmd_help_widow
		}

		set line [$cmd_line get {insert linestart} {insert lineend}]
		if {$line == {}} {return}

		if {[catch {
			set command [string tolower [lindex $line 0]]
		}]} then {
			Sbar "Invalid command"
			return
		}
		set options {}
		if {[regexp {\:\w*$} $command options]} {
			set options [string range $options 1 end]
			set options [split $options {}]
			regsub {\:\w*$} $command {} command
		}
		set command [cmd_line_get_possible_cmds $command]
		if {$command == {}} {
			Sbar "EDITOR COMMAND LINE: invalid command, type `help list' to get list of avaliable commands"
			return
		} elseif {[llength $command] > 1} {
			Sbar "Ambiguous command"
			return
		}
		regsub {^[\w\-:]+\s*} $line {} args
		set args [string trim $args]

		switch -- $command {
			{help} {
				if {[llength $args] > 1} {
					Sbar "EDITOR COMMAND LINE: wrong # args (command: char)"
				}
				if {![llength $args]} {
					cmd_line_help_window {Help}	\
					"This is MCU 8051 IDE command line

					usage: <b>command \[argumets\]</b>

					Enter <b>help list</b> for list of avaliable commands or
					<b>help <STRING command></b> for help for individual command"
					return
				}
				set command [lindex $args 0]
				if {$command != {list}} {
					set command [cmd_line_get_possible_cmds $command]
				}
				if {[llength $command] > 1} {
					Sbar "Ambiguous command"
					return
				}

				switch -- $command {
					{list} {
						cmd_line_help_window {Avaliable commands}	\
						"<b>d2h</b>		DEC -> HEX
						<b>d2o</b>		DEC -> OCT
						<b>d2b</b>		DEC -> BIN
						<b>h2d</b>		HEX -> DEC
						<b>h2o</b>		HEX -> OCT
						<b>h2b</b>		HEX -> BIN
						<b>o2h</b>		OCT -> HEX
						<b>o2d</b>		OCT -> DEC
						<b>o2b</b>		OCT -> BIN
						<b>b2h</b>		BIN -> HEX
						<b>b2d</b>		BIN -> DEC
						<b>b2o</b>		BIN -> OCT
						<b>animate</b>		Animate program
						<b>assemble</b>	Compile current file
						<b>auto-indent</b>	Reformat code
						<b>bookmark</b>	Bookmark current line
						<b>breakpoint</b>	Add/Remove breakpoint
						<b>capitalize</b>	Capitalize selection
						<b>clear</b>		Clear history
						<b>comment</b>		Comment slected text
						<b>copy</b>		Copy selection
						<b>custom</b>		Custom command
						<b>cut</b>		Cut selection
						<b>date</b>		Insert date
						<b>exit</b>		Exit command line
						<b>exit-program</b>	Exit program
						<b>find</b>		Find string
						<b>goto</b>		Go to line
						<b>help</b>		Help
						<b>char</b>		Insert literal character
						<b>indent</b>		Indent selection
						<b>kill-line</b>	Remove current line
						<b>open</b>		Open file
						<b>paste</b>		Paste clipboard
						<b>redo</b>		Take back last undo
						<b>reload</b>		Reload current document
						<b>replace</b>		Replace strings
						<b>run</b>		Run simulation
						<b>save</b>		Save current line
						<b>set-icon-border</b>	Show/Hide icon border
						<b>set-line-numbers</b> Show/Hide line n. bar
						<b>sim</b>		Start/Stop simulator
						<b>step</b>		Step program
						<b>tolower</b>		To lowercase
						<b>toupper</b>		To uppercase
						<b>uncomment</b>	Uncomment selection
						<b>undo</b>		Take back last operation
						<b>unindent</b>	Unindent selection
						<b>hibernate</b>	Hibernate running program
						<b>resume</b>		Resume hibernated program
						<b>switch-mcu</b>	Change current MCU
						<b>set-xcode</b>	Set XCODE memory size for current MCU
						<b>set-xdata</b>	SET XDATA memory size for current MCU"
					}
					{hibernate} {
						cmd_line_help_window {Command hibernate}	\
						"<b>hibernate</b> \[<STRING target-file>\]
						Hibernate running program (avaliliable only when simulator is stated).

						This function saves current state of the simulator engine for future resumption. If no target is not specified it will invoke file selection dialog"
					}
					{resume} {
						cmd_line_help_window {Command resume}		\
						"<b>resume</b> \[<STRING source-file>\]
						Resume hibernated program (avaliliable only when simulator is stated).

						This function restores previous state of the simulator engine stored in the given file. If no source is not specified it will invoke file selection dialog"
					}
					{switch-mcu} {
						cmd_line_help_window {Command switch-mcu}	\
						"<b>switch-mcu</b> <STRING processor-type>
						Change current MCU. Type `switch-mcu list' for list of supported microcontrollers"
					}
					{set-xcode} {
						cmd_line_help_window {Command set-xcode}	\
						"<b>set-xcode</b> <INT size>
						Change capacity of external program memory.
						Note: this command also close CODE memory hexeditor"
					}
					{set-xdata} {
						cmd_line_help_window {Command set-xdata}	\
						"<b>set-xdata</b> <INT size>
						Change capacity of external data memory.
						Note: this command also close XDATA memory hexeditor"
					}
					{run} {
						cmd_line_help_window {Command run}	\
						"Run simulation (avaliliable only when simulator is stated)"
					}
					{exit} {
						cmd_line_help_window {Command exit}	\
						"Exits this command line"
					}
					{exit-program} {
						cmd_line_help_window {Command exit-program}	\
						"Quit MCU 8051 IDE"
					}
					{set-icon-border} {
						cmd_line_help_window {Command set-icon-border}	\
						"Sets the visibility of the icon border"
					}
					{set-line-numbers} {
						cmd_line_help_window {Command set-line-numbers}	\
						"Sets the visibility of the line numbers."
					}
					{help} {
						cmd_line_help_window {Command help}	\
						"<b>help</b> <STRING command>
						Shows help for the given command

						<b>help list</b>
						Shows list of avaliable command"
					}
					{open} {
						cmd_line_help_window {Command open}	\
						"<b>open</b> <STRING full_filename>
						Opens the given file in new editor"
					}
					{indent} {
						cmd_line_help_window {Command indent}	\
						"Indents current line or selected area"
					}
					{unindent} {
						cmd_line_help_window {Command unindent}	\
						"Unindents current line or selected area"
					}
					{comment} {
						cmd_line_help_window {Command comment}	\
						"Comments current line or selected area"
					}
					{uncomment} {
						cmd_line_help_window {Command uncomment}	\
						"Uncomments current line or selected area"
					}
					{kill-line} {
						cmd_line_help_window {Command kill-line}	\
						"Removes the current line"
					}
					{date} {
						cmd_line_help_window {Command date}	\
						"<b>date</b> <STRING format>
						Inserts formated date at the current position in text

						<b>Format string:</b>
						%%	=> %
						%a	=> Weekday name (Mon, Tue, etc.)
						%A	=> Weekday name (Monday, Tuesday, etc.)
						%b	=> Month name (Jan, Feb, etc.)
						%B	=> Full month name
						%C	=> Year (19 or 20)
						%d	=> Day of month (01 - 31)
						%D	=> %m/%d/%y
						%h	=> Abbreviated month name.
						%H	=> Hour (00 - 23)
						%I	=> Hour (01 - 12)
						%j	=> Day of year (001 - 366)
						%k	=> Hour (0 - 23)
						%l	=> Hour (1 - 12).
						%m	=> Month (01 - 12)
						%M	=> Minute (00 - 59)
						%n	=> Newline
						%p	=> AM/PM
						%R	=> %H:%M.
						%s	=> Unix timestamp
						%S	=> Seconds (00 - 59)
						%t	=> Tab
						%T	=> %H:%M:%S.
						%u	=> Weekday number (Monday = 1, Sunday = 7)
						%w	=> Weekday number (Sunday = 0, Saturday = 6)
						%y	=> Year without century (00 - 99)
						%Y	=> Year with century (e.g. 1459)"
					}
					{clear} {
						cmd_line_help_window {Command clear}	\
						"Clears command line history"
					}
					{char} {
						cmd_line_help_window {Command char}	\
						"<b>char</b> <NUMBER identifier>
						Inserts literal characters by their numerical identifier.
						Identifier can be in decimal hexadecimal or octal form."
					}
					{goto} {
						cmd_line_help_window {Command goto}	\
						"<b>goto</b> <INT line>
						Go to the given line"
					}
					{replace} {
						cmd_line_help_window {Command replace}	\
						"<b>replace\[:options\]</b> <STRING pattern> <STRING replacement>

						<b>options:</b>
						b	Search backwards
						c	Search from cursor position
						r	Regular expression search
						s	Case sensitive search
						p	Ask before replacement"
					}
					{find} {
						cmd_line_help_window {Command find}	\
						"<b>find\[:options\]</b> <STRING pattern>

						<b>options:</b>
						b	Search backwards
						c	Search from cursor position
						e	Search in the selection only
						r	Regular expression search
						s	Case sensitive search"
					}
					{cut} {
						cmd_line_help_window {Command cut}	\
						"Cut selected text"
					}
					{copy} {
						cmd_line_help_window {Command copy}	\
						"Copy selected text to clipboard"
					}
					{paste} {
						cmd_line_help_window {Command paste}	\
						"Paste clipboard content"
					}
					{tolower} {
						cmd_line_help_window {Command tolower}	\
						"Convert selected text to lowercase"
					}
					{toupper} {
						cmd_line_help_window {Command toupper}	\
						"Convert selected text to upppercase"
					}
					{capitalize} {
						cmd_line_help_window {Command capitalize}	\
						"Capitalize the selected text (convert 1st character to uppercase)"
					}
					{save} {
						cmd_line_help_window {Command save}	\
						"Save the current document"
					}
					{bookmark} {
						cmd_line_help_window {Command bookmark}	\
						"Bookmark the current line"
					}
					{custom} {
						cmd_line_help_window {Command custom}	\
						"<b>custom</b> <INT command_number>
						Execute custom command (see menu Configuration -> Custom commands)"
					}
					{breakpoint} {
						cmd_line_help_window {Command breakpoint}	\
						"Add / Remove breakpoint to the current line"
					}
					{undo} {
						cmd_line_help_window {Command undo}	\
						"Take back last operation"
					}
					{redo} {
						cmd_line_help_window {Command redo}	\
						"Take back last undo"
					}
					{auto-indent} {
						cmd_line_help_window {Command auto-indent}	\
						"Reformat code"
					}
					{reload} {
						cmd_line_help_window {Command reload}	\
						"Reload the current document"
					}
					{assemble} {
						cmd_line_help_window {Command assemble}	\
						"Compile the current document"
					}
					{sim} {
						cmd_line_help_window {Command sim}	\
						"Start / Stop simulator"
					}
					{step} {
						cmd_line_help_window {Command step}	\
						"Step program (avaliliable only when simulator is stated)"
					}
					{animate} {
						cmd_line_help_window {Command animate}	\
						"Animate program (avaliliable only when simulator is stated)"
					}
					{d2h} {
						cmd_line_help_window {Command d2h}	\
						"Convert decimal number to hexadecimal and write result to editor"
					}
					{d2o} {
						cmd_line_help_window {Command d2o}	\
						"Convert decimal number to octal and write result to editor"
					}
					{d2b} {
						cmd_line_help_window {Command d2b}	\
						"Convert decimal number to binary and write result to editor"
					}
					{h2d} {
						cmd_line_help_window {Command h2d}	\
						"Convert hexadecimal number to decimal and write result to editor"
					}
					{h2o} {
						cmd_line_help_window {Command h2o}	\
						"Convert hexadecimal number to octal and write result to editor"
					}
					{h2b} {
						cmd_line_help_window {Command h2b}	\
						"Convert hexadecimal number to binary and write result to editor"
					}
					{o2h} {
						cmd_line_help_window {Command o2h}	\
						"Convert octal number to hexadecimal and write result to editor"
					}
					{o2d} {
						cmd_line_help_window {Command o2d}	\
						"Convert octal number to decimal and write result to editor"
					}
					{o2b} {
						cmd_line_help_window {Command o2b}	\
						"Convert octal number to binary and write result to editor"
					}
					{b2h} {
						cmd_line_help_window {Command b2h}	\
						"Convert binary number to hexadecimal and write result to editor"
					}
					{b2d} {
						cmd_line_help_window {Command b2d}	\
						"Convert binary number to decimal and write result to editor"
					}
					{b2o} {
						cmd_line_help_window {Command b2o}	\
						"Convert binary number to octal and write result to editor"
					}
					default {
						Sbar "EDITOR COMMAND LINE: Unknown command: `[lindex $args 0]'"
					}
				}
			}
			{hibernate} {
				if {[llength $args] > 2} {
					Sbar "EDITOR COMMAND LINE: wrong # args (command: hibernate)"
					return
				}

				if {![llength $args]} {
					::X::__hibernate
				} {
					::X::__hibernate_to [lindex $args 0] $filename
				}
				Sbar "Success"
			}
			{resume} {
				if {[llength $args] > 2} {
					Sbar "EDITOR COMMAND LINE: wrong # args (command: resume)"
					return
				}

				if {![llength $args]} {
					::X::__resume
				} {
					::X::__resume_from [lindex $args 0] $filename
				}
				Sbar "Success"
			}
			{switch-mcu} {
				if {[llength $args] > 2 || ![llength $args]} {
					Sbar "EDITOR COMMAND LINE: wrong # args (command: switch-mcu)"
					return
				}

				set arg [string toupper [lindex $args 0]]
				if {$arg == {list}} {
					set arg {}
					foreach mcu ${::X::avaliable_processors} {
						append arg $mcu
						append arg "\n"
					}
					cmd_line_help_window {Supported microcontrollers} $arg
				} {
					if {[lsearch ${::X::avaliable_processors} $arg] == -1} {
						Sbar "EDITOR COMMAND LINE: Unsupported processor `$arg'"
					} {
						::X::change_processor $arg
					}
				}
				Sbar "Success"
			}
			{set-xcode} {
				if {[llength $args] > 2 || ![llength $args]} {
					Sbar "EDITOR COMMAND LINE: wrong # args (command: set-xcode)"
					return
				}

				set arg [lindex $args 0]
				set error 0
				if {![string first {0x} $arg]} {
					if {![string is xdigit -strict [string range $arg 2 end]]} {set error 1}
					set arg [expr "$arg"]
				} elseif {[string index $arg 0] == {0}} {
					if {![regexp {^[0-7]+$} $arg]} {set error 1}
					set arg [expr "$arg"]
				} else {
					if {![string is digit -strict $arg]} {set error 1}
				}

				if {$error} {
					Sbar "EDITOR COMMAND LINE: Expected integer but got `$arg' (command: set-xcode)"
					return
				}

				set icode [expr {[lindex [${::X::actualProject} cget -procData] 2] * 1024}]
				if {$arg > (0xFFFF - $icode)} {
					Sbar "EDITOR COMMAND LINE: This MCU has CODE memory limit 0x10000 B (65536) (command: set-xdata)"
					return
				}

				if {[lindex [${::X::actualProject} cget -procData] 1] != {yes}} {
					Sbar "This MCU cannot have connected external program memory"
				} {
					${::X::actualProject} configure -P_option_mcu_xcode $arg
					::X::close_hexedit code ${::X::actualProject}
					${::X::actualProject} simulator_resize_code_memory $arg
					Sbar "Success"
				}
			}
			{set-xdata} {
				if {[llength $args] > 2 || ![llength $args]} {
					Sbar "EDITOR COMMAND LINE: wrong # args (command: set-xdata)"
					return
				}

				set arg [lindex $args 0]
				set error 0
				if {![string first {0x} $arg]} {
					if {![string is xdigit -strict [string range $arg 2 end]]} {set error 1}
					set arg [expr "$arg"]
				} elseif {[string index $arg 0] == {0}} {
					if {![regexp {^[0-7]+$} $arg]} {set error 1}
					set arg [expr "$arg"]
				} else {
					if {![string is digit -strict $arg]} {set error 1}
				}

				if {$error} {
					Sbar "EDITOR COMMAND LINE: Expected integer but got `$arg' (command: set-xdata)"
					return
				}

				if {$arg > 0xFFFF} {
					Sbar "EDITOR COMMAND LINE: This MCU has XDATA memory limit 0x10000 B (65536) (command: set-xdata)"
					return
				}

				if {[lindex [${::X::actualProject} cget -procData] 0] != {yes}} {
					Sbar "This MCU cannot have connected external data memory"
				} {
					${::X::actualProject} configure -P_option_mcu_xdata $arg
					::X::close_hexedit xdata ${::X::actualProject}
					${::X::actualProject} simulator_resize_xdata_memory $arg
					Sbar "Success"
				}
			}
			{d2h} {command_line_X2X_command {d2h} $args}
			{d2o} {command_line_X2X_command {d2o} $args}
			{d2b} {command_line_X2X_command {d2b} $args}
			{h2d} {command_line_X2X_command {h2d} $args}
			{h2o} {command_line_X2X_command {h2o} $args}
			{h2b} {command_line_X2X_command {h2b} $args}
			{o2h} {command_line_X2X_command {o2h} $args}
			{o2d} {command_line_X2X_command {o2d} $args}
			{o2b} {command_line_X2X_command {o2b} $args}
			{b2h} {command_line_X2X_command {b2h} $args}
			{b2d} {command_line_X2X_command {b2d} $args}
			{b2o} {command_line_X2X_command {b2o} $args}
			{set-icon-border} {
				::X::__show_hine_IconB
				command_without_args $args
			}
			{set-line-numbers} {
				::X::__show_hine_LineN
				command_without_args $args
			}
			{open} {
				if {![llength $args] || [llength $args] > 1} {
					Sbar "EDITOR COMMAND LINE: wrong # args (command: open)"
				}
				if {$fullFileName != {}} {
					set dir [file dirname $fullFileName]
				} {
					set dir $projectPath
				}
				set filename [file join $dir [lindex $args 0]]
				if {![file exists $filename] || ![file isfile $filename]} {
					Sbar "EDITOR COMMAND LINE: wrong # args (command: unindent)"
				}
				if {[${X::actualProject} openfile $filename 1 . def def 0 0] != {}} {
					${X::actualProject} switch_to_last
					update idle
					${X::actualProject} editor_procedure {} parseAll {}
					Sbar "Success"
				}
			}
			{indent} {
				indent
				command_without_args $args
			}
			{unindent} {
				unindent
				command_without_args $args
			}
			{comment} {
				comment
				command_without_args $args
			}
			{uncomment} {
				uncomment
				command_without_args $args
			}
			{kill-line} {
				detete_text_in_editor {insert linestart} {insert+1l linestart}
				goto [expr {int([$editor index insert])}]
				Sbar "Success"
			}
			{date} {
				if {[catch {$editor insert insert [clock format [clock seconds] -format $args]}]} {
					Sbar "EDITOR COMMAND LINE: Invalid format string"
				} {
					parse [expr {int([$editor index insert])}]
					Sbar "Success"
				}
			}
			{char} {
				if {[llength $args] > 1} {
					Sbar "EDITOR COMMAND LINE: wrong # args (command: char)"
				} {
					Sbar "Success"
				}
				set char [lindex $args 0]
				if {![regexp {^(0|0x|0X)?[0-9]+$} $char]} {
					Sbar "EDITOR COMMAND LINE: syntax error: expected integer (command: char)"
					return
				}
				$editor insert insert [format %c $char]
				parse [expr {int([$editor index insert])}]
			}
			{goto} {
				if {[llength $args] > 1} {
					Sbar "EDITOR COMMAND LINE: wrong # args (command: goto)"
				} {
					Sbar "Success"
				}
				set target_line [lindex $args 0]

				if {![regexp {^(0|0x|0X)?[0-9]+$} $target_line]} {
					Sbar "EDITOR COMMAND LINE: syntax error: expected integer (command: goto)"
					return
				}
				if {$target_line > [editor_linescount]} {
					Sbar "Target line out of range"
				} {
					goto $target_line
					Sbar "Success"
				}
			}
			{replace} {
				if {[llength $args] > 2} {
					Sbar "EDITOR COMMAND LINE: wrong # args (command: replace)"
				} {
					Sbar "Success"
				}
				set pattern [lindex $args 0]
				set replacement [lindex $args 1]
				set Backwards 0
				set fromCursor 0
				set regExp 0
				set noCase 0
				set confirm 0
				set options {}
				foreach opt $options {
					switch -- $opt {
						{b} {	;# Search backwards
							set Backwards 1
						}
						{c} {	;# Search from cursor position
							set fromCursor 1
						}
						{r} {	;# Regular expression search
							set regExp 1
						}
						{s} {	;# Do case sensitive search
							set noCase 1
						}
						{p} {	;# Prompt before replacement
							set confirm 1
						}
						default {
							Sbar "Invalid option: $opt"
						}
					}
				}
				replace	$fromCursor	$Backwards	$regExp		\
					$noCase		$pattern	$replacement	\
					$confirm	::X::replace_prompt
			}
			{find} {
				if {[llength $args] > 1} {
					Sbar "EDITOR COMMAND LINE: wrong # args (command: find)"
				}
				set pattern [lindex $args 0]
				set Backwards 0
				set fromCursor 0
				set regExp 0
				set noCase 0
				set confirm 0
				set inSelection 0
				set options {}
				foreach opt $options {
					switch -- $opt {
						{b} {	;# Search backwards
							set Backwards 1
						}
						{e} {	;# Search in the selection only
							set inSelection 1
						}
						{c} {	;# Search from cursor position
							set fromCursor 1
						}
						{r} {	;# Regular expression search
							set regExp 1
						}
						{s} {	;# Do case sensitive search
							set noCase 1
						}
						default {
							Sbar "Invalid option: $opt"
						}
					}
				}
				set result [find $fromCursor $Backwards $regExp $noCase $inSelection 1.0 $pattern]
				if {[lindex $result 0] == -1} {
					Sbar "Search failed: [lindex $result 1]"
				} {
					Sbar "Found [lindex $result 2] occurences"
				}
			}
			{cut} {
				cut
				command_without_args $args
			}
			{copy} {
				copy
				command_without_args $args
			}
			{paste} {
				paste
				command_without_args $args
			}
			{tolower} {
				lowercase
				command_without_args $args
			}
			{toupper} {
				uppercase
				command_without_args $args
			}
			{capitalize} {
				capitalize
				command_without_args $args
			}
			{save} {
				save
				command_without_args $args
			}
			{bookmark} {
				Bookmark
				command_without_args $args
			}
			{breakpoint} {
				Breakpoint
				command_without_args $args
			}
			{undo} {
				undo
				command_without_args $args
			}
			{redo} {
				redo
				command_without_args $args
			}
			{auto-indent} {
				::X::__reformat_code
				command_without_args $args
			}
			{reload} {
				::X::__reload
				command_without_args $args
			}
			{assemble} {
				::X::__compile 0
				command_without_args $args
			}
			{sim} {
				::X::__initiate_sim
				command_without_args $args
			}
			{step} {
				::X::__step
				command_without_args $args
			}
			{animate} {
				::X::__animate
				command_without_args $args
			}
			{run} {
				::X::__run
				command_without_args $args
			}
			{custom} {
				set cmd [lindex $args 0]
				if {$cmd != 0 && $cmd != 1 && $cmd != 2}
				__exec_custom_cmd $cmd
				Sbar "Success"
			}
			{clear} {
				$cmd_line delete 1.0 end
				command_without_args $args
				return
			}
			{exit-program} {
				::X::__exit
			}
			{exit} {
				pack forget $cmd_line
				focus $editor
				command_without_args $args
			}
			default {
				Sbar "EDITOR COMMAND LINE: invalid command, type `help list' to get list of avaliable commands"
			}
		}

		# Manage command line history
		if {int([$cmd_line index end-1l]) == int([$cmd_line index insert])} {
			$cmd_line insert {insert lineend} "\n"
		} {
			set txt [$cmd_line get {insert linestart} {insert lineend}]
			$cmd_line mark set insert end
			$cmd_line insert insert $txt
			cmd_line_highlight 1
			$cmd_line insert insert "\n"
		}
		$cmd_line mark set insert end
		$cmd_line see end
	}

	## Auxiliary procedure for "cmd_line_enter"
	 # Performs number base conversions
	 # @parm String command		- Type of conversion (e.g h2b, d2o)
	 # @parm String argument	- Number to convert
	 # @retrun void
	private method command_line_X2X_command {command argument} {
		set len [llength $argument]
		if {!$len} {
			Sbar "EDITOR COMMAND LINE: This command requires exactly one argument"
			return
		}

		set argument [lindex $argument 0]

		switch -- $command {
			{d2h} {	;# DEC -> HEX
				set ver_cmd	{isdec}
				set conv_cmd	{dec2hex}
			}
			{d2o} {	;# DEC -> OCT
				set ver_cmd	{isdec}
				set conv_cmd	{dec2oct}
			}
			{d2b} {	;# DEC -> BIN
				set ver_cmd	{isdec}
				set conv_cmd	{dec2bin}
			}
			{h2d} {	;# HEX -> DEC
				set ver_cmd	{ishex}
				set conv_cmd	{hex2dec}
			}
			{h2o} {	;# HEX -> OCT
				set ver_cmd	{ishex}
				set conv_cmd	{hex2oct}
			}
			{h2b} {	;# HEX -> BIN
				set ver_cmd	{ishex}
				set conv_cmd	{hex2bin}
			}
			{o2h} {	;# OCT -> HEX
				set ver_cmd	{isoct}
				set conv_cmd	{oct2hex}
			}
			{o2d} {	;# OCT -> BIN
				set ver_cmd	{isoct}
				set conv_cmd	{oct2dec}
			}
			{o2b} {	;# OCT -> BIN
				set ver_cmd	{isoct}
				set conv_cmd	{oct2bin}
			}
			{b2h} {	;# BIN -> HEX
				set ver_cmd	{isbin}
				set conv_cmd	{bin2hex}
			}
			{b2d} {	;# BIN -> DEC
				set ver_cmd	{isbin}
				set conv_cmd	{bin2dec}
			}
			{b2o} {	;# BIN -> OCT
				set ver_cmd	{isbin}
				set conv_cmd	{bin2oct}
			}
			default {
				error "Unknown error in Editor.command_line_X2X_command()"
			}
		}

		if {![::NumSystem::$ver_cmd $argument]} {
			Sbar "EDITOR COMMAND LINE: Invalid number format"
			return
		}

		$editor insert insert [::NumSystem::$conv_cmd $argument]
		parse [expr {int([$editor index insert])}]
		Sbar "Success"
	}

	## Check if the given list of arguments is empty and display result on main statusbar
	 # @parm List args - list of arguments
	 # @retrun void
	private method command_without_args {args} {
		if {[llength $args]} {
			if {[llength $args] != 1 || [lindex $args 0] != {}} {
				Sbar "EDITOR COMMAND LINE: This command takes no arguments"
			}
		}
		Sbar "Success"
	}

	## Highlight current contents of editor command line
	 # @parm Bool = 0 - Disable popup-based completion
	 # @return void
	private method cmd_line_highlight args {
		# Remove all tags from command line
		foreach tag {tag_cmd tag_argument tag_option tag_error} {
			$cmd_line tag remove $tag {insert linestart} {insert lineend}
		}

		# Get line contents and line number
		set line [$cmd_line get {insert linestart} {insert lineend}]
		set lineNumber [expr {int([$cmd_line index insert])}]

		# Split line into command and arguments
		set cmd {}
		set opt {}
		if {[regexp {^[\w:-]+} $line cmd]} {
			if {[regexp {\:\w*$} $cmd opt]} {
				set cmd [regsub {\:\w*$} $cmd {}]
			}
		}

		# Resolve possible commands, start index and end index
		set command [cmd_line_get_possible_cmds $cmd]
		set startIdx [$cmd_line index {insert linestart}]
		set endIdx [string length $cmd]
		# Highlight command (+ post down list of possible commands | + return)
		if {[llength $command] == 1} {
			$cmd_line tag add tag_cmd $startIdx $lineNumber.$endIdx
		} elseif {$endIdx && [llength $command] > 1 && [$cmd_line compare $lineNumber.$endIdx == insert]} {
			if {$cline_completion && $args != 1} {
				# Automaticaly complete command
				set possible_cmd [lindex $command 0]
				set insert [$cmd_line index insert]
				$cmd_line insert insert [string range $possible_cmd $endIdx end]
				$cmd_line tag remove sel 1.0 end
				$cmd_line tag add sel $lineNumber.$endIdx $lineNumber.[string length $possible_cmd]
				$cmd_line mark set insert $insert
				# Postdown completion menu
				cmd_line_menu_postdown $command
			}
			return
		} else {
			$cmd_line tag add tag_error $startIdx $lineNumber.$endIdx
		}
		# Insure than list of possible commands is not visible
		cmd_line_menu_close

		# Highlight command options (characters after colon)
		if {$opt != {}} {
			set startIdx $endIdx
			incr endIdx [string length $opt]
			if {[lsearch $commands_with_option $command] != -1} {
				$cmd_line tag add tag_option $lineNumber.$startIdx $lineNumber.$endIdx
			} {
				$cmd_line tag add tag_error $lineNumber.$startIdx $lineNumber.$endIdx
			}
		}

		# Highlight remaining as arguments
		$cmd_line tag add tag_argument $lineNumber.$endIdx {insert lineend}
	}

	## Postdown list of possible commands for command line
	 # @parm List commands - contents of the list
	 # @retrun void
	private method cmd_line_menu_postdown {commands} {
		# List is not mapped -> create new toplevel window
		if {![winfo exists $cmd_line_listbox]} {
			# Determinate window coordinates and width
			set x [winfo rootx $cmd_line]
			set y [expr {[winfo rooty $cmd_line] + [winfo height $cmd_line]}]
			set width [winfo width $cmd_line]

			# Create window
			set win {.editor_cmd_help_widow}
			if {[winfo exists $win]} {
				grab release $win
				destroy $win
			}
			toplevel $win -background {#000000} -class {Menu}
			wm overrideredirect $win 1
			wm geometry $win "=${width}x100+${x}+${y}"
			bind $win <ButtonPress-1> "$this cmd_line_win_B1 %X %Y"
			bind $win <FocusOut> "$this cmd_line_menu_close_now"

			# Create listbox of possible commands
			set frame [frame $win.frame]
			set cmd_line_listbox [ListBox $frame.lisbox	\
				-relief flat -bd 0 -selectfill 1	\
				-selectbackground {#AAAAFF}		\
				-yscrollcommand "$frame.scrollbar set"	\
				-selectmode single -highlightthickness 0\
			]
			pack $cmd_line_listbox -fill both -expand 1 -side left
			# Create scrollbar
			pack [scrollbar $frame.scrollbar		\
				-orient vertical			\
				-command "$cmd_line_listbox yview"	\
			] -side right -after $cmd_line_listbox -fill y
			pack $frame -padx 1 -pady 1 -fill both -expand 1

			# Configure listbox event bindings
			if {[winfo exists $cmd_line_listbox.c]} {
				bind $cmd_line_listbox.c <Button-5>	{%W yview scroll +5 units; break}
				bind $cmd_line_listbox.c <Button-4>	{%W yview scroll -5 units; break}
			}
			bind $cmd_line_listbox <Key-Escape> "$this cmd_line_menu_close_now"
			bind $cmd_line_listbox <Key-Return>	\
				"$this cmd_line_listbox_sel \[$cmd_line_listbox selection get\]"
			bind $cmd_line_listbox <Key-KP_Enter>	\
				"$this cmd_line_listbox_sel \[$cmd_line_listbox selection get\]"
			$cmd_line_listbox bindText <Button-1>	"$this cmd_line_listbox_sel"
			$cmd_line_listbox bindText <Enter>	"
				$cmd_line_listbox selection clear
				$cmd_line_listbox selection set"

			# Finalize window initialization (global grab)
			update idle
			catch {
				grab -global $win
				raise $win
			}
		}

		# Fill listbox with the given list of commands
		$cmd_line_listbox delete [$cmd_line_listbox items]
		foreach cmd $commands {
			$cmd_line_listbox insert end #auto -text $cmd
		}
	}

	## Binding for command line event <Key-Down>
	 # Possible results:
	 #	A) Focus on list of possible commands
	 #	C) Return 0
	 # @retrun Bool - $cmd_line_listbox focused
	public method cmd_line_down {} {
		if {![winfo exists $cmd_line_listbox]} {
			return 0
		} {
			$cmd_line_listbox selection set [$cmd_line_listbox item 0]
			focus -force $cmd_line_listbox
			return 1
		}
	}

	## Clear command line and write there text
	 # of the given item in list of possible commands.
	 # This procedure also closes command line help window
	 # @parm String item - ID of choosen item
	 # @retrun void
	public method cmd_line_listbox_sel {item} {
		$cmd_line delete {insert linestart} {insert lineend}
		$cmd_line insert end [$cmd_line_listbox itemcget $item -text]
		$cmd_line tag add tag_cmd {insert linestart} {insert lineend}
		cmd_line_menu_close
	}

	## Binding common for all command line help windows: event <ButtonPress-1>
	 # This procedure will close help window if user click outside it.
	 # @parm Int X - absolute horizontal position of mouse pointer
	 # @parm Int Y - absolute vertical position of mouse pointer
	 # @retrun void
	public method cmd_line_win_B1 {X Y} {
		set win {.editor_cmd_help_widow}
		set min_x [winfo rootx $win]
		set min_y [winfo rooty $win]
		set max_x [expr {$min_x + [winfo width $win]}]
		set max_y [expr {$min_y + [winfo height $win]}]

		if {$X > $max_x || $X < $min_x || $Y > $max_y || $Y < $min_y} {
			cmd_line_menu_close_now
		}
	}

	## Close command line help window if opened
	 # @retrun void
	public method cmd_line_menu_close {} {
		if {![winfo exists $cmd_line_listbox]} {
			return
		}
		cmd_line_menu_close_now
	}

	## Unconditionaly close command line help window
	 # @retrun void
	public method cmd_line_menu_close_now {} {
		grab release .editor_cmd_help_widow
		destroy .editor_cmd_help_widow
		if {[winfo viewable $cmd_line]} {
			focus -force $cmd_line
		}
	}

	## Create command line help window for purpose of help
	 # @parm String header	- Window header
	 # @parm String content	- Window contents (can contain tags <b>bold</b>)
	 # @return Widget - Text widget
	private method cmd_line_help_window {header content} {
		# Destroy the previous window
		if {[winfo exists .editor_cmd_help_widow]} {
			grab release .editor_cmd_help_widow
			destroy .editor_cmd_help_widow
		}

		# Create window
		set win [toplevel .editor_cmd_help_widow]
		wm overrideredirect $win 1

		bind $win <ButtonPress-1> "$this cmd_line_win_B1 %X %Y"
		bind $win <FocusOut> "$this cmd_line_menu_close_now"

		# Create header
		set header_frame [frame $win.header]
		pack [label $header_frame.lbl_heder	\
			-text $header -fg {#FF0000}	\
			-bg {#AAAAFF} -bd 0 -anchor w	\
			-relief flat			\
			-font $cmd_line_win_font	\
		] -fill x -expand 1 -side left -ipadx 5
		pack [Button $header_frame.lbl_close	\
			-text "Close" -bd 0 -pady 0	\
			-compound left -cursor hand1	\
			-bg {#AAAAFF} -relief flat	\
			-fg {#FFFFFF} -anchor e		\
			-image ::ICONS::16::button_cancel	\
			-font $cmd_line_win_font	\
			-helptext "Close this window"	\
			-command "destroy $win"		\
		] -fill none -side left -pady 0 -ipady 0
		pack $header_frame -fill x -side top

		# Create text widget
		set text [text $win.text	\
			-bg {#FFFFFF}		\
			-cursor left_ptr	\
			-bd 1			\
			-width 0 -height 0	\
			-font $cl_hw_nrml_font	\
			-yscrollcommand "$win.scrollbar set"	\
		]
		$text tag configure tag_bold -font $cl_hw_bold_font

		# Create map of bold font tags
		regsub -all -line {^\t+} $content {} content
		set bold_tag_map {}
		while 1 {
			set tag_pair {}

			set idx [string first {<b>} $content]
			if {$idx == -1} {break}
			regsub {<b>} $content {} content
			lappend tag_pair $idx

			set idx [string first {</b>} $content]
			if {$idx == -1} {break}
			regsub {</b>} $content {} content
			lappend tag_pair $idx

			lappend bold_tag_map $tag_pair
		}

		# Adjust content and insert tags
		set start [$text index insert]
		$text insert end $content
		foreach pair $bold_tag_map {
			$text tag add tag_bold $start+[lindex $pair 0]c $start+[lindex $pair 1]c
		}
		$text configure -state disabled

		# Create and pack scrollbar
		pack $text -side left -fill both -expand 1
		pack [scrollbar $win.scrollbar	\
			-orient vertical	\
			-command "$text yview"	\
		] -side right -fill y -after $text

		# Show the window
		set x [winfo rootx $cmd_line]
		set y [expr {[winfo rooty $cmd_line] + [winfo height $cmd_line]}]
		update idle
		if {150 > ([winfo height .] - $y)} {
			incr y -150
			incr y -[winfo height $cmd_line]
		}
		catch {
			wm transient $win .
			grab -global $win
			update
			wm geometry $win "=[winfo width $cmd_line]x150+${x}+${y}"
		}
	}

	## Focus on editor / editor command line
	 # @parm Bool = 0 - Do not call proc. "cmd_line_on"
	 # @return void
	public method cmd_line_focus args {
		# Show command line
		if {![winfo viewable $cmd_line]} {
			pack $cmd_line -side top -fill x
			if {$args != 1} {
				${::X::actualProject} cmd_line_on
			}
		}

		if {[focus] == $cmd_line} {
			focus $editor
		} {
			focus $cmd_line
		}
		update
	}


	# -------------------------------------------------------------------
	# GENERAL PURPOSE PROCEDURES
	# -------------------------------------------------------------------

	## Insure than simulator cursor is in visible area
	 # @return void
	public method see_sim_cursor {} {
		if {[llength [$editor tag nextrange tag_simulator_curr 1.0 end]]} {
			$editor see tag_simulator_curr.first
		}
	}

	## Clear list of words for autocompletion window
	 # @return void
	public method clear_autocompletion_list {} {
		for {set i 0} {$i < 4} {incr i} {
			set autocompletion_list($i) {}
		}
	}

	## Delete some string in the editor
	 # It's important to delete strings in this way in order to keep
	 #+ autocompletion list up to date
	 # @parm TextIndex start_index	- Start
	 # @parm TextIndex end_index	- End
	 # @return void
	public method detete_text_in_editor {start_index end_index} {
		autocompletion_maybe_important_change $start_index $end_index
		$editor delete $start_index $end_index
	}

	## Inform autocompletion mechanism about possibly deleted symbol
	 # @parm TextIndex start_index	- Begining on area to to analyze
	 # @parm TextIndex end_index	- End on area to to analyze
	 # @return void
	private method autocompletion_maybe_important_change {start_index end_index} {
		# Abort if autocompletion was disabled
		if {!$auto_completion} {
			return
		}

		# Detect new symvol
		foreach tag_name {tag_label tag_constant_def c_lang_var_or_func tag_macro_def} \
			index {0 1 2 3} \
		{
			# Detect ...
			set range [$editor tag nextrange $tag_name			\
				[list $start_index linestart] [list $end_index lineend]	\
			]

			# Nothing detected ...
			if {![llength $range]} {
				continue
			}

			# Get symbol name
			set string [$editor get [lindex $range 0] [lindex $range 1]]

			# Adjust case (all to uppercase except C lang. symbols)
			if {$index != 2} {
				set string [string toupper $string]
			}

			# Remove semicolon from labels in assembly
			if {!$index} {
				set string [string replace $string end end]
			}

			# Adjust autocompletion list
			set idx [lsearch -ascii -exact $autocompletion_list($index) $string]
			if {$idx != -1} {
				set autocompletion_list($index)	\
					[lreplace $autocompletion_list($index) $idx $idx]
			}
		}

		# Remove tag created by "autocompletion_c_syntax_analyze"
		$editor tag remove c_lang_var_or_func $start_index $end_index
	}
	
	## Significant part of autocompletion mechanism related to C language
	 # Creates tag "c_lang_var_or_func" to mark C symbols
	 # @parm Int line_number - Line number
	 # @return void
	private method autocompletion_c_syntax_analyze {line_number} {
		# Abort if autocompletion was disabled
		if {!$auto_completion} {
			return
		}

		# Find word after data type specification
		set prev_range {}
		set range [list $line_number.0 $line_number.0]
		while {1} {
			set range [$editor tag nextrange tag_c_data_type	\
				[lindex $range 1] [list $line_number.0 lineend]	\
			]
			if {![llength $range]} {
				break
			}
			set prev_range $range
		}

		# Nothing found -> abort
		if {![llength $prev_range]} {
			return
		}

		# Gain details about the word
		set range $prev_range
		set line [$editor get $line_number.0 [list $line_number.0 lineend]]
		set start [lindex [split [lindex $range 1] {.}] 1]
		set end 0
		set string {}

		# Find firts part which consist of alfanumeric characters
		if {![regexp -start $start -- {\w+} $line string]} {
			return
		}
		set start [string first $string $line $start]
		set end [expr {$start + [string length $string]}]

		# Mark the word
		$editor tag add c_lang_var_or_func $line_number.$start $line_number.$end
	}

	## Inform autocompletion mechanism about possibly newly defined symbol
	 # @parm Int line_number - Line number
	 # @return void
	private method manage_autocompletion_list {line_number} {
		# Abort if autocompletion was disabled
		if {!$auto_completion} {
			return
		}
		
		# Detect new symvol
		foreach tag_name {tag_label tag_constant_def c_lang_var_or_func tag_macro_def} \
			index {0 1 2 3} \
		{
			# Detect ...
			set range [$editor tag nextrange $tag_name		\
				$line_number.0 [list $line_number.0 lineend]	\
			]

			# Nothing detected ...
			if {![llength $range]} {
				continue
			}

			# Get symbol name
			set string [$editor get [lindex $range 0] [lindex $range 1]]

			# Check if it not already defined
			if {$index != 2} {
				set string [string toupper $string]
			} {
				if {
					[lsearch -ascii -exact ${::CsyntaxHighlight::data_types} $string] != -1
						||
					[lsearch -ascii -exact ${::CsyntaxHighlight::keywords} $string] != -1
				} then {
					return
				}
			}

			# Remove semicolon from labels in assembly
			if {!$index} {
				set string [string replace $string end end]
			}

			# Append to the autocompletion list
			if {[lsearch -ascii -exact $autocompletion_list($index) $string] == -1} {
				lappend autocompletion_list($index) $string
			}
		}

		# Sort autocompletion list every 20nd iteration
		incr macl_invocations
		if {$macl_invocations > 20} {
			set macl_invocations 0
			for {set i 0} {$i < 4} {incr i} {
				set autocompletion_list($i) [lsort -ascii $autocompletion_list($i)]
			}
		}
	}

	## Invoke popup menu completon window
	 # @parm Bool mode	- Mode of autocompletion
	 #	0 - Instructions, directives and macro's
	 #	1 - Constants and labels
	 #	2 - C functions
	 #	3 - Indirect values
	 #	4 - Doxygen tags
	 # @parm String str	- Incomplite instruction or directive
	 # @parm Int x		- Relative X position of the popup window (relative to editor)
	 # @parm Int y		- Relative Y position of the popup window (relative to editor)
	 # @return void
	private method invoke_completion_popup_window {mode start_idx end_idx} {
		if {$invoke_com_win_in_p} {
			update
			return
		}
		set invoke_com_win_in_p 1

		set bbox [$editor bbox $start_idx]
		if {![llength $bbox]} {
			set invoke_com_win_in_p 0
			return
		}
		set x [lindex $bbox 0]
		set y [expr {[lindex $bbox 1] + [lindex $bbox 3]}]
		set str [$editor get $start_idx $end_idx]

		if {![string length $str]} {
			close_completion_popup_window
			set invoke_com_win_in_p 0
			return
		}

		set loading 0
		if {!$comp_win_loading_in_p && [string first 0 $highlighted_lines 1] > 0} {
			set loading 1
			set ::X::compilation_progress 0
			set comp_win_loading_max [highlight_all_count_of_iterations]
		}
 
		# Close current window if any
		close_completion_popup_window

		# Adjust arguments
		set str_org $str

		# Set opened flag
		set completion_win_opened 1
		set do_not_hide_comp_win 1
		
		set completion_win_str_i $start_idx
		set completion_win_end_i $end_idx
		set completion_win_mode $mode

		# Create window
		if {![winfo exists .completion_win]} {
			set win [frame .completion_win -background {#0000DD}]
			bind $win <Button-1> "$this completion_popup_window_but1 %X %Y"
			bind $win <FocusOut> "$this close_completion_popup_window"
			bind $win <Destroy> "
				catch {$this detete_text_in_editor sel.first sel.last}
				$this parse \[expr {int(\[$editor index insert\])}\]"
			bind $win <Key-Escape> "
				catch {$this detete_text_in_editor sel.first sel.last}
				$this close_completion_popup_window"

			# Create lisbox and scrollbar
			set frame [frame $win.frame]
			set listbox [ListBox $frame.listbox		\
				-relief flat -bd 0 -selectfill 1	\
				-selectbackground {#AAAAFF}		\
				-bg white -cursor left_ptr	\
				-yscrollcommand "$frame.scrollbar set"	\
				-selectmode single -width 0 -height 0	\
				-highlightthickness 0 -padx 2		\
			]
			set completion_listbox $listbox
			pack $listbox -side left -fill both -expand 1
			pack [scrollbar $frame.scrollbar	\
				-orient vertical		\
				-command "$listbox yview"	\
				-width 10 -bd 0			\
			] -side right -after $listbox -fill y
			
			ProgressBar .completion_win.progress_bar 	\
				-troughcolor #DDDDDD			\
				-type normal -height 4 -bd 0		\
				-variable {::X::compilation_progress}	\
			
			pack $frame -padx 1 -pady 1 -fill both -expand 1
			
			$listbox bindText <Button-1>	"$this completion_accept"
			$listbox bindText <Escape>	"$this close_completion_popup_window"
			bind $listbox <Key-Return>	"$this completion_accept \[$listbox selection get\]"
			bind $listbox <KP_Enter>	"$this completion_accept \[$listbox selection get\]"
			bind $listbox <Escape>		"$this close_completion_popup_window"
			if {[winfo exists $listbox.c]} {
				bind $listbox.c <Button-5>	{%W yview scroll +1 units; break}
				bind $listbox.c <Button-4>	{%W yview scroll -1 units; break}
			}
		}
		set listbox ".completion_win.frame.listbox"
		$listbox selection clear
		$listbox delete [$listbox items]
		update idle

		if {$loading || $comp_win_loading_in_p} {
			if {!($comp_win_loading_max > 1)} {
				set comp_win_loading_max 1
			}
			.completion_win.progress_bar configure -maximum $comp_win_loading_max
			catch {
				pack .completion_win.progress_bar -fill x -pady 0
			}
		}

		# Fill up listbox
		set end [string length $str]
		incr end -1

		set last_inserted {}
		set string_width 0
		set required_width 70
		if {$mode != 2 && $mode != 4} {
			set str [string toupper $str]
		}
		if {$mode == 3} {
			foreach command {@R0 @R1 @DPTR @A+DPTR @A+PC} {
				set shortcmd [string range $command 0 $end]
				if {$shortcmd == $str} {
					set last_inserted $command
					$listbox insert end #auto	\
						-text $command		\
						-fg $indirect_fg	\
						-font $indirect_font

					set string_width [font measure $defaultFont $command]
					if {$required_width < $string_width} {
						set required_width $string_width
					}
				}
			}
		} else {
			for {set i 0} {$i < 7} {incr i} {
				switch -- $i {
					0 {
						if {$mode != 1} {
							continue
						}
						set color $label_fg
						set font $label_font
					}
					1 {
						if {$mode != 1} {
							continue
						}
						set color $const_fg
						set font $const_font
					}
					2 {
						if {$mode != 2 || $prog_language != 1} {
							continue
						}
						set color {black}
						set font $defaultFont
					}
					3 {
						if {$mode != 0} {
							continue
						}
						set color $macro_fg
						set font $macro_font
					}
					4 {
						if {$mode != 1} {
							continue
						}
						set color $sfr_fg
						set font $sfr_font
					}
					5 {
						if {$mode != 1} {
							continue
						}
						set color $symbol_fg
						set font $symbol_font
					}
					6 {
						if {$mode != 4} {
							continue
						}
						set color $doxytag_fg
						set font $doxytag_font
					}
				}
				foreach command $autocompletion_list($i) {
					set shortcmd [string range $command 0 $end]
					if {$shortcmd == $str} {
						set last_inserted $command
						$listbox insert end #auto	\
							-text $command		\
							-fg $color		\
							-font $font

						set string_width [font measure $defaultFont $command]
						if {$required_width < $string_width} {
							set required_width $string_width
						}
					}
				}
			}
		}
		if {$mode == 0} {
			foreach command ${::ASMsyntaxHighlight::instructions} {
				set shortcmd [string range $command 0 $end]
				if {$shortcmd == $str} {
					set last_inserted $command
					$listbox insert end #auto	\
						-text $command		\
						-fg $ins_fg		\
						-font $ins_font
				}
			}
			foreach command ${::ASMsyntaxHighlight::all_directives} {
				set shortcmd [string range $command 0 $end]
				if {$shortcmd == $str} {
					set last_inserted $command
					$listbox insert end #auto	\
						-text $command		\
						-fg $dir_fg		\
						-font $dir_font
				}
			}
		}

		set num_of_items [llength [$listbox items]]

		# If the listbox is empty -> delete window
		set do_not_show 0
		if {!$num_of_items} {
			set do_not_show 1

		} elseif {$num_of_items == 1 && $last_inserted == $str} {
			set do_not_show 1

		# Automaticaly complete the incomplete command
		} else {
			set command	[$listbox itemcget [$listbox item 0] -text]
			set insert	[$editor index insert]
			set cmd_len	[string length $command]
			set str_len	[string length $str]

			if {$mode != 2 && ![string is upper [regsub -all {[_\d@]} $str_org {}]]} {
				set command [string tolower $command]
			}

			$editor configure -autoseparators 0
			catch {$editor tag remove sel 1.0 end}
			$editor insert insert [string range $command $str_len end]
			$editor mark set insert $insert
			$editor tag add sel insert insert+[expr {$cmd_len - $str_len}]c
			$editor edit separator
			$editor configure -autoseparators 1
			parse [expr {int([$editor index insert])}]
		}

		# Do not display the window
		if {$do_not_show} {
			set invoke_com_win_in_p 0
			close_completion_popup_window
			
		# Display the window
		} else {
			place .completion_win -width [expr {$required_width + 30}]	\
				-height 105 -anchor nw -x $x -y $y -in $editor
			raise .completion_win
			update
			catch {
				grab -global .completion_win
			}
		}
		set invoke_com_win_in_p 0

		# Highlight all in background to gain autocompletion list
		if {$loading && !$comp_win_loading_in_p} {
			comp_win_highlight_all_in_background
		}
	}

	## Auxiliary method for "comp_win_highlight_all_in_background"
	 #+ (Highlight all in background), part of autocompletion mechanism
	 # @return void
	public method comp_win_highlight_all_in_background_AUX {} {
		if {!$comp_win_loading_in_p} {
			set comp_win_loading_in_p 1
			highlight_all
			set comp_win_loading_in_p 0
			catch {
				pack forget .completion_win.progress_bar
			}
		}
	}

	## Highlight all in background
	 # @return void
	private method comp_win_highlight_all_in_background {} {
		after 0 "$this comp_win_highlight_all_in_background_AUX"
	}

	## Informs editor about that than autocompletion has been turned on
	 # @return void
	public method autocompletion_turned_on {} {
		set highlighted_lines [string repeat 0 [string bytelength $highlighted_lines]]
	}

	## Completion -- accept selection
	 # @parm Sring item - Listbox item
	 # @return void
	public method completion_accept {item} {
		if {$item == {}} {
			return
		}

		if {[llength [$editor tag nextrange sel 1.0]]} {
			$editor delete sel.first sel.last
		}

		set item [$completion_listbox itemcget $item -text]
		set text_org [$editor get $completion_win_str_i $completion_win_end_i]

		if {$completion_win_mode != 2 && ![string is upper [regsub -all {[_\d@]} $text_org {}]]} {
			set item [string tolower $item]
		}
		$editor delete $completion_win_str_i $completion_win_end_i
		$editor insert $completion_win_str_i $item

		set line [expr {int([$editor index insert])}]
		recalc_status_counter {} 0
		parse $line
		rightPanel_adjust $line

		close_completion_popup_window
	}

	## Close completion popup window if user clicked out of it
	 # @parm Int X - absolute horizontal position of mouse pointer
	 # @parm Int Y - absolute vertical position of mouse pointer
	 # @retrun void
	public method completion_popup_window_but1 {X Y} {
		set min_x [winfo rootx .completion_win]
		set min_y [winfo rooty .completion_win]
		set max_x [expr {$min_x + [winfo width .completion_win]}]
		set max_y [expr {$min_y + [winfo height .completion_win]}]

		if {$X > $max_x || $X < $min_x || $Y > $max_y || $Y < $min_y} {
			close_completion_popup_window
		}
	}

	## Unconditionaly safely close completion popup window
	 # @return void
	public method close_completion_popup_window {} {
		if {$invoke_com_win_in_p} {return}
		set invoke_com_win_in_p 1

		if {$completion_win_opened} {
			catch {$editor delete sel.first sel.last}
			grab release .completion_win
			place forget .completion_win
			focus -force $editor
			parse [expr {int([$editor index insert])}]
		}

		set completion_win_opened 0
		set invoke_com_win_in_p 0
	}

	## Binding for event <Configure>
	 # Commit new width of the editor widget -- for correct line wrapping
	 # @return void
	public method Configure {} {
		# Check if program is loaded
		if {!${::APPLICATION_LOADED}} {return}

		# Determinate width of active area
		set width [winfo width $editor]
		incr width -6
		incr width [expr {-($width % $defaultCharWidth)}]
		# If width chaged -> adjust line wrapping
		if {$editor_width != $width} {
			set editor_width $width
			set highlighted_lines [string repeat 0 [string bytelength $highlighted_lines]]
			highlight_visible_area
		}
	}

	## Refresh color setting (excluding highlightind)
	 # @return void
	public method change_colors {} {
		if {$editor_to_use} {return}

		$lineNumbers	configure -bg $lineNumbers_bg -fg $lineNumbers_fg
		$left_frame_R	configure -bg $lineNumbers_bg

		$iconBorder	configure -bg $iconBorder_bg
		$left_frame_L	configure -bg $iconBorder_bg

		$editor		configure -bg $normal_text_bg
	}

	## Refresh font setting (excluding highlightind)
	 # @return void
	public method refresh_font_settings {} {
		if {$editor_to_use} {return}

		# Remove all text tags
		foreach tag [$editor tag names] {
			if {[lsearch {
					sel		tag_current_line	tag_bookmark
					tag_breakpoint	tag_simulator_curr	tag_error_line
				} $tag] != -1
			} then {
				break
			}
			$editor tag remove $tag 1.0 end
		}
		# Enable writing to the left border
		$iconBorder configure -state normal
		# Set new font
		set defaultCharHeight_org $defaultCharHeight
		set defaultFont	[font create -size -$fontSize -family $fontFamily]
		set defaultCharWidth [font measure $defaultFont -displayof $editor { }]
		set defaultCharHeight [font metrics $defaultFont -displayof $editor -linespace]
		# Adjust bookmark images
		if {$defaultCharHeight_org != $defaultCharHeight} {
			set indexes {}
			if {$defaultCharHeight_org < 9} {
				set idx 1.0
				set idx_prev $idx
				while 1 {
					set idx [$iconBorder search -exact -- {*} $idx]
					if {$idx == {}} {break}
					if {[$iconBorder compare $idx_prev >= $idx]} {break}
					lappend indexes $idx
					set idx_prev $idx
					set idx [$iconBorder index "$idx+1c"]
				}
			} {
				foreach img [$iconBorder image names] {
					lappend indexes [$iconBorder index $img]
				}
			}
			if {$defaultCharHeight < 9} {
				foreach idx $indexes {
					$iconBorder delete $idx "$idx+1c"
					$iconBorder insert $idx {*}
				}
			} {
				if {$defaultCharHeight < 15} {
					set image {dot}
				} {
					set image {bookmark}
				}
				foreach idx $indexes {
					$iconBorder delete $idx "$idx+1c"
					$iconBorder image create $idx		\
						-image ::ICONS::16::$image	\
						-align center
				}
			}
		}
		# Change fonts
		$iconBorder	configure -font $defaultFont
		$lineNumbers	configure -font $defaultFont
		$editor		configure -font $defaultFont
		# Disable writing to the left border
		$iconBorder configure -state disabled
		# Reset line wrap settings
		set highlighted_lines [string repeat 0 [string bytelength $highlighted_lines]]
		update idle
		highlight_visible_area
	}

	## Switch betweeen selection modes (Normal / Block)
	 # @return void
	public method switch_sel_mode {} {
		set selection_mode [expr {!$selection_mode}]
		if {$selection_mode} {
			adjust_selection_to_block
			$sel_mode_lbl configure -text "BLK" -fg #0088CC
		} {
			if {[llength [$editor tag nextrange sel 1.0]]} {
				$editor tag add sel sel.first sel.last
			}
			$sel_mode_lbl configure -text "NORM" -fg #000000
		}
	}

	## Switch between modes "Insert" and "Overwrite"
	 # @return void
	public method switch_ins_ovr {} {
		if {$ro_mode} {return}
		set ins_ovr_mode [expr {!$ins_ovr_mode}]
		adjust_INS_OVR_label
	}

	## Adjust Insert/Overwrite label on status bar
	 # @return void
	private method adjust_INS_OVR_label {} {
		if {$ins_ovr_mode} {
			$ins_mode_lbl configure -text "INS" -fg #000000
			$editor configure -insertbackground #000000
		} {
			$ins_mode_lbl configure -text "OVR" -fg #FF0000
			$editor configure -insertbackground #FF0000
		}
	}

	## Get list of breapoints
	 # @return List - result (eg. '{0 1 1 0 0 0}')
	public method getBreakpoints {} {
		return $breakpoints
	}

	## This function should be called after each column change
	 # -- Close popup completion menu
	 # @return void
	public method resetUpDownIndex {} {
		set lastUpDownIndex 0
	}

	## Find a string in the text, scroll to it and select it
	 # @parm Bool fromCursor	- Search from cursor / whole document
	 # @parm Bool Backwards		- Search backwards from cursor / forwards
	 # @parm Bool regExp		- Use regular expressions / exact matching
	 # @parm Bool noCase		- Case insensitive / sensitive
	 # @parm Bool inSelection	- Search in the selected block / search globaly
	 # @parm String Sindex		- index in the text where the search should start
	 # @parm String String		- String to search
	 # @return List - {indexMatchBegining indexMatchEnd matchesCount}
	public method find {fromCursor Backwards regExp noCase inSelection Sindex String} {

		## adjust search options

		# set Stop-Index (and Start-index) depending on variable $inSelection
		if {![llength [$editor tag nextrange sel 1.0]]} {
			set inSelection 0
		}
		if {$inSelection} {
			set Sindex sel.first
			set Eindex sel.last
		} {
			if {$Backwards} {
				set Eindex 1.0
			} {
				set Eindex {end}
			}
		}

		# set direction and possibly Start-index too
		if {$fromCursor} {
			# Sindex
			set Sindex {insert}
		} {
			# Sindex
			if {$Sindex == {}} {
				if {$Backwards} {
					set Sindex end
				} {
					set Sindex 1.0
				}
			}
		}

		# direction
		if {$Backwards} {
			append Sindex -[expr {[string length $String] - 1}]c
			set direction {-backwards}
		} {
			set direction {-forwards}
		}

		# set exact or regexp based on search and case sensitivity
		if {$regExp}	{set regexp {-regexp}} {set regexp {-exact}}

		## Perform search
		if {[catch {
			if {$noCase} {
				set index [$editor search $direction $regexp -nocase	\
					-count {::editor_search_count} --		\
					$String $Sindex $Eindex]
			} {
				set index [$editor search $direction $regexp	\
					-count {::editor_search_count} --	\
					$String $Sindex $Eindex]
			}
		} result]} then {
			return [list -1 $result]
		}

		## Focus on the found string
		if {$index != {}} {
			# Determinate number of column (begin) and row of matched string
			set lineNumber [expr {int($index)}]
			regexp {\d+$} $index colNumber
			# Determinate lenght of matched string
			if {$regExp} {
				regexp $String [$editor get $Sindex $Eindex] String
			}
			# Determinate number of end column of matched string
			set end_col [string length $String]
			set end_col [expr {$end_col + $colNumber}]
			# Goto line with the found match and select that matched string
			if {$Backwards} {
				goto $lineNumber.$colNumber
			} {
				goto $lineNumber.$end_col
			}
			$editor tag remove sel 1.0 end
			$editor tag add sel $index $lineNumber.$end_col
			# set result acording to values determinated above
			set matches "$index $lineNumber.$end_col ${::editor_search_count}"
		} {
			# set result to something like 'nothing found'
			set matches "$Sindex $Sindex 0"
		}

		# return result (see procedure header for details)
		return $matches
	}

	## Find a string in the text and replce it by something else ...
	 # note: using 'find' procedure
	 # @parm Bool fromCursor	- Search from cursor / whole document
	 # @parm Bool inSelection	- Search in the selected block / search globaly
	 # @parm Bool Backwards		- Search backwards from cursor / forwards
	 # @parm Bool regExp		- Use regular expressions / exact matching
	 # @parm Bool noCase		- Case insensitive / sensitive
	 # @parm String SearchString	- String to search
	 # @parm String Replacement	- String to replace SearchString
	 # @parm Bool confirm		- (see attribute confirmCMD below)
	 # @parm String confirmCMD	- command which will be executed on each match if cofirm is 1,
	 #				  acceptable return values are:
	 #					0 : Replace and search next
	 #					1 : Replace and close
	 #					2 : Replace all without prompt
	 #					3 : Search next
	 #					4 : Close
	 # @return Bool - 1 == Ok; 0 == last replacement was refused by user
	public method replace {fromCursor Backwards regExp noCase SearchString Replacement confirm confirmCMD} {
		# Local variables
		set remaining	1	;# Int - Count of remaining matches to replace
		set repl_made	0	;# Int - Count of replacements made
		set close	0	;# Bool - Close after this replace
		set cnd		1	;# Bool - Perform replacement in this iteration

		# Save the current insertion cursor index
		set ins_index [$editor index insert]

		if {$critical_edit_proc} {return 0}
		set critical_edit_proc 1

		## Derminate indexes of area to be affected
		if {$fromCursor} {
			# from actual cursor position
			set index {insert}
		} {
			if {$Backwards} {
				set index {end}
			} {
				set index 1.0
			}
		}

		## Perform replacement for each match
		while $remaining {
			# Initiate search and determinate count of remaining matches
			set result [find $fromCursor $Backwards $regExp $noCase 0 $index $SearchString]

			set remaining [lindex $result 2]
			if {$remaining == 0} {break}

			# Determinate index where to reinitiate search
			set index [lindex $result [expr {!$Backwards}]]

			# "Annoy user" if there is requested confirmation before each replace
			if {$confirm} {
				# invoke confirmation command and setup new parameters
				switch [$confirmCMD] {
					0	{	;# Replace
						set cnd 1
					}

					1	{	;# Replace & close
						set cnd 1
						set close 1
					}

					2	{	;# Replace all
						set cnd 1;
						set confirm 0
					}

					3	{	;# Find next
						set cnd 0
					}

					4	{	;# Close
						set cnd 1
						break
					}
				}
			} {
				# automaticaly replace all without any prompt
				set cnd 1
			}

			# Perform replace if it's allowed
			if {$cnd} {
				# Determinate indexes of string to replace
				set start 	[lindex $result 0]
				set end		[lindex $result 1]
				# Replace
				$editor configure -autoseparators 0
				detete_text_in_editor $start $end
				$editor insert $start $Replacement
				if {!$Backwards} {
					set index [$editor index insert]
				}
				$editor edit separator
				$editor configure -autoseparators 1
				# restore syntax highlight on all affected lines
				set start [expr {int($start)}]
				set end [expr {int($end)}]
				if {$confirm} {
					for {set line $start} {$line <= $end} {incr line} {
						parse $line
					}
				} {
					for {set line $start} {$line <= $end} {incr line} {
						restore_line_markers $line
					}
					set highlighted_lines [string replace $highlighted_lines	\
						$start $end [string repeat 0 [expr {$end - $start + 1}]]]
				}
				# increment counter of made replacemetns
				incr repl_made
				# contitionaly break replacing loop
				if {$close} {break}
			}
			# decrease counter of remining replacements
			incr remaining -1
		}

		$editor tag remove sel 1.0 end

		## Change application status bar (show results)
		Sbar "Replace: $repl_made replacements made"

		goto $ins_index
		highlight_visible_area
		set critical_edit_proc 0
		return $cnd
	}

	## Select all content of the editor's text widget
	 # @return void
	public method select_all {} {
		catch {
			$editor tag remove	sel 1.0 end
		}
		$editor tag add		sel 1.0 end
	}

	## Restore tags "Bookmark" and "Error" on the given line
	 # @parm Int line - line number
	 # @return void
	private method restore_line_markers {line} {
		# Restore bookmark
		if {[lindex $bookmarks $line] == 1} {
			$editor tag add tag_bookmark $line.0 "$line.0+1l"
		}
		# Restore tag error
		if {
			$prog_language != 2 &&
			[llength [$editor tag nextrange tag_error $line.0 "$line.0 lineend"]]
		} then {
			$editor tag add tag_error_line $line.0 "$line.0+1l"
		}
	}

	## Comment the selected area or current line
	 # @return void
	public method comment {} {
		if {$editor_to_use} {return}
		if {$completion_win_opened} {return}

		set start [expr {int([$editor index insert])}]
		$editor configure -autoseparators 0

		# Assembly language
		if {$prog_language == 0 || $prog_language == 3 || [string index $highlighted_lines $start] == 6} {
			# determinate indexes of area to comment
			if {[getselection] == {}} {
				set restore_sel 0
				set end $start
			} {
				set restore_sel 1
				set start [expr {int([$editor index sel.first])}]
				set end_o [$editor index sel.last]
				set end [expr {int($end_o)}]
				if {$end == $end_o} {incr end -1}
			}

			# comment each line in the block
			autocompletion_maybe_important_change $start.0 $end.0
			for {set line $start} {$line <= $end} {incr line} {
				$editor insert $line.0 {;}
				restore_line_markers $line
			}

		# C language
		} elseif {$prog_language == 1} {
			# determinate indexes of area to comment
			if {[getselection] == {}} {
				set by_lines 1
				set restore_sel 0
				set end $start
			} {
				set start_o [$editor index sel.first]
				set start [expr {int($start_o)}]
				set end_o [$editor index sel.last]
				set end [expr {int($end_o)}]
				if {$end == $end_o && $start == $start_o} {
					set restore_sel 1
					set by_lines 1
				} {
					set restore_sel 0
					set by_lines 0
				}
				if {$end == $end_o} {
					incr end -1
				}
			}

			# Comment each line in the block
			autocompletion_maybe_important_change $start.0 $end.0
			if {$by_lines} {
				for {set line $start} {$line <= $end} {incr line} {
					$editor insert $line.0 {// }
					restore_line_markers $line
				}
			# Comment only selected characters
			} {
				$editor insert $end_o { */}
				$editor insert $start_o {/* }
				$editor tag add sel sel.first-3c sel.last+3c
			}
		} else {
			$editor edit separator
			$editor configure -autoseparators 1
			return
		}

		$editor edit separator
		$editor configure -autoseparators 1

		if {$prog_language != -1} {
			# Restore highlight
			if {$prog_language == 1 && ![string index $highlighted_lines $start] == 6} {
				parse $start
			} {
				for {set i $start} {$i <= $end} {incr i} {
					parse $i
				}
			}

			# Restore selection shape
			if {$restore_sel} {
				$editor tag add sel "$start.0 linestart" "$end.0 lineend"
			}
		}
	}

	## Remove first semicolon in selected area or current line
	 # @return bool - result
	public method uncomment {} {
		if {$editor_to_use} {return}
		if {$completion_win_opened} {return}

		set succesful 0
		set start [expr {int([$editor index insert])}]
		$editor configure -autoseparators 0

		# Assembly language
		if {$prog_language == 0 || $prog_language == 3 || [string index $highlighted_lines $start] == 6} {
			# determinate indexes of area to uncomment
			if {[getselection] == {}} {
				set restore_sel 0
				set end $start
			} {
				set restore_sel 1
				set start [expr {int([$editor index sel.first])}]
				set end_o [$editor index sel.last]
				set end [expr {int($end_o)}]
				if {$end == $end_o} {incr end -1}
			}

			# Uncomment each line in the block
			for {set line $start} {$line <= $end} {incr line} {
				# get line
				set line_data [$editor get $line.0 "$line.0 lineend"]

				if {[regexp {^\s*;\s*} $line_data comment]} {
					detete_text_in_editor $line.0 $line.[string length $comment]
					regsub {;} $comment {} comment
					$editor insert $line.0 $comment
					restore_line_markers $line
					set succesful 1
				}
			}

		# C language
		} elseif {$prog_language == 1} {
			# determinate indexes of area to comment
			if {[getselection] == {}} {
				set by_lines 1
				set restore_sel 0
				set end $start
			} {
				set start_o [$editor index sel.first]
				set start [expr {int($start_o)}]
				set end_o [$editor index sel.last]
				set end [expr {int($end_o)}]
				if {$end == $end_o && $start == $start_o} {
					set restore_sel 1
					set by_lines 1
				} {
					set restore_sel 0
					set by_lines 0
				}
				if {$end == $end_o} {
					incr end -1
				}
			}

			# Uncomment only selected characters
			if {!$by_lines} {
				set start_data	[$editor get $start_o [list $start_o lineend]]
				set end_data	[$editor get [list $end_o linestart] $end_o]
				if {
					[regexp {^\s*/\* ?} $start_data start_data]
						&&
					[regexp { ?\*/\s*$} $end_data end_data]
				} then {
					set succesful 1
					set start_data [string length $start_data]
					set end_data [string length $end_data]
					detete_text_in_editor $end_o-${end_data}c $end_o
					detete_text_in_editor $start_o $start_o+${start_data}c
				}
			}

			# Uncomment each line in the block
			for {set line $start} {$line <= $end} {incr line} {
				set line_data [$editor get $line.0 [list $line.0 lineend]]
				if {[regexp {^\s*// ?} $line_data line_data]} {
					set line_data [string length $line_data]
					detete_text_in_editor $line.0 $line.$line_data
					set succesful 1
				}
				restore_line_markers $line
			}
		} {
			$editor edit separator
			$editor configure -autoseparators 1
			return $succesful
		}
		$editor edit separator
		$editor configure -autoseparators 1

		if {$prog_language != -1} {
			# Restore highlight
			if {$prog_language == 1 && ![string index $highlighted_lines $start] == 6} {
				parse $start
			} {
				for {set i $start} {$i <= $end} {incr i} {
					parse $i
				}
			}

			# Restore selection shape
			if {$restore_sel} {
				$editor tag add sel "$start.0 linestart" "$end.0 lineend"
			}

		}

		# Return result
		return $succesful
	}

	## Go to line number or text index in the text
	 # @parm Number textIndex - Can be line number (like 154) or text index (like 32.17)
	 # @return void
	public method goto {textIndex} {
		if {$editor_to_use} {return}

		# Check for validity of the given argument
		if {![regexp {^\d+(\.\d+)?$} $textIndex]} {
			return
		}
		# Adjust the given parameter
		if {![regexp {\.} $textIndex]} {
			set textIndex "$textIndex.0"
		}
		# Scroll to the requered index and move cursor there
		rightPanel_adjust [expr {int($textIndex)}]
		$editor mark set insert $textIndex
		if {!$frozen} {
			$editor tag remove tag_current_line 1.0 end
			$editor tag add tag_current_line			\
				[$editor index "$textIndex linestart"]		\
				[$editor index "$textIndex +1 line linestart"]
		}
		recalc_status_counter {} 0
		update idle
		$editor see insert
	}

	## Delete all selected characters
	 # @return Bool - Anything deleted
	public method deleteselection {} {
		set ranges [$editor tag ranges sel]
		set len [llength $ranges]
		for {set i 0; set j 1} {$i < $len} {incr i 2; incr j 2} {
			detete_text_in_editor [lindex $ranges $i] [lindex $ranges $j]

			if {$prog_language == 1} {
				c_syntax_highlight [expr {int([lindex $ranges $i])}]
			}
		}

		return [expr {!(!$len)}]
	}

	## Get currently selected text
	 # @return String - content of the selected area
	public method getselection {} {
		set data {}
		set ranges [$editor tag ranges sel]
		set len [llength $ranges]
		for {set i 0; set j 1} {$i < $len} {incr i 2; incr j 2} {
			if {$i} {
				append data "\n"
			}
			append data [$editor get [lindex $ranges $i] [lindex $ranges $j]]
		}
		return $data
	}

	## Indent content of the selected area or current line
	 # @return void
	public method indent {} {
		if {$completion_win_opened} {return}

		# Determinate indexes of area to be affected
		if {[getselection] == {}} {
			set restore_sel 0
			set start [expr {int([$editor index insert])}]
			set end $start
		} {
			set restore_sel 1
			set start [expr {int([$editor index sel.first])}]
			set end_o [$editor index sel.last]
			set end [expr {int($end_o)}]
			if {$end == $end_o} {incr end -1}
		}

		# indent each line in the block
		if {$spaces_no_tabs} {
			set indent_char [string repeat { } $number_of_spaces]
		} {
			set indent_char "\t"
		}
		for {set line $start} {$line <= $end} {incr line} {
			$editor insert $line.0 $indent_char
			rightPanel_changeLineContent $line
			restore_line_markers $line
		}
		# Restore selection shape
		if {$restore_sel} {
			$editor tag add sel "$start.0 linestart" "$end.0 lineend"
		}
		$editor tag add sel $start.0 [expr {$end + 1}].0
	}

	## Unindent content of the selected area or current line
	 # @return void
	public method unindent {} {
		if {$completion_win_opened} {return}

		# Determinate indexes of area to be affected
		if {[getselection] == {}} {
			set restore_sel 0
			set start [expr {int([$editor index insert])}]
			set end [expr {int([$editor index insert])}]
		} {
			set restore_sel 1
			set start [expr {int([$editor index sel.first])}]
			set end_o [$editor index sel.last]
			set end [expr {int($end_o)}]
			if {$end == $end_o} {incr end -1}
		}
		# unindent each line in the block
		for {set line $start} {$line <= $end} {incr line} {
			set line_data [$editor get $line.0 "$line.0 lineend"]
			if {$spaces_no_tabs} {
				if {[regexp {^ +} $line_data space]} {
					set space [string length $space]
					if {$space > $number_of_spaces} {
						set space $number_of_spaces
					}
					detete_text_in_editor $line.0 $line.$space
				} elseif {[regexp {^\t} $line_data]} {
					detete_text_in_editor $line.0 $line.1
				}
			} {
				if {[regexp {^[\t(        )]} $line_data]} {
					detete_text_in_editor $line.0 $line.1
				}
			}
			rightPanel_changeLineContent $line
		}
		# Restore selection shape
		if {$restore_sel} {
			$editor tag add sel "$start.0 linestart" "$end.0 lineend"
		}
		$editor tag add sel $start.0 [expr {$end + 1}].0
	}

	## Get contents of the text widget
	 # EOL: LF
	 # Encoding: UTF-8
	 # @return String
	public method getdata {} {
		return [regsub {\n$} [$editor get 1.0 end] {}]
	}

	## Get contents of the text widget
	 # EOL: $eol
	 # Encoding: $encoding
	 # @return String
	public method getdata_adjusted_ENC_and_EOL {} {
		switch -- $eol {
			{lf}	{set eol_char "\n"}
			{cr}	{set eol_char "\r"}
			{crlf}	{set eol_char "\r\n"}
		}
		return	[encoding convertto $encoding		\
			[regsub -all {\n}			\
				[regsub {\n$}			\
					[$editor get 1.0 end]	\
				{}]	\
			$eol_char]	\
		]			\
	}

	## Get MD5 of the opened file
	 # @return String - MD5 hash
	public method get_md5 {} {
		switch -- $eol {
			{lf}	{set eol_char "\n"}
			{cr}	{set eol_char "\r"}
			{crlf}	{set eol_char "\r\n"}
		}
		return	[md5::md5 -hex					\
			[encoding convertto $encoding			\
				[regsub -all {\n}			\
					[regsub {\n$}			\
						[$editor get 1.0 end]	\
					{}]				\
				$eol_char]				\
			]						\
		]
	}

	## Determinate whether editor text has been modified
	 # and adjust internal variables
	 # @parm bool force	- 1: "I'm sure it has been modified !"
	 #			  0: "discover it automaticaly"
	 # @return bool - a new modified flag or {}
	public method recalc_status_modified {force} {

		# Modified
		if {[$editor edit modified] || $force} {

			# Adjust editor status bar
			$Sbar_image configure	\
				-image ::ICONS::16::filesave	\
				-helptext "File has been modified, click to save"
			pack $Sbar_image -side left

			# Set modified flag
			set modified 1

			# Start autosave timer
			if {$autosave} {
				catch {
					after cancel $autosave_timer
				}
				set autosave_timer [after [expr {$autosave * 60000}] [list $this save]]
			}

		# Not modifed
		} {
			# Adjust editor status bar
			pack forget $Sbar_image
			
			# Set modified flag
			set modified 0

			# Stop autosave timer
			catch {
				after cancel $autosave_timer
			}
		}

		::X::adjust_title

		# Return modified flag
		return $modified
	}

	## Add/Remove bookmark to/from current line
	 # Directly depends on variable "bookmarks" (managed by proc. recalc_left_frame)
	 # @parm Int = NULL - target text index
	 # @return bool - 0: bookmark removed; 1: bookmark created
	public method Bookmark args {
		if {$editor_to_use} {return}

		# Determinate line number
		if {$args != {}} {
			set lineNumber [expr {int($args)}]
		} {
			set lineNumber [expr {int([$editor index insert])}]
		}

		# Check for maximum line number value
		if {[expr {$lineNumber - [llength $bookmarks]}] > -1} {
			recalc_left_frame
			return
		}

		# Add or remove bookmark ?
		if {[lindex $bookmarks $lineNumber] == 1} {set make 0} {set make 1}
		lset bookmarks $lineNumber $make

		# Adjust line number
		set lineNumber_i [wrap_aux_idx2line $lineNumber]

		## Add/remove bookmark icon to/from iconBorder


		$iconBorder configure -state normal	;# Enable the text widget
		set scroll_in_progress 1		;# Disable scrolling

		# Add icon
		if {$make} {
			if {$defaultCharHeight < 9} {
				$iconBorder insert $lineNumber_i.0 {*}
			} elseif {$defaultCharHeight < 15} {
				$iconBorder image create $lineNumber_i.0	\
					-image ::ICONS::16::dot			\
					-align center
			} {
				$iconBorder image create $lineNumber_i.0	\
					-image ::ICONS::16::bookmark		\
					-align center
			}
		# Remove icon
		} {
			$iconBorder delete $lineNumber_i.0 $lineNumber_i.1
		}

		# Disable the text widget
		$iconBorder configure -state disabled

		# Take care of bookmark tag
		set tmp $lineNumber
		incr tmp
		# Add the tag
		if {$make} {
			$editor tag add tag_bookmark $lineNumber.0 $tmp.0
			$parentObject rightPanel_add_bookmark $lineNumber
			$parentObject rightPanel_bm_select $lineNumber
		# Remove the tag
		} {
			$editor tag remove tag_bookmark $lineNumber.0 $tmp.0
			$parentObject rightPanel_remove_bookmark $lineNumber
			$parentObject rightPanel_bm_unselect
		}

		# Enable scrolling
		update idle
		set scroll_in_progress 0

		# Done ...
		return $make
	}

	## Add/Remove breapoint to/from current line
	 # Directly depends on variable "breakpoints" (managed by proc. recalc_left_frame)
	 # @parm Int = NULL - target text index
	 # @return bool - 0: breakpoint removed; 1: breakpoint created; or {}
	public method Breakpoint args {

		# Determinate line number
		if {$args != {}} {
			set lineNumber [expr {int($args)}]
		} {
			set lineNumber [expr {int([$editor index insert])}]
		}

		# Check for maximum line number value
		if {[expr {$lineNumber - [llength $breakpoints]}] > -1} {
			recalc_left_frame
			return
		}

		# Add or remove breakpoint ?
		if {[lindex $breakpoints $lineNumber] == 1} {set make 0} {set make 1}
		# Set breakpoint flag
		lset breakpoints $lineNumber $make

		# Adjust line number
		set lineNumber [wrap_aux_idx2line $lineNumber]

		## Add/remove breakpoint tag to/from LineNumbers
		set tmp $lineNumber
		incr tmp

		$lineNumbers configure -state normal	;# Enable the text widget
		set scroll_in_progress 1		;# Disable scrolling

		# Add the tag
		if {$make} {
			$lineNumbers tag add tag_breakpoint $lineNumber.0 $tmp.0
			$parentObject rightPanel_add_breakpoint $lineNumber
			$parentObject rightPanel_bp_select $lineNumber
		# Remove the tag
		} {
			$lineNumbers tag remove tag_breakpoint $lineNumber.0 $tmp.0
			$parentObject rightPanel_remove_breakpoint $lineNumber
			$parentObject rightPanel_bp_unselect
		}

		# Disable the text widget
		$lineNumbers configure -state disabled

		# Refresh breakpoint settings in simulator engine
		if {[lindex ${::X::simulator_enabled} ${::X::actualProjectIdx}] == 1} {
			$parentObject Simulator_import_breakpoints $fullFileName [getBreakpoints]
		}

		# Enable scrolling
		update idle
		set scroll_in_progress 0

		# done ...
		return $make
	}

	## Remove all bookmarks from the editor and from right panel
	 # @return void
	public method clear_all_bookmarks {} {
		if {$editor_to_use} {return}

		# Clear icon border
		$iconBorder configure -state normal
		set idx -1
		foreach bool $bookmarks {
			incr idx
			if {!$bool} {continue}
			$iconBorder delete $idx.0 $idx.1
		}
		$iconBorder configure -state disabled

		# Clear text tags
		$editor tag remove tag_bookmark 1.0 end

		# Clear list of bookmarks
		set len [llength $bookmarks]
		incr len -1
		set bookmarks 0
		append bookmarks [string repeat { 0} $len]

		# Clear list of bookmakrs in the right panel
		$parentObject rightPanel_clear_all_bookmarks
	}

	## Remove all breakpoints from the editor and from right panel
	 # @return void
	public method clear_all_breakpoints {} {

		# Clear breakpoints in  object variable
		set len [llength $bookmarks]
		incr len -1
		set breakpoints 0
		append breakpoints [string repeat { 0} $len]

		# Clear breakpoint tags from line numbers
		$lineNumbers tag remove tag_breakpoint 1.0 end

		# Clear right panel
		$parentObject rightPanel_clear_all_breakpoints
	}

	## Get number of the current line
	 # @return Int - current line num.
	public method get_current_line_number {} {
		set line [expr {int([$editor index insert])}]
		return $line
	}

	## Call ::X::__show_hine_IconB
	 # @return void
	public method show_hine_IconB {} {
		::X::__show_hine_IconB
	}

	## Call ::X::__show_hine_LineN
	 # @return void
	public method show_hine_LineN {} {
		::X::__show_hine_LineN
	}

	## Call ::configDialogs::mkDialog $args
	 # @return void
	public method configDialogs_mkDialog args {
		::configDialogs::editor::mkDialog $args
	}

	## Show the Icon Border
	 # @return bool - result
	public method showIconBorder {} {
		if {!$show_iconBorder} {
			pack $left_frame_L -fill y -side left
			set show_iconBorder 1
			recalc_left_frame
			return 1
		}
		return 0
	}

	## Hide the Icon Border
	 # @return bool - result
	public method hideIconBorder {} {
		if {$show_iconBorder} {
			pack forget $left_frame_L
			set show_iconBorder 0
			return 1
		}
		return 0
	}

	## Show the Line Numbers
	 # @return bool - result
	public method showLineNumbers {} {
		if {!$show_lineNum} {
			pack $left_frame_R -fill y -side right
			set show_lineNum 1
			recalc_left_frame
			return 1
		}
		return 0
	}

	## Hide the Line Numbers
	 # @return bool - result
	public method hideLineNumbers {} {
		if {$show_lineNum} {
			pack forget $left_frame_R
			set show_lineNum 0
			return 1
		}
		return 0
	}

	## Insert given data into the text
	 # @parm String data		- Data to insert
	 # @parm TextIndex position	- Target text index ({} == "end")
	 # @return void
	public method insertData {data position} {
		if {$position == ""} {
			set position end
		}
		# Insert data
		$editor insert $position [regsub -all {[\u0000-\u0008\u000B-\u000C\u000E-\u001F\u007F-\u009F]} $data {}]
		# Restore highlight
		update idle
		parseAll
	}

	## Restore syntax highlight in whole text
	 # @return void
	public method parseAll {} {
		if {$editor_to_use} {return}

		# Disable this function
		set enable_parseAll 0

		# Determinate number of the last line in the editor
		set lastEnd [expr {int([$editor index end])}]

		# Initialize list of highlighted lines
		set highlighted_lines [string repeat 0 $lastEnd]

		# Reevaluate bookmarks and breakpoints
		$lineNumbers configure -state normal
		$lineNumbers delete 1.0 end
		$lineNumbers insert end 1
		import_line_markers_data [join $bookmarks {}] [join $breakpoints {}]

		# Highlight all visible lines
		highlight_visible_area

		# Recalculate left frame and status bar counters
		scrollSet [lindex [$editor yview] 0] [lindex [$editor yview] 1]
		set lastEnd [expr {int([$editor index end])}]
		recalc_status_counter {} 0

		# Enable this function
		set enable_parseAll 1
	}

	## Recalculate variables related to bookmarks, line numbers and list of highlighted lines
	 # @parm Bool force = 1	- perform recalcutaion even if length of text wasn't changed
	 # @return bool - 0: failed; 1: successful
	public method recalc_left_frame args {
		if {$editor_to_use} {return}

		# Parse arguments
		set force [lindex $args 0]
		if {$force == {}} {
			set force 0
		}

		# Determinate editor lines count and End index
		set End [$editor index end]
		set Tlines [expr {int($End)}]

		# Current line number -> Actline
		set insert [$editor index insert]
		set Actline [expr {int($insert)}]

		# Return if lines count has not been changed
		if {!$force && $lastEnd == $Tlines} {
			return 0
		}
		incr Tlines -1

		# Determinate iconBorder lines count - 1
		set Ilines [expr {int([$iconBorder index end]) - $number_of_wraps}]
		incr Ilines -1

		# Remove wrap markers from Line Numbers
		if {$number_of_wraps && ($Ilines != $Tlines)} {
			$lineNumbers configure -state normal
			set remaining $number_of_wraps
			set i 1
			foreach wrap $map_of_wraped_lines {
				if {$wrap} {
					$lineNumbers delete $i.0 $i.0+${wrap}l
					incr remaining -$wrap
				}
				if {!$remaining} {break}
				incr i
			}
		}

		## Some lines have been removed
		if {$Ilines > $Tlines} {
			# Determinate how many lines should be removed
			regexp {\d+$} $insert cur_line_col
			if {$cur_line_col != 0} {
				set Actline_m1 $Actline
				incr Actline
				restore_line_markers $Actline_m1
			}

			set diff $Actline
			incr diff -$Tlines
			incr diff $Ilines

			# Delete bookmark icon(s) from left bar
			$iconBorder configure -state normal
			set Actline_tmp $Actline
			set diff_tmp $diff
			if {$number_of_wraps} {
				set remaining $number_of_wraps
				for {set i 1} {$i < $diff} {incr i} {
					set wrap [lindex $map_of_wraped_lines $i]
					incr diff_tmp $wrap
					if {$i < $Actline} {
						incr Actline_tmp $wrap
					}
					incr remaining -$wrap
					if {!$remaining} {break}
				}
			}
			$iconBorder delete $Actline_tmp.0 $diff_tmp.0
			$iconBorder configure -state disabled

			# Unregister bookmarks for deletion
			incr diff -1
			set bookmarks [lreplace $bookmarks $Actline $diff]
			set breakpoints [lreplace $breakpoints $Actline $diff]
			for {set i $Actline} {$i <= $diff} {incr i} {
				incr number_of_wraps -[lindex $map_of_wraped_lines $i]
			}
			set map_of_wraped_lines [lreplace $map_of_wraped_lines $Actline $diff]
			set highlighted_lines [string replace $highlighted_lines $Actline $diff]

			# Remove bookmarks from right panel
			$parentObject rightPanel_remove_bookmarks $Actline $diff
			$parentObject rightPanel_remove_breakpoints $Actline $diff

			# rewrite breakpoints
			rewrite_breakpoint_tags

		## Some lines have been added
		} elseif {$Ilines < $Tlines} {
			# Determinate how many lines should be added
			set diff $Tlines
			incr diff -$Ilines
			set ins [string repeat "\n" $diff]
			set BMStr [string range [string repeat {0 } $diff] 0 end-1]
			set insIndex $Actline
			incr insIndex -$diff
			if {$insIndex == 0} {incr insIndex}
			set insLineEnd [$editor index "$insIndex.0 lineend"]
			regexp {\d+$} $insLineEnd ins_line_col

			set insIndex_1 $insIndex
			incr insIndex_1

			if {$ins_line_col != 0} {
				# Remove tag "Bookmark"
				if {[lindex $bookmarks $insIndex] == 1} {
					$editor tag remove tag_bookmark $insIndex.0 [expr {$Actline + 1}].0
				}

				# Adjust insert index
				incr insIndex
				incr insIndex_1

				# Prepare lists
				if {[string length $highlighted_lines] == $insIndex} {
					append highlighted_lines 0
				}
				if {[llength $bookmarks] == $insIndex} {
					lappend bookmarks 0
				}
				if {[llength $breakpoints] == $insIndex} {
					lappend breakpoints 0
				}
				if {[llength $map_of_wraped_lines] == $insIndex} {
					lappend map_of_wraped_lines 0
				}

				# Adjust list of highlighted lines
				set highlighted_lines [string replace $highlighted_lines	\
					[expr {$insIndex - 1}] [expr {$insIndex - 1}]		\
					[string repeat 0 [expr {$diff + 1}]]]

				# Adjust bookmark and breakpoint lists
				set map_of_wraped_lines [linsert $map_of_wraped_lines $insIndex $BMStr]
				set bookmarks [linsert $bookmarks $insIndex $BMStr]
				set breakpoints [linsert $breakpoints $insIndex $BMStr]
				regsub -all {[\{\}]} $map_of_wraped_lines {} map_of_wraped_lines
				regsub -all {[\{\}]} $bookmarks {} bookmarks
				regsub -all {[\{\}]} $breakpoints {} breakpoints

				# Shift bookmarks and breakpoints on the right panel
				$parentObject rightPanel_shift_bookmarks $insIndex $diff
				$parentObject rightPanel_shift_breakpoints $insIndex $diff

				if {[lindex $bookmarks [expr {$insIndex - 1}]] == 1} {
					$editor tag add tag_bookmark [expr {$insIndex - 1}].0 $insIndex.0
				}

			} {
				# Adjust list of highlighted lines
				set highlighted_lines [string replace		\
					$highlighted_lines $insIndex $insIndex	\
					[string repeat 0 [expr {$diff + 1}]]]

				# Adjust bookmark and breakpoint lists
				set map_of_wraped_lines [linsert $map_of_wraped_lines $insIndex $BMStr]
				set bookmarks [linsert $bookmarks $insIndex $BMStr]
				set breakpoints [linsert $breakpoints $insIndex $BMStr]
				regsub -all {[\{\}]} $map_of_wraped_lines {} map_of_wraped_lines
				regsub -all {[\{\}]} $bookmarks {} bookmarks
				regsub -all {[\{\}]} $breakpoints {} breakpoints

				$parentObject rightPanel_shift_bookmarks $insIndex $diff
				$parentObject rightPanel_shift_breakpoints $insIndex $diff
			}

			$iconBorder configure -state normal
			if {$number_of_wraps} {
				set insIndex_tmp $insIndex
				for {set i 1} {$i < $insIndex_tmp} {incr i} {
					incr insIndex [lindex $map_of_wraped_lines $i]
				}
			}
			$iconBorder insert $insIndex.0 $ins
			$iconBorder configure -state disabled

			# rewrite breakpoints
			rewrite_breakpoint_tags
		}

		## Recalculate Line Numbers
		# Prepare Line Numbers
		if {$Tlines != [expr {int([$lineNumbers index end]) - 1}]} {
			# Enable the widget
			$lineNumbers configure -state normal
		}

		# Determinate LineNumbers lines count - 1
		set Llines [expr {int([$lineNumbers index end])}]
		incr Llines -1

		if {$Llines > $Tlines} {	;# too many lines -> remove some ones
			$lineNumbers delete $End end

		} elseif {$Llines < $Tlines} {	;# not enought lines -> add some ones
			# Create string to insert to Line Numbers
			set ins {}
			for {set i [expr {$Llines + 1}]} {$i <= $Tlines} {incr i} {
				append ins "\n$i"
			}

			# Insert it at the end of widget and adjust widget width
			$lineNumbers insert end $ins
		}

		# Finalize
		if {$Llines != $Tlines} {
			# Adjust widget width
			$lineNumbers configure -width [string length [expr {int([$lineNumbers index end]) - 1}]]
			# Restore wrap markers
			if {$number_of_wraps} {
				set remaining $number_of_wraps
				set i 1
				foreach wrap $map_of_wraped_lines {
					if {$wrap} {
						$lineNumbers insert $i.0 [string repeat "$wrap_char\n" $wrap]
						$lineNumbers tag add center $i.0 [expr {$i + $wrap}].0
						incr i $wrap
						incr remaining -$wrap
					}
					if {!$remaining} {break}
					incr i
				}
			}
			# Disable Line Numbers
			$lineNumbers configure -state disabled
		}
		scrollSet [lindex [$editor yview] 0] [lindex [$editor yview] 1]
		$lineNumbers tag add right 1.0 end
		$iconBorder tag add center 1.0 end

		# Parse the current line
		parse $Actline
		highlight_visible_area

		# Save last lines count
		set lastEnd [expr {int($End)}]
		# done ...
		return 1
	}

	## Get content of the given line
	 # @parm Int lineNumber - number of the target line
	 # @return String - result
	public method getLineContent {lineNumber} {
		# Check lineNumber validity
		set end [$editor index end]
		if {$end <= $lineNumber} {return {}}

		# Return the data
		return [$editor get $lineNumber.0 "$lineNumber.0 lineend"]
	}

	## Select appropriate item in right panel lists (bookmarks and breapoints)
	 # @parm Int lineNumber - current line
	 # @return void
	public method rightPanel_adjust {lineNumber} {

		# Adjust list of bookmarks
		if {[lindex $bookmarks $lineNumber] == 1} {
			$parentObject rightPanel_bm_select $lineNumber
		} {
			$parentObject rightPanel_bm_unselect
		}

		# Adjust list of breakpoints
		if {[lindex $breakpoints $lineNumber] == 1} {
			$parentObject rightPanel_bp_select $lineNumber
		} {
			$parentObject rightPanel_bp_unselect
		}
	}

	## Parse the current line
	 # @return void
	public method parse_current_line {} {
		if {$editor_to_use} {return}
		set line [expr {int([$editor index insert])}]
		parse $line
		set highlighted_lines [string replace $highlighted_lines $line $line 0]
	}

	## Get number of lines between the given indexes
	 # @parm TextIndex index0 - Start index
	 # @parm TextIndex index1 - End index
	 # @return Int - lines count
	private method get_count_of_lines {index0 index1} {
		# Check if editor width is properly set
		if {!$editor_width} {return 1}

		# Determinate text between the given indexes
		set lineText [$editor get $index0 $index1]

		# Line contains tabulators
		if {[regexp {\t} $lineText]} {
			# Translate tabulators to spaces
			set idx -1
			set cor 0
			while 1 {
				set idx [string first "\t" $lineText [expr {$idx + 1}]]
				if {$idx == -1} {break}

				incr cor [expr {7 - (($idx + $cor) % 8)}]
			}
			regsub -all {\t} $lineText { } lineText
			# Determinate line width in pixels
			set line_width [font measure $defaultFont -displayof $editor $lineText]
			incr line_width [expr {$cor * $defaultCharWidth}]
		# Line doesn't contain tabulators
		} {
			set line_width [font measure $defaultFont -displayof $editor $lineText]
		}

		# Determinate number of lines
		set new_wrap [expr {$line_width / $editor_width}]
		if {[expr {$line_width % $editor_width}]} {
			incr new_wrap
		}

		# Return result
		return $new_wrap
	}

	## Line wrapping manager - variant 2
	 # Adjust map of wrapped lines and left border to wrap of the specified line
	 # This function using function 'get_count_of_lines' (slow)
	 # @parm Int line_number - Line number
	 # @return Bool - result
	private method wrap_mgr2 {line_number} {
		# Check if editor is properly initialized
		if {!$editor_width || $map_of_wraped_lines == {}} {return 1}

		# Not empty line
		if {[$editor compare [$editor index "$line_number.0 linestart"] != "$line_number.0 lineend"]} {
			set new_wrap [get_count_of_lines $line_number.0 "$line_number.0 lineend"]
			incr new_wrap -1
		# Empty line
		} {
			set new_wrap 0
		}

		# Determinate the current wrap factor
		set wrap [lindex $map_of_wraped_lines $line_number]
		if {$new_wrap == $wrap} {return 1}

		# Adjust map of wrapped lines
		if {$line_number >= [llength $map_of_wraped_lines]} {
			Configure
			return
		} {
			lset map_of_wraped_lines $line_number $new_wrap
		}

		# Adjust line number
		set line_number_tmp $line_number
		for {set i 1} {$i < $line_number_tmp} {incr i} {
			incr line_number [lindex $map_of_wraped_lines $i]
		}
		incr line_number

		# Adjust left border and number of line wraps
		set scroll_in_progress 1
		$lineNumbers configure -state normal
		$iconBorder configure -state normal
		if {$new_wrap > $wrap} {
			set diff [expr {$new_wrap - $wrap}]
			set ins [string repeat "$wrap_char\n" $diff]
			incr number_of_wraps $diff
			$lineNumbers insert $line_number.0 $ins
			$iconBorder insert $line_number.0 $ins
			$lineNumbers tag add center $line_number.0 [expr {$line_number + $diff}].0
		} elseif {$new_wrap < $wrap} {
			set diff [expr {$wrap - $new_wrap}]
			incr number_of_wraps -$diff
			$lineNumbers delete $line_number.0 $line_number.0+${diff}l
			$iconBorder delete $line_number.0 $line_number.0+${diff}l
		}
		$lineNumbers configure -state disabled
		$iconBorder configure -state disabled
		set scroll_in_progress 0

		# Success
		return 1
	}

	## Line wrapping manager - variant 1
	 # Adjust map of wrapped lines and left border to wrap of the specified line
	 # This function is using method 'dlineinfo' (fast)
	 # @parm Int line_number - Line number
	 # @return Bool - result
	private method wrap_mgr {line_number} {
		# Check if editor is properly initialized
		if {!$editor_width || $map_of_wraped_lines == {}} {return 1}

		# Deterinate current and previous wrap factor
		set new_wrap [lindex [$editor dlineinfo $line_number.0+1l] 1]
		set wrap [lindex [$editor dlineinfo $line_number.0] 1]
		if {$wrap == {} || $new_wrap == {}} {
			return 0
		}
		set new_wrap [expr {($new_wrap - $wrap) / $defaultCharHeight - 1}]
		set wrap [lindex $map_of_wraped_lines $line_number]
		if {$new_wrap == $wrap} {return 1}

		# Adjust map of wrapped lines
		if {$line_number >= [llength $map_of_wraped_lines]} {
			Configure
			return
		} {
			lset map_of_wraped_lines $line_number $new_wrap
		}

		# Adjust line number
		set line_number_tmp $line_number
		for {set i 1} {$i < $line_number_tmp} {incr i} {
			incr line_number [lindex $map_of_wraped_lines $i]
		}
		incr line_number

		# Adjust left border
		set scroll_in_progress 1
		$lineNumbers configure -state normal
		$iconBorder configure -state normal
		if {$new_wrap > $wrap} {
			set diff [expr {$new_wrap - $wrap}]
			set ins [string repeat "$wrap_char\n" $diff]
			incr number_of_wraps $diff
			$lineNumbers insert $line_number.0 $ins
			$iconBorder insert $line_number.0 $ins
			$lineNumbers tag add center $line_number.0 [expr {$line_number + $diff}].0
		} elseif {$new_wrap < $wrap} {
			set diff [expr {$wrap - $new_wrap}]
			incr number_of_wraps -$diff
			$lineNumbers delete $line_number.0 $line_number.0+${diff}l
			$iconBorder delete $line_number.0 $line_number.0+${diff}l
		}
		$lineNumbers configure -state disabled
		$iconBorder configure -state disabled
		set scroll_in_progress 0

		# Success
		return 1
	}

	## Reset map of wrapped lines and count of line wraps
	 # @return void
	public method reset_wraped_lines {} {
		set number_of_wraps 0
		set map_of_wraped_lines [string repeat {0 } [expr {int([$editor index end])}]]
		return
	}

	## Create highlight dialog
	 # @return void
	public method open_highlight_wait_dialog {} {
		# Create dialog frame
		set finishigh_hg_dlg_wdg [frame $ed_sc_frame.hg_dialog	\
			-bg {#EEEEFF} -bd 2 -relief raised		\
		]

		# Create heder label
		pack [label $finishigh_hg_dlg_wdg.label	\
			-text "Finishing highlight"	\
			-bg {#EEEEFF} -fg {#0000FF}	\
		] -fill x
		# Create progress bar
		pack [ProgressBar $finishigh_hg_dlg_wdg.progressbar	\
			-type normal -orient horizontal -relief flat	\
			-troughcolor {#CCCCCC} -fg {#5555FF} -height 15	\
			-maximum $finishigh_hg_dlg_max -bg {#EEEEFF}	\
			-variable ::Editor::finishigh_hg_dlg_const	\
		] -fill x -pady 5 -padx 5

		# Show dialog
		place $finishigh_hg_dlg_wdg			\
			-width 200 -height 45 -in $ed_sc_frame	\
			-x -100 -y -25 -relx 0.5 -rely 0.5
		raise $finishigh_hg_dlg_wdg
		grab $finishigh_hg_dlg_wdg
		update
	}

	## Close highlight dialog
	 # @return void
	private method close_highlight_wait_dialog {} {
		if {$finishigh_hg_dlg_tmr != {}} {
			after cancel $finishigh_hg_dlg_tmr
			set finishigh_hg_dlg_tmr {}
		}
		if {[winfo exists $finishigh_hg_dlg_wdg]} {
			grab release $finishigh_hg_dlg_wdg
			destroy $finishigh_hg_dlg_wdg
		}
	}

	## Perform syntax highlight for C language on specified line
	 # @return void
	private method c_syntax_highlight {lineNumber} {
		# Get highlight status of the previous line
		if {$lineNumber > 1} {
			set highlight_status [string index $highlighted_lines [expr {$lineNumber - 1}]]
			if {$highlight_status == {}} {
				set highlight_status 0
			}
		} {
			set highlight_status 1
		}

		# Highlighted all lines before the current one
		if {!$highlight_status} {
			# determinate highlight status of previous line
			set i [string first 0 $highlighted_lines 1]
			set highlight_status [string index $highlighted_lines [expr {$i - 1}]]
			if {$highlight_status == {}} {
				set highlight_status 1
			}

			# Highlight dialog
			set finishigh_hg_dlg_const 0
			set finishigh_hg_dlg_max [expr {($lineNumber - $i) / 500}]
			if {$object_initialized && $finishigh_hg_dlg_tmr == {}} {
				set finishigh_hg_dlg_tmr [after 500 "$this open_highlight_wait_dialog"]
			}

			# Highlight preceeding lines
			for {set j 0} {$i < $lineNumber} {incr i; incr j} {
				highlight_trailing_space $i
				set highlight_status [CsyntaxHighlight::highlight $editor $i $highlight_status]
				autocompletion_c_syntax_analyze $i
				set highlighted_lines [string replace $highlighted_lines $i $i $highlight_status]

				if {$j > 500} {
					set j 0
					incr finishigh_hg_dlg_const
					update
				}
			}

			# Close highlight dialog
			if {$object_initialized} {
				close_highlight_wait_dialog
			}
		}

		# Highlight this line
		set i $lineNumber
		set last_visible_line [expr {int([lindex [$editor yview] 1] * int([$editor index end])) + 1}]

		# Highlight all line after the current one until it is nessesary
		while 1 {
			autocompletion_maybe_important_change $i.0 $i.0
			set highlight_status_org [string index $highlighted_lines $i]
			set highlight_status [CsyntaxHighlight::highlight $editor $i $highlight_status]
			autocompletion_c_syntax_analyze $i
			set highlighted_lines	\
				[string replace $highlighted_lines $i $i $highlight_status]
			if {
				$highlight_status_org != 0	&&
				$highlight_status_org != {}	&&
				$highlight_status_org != $highlight_status
			} then {
				incr i
			} else {
				break
			}
			if {$i > $last_visible_line} {
				set highlighted_lines					\
					[string replace $highlighted_lines $i end	\
						[string repeat 0 [expr {		\
								[string length $highlighted_lines] - $i
							}]	\
						]		\
					]
				break
			}
		}
	}

	## Highlight trailing space
	 # @parm Int lineNumber - number of the target line
	 # @return void
	private method highlight_trailing_space {lineNumber} {
		$editor tag remove tag_trailing_space $lineNumber.0 [list $lineNumber.0 lineend]
		if {$hg_trailing_sp && [regexp {[\t  ]+$} [$editor get $lineNumber.0 [list $lineNumber.0 lineend]] space]} {
			$editor tag add tag_trailing_space				\
				[list $lineNumber.0 lineend]-[string length $space]c	\
				[list $lineNumber.0 lineend]
		}
	}

	## Parse given line
	 # Restore highlight, recalculate counters on status bar, adjust right panel
	 # @parm Int lineNumber - number of the target line
	 # @return Bool - result from wrap manager
	public method parse {lineNumber} {
		# Check if the given line number is valid
		if {$lineNumber >= int([$editor index end])} {
			set lineNumber [expr {int([$editor index end]) - 1}]
		}

		# Is the given line number is the current line ?
		if {int([$editor index insert]) == $lineNumber} {
			set curLine 1
		} {
			set curLine 0
		}

		# Highlight trailing space
		highlight_trailing_space $lineNumber

		# Basic validation
		if {!$curLine || !${::ASMsyntaxHighlight::validation_L1}} {
			## Restore highlight
			 # Assembly language
			if {$prog_language == 0} {
				ASMsyntaxHighlight::highlight $editor $lineNumber

				# Adjust list of highlighted lines
				if {[string index $highlighted_lines $lineNumber] == 0} {
					set highlighted_lines	\
						[string replace $highlighted_lines $lineNumber $lineNumber 1]
				}

			 # C language
			} elseif {$prog_language == 1} {
				c_syntax_highlight $lineNumber

			 # Code listing
			} elseif {$prog_language == 2} {
				LSTsyntaxHighlight::highlight $editor $lineNumber

				# Adjust list of highlighted lines
				if {[string index $highlighted_lines $lineNumber] == 0} {
					set highlighted_lines	\
						[string replace $highlighted_lines $lineNumber $lineNumber 1]
				}

			# ASX8051
			} elseif {$prog_language == 3} {
				R_ASMsyntaxHighlight::highlight $editor $lineNumber

				# Adjust list of highlighted lines
				if {[string index $highlighted_lines $lineNumber] == 0} {
					set highlighted_lines	\
						[string replace $highlighted_lines $lineNumber $lineNumber 1]
				}

			 # No highlighting
			} else {
				set highlight_status 1
			}
			
			manage_autocompletion_list $lineNumber

			# Finalize validation
			validate_line $lineNumber 0
		}

		# Recalculate counters on status bar
		if {$curLine} {
			recalc_status_counter {}
		}

		# Put tag "tag_error_line" (but not for code listing)
		if {$prog_language != 2} {
			if {${::ASMsyntaxHighlight::validation_L0}} {
				if {[llength [$editor tag nextrange tag_error $lineNumber.0 [list $lineNumber.0 lineend]]]} {
					$editor tag add tag_error_line $lineNumber.0 $lineNumber.0+1l
				} {
					$editor tag remove tag_error_line $lineNumber.0 $lineNumber.0+1l
				}
			# Remove tag "tag_error_line"
			} elseif {!${::ASMsyntaxHighlight::validation_L1}} {
				$editor tag remove tag_error_line $lineNumber.0 $lineNumber.0+1l
			}
		}

		if {$curLine} {
			resetUpDownIndex
			recalc_status_modified 0
		}

		# Agjust right panel
		rightPanel_changeLineContent $lineNumber

		set result [wrap_mgr $lineNumber]
		if {!$result} {
			wrap_mgr2 $lineNumber
		}
		return $result
	}

	## Finalize syntax validation on the given line (validates operands only)
	 # @parm Int		- Number of line in source code
	 # @parm Bool = 1	- Affect panel "Instruction details"
	 # @return void
	private method validate_line args {
		# Parse input arguments
		set line	[lindex $args 0]
		set ins_det	[lindex $args 1]
		if {$ins_det == {}} {
			set ins_det 1
		}

		# Check if basic validation is enabled
		if {!${::ASMsyntaxHighlight::validation_L0}} {return}

		# Detereminate range of instruction tag
		set ins_range [$editor tag nextrange tag_instruction $line.0 "$line.0 lineend"]

		# Detereminate instruction name
		if {[llength $ins_range]} {
			set instruction [$editor get [lindex $ins_range 0] [lindex $ins_range 1]]
			set instruction [string tolower $instruction]

			if {[lsearch -ascii -exact ${CompilerConsts::AllInstructions} $instruction] == -1} {
				return
			}
		} {
			return
		}

		# Inset selection in "Instruction details" tab on the Right Panel
		if {$ins_det} {
			$parentObject rightPanel_ins_unselect
		}

		# Check for allowed number of operands
		if {
			${::ASMsyntaxHighlight::operands_count}
				!=
			[lindex $::CompilerConsts::InstructionDefinition($instruction) 0]
		} then {
			$editor tag add tag_error [lindex $ins_range 0] [lindex $ins_range 1]
			return
		}

		# Handle instruction without operands
		if {!${::ASMsyntaxHighlight::operands_count}} {
			if {$ins_det} {
				$parentObject rightPanel_ins_select 1 0
			}
			return
		}

		# Check for valid operand types
		if {${::ASMsyntaxHighlight::validation_L1}} {
			# Local variables
			set matches	{}	;# List of matched operand sets
			set matches0	{}	;# List of not perfectly matched operand sets

			# Iterate over simple definitions and find matches
			set operands		${::ASMsyntaxHighlight::opr_types}
			set operands_org	$operands
			for {set i 0} {$i < 3} {incr i} {

				set idx 0
				foreach opr_set $CompilerConsts::SimpleOperandDefinitions($instruction) {
					if {$opr_set == $operands} {
						if {$i} {
							lappend matches0 $idx
						} {
							lappend matches $idx
						}
					}
					incr idx
				}

				# Try to change operand set without changing meaning
				while {$i < 2} {
					if {
						[lindex $operands $i] != {A}
							&&
						[lindex $operands $i] != {C}
					} then {
						incr i
						continue
					}

					set operands $operands_org
					if {[lindex $operands $i] == {A}} {
						lset operands $i {D}
					} elseif {[lindex $operands $i] == {C}} {
						lset operands $i {D}
					}
					break
				}
			}

			# Highlight coresponding operand sets in "Instruction details"
			if {[llength $matches] || [llength $matches0]} {
				if {$ins_det} {
					if {[llength $matches]} {
						$parentObject rightPanel_ins_select 1 $matches
					}
					if {[llength $matches0]} {
						$parentObject rightPanel_ins_select 0 $matches0
					}
				}
			} {
				$editor tag add tag_error [lindex $ins_range 0] [lindex $ins_range 1]
			}

			# Check for legal usege of SFRs and SFBs
			set sfr_range_start	$line.0
			set sfr_range		{}
			set sfr_name		{}
			while 1 {
				# Try to find SFR
				set sfr_range [$editor tag nextrange tag_sfr $sfr_range_start [list $line.0 lineend]]
				if {![llength $sfr_range]} {
					break
				}

				# Check for its legality
				set sfr_range_start [lindex $sfr_range 1]
				set sfr_name [$editor get [lindex $sfr_range 0] [lindex $sfr_range 1]]
				if {[lsearch -ascii -exact [$parentObject cget -avaliable_SFR] [string toupper $sfr_name]] == -1} {
					$editor tag add tag_error [lindex $sfr_range 0] [lindex $sfr_range 1]
				}
			}
		}
	}

	## Adjust content of the given line in list of bookmakrs and list of breakpoint (in right panel)
	 # This function should be called after change content of any line
	 # @parm Int lineNumber - line number
	 # @return void
	private method rightPanel_changeLineContent {lineNumber} {
		# Adjust list of bookmarks
		if {[lindex $bookmarks $lineNumber] == 1} {
			$parentObject rightPanel_remove_bookmark	$lineNumber
			$parentObject rightPanel_add_bookmark		$lineNumber
			$parentObject rightPanel_bm_select		$lineNumber
		}
		# Adjust list of breakpoints
		if {[lindex $breakpoints $lineNumber] == 1} {
			$parentObject rightPanel_remove_breakpoint	$lineNumber
			$parentObject rightPanel_add_breakpoint		$lineNumber
			$parentObject rightPanel_bp_select		$lineNumber
		}
	}

	## Convert editor line number to left border line number
	 # @parm Int idx	- line number (in editor)
	 # @return Int		- line number (in left border)
	private method wrap_aux_idx2line {idx} {
		if {$number_of_wraps} {
			set remaining $number_of_wraps
			set line $idx
			for {set i 1} {$i < $idx} {incr i} {
				set wrap [lindex $map_of_wraped_lines $i]
				incr line $wrap
				incr remaining -$wrap
				if {!$remaining} {break}
			}
			return $line
		} else {
			return $idx
		}
	}

	## Convert left border line number to editor line number
	 # @parm Int idx	- line number (in left border)
	 # @return Int		- line number (in editor)
	private method wrap_aux_line2idx {line} {
		if {$number_of_wraps} {
			set i 1
			while 1 {
				incr line [expr { -1 - [lindex $map_of_wraped_lines $i]}]
				if {$line < 1 || $line == {}} {break}
				incr i
			}
			return $i
		} else {
			return $line
		}
	}

	## Handle click on "Line numbers"
	 # Add/Remove breakpoint
	 # @parm Int x - realative X coordinate
	 # @parm Int y - realative Y coordinate
	 # @return void
	public method lineNumbers_click {x y} {
		set line [expr {int([$lineNumbers index @$x,$y])}]
		Breakpoint [wrap_aux_line2idx $line]
	}

	## Handle click on "Icon border"
	 # Add/Remove bookmarks
	 # @parm Int x - realative X coordinate
	 # @parm Int y - realative Y coordinate
	 # @return void
	public method iconBorder_click {x y} {
		set line [expr {int([$iconBorder index @$x,$y])}]
		Bookmark [wrap_aux_line2idx $line]
	}

	## Focus on the editor widget
	 # @return void
	public method focus_in {} {
		focus $editor
	}

	## Get ranges for all highlighting tags on the given line
	 # @parm Int lineNum - line number
	 # @return List - tag ranges {{tag_name {start_idx end_idx ...}} ... }
	public method getTagsRanges {lineNum} {
		# Initialize resulting ranges
		set ranges {}

		# Determnate end index
		set endIdx [$editor index "$lineNum.0 lineend"]

		# Iterate over defined highlighting tags
		foreach tag [concat				\
			${ASMsyntaxHighlight::hightlight_tags}	\
			${CsyntaxHighlight::hightlight_tags}	\
			${LSTsyntaxHighlight::hightlight_tags}	\
			{tag_macro_def tag_constant_def}	\
		] {

			# Determinate tag name
			set tag [lindex $tag 0]

			# Determinate range of the tag
			set range {}
			while 1 {
				# Determinate start index
				set startIdx [lindex $range [expr {[llength $range] - 1}]]
				if {$startIdx == {}} {
					set startIdx $lineNum.0
				}
				# Gain tag range
				set rng [$editor tag nextrange $tag $startIdx $endIdx]
				if {![llength $rng]} {break}
				# Append range
				append range $rng
				append range { }
			}
			set range [string range $range 0 {end-1}]

			# Skip empty ranges
			if {[llength $range] == 0} {continue}

			# Append the range to result
			lappend tag $range
			lappend ranges $tag
		}

		# Return resulting range
		return $ranges
	}

	## Get data of bookmarks and breapoints
	 # @return List - {bookmarks breakpoints} (eg. {00011 10100})
	public method export_line_markers_data {} {
		set result [join $bookmarks {}]
		lappend result [join $breakpoints {}]
		return $result
	}

	## Get total number of lines in editor
	 # @return Int - result
	public method getLinesCount {} {
		if {$editor_to_use} {return 0}

		set result [$editor index end]
		return [expr {int($result) - 1}]
	}

	## Import list of bookmarks and breapoints
	 # This function also validates given input data
	 # This function does not do anything with the right panel
	 # @parm String Bookmarks	- bookmakrs	(eg. 00011010)
	 # @parm String Breakpoints	- breakpoints	(eg. 011010000000)
	 # @return void
	public method import_line_markers_data {Bookmarks Breakpoints} {
		if {$editor_to_use} {return}

		# Check validity of the given data
		if {![regexp {^[01]*$} $Bookmarks]} {
			puts stderr "Invalid list of bookmarks -- bookmarks discarded"
			set Bookmarks {}
		}
		if {![regexp {^[01]*$} $Breakpoints]} {
			puts stderr "Invalid list of breakpoints -- bookmarks discarded"
			set Breakpoints {}
		}

		# Determinate number of the last line in the editor
		set lastEnd [expr {int([$editor index end])}]

		# Initialize list of highlighted lines
		set highlighted_lines [string repeat 0 $lastEnd]

		# Enable left panel
		$iconBorder configure -state normal
		$lineNumbers configure -state normal

		# Fill in text widgets in left frame
		$iconBorder delete 1.0 end
		$iconBorder insert end [string repeat "\n" [expr {$lastEnd-2}]]
		for {set i 2} {$i < $lastEnd} {incr i} {
			$lineNumbers insert end "\n$i"
		}
		$lineNumbers configure -width [string length [expr {$lastEnd-1}]]

		## Import bookmarks
		set len [string bytelength $Bookmarks]	;# Number of bookmark flags
		set bookmarks [split $Bookmarks {}]	;# Bookmarks -> List
		# Adjust given input data (length)
		if {$lastEnd > $len} {
			append bookmarks [string repeat { 0} [expr {$lastEnd - $len}]]
		} elseif {$lastEnd < $len} {
			set bookmarks [lrange $bookmarks 0 [expr {$lastEnd - 1}]]
		}
		# Determinate list of bookmarked lines
		foreach line [lsearch  -ascii -exact -all $bookmarks 1] {
			if {!$line} {continue}

			# Create bookmark image
			if {$defaultCharHeight < 9} {
				$iconBorder insert $line.0 {*}
			} elseif {$defaultCharHeight < 15} {
				$iconBorder image create $line.0	\
					-image ::ICONS::16::dot		\
					-align center
			} {
				$iconBorder image create $line.0	\
					-image ::ICONS::16::bookmark	\
					-align center
			}
			# Create bookmark text tag
			$editor tag add tag_bookmark $line.0 [expr {$line + 1}].0
			parse $line
		}

		## Import breakpoints
		set len [string bytelength $Breakpoints]	;# Number of breakpoint flags
		set breakpoints [split $Breakpoints {}]		;# Breakpoints -> List
		# Adjust given input data (length)
		if {$lastEnd > $len} {
			set ins [string repeat { 0} [expr {$lastEnd - $len}]]
			append breakpoints $ins
		} elseif {$lastEnd < $len} {
			set breakpoints [lrange $breakpoints 0 [expr {$lastEnd - 1}]]
		}
		# Determinate list of lines marked with bookmark
		foreach line [lsearch  -ascii -exact -all $breakpoints 1] {
			if {!$line} {continue}

			set line_1 $line
			incr line_1
			$lineNumbers tag add tag_breakpoint $line.0 $line_1.0
			parse $line
		}

		# Disable left panel
		$lineNumbers tag add right 1.0 end
		$iconBorder tag add center 1.0 end
		$iconBorder configure -state disabled
		$lineNumbers configure -state disabled

		reset_wraped_lines
	}

	## Restore breakpoint tags in "Line numbers"
	 # @return void
	private method rewrite_breakpoint_tags {} {
		if {$editor_to_use} {return}

		# Enable line numbers
		$lineNumbers configure -state normal

		# Remove current tags
		$lineNumbers tag remove tag_breakpoint 1.0 end
		# Restore tags
		foreach line [lsearch  -ascii -exact -all $breakpoints 1] {
			append line {.0}
			$lineNumbers tag add tag_breakpoint $line "$line+1l"
		}

		# Disable line numbers
		$lineNumbers configure -state disabled
	}

	## Define line markers (bookmark, breakpoint, simulator line, etc.)
	 # @return void
	public method define_line_markers {} {
		if {$editor_to_use} {return}

		# Iterate over definition
		foreach tag_definition $line_markers {
			# Create tag in editor
			$editor tag configure [lindex $tag_definition 0] -background [lindex $tag_definition 1]
			# Create tag in line numbers
			if {[lindex $tag_definition 0] == {tag_breakpoint}} {
				$lineNumbers tag configure [lindex $tag_definition 0]	\
					-background [lindex $tag_definition 1] -relief raised -borderwidth 1
			}
		}
		$editor tag configure tag_current_line -borderwidth 0 -relief flat
	}

	## Execute any editor procedure
	 # @parm String null		- anything (doesn't matter)
	 # @parm String procedure	- procudure name
	 # @parm String arguments	- procedure arguments
	 # @return String - procedure result
	public method editor_procedure {null procedure arguments} {
		# call editor's procedure
		return [eval "$procedure $arguments"]
	}

	## Jump to the bookmark below the current line
	 # @return Bool - result
	public method goto_next_bookmark {} {
		if {$editor_to_use} {return}

		# Local varibales
		set line $last_cur_line			;# Current line
		set linesMax [llength $bookmarks]	;# Maximal line number

		incr line

		# Search for the nearest bookmark
		for {set i $line} {$i < $linesMax} {incr i} {
			if {[lindex $bookmarks $i] == 1} {
				goto $i
				return 1
			}
		}

		# Failed
		return 0
	}

	## Jump to the bookmark above the current line
	 # @return Bool - result
	public method goto_prev_bookmark {} {
		if {$editor_to_use} {return}

		# Local varibales
		set line $last_cur_line			;# Current line
		set linesMax [llength $bookmarks]	;# Maximal line number

		incr line -1

		# Search for the nearest bookmark
		for {set i $line} {$i > 0} {incr i -1} {
			if {[lindex $bookmarks $i] == 1} {
				goto $i
				return 1
			}
		}

		# Failed
		return 0
	}

	## Set state of editor lock on status bar
	 # @parm Bool bool - 1 == Locked; 0 == Unlocked
	 # @return void
	public method set_lock {bool} {
		if {$bool} {
			setStatusTip -widget $Sbar_lock_file -text "File switching locked"
			Sbar -freeze "File switching locked"
			$Sbar_lock_file configure		\
				-image ::ICONS::16::lock	\
				-helptext "Unlock file switching"
		} {
			setStatusTip -widget $Sbar_lock_file -text "File switching unlocked"
			Sbar -freeze "File switching unlocked"
			$Sbar_lock_file configure		\
				-image ::ICONS::16::unlock	\
				-helptext "Lock file switching"
		}
		set auto_switching_lock $bool
	}

	## Invert simulator lock
	 # @retunr void
	public method invert_lock {} {
		set_lock [expr {!$auto_switching_lock}]
		$parentObject set_editor_lock $this $auto_switching_lock
	}

	## Switch from editor mode to simulator mode
	 # This operation will cause error if editor is in mode disabled
	 # @return void
	public method freeze {} {
		if {$editor_to_use} {return}

		# Adjust editor
		$editor configure -state disabled
		$editor tag remove tag_current_line 1.0 end
		pack forget $Sbar_CRT_frame
		catch {
			pack forget $Sbar_dis_mode
		}
		catch {
			pack forget $Sbar_ssim_mode
		}
		catch {
			pack forget $Sbar_sim_mode
		}
		if {!$frozen} {
			pack $Sbar_ssim_mode -side right
			pack $Sbar_lock_file -side left
		} {
			pack $Sbar_sim_mode -side right
		}
		# Disable some popup menu items
		foreach entry $freezable_menu_items {
			$menu entryconfigure $entry -state disabled
		}
		# Set mode flag
		set frozen 1
	}

	## IDE is now in "Simulator mode" (previous state was "Starting simulator")
	 # @return void
	public method now_frozen {} {
		if {[winfo ismapped $Sbar_ssim_mode]} {
			pack forget $Sbar_ssim_mode
			pack $Sbar_sim_mode -side right
		}
	}

	## Switch from editor mode to disabled
	 # This operation will cause error if editor is in simulator mode
	 # @return void
	public method disable {} {
		if {$editor_to_use} {return}

		# Adjust editor
		$editor configure -state disabled
		$editor tag remove tag_current_line 1.0 end
		pack forget $Sbar_CRT_frame
		catch {
			pack forget $Sbar_sim_mode
		}
		pack $Sbar_dis_mode -side right
		if {!$frozen} {
			pack $Sbar_lock_file -side left
		}
		# Disable some popup menu items
		foreach entry $freezable_menu_items {
			$menu entryconfigure $entry -state disabled
		}
		# Set mode flag
		set frozen 1
	}

	## Switch from simulator mode to editor mode
	 # @return void
	public method thaw {} {
		if {$editor_to_use} {return}

		# Set mode flag
		set frozen 0
		# Adjust editor
		if {!$ro_mode} {
			$editor configure -state normal
		}
		focus -force $editor
		set idx [$editor index "insert linestart"]
		$editor tag add tag_current_line $idx "$idx + 1 line"
		pack $Sbar_CRT_frame
		pack forget $Sbar_lock_file
		catch {
			pack forget $Sbar_sim_mode
		}
		catch {
			pack forget $Sbar_ssim_mode
		}
		catch {
			pack forget $Sbar_dis_mode
		}
		# Enable all popup menu items
		if {!$ro_mode} {
			foreach entry $freezable_menu_items {
				$menu entryconfigure $entry -state normal
			}
		}
		# Recalculate counters
		recalc_status_counter {} 0
	}

	## Move simulator line (line representing current position in simulator engine)
	 # @parm Int lineNum - target line number
	 # @return void
	public method move_simulator_line {lineNum} {
		if {$editor_to_use} {return}
		set lineNum_1 $lineNum
		incr lineNum_1
		unset_simulator_line
		$editor tag add tag_simulator_curr $lineNum.0 $lineNum_1.0
		$editor see $lineNum.0
	}

	## Unset simulator line tag and restore current line tag
	 # @return void
	public method unset_simulator_line {} {
		$editor tag remove tag_simulator_curr 1.0 end
	}

	## Create bindings for defined key shortcuts
	 # @return void
	public method shortcuts_reevaluate {} {
		# Unset previous configuration
		foreach key $set_shortcuts {
			bind $editor <$key> {}
		}
		set set_shortcuts {}

		# Iterate over shortcuts definition
		foreach block ${::SHORTCUTS_LIST} {
			# Determinate category
			set category	[lindex $block 0]
			if {[lsearch $shortcuts_cat $category] == -1} {continue}

			# Determinate definition list and its length
			set block	[lreplace $block 0 2]
			set len		[llength $block]

			# Iterate over definition list and create bindings
			for {set i 0; set j 1} {$i < $len} {incr i 2; incr j 2} {
				# Determinate key sequence
				set key [lindex $block $i]
				if {$key == {cmd_line}} {
					catch {
						bind $cmd_line <$::SHORTCUTS_DB($category:$key)>	\
							"$this cmd_line_focus; break"
						bind . <$::SHORTCUTS_DB($category:$key)>		\
							"\${::X::actualProject} cmd_line_on; break"
					}
					continue
				}
				if {[catch {
					set key $::SHORTCUTS_DB($category:$key)
				}]} then {
					continue
				}
				if {$key == {}} {continue}

				# Create and register new binding
				lappend set_shortcuts $key
				set cmd [subst [lindex $block [list $j 1]]]
				append cmd {;break}

				bind $editor <$key> $cmd
			}
		}
	}

	## Define popup menu
	 # @return void
	public method makePopupMenu {} {
		if {[winfo exists $menu]} {destroy $menu}
		menuFactory $EDITORMENU $menu 0 $cmd_prefix 0 {}

		if {[winfo exists $stat_menu]} {destroy $stat_menu}
		menuFactory $STATMENU $stat_menu 0 {::X::} 0 {}

		if {[winfo exists $IB_menu]} {destroy $IB_menu}
		menuFactory $IBMENU $IB_menu 0 "$this " 0 {}

		if {[winfo exists $LN_menu]} {destroy $LN_menu}
		menuFactory $LNMENU $LN_menu 0 "$this " 0 {}
	}

	## Do the same as scroll + set up the scollbar
	 # @parm Float fraction0	- Freaction where to move
	 # @parm Float fraction1	- Freaction where to setup end of visible area for the scrollbar
	 # @return void
	public method scrollSet {fraction0 fraction1} {
		$scrollbar set $fraction0 $fraction1
		scroll moveto $fraction0
	}

	## Scroll simultaneously Icon border, Lines count and editor widget
	 # @parm String		- Scroll command (eg. 'moveto')
	 # @parm Float		- Scroll fraction
	 # @parm String	= {}	- Units
	 # @return void
	public method scroll args {
		# This function cannot recursively
		if {$scroll_in_progress} {return}
		set scroll_in_progress 1

		eval "$editor yview $args"

		# Highlight lines which hasn't been highlighted yet
		highlight_visible_area

		set idx [$editor index @5,5]
		scan $idx "%d.%d" row col

		set tmp_row $row
		if {$number_of_wraps} {
			set remaining $number_of_wraps
			for {set i 1} {$i < $row} {incr i} {
				set wrap [lindex $map_of_wraped_lines $i]
				incr tmp_row $wrap
				incr remaining -$wrap
				if {!$remaining} {break}
			}
		}
		if {$col != 0} {
			incr tmp_row [get_count_of_lines "$idx linestart" "$idx"]
		}
		incr tmp_row -1
		$iconBorder yview $tmp_row
		$lineNumbers yview $tmp_row
		if {$number_of_wraps && [lindex [split ${::tk_version} {.}] 0] >= 8 && [lindex [split ${::tk_version} {.}] 1] >= 5} {
			$editor yview $tmp_row
		}

		# Done ...
		update idle
		set scroll_in_progress 0
	}

	## Highlight lines which hasn't been highlighted yet
	 # @return void
	public method highlight_visible_area {} {
		# Determinate indexes of the current view
		set view	[$editor yview]
		set lastLine	[expr {int([$editor index end])}]
		set start	[expr {int([lindex $view 0] * $lastLine) + 1}]
		set end		[expr {int([lindex $view 1] * $lastLine) + 1}]

		# Adjust start and end index
		if {$start < 1} {
			set start 1
		}
		if {$end > $lastLine} {
			set end $lastLine
		}

		# Abort if there is nothing to do
		if {[string first 0 [string range $highlighted_lines $start $end]] == -1} {
			return
		}

		# Enable editor if it is disabled
		if {$frozen} {$editor configure -state normal}

		# Highlight the current view
		for {set line $start} {$line <= $end} {incr line} {
			if {[string index $highlighted_lines $line] == 0} {
				if {![parse $line]} {
					if {$line != $start} {break}
				}
			}
		}

		# Restore previous editor state
		if {$frozen} {$editor configure -state disabled}
	}

	## Invoke editor popup menu
	 # @parm Int X - absolute X coordinate
	 # @parm Int Y - absolute Y coordinate
	 # @parm Int x - relative X coordinate
	 # @parm Int y - relative Y coordinate
	 # @return void
	public method popupMenu {X Y x y} {
		if {![winfo exists $menu]} {return}

		if {$frozen} {
			set address [$parentObject simulator_line2address	\
				[expr {int([$editor index @$x,$y])}]		\
				[$parentObject simulator_get_filenumber $fullFileName]	\
			]
			if {$address == {}} {
				set state {disabled}
			} {
				set state {normal}
			}
			$menu entryconfigure {LJMP this line} -state $state
			$menu entryconfigure {LCALL this line} -state $state
		} {
			$menu entryconfigure {LJMP this line} -state disabled
			$menu entryconfigure {LCALL this line} -state disabled
		}

		tk_popup $menu $X $Y
		$editor mark set insert "@$x,$y"
		recalc_status_counter {} 0
	}

	## Invoke Icon Border popup menu
	 # @parm Int X - absolute X coordinate
	 # @parm Int Y - absolute Y coordinate
	 # @parm Int x - relative X coordinate
	 # @parm Int y - relative Y coordinate
	 # @return void
	public method iconBorder_popup_menu {X Y x y} {
		if {![winfo exists $IB_menu]} {return}
		set line [expr {int([$iconBorder index @$x,$y])}]
		set line [wrap_aux_line2idx $line]
		set pmenu_cline $line
		set bookmark [lindex $bookmarks $line]
		tk_popup $IB_menu $X $Y
	}

	## Invoke Line Numbers popup menu
	 # @parm Int X - absolute X coordinate
	 # @parm Int Y - absolute Y coordinate
	 # @parm Int x - relative X coordinate
	 # @parm Int y - relative Y coordinate
	 # @return void
	public method lineNumbers_popup_menu {X Y x y} {
		if {![winfo exists $LN_menu]} {return}
		set line [expr {int([$lineNumbers index @$x,$y])}]
		set line [wrap_aux_line2idx $line]
		set pmenu_cline $line
		set breakpoint [lindex $breakpoints $line]
		tk_popup $LN_menu $X $Y
	}

	## Invoke statusbar popup menu
	 # @parm Int X - absolute X coordinate
	 # @parm Int Y - absolute Y coordinate
	 # @return void
	public method statusbar_popup_menu {X Y} {
		if {![winfo exists $stat_menu]} {return}

		if {[lindex $statusbar_menu_config 0] != 0} {
			set state normal
		} {
			set state disabled
		}
		$stat_menu entryconfigure "Split vertical" -state $state
		$stat_menu entryconfigure "Split horizontal" -state $state

		if {[lindex $statusbar_menu_config 1] != 0} {
			set state normal
		} {
			set state disabled
		}
		$stat_menu entryconfigure "Close current view" -state $state

		if {[lindex $statusbar_menu_config 2] != 0} {
			set state normal
		} {
			set state disabled
		}
		$stat_menu entryconfigure "Back" -state $state

		if {[lindex $statusbar_menu_config 3] != 0} {
			set state normal
		} {
			set state disabled
		}
		$stat_menu entryconfigure "Forward" -state $state
		set ::X::selectedView $this
		tk_popup $stat_menu $X $Y
	}

	## Configure state of statusbar popup menu entries
	 # @parm Bool split	- Enable Spit vertial / horizontal	or {} == keep previous value
	 # @parm Bool close	- Enable "Close current view"		or {} == keep previous value
	 # @parm Bool prev	- Enable "Back"				or {} == keep previous value
	 # @parm Bool next	- Enable "Forward"			or {} == keep previous value
	 # @return void
	public method configure_statusbar_menu {split close prev next} {
		if {[llength $statusbar_menu_config] != 4} {
			set statusbar_menu_config [list 1 1 1 1]
		}
		if {$split != {}} {
			lset statusbar_menu_config 0 $split
		}
		if {$close != {}} {
			lset statusbar_menu_config 1 $close
		}
		if {$prev != {}} {
			lset statusbar_menu_config 2 $prev
		}
		if {$next != {}} {
			lset statusbar_menu_config 3 $next
		}
	}

	## Rewrite left site of editor status bar
	 # @parm List 		- Relative mouse cursor coordinates ({%x,%y})
	 #			  {} == keyboard input (eg. leftArrow pressed)
	 # @parm Bool = 1	- Highlight current line and such things
	 # @return void
	public method recalc_status_counter args {
		if {$editor_to_use} {return}

		# Procedure can executed only in editor mode
		if {$frozen} {return}

		# Parse arguments
		if {[lindex $args 0] == {}} {
			set coord insert
		} {
			set coord "@[lindex $args {0 0}],[lindex $args {0 1}]"
		}
		if {[lindex $args 1] == {}} {
			set perform_highlight 1
		} {
			set perform_highlight [lindex $args 1]
		}

		# Translate text index into number
		set Index [$editor index $coord]
		# Determinate line number and column
		set line [expr {int($Index)}]
		regexp {\d+$} $Index col

		# Adjust column number
		set lineText [$editor get "$Index linestart" $Index]
		set Index $col
		if {[regexp {\t} $lineText]} {
			set idx -1
			set cor 0
			while 1 {
				set idx [string first "\t" $lineText [expr {$idx + 1}]]
				if {$idx == -1 || $idx > $Index} {break}

				incr cor [expr {7 - (($idx + $cor) % 8)}]
			}
			incr col $cor
		}

		# Restore tag current line
		set tmp $last_cur_line
		incr tmp
		if {$perform_highlight || $last_cur_line != $line} {
			$editor tag remove tag_current_line 1.0 end
			set tmp $line
			incr tmp
			$editor tag add tag_current_line $line.0 $tmp.0
			set last_cur_line $line
		} {
			$editor tag add tag_current_line $line.0 $tmp.0
		}

		# Restore highlight
		if {$perform_highlight && ${::ASMsyntaxHighlight::validation_L1}} {
			if {$prog_language == 0} {
				ASMsyntaxHighlight::highlight $editor $line
			} elseif {$prog_language == 1} {
				if {$line > 1 && [string index $highlighted_lines [expr {$line - 1}]] != 0} {
					c_syntax_highlight $line
				} elseif {$line == 1} {
					c_syntax_highlight 1
				}
			} elseif {$prog_language == 2} {
				LSTsyntaxHighlight::highlight $editor $line
			} elseif {$prog_language == 3} {
				R_ASMsyntaxHighlight::highlight $editor $line
			}

			manage_autocompletion_list $line
		}

		# Highlight trailing space
		if {$perform_highlight} {
			highlight_trailing_space $line
		}

		# Adjust content of "Instruction details" on Right Panel
		adjust_instruction_details

		# Advanced validation
		if {$perform_highlight && ${::ASMsyntaxHighlight::validation_L1}} {
			validate_line $line
		}

		restore_line_markers $line

		# Change content of editor status bar
		incr col
		set total	[expr {$lastEnd - 1}]
		set line_len	[string length $line]
		set col_len	[string length $col]
		set total_len	[string length $total]
		if {$line_len < $total_len} {
			set line "[string repeat { } [expr {$total_len - $line_len}]]$line"
		}
		if {$col_len < 3} {
			set col "[string repeat { } [expr {3 - $col_len}]]$col"
		}
		$Sbar_row	configure -text $line
		$Sbar_col	configure -text $col
		$Sbar_total	configure -text $total
	}

	## Adjust content of "Instruction details" on Right Panel
	 # @return void
	public method adjust_instruction_details {} {
		set ins_range [$editor tag nextrange tag_instruction {insert linestart} {insert lineend}]
		if {[llength $ins_range]} {
			$parentObject rightPanel_ins_change [$editor get [lindex $ins_range 0] [lindex $ins_range 1]]
		} {
			$parentObject rightPanel_ins_clear
		}
	}

	## Save content of editor text widget
	 # note: Name of the target file should be stored in $fullFileName,
	 #	 if it is not then invoke procedure 'X::__save_as'
	 # @return Bool - result
	public method save {} {
		if {$editor_to_use} {return 1}
		if {$save_in_progress} {return}
		set save_in_progress 1

		# Check previously set filename
		if {$fullFileName == {}} {
			# Ask user for a new filename
			set ::X::critical_procedure_in_progress 0
			X::__save_as
		} {
			# save data to file
			if {[file exists $fullFileName]} {
				catch {
					file rename -force $fullFileName "$fullFileName~"
				}
			}
			if {[catch {
				set chanel [open $fullFileName w 420]
			}]} then {
				tk_messageBox		\
					-parent .	\
					-icon warning	\
					-type ok	\
					-title "Permission denied"	\
					-message "Unable to open file:\n\"$fullFileName\"\nfor writing"
				set save_in_progress 0
				return 0
			}
			fconfigure $chanel -translation $eol -encoding $encoding
			puts -nonewline $chanel [getdata]
			close $chanel
			pack forget $Sbar_image
			$editor edit modified 0
			set modified 0

			# Stop autosave timer
			catch {
				after cancel $autosave_timer
			}
		}

		# Change application status
		Sbar "$fullFileName saved"
		::X::adjust_title

		set save_in_progress 0
		if {$fullFileName == {}} {
			return 0
		} {
			return 1
		}
	}

	## Set variable 'fullFileName' for later file save (method 'save')
	 # note: also change editors status bar
	 # @parm String full_filename	- the full filename (including path)
	 # @parm String rootName	- only filename with extension
	 # @return void
	public method set_FileName {full_filename rootName} {
		if {$editor_to_use} {return}

		# set variables
		set fullFileName $full_filename
		set filename $rootName
		# change etitor status bar
		$Sbar_fileName configure -text $filename -helptext $filename
		# Determinate programming language
		determinate_prog_lang 1
	}

	## Get current filename
	 # @return String - the filename
	public method getFileName {} {
		return [list [file dirname $fullFileName] $filename]
	}

	## Change letter case acording to the given options
	 # @parm List options	- list of 21 values, each must be one of {- L -U}
	 #			  '-' - keep case
	 #			  'U' - Uppercase
	 #			  'L' - Lowercase
	 # @return void
	public method change_letter_case {options} {

		# Reset abort condition
		set changeLCase_abort 0

		# Initialize conter of iterations
		set i 0

		# Perform case change
		foreach option $options	\
			tags {
				tag_hex			tag_oct
				tag_dec			tag_bin
				tag_constant		tag_unknown_base
				tag_comment		tag_control
				tag_symbol		tag_directive
				tag_label		tag_instruction
				tag_sfr			tag_indirect
				tag_imm_hex		tag_imm_oct
				tag_imm_dec		tag_imm_bin
				tag_imm_constant	tag_imm_unknown
				tag_macro
			}	\
		{
			# Evaluate option
			if {$option == {-}} {continue}
			if {$option == {U}} {
				set option {toupper}
			} {
				set option {tolower}
			}

			if {$tags == {tag_constant}} {
				lappend tags {tag_constant_def}
			} elseif {$tags == {tag_macro}} {
				lappend tags {tag_macro_def}
			}

			# Iterate over tag ranges and change their letter case
			foreach tag $tags {
				set ranges [$editor tag ranges $tag]
				for {set j 0} {$j < [llength $ranges]} {incr j} {
					# Determinate string indexes
					set firts [lindex $ranges $j]
					incr j
					set last [lindex $ranges $j]

					# Perform letter case change
					set string [string $option [$editor get $firts $last]]
					$editor delete $firts $last
					$editor insert $firts $string
					$editor tag add $tag $firts $last

					# Manage GUI
					if {![expr {$i % 50}]} {
						# Update progress bar
						incr ::X::compilation_progress
						update

						# Conditional abort
						if {$changeLCase_abort} {
							return
						}
					}

					# Increment counter of iterations
					incr i
				}
			}
		}
	}

	## Abort procedure 'change_letter_case'
	 # @return void
	public method change_letter_case_abort_now {} {
		set changeLCase_abort 1
	}

	## Get maximum value for progressbar showing change letter case progress (proc. 'change_letter_case')
	 # @parm List options - same as with proc. 'change_letter_case'
	 # @return Int - Number of iterations divided by 50
	public method change_letter_case_get_count_of_iterations {options} {
		set result 0
		foreach option $options	\
			tag {	tag_hex			tag_oct
				tag_dec			tag_bin
				tag_constant		tag_unknown_base
				tag_comment		tag_control
				tag_symbol		tag_directive
				tag_label		tag_instruction
				tag_sfr			tag_indirect
				tag_imm_hex		tag_imm_oct
				tag_imm_dec		tag_imm_bin
				tag_imm_constant	tag_imm_unknown
				tag_macro}	\
		{
			if {$option == {-}} {continue}

			incr result [llength [$editor tag ranges $tag]]
			if {![expr {$result % 1000}]} {update}
		}
		return [expr {$result / 50}]
	}

	# -------------------------------------------------------------------
	# EXPORTS TO OTHER DATA FORMATS (XHTML && LaTeX)
	# -------------------------------------------------------------------

	## Get maximum value for progressbar showing highlightion progress (proc. highlight_all)
	 # @return Int - Number of lines to highlight divided by 50
	public method highlight_all_count_of_iterations {} {
		set result 0
		for {set i 1} {$i < [string bytelength $highlighted_lines]} {incr i} {
			if {[string index $highlighted_lines $i] == 0} {
				incr result
			}
			if {![expr {$i % 1000}]} {update}
		}
		return [expr {$result / 50}]
	}

	## Highlight all lines in the editor (can take a long time !)
	 # @return void
	public method highlight_all {} {
		# Reset abort variables
		set getDataAsLaTeX_abort 0
# 		$parentObject rightPanel_disable

		# Highlight all lines
		set len [string bytelength $highlighted_lines]
		for {set i 1} {$i < $len} {incr i} {
			if {[string index $highlighted_lines $i]} {continue}

			# Highlight line
			parse $i

			# Update progress bar
			if {![expr {$i % 50}]} {
				incr ::X::compilation_progress
				update
			}

			# Conditional abort
			if {$getDataAsLaTeX_abort} {
				set getDataAsLaTeX_abort 0
# 				$parentObject rightPanel_enable
				return
			}
		}
# 		$parentObject rightPanel_enable
	}

	## Get maximum value for export progress bar
	 # @return Int - the value
	public method getDataAsXHTML_count_of_iterations {} {
		set result 0
		foreach tag_def [concat				\
			${ASMsyntaxHighlight::hightlight_tags}	\
			${CsyntaxHighlight::hightlight_tags}	\
			${LSTsyntaxHighlight::hightlight_tags}	\
		] {
			set range [$editor tag ranges [lindex $tag_def 0]]
			incr result [llength $range]
			if {![expr {$result % 1000}]} {update}
		}
		return [expr {$result / 50}]
	}

	## Abort export to LaTeX
	 # @return void
	public method getDataAsLaTeX_abort_now {} {
		set getDataAsLaTeX_abort 1
	}

	## Export editor content as LaTeX source (include colors)
	 # @return String - LaTeX source code
	public method getDataAsLaTeX {} {

		# Reset abort variables
		set getDataAsLaTeX_abort 0

		# Local variables
		set end [$editor index end]	;# Editor end index
		set last_index 0		;# Current position (by characters)
		set line(1) 0			;# Map of indexes ($line(num) == scalar_index)

		# Create map of indexes
		for {set i 1; set j 2} {$i < $end} {incr i; incr j} {

			# Conditional abort
			if {$getDataAsLaTeX_abort} {
				set getDataAsLaTeX_abort 0
				return {}
			}

			# Determinate last column of the line
			set idx [$editor index "$i.0 lineend"]
			regexp {\d+$} $idx idx

			# Adjust map of indexes
			incr last_index $idx
			incr last_index
			set line($j) $last_index
		}

		# Create LaTeX preamble
		set latex "\\documentclass\[a4paper,12pt\]{article}"
		append latex "\n\n% Creator: ${::APPNAME}\n\n"
		append latex "\\usepackage\[utf-8\]{inputenc}\n"
		append latex "\\usepackage\[T1\]{fontenc}\n"
		append latex "\\usepackage{color}\n"
		append latex "\\title{$filename}\n"
		append latex "\\date{[clock format [clock seconds] -format {%D}]}\n"
		append latex "\n% define highlighting\n"

		## Determinate highlighting tag ranges and define colors for 'color' package
		set ranges {}
		# Iterate over predefined highlighting tags
		foreach tag_def [concat				\
			${ASMsyntaxHighlight::hightlight_tags}	\
			${CsyntaxHighlight::hightlight_tags}	\
			${LSTsyntaxHighlight::hightlight_tags}	\
		] {

			# Conditional abort
			if {$getDataAsLaTeX_abort} {
				set getDataAsLaTeX_abort 0
				return {}
			}

			# Local variables
			set color	[lindex $tag_def 1]		;# RGB color
			set red		[string range $color 1 2]	;# Color - RED
			set green	[string range $color 3 4]	;# Color - GREEN
			set blue	[string range $color 5 6]	;# Color - BLUE
			set tag		[lindex $tag_def 0]		;# Tag name
			set range	[$editor tag ranges $tag]	;# List of tag ranges
			set len		[llength $range]		;# Number of ranges
			set mirror_tag {}			;# Tag with exatly the same highlight

			# Determinate mirror tag
			switch -- $tag {
				{tag_constant}	{set mirror_tag tag_constant_def}
				{tag_macro}	{set mirror_tag tag_macro_def}
			}
			if {$mirror_tag != {}} {
				set mirror_range [$editor tag ranges $mirror_tag]
			} {
				set mirror_range {}
			}

			# If the tag isn't present in the text -> skip
			if {$len == 0 && ![llength $mirror_range]} {
				continue
			}
			# Adjust tag name
			set tag [string replace $tag 0 3]

			# Convert hexadecimal color values to decimal representation
			set red		[string range [expr "0x$red	/ 255.0"] 0 4]
			set green	[string range [expr "0x$green	/ 255.0"] 0 4]
			set blue	[string range [expr "0x$blue	/ 255.0"] 0 4]
			# Define color (for package color)
			append latex "\\definecolor{highlight_$tag}{rgb}{$red, $green, $blue}\n"

			# Adjust map of text tags
			set mirror_tag {}
			switch -- $tag {
				{constant}	{set mirror_tag tag_constant_def}
				{macro}		{set mirror_tag tag_macro_def}
			}
			for {set i 0} {$i < $len} {incr i} {
				lappend ranges [list [lindex $range $i] $tag 1]
				incr i
				lappend ranges [list [lindex $range $i] $tag 0]
			}
			if {$mirror_tag != {}} {
				set range [$editor tag ranges $mirror_tag]
				set len [llength $range]
				for {set i 0} {$i < $len} {incr i} {
					lappend ranges [list [lindex $range $i] $tag 1]
					incr i
					lappend ranges [list [lindex $range $i] $tag 0]
				}
			}
		}

		# Sort map of text tags (recursive)
		set ranges [lsort -command "::FileList::editor__sort_tag_ranges" $ranges]

		# Get plain text
		set text [$editor get 1.0 end]
		regsub -all {'} $text "\a" text

		## Create map of tabulators ("\t")
		set tab_map {}
		# Iterate ovet lines in editor
		foreach textLine [split $text "\n"] {

			if {$textLine == {}} {continue}
			set idx -1
			set spaces 0
			set correction 0

			while 1 {
				set idx [string first "\t" $textLine [expr {$idx + 1}]]
				if {$idx == -1} {break}

				set spaces [expr {8 - (($idx + $correction) % 8)}]
				incr correction [expr {$spaces - 1}]

				lappend tab_map $spaces
			}
		}

		# Write LaTeX control sequences
		set i 0
		foreach range $ranges {

			# Conditional abort
			if {$getDataAsLaTeX_abort} {
				set getDataAsLaTeX_abort 0
				return {}
			}

			# Update progress bar
			if {![expr {$i % 50}]} {
				incr ::X::compilation_progress
				update
			}

			set idx [split [lindex $range 0] {.}]	;# Text index
			set row [lindex $idx 0]			;# Line number
			set col [lindex $idx 1]			;# Column number

			# Determinate scalar text index
			set idx [expr {$line($row) + $col}]
			if {$idx < 0} {set idx 0}

			# Determinate string to insert
			if {[lindex $range 2]} {
				set tag "'\{\\color{highlight_[lindex $range 1]}\\verb'"
			} {
				set tag "'\}\\verb'"
			}

			# Insert control sequence into plain text
			set char [string index $text $idx]
			set text [string replace $text $idx $idx "$tag$char"]

			incr i
		}

		# Covert tabs to spaces
		set i 0
		foreach spaces $tab_map {
			set idx [string first "\t" $text]
			if {$idx == -1} {break}

			set text [string replace $text $idx $idx [string repeat { } $spaces]]
			if {![expr {$i % 1000}]} {update}
			incr i
		}

		# Adjust lines
		regsub -all -line {^} $text {\\verb'} text
		regsub -all -line {$} $text "'\\\\\\" text
		regsub -all -line {\s+'\\\\$} $text {'\\\\} text
		regsub -all {\\verb''} $text {} text
		regsub -all -line {^\\\\$} $text {\\verb''&} text
		regsub -all "\a" $text {'\\verb"'"\\verb'} text

		# Create final LaTeX document
		append latex "\n\n\\begin{document}\n"
		append latex "\\ \\\\\n"
		append latex $text
		append latex "\n\\end{document}"

		# Return result
		return $latex
	}

	## Abort export to XHTML
	 # @return void
	public method getDataAsXHTML_abort_now {} {
		set getDataAsXHTML_abort 1
	}

	## Export editor content as XHTML source (include colors)
	 # @return String - XHTML source code
	public method getDataAsXHTML {} {

		# Reset abort variables
		set getDataAsXHTML_abort 0

		# Local variables
		set end [$editor index end]	;# Editor end index
		set last_index 0		;# Current position (by characters)
		set line(1) 0			;# Map of indexes ($line(num) == scalar_index)

		# Create map of indexes
		for {set i 1; set j 2} {$i < $end} {incr i; incr j} {

			# Conditional abort
			if {$getDataAsXHTML_abort} {
				set getDataAsXHTML_abort 0
				return {}
			}

			# Determinate last column of the line
			set idx [$editor index [list $i.0 lineend]]
			regexp {\d+$} $idx idx

			# Adjust map of indexes
			incr last_index $idx
			incr last_index
			set line($j) $last_index
		}

		# Create XHTML header
		set html "<?xml version='1.0' encoding='utf-8' standalone='no'?>\n"
		append html "<!DOCTYPE html PUBLIC\n"
		append html "\t'-//W3C//DTD XHTML 1.1//EN'\n"
		append html "\t'http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd'>\n"
		append html "<html xmlns='http://www.w3.org/1999/xhtml' xml:lang='en'>\n"
		append html "<!--\n\tCreator: ${::APPNAME}\n\tDate: [clock format [clock seconds] -format {%D}]\n-->\n"
		append html "\t<head>\n"
		append html "\t\t<title>$filename</title>\n"
		append html "\t\t<meta http-equiv=\"Content-Type\" content=\"application/xhtml+xml; charset=UTF-8\" />\n"
		append html "\t\t<meta name=\"Generator\" content=\"${::APPNAME}\" />\n"
		append html "\t\t<style type=\"text/css\">\n"
		append html "\t\t\tbody {\n\t\t\t\tfont-family: $fontFamily;\n\t\t\t\tfont-size: ${fontSize}px;\n\t\t\t}\n"

		## Determinate highlighting tag ranges and define inline CSS
		set ranges {}
		# Iterate over predefined highlighting tags
		foreach tag_def [concat				\
			${ASMsyntaxHighlight::hightlight_tags}	\
			${CsyntaxHighlight::hightlight_tags}	\
			${LSTsyntaxHighlight::hightlight_tags}	\
		] {

			# Conditional abort
			if {$getDataAsXHTML_abort} {
				set getDataAsXHTML_abort 0
				return {}
			}

			# Local variables
			set tag [lindex $tag_def 0]		;# Tag name
			set range [$editor tag ranges $tag]	;# List of tag ranges
			set len [llength $range]		;# Number of ranges
			set mirror_tag {}			;# Tag with exatly the same highlight

			# Determinate mirror tag
			switch -- $tag {
				{tag_constant}	{set mirror_tag tag_constant_def}
				{tag_macro}	{set mirror_tag tag_macro_def}
			}
			if {$mirror_tag != {}} {
				set mirror_range [$editor tag ranges $mirror_tag]
			} {
				set mirror_range {}
			}

			# If the tag isn't present in the text -> skip
			if {$len == 0 && ![llength $mirror_range]} {
				continue
			}
			# Adjust tag name
			set tag [string replace $tag 0 3]

			# create CSS
			append html "\t\t\t.$tag {\n"
			append html "\t\t\t\tcolor: [lindex $tag_def 1];\n"
			if {[lindex $tag_def 2]} {
				append html "\t\t\t\ttext-decoration: line-through;\n"
			}
			if {[lindex $tag_def 3]} {
				append html "\t\t\t\tfont-style: italic;\n"
			}
			if {[lindex $tag_def 4]} {
				append html "\t\t\t\tfont-weight: bold;\n"
			}
			append html "\t\t\t}\n"

			for {set i 0} {$i < $len} {incr i} {
				lappend ranges [list [lindex $range $i] $tag 1]
				incr i
				lappend ranges [list [lindex $range $i] $tag 0]
			}
			if {$mirror_tag != {}} {
				set len [llength $mirror_range]
				for {set i 0} {$i < $len} {incr i} {
					lappend ranges [list [lindex $mirror_range $i] $tag 1]
					incr i
					lappend ranges [list [lindex $mirror_range $i] $tag 0]
				}
			}
		}
		append html "\t\t</style>\n"
		append html "\t</head>\n"

		# Sort tag ranges (recursive)
		set ranges [lsort -command "::FileList::editor__sort_tag_ranges" $ranges]

		# Get plain text
		set text [$editor get 1.0 end]
		# Translate '<' and '>' to '\a' and '\b'
		regsub -all {<} $text "\a" text
		regsub -all {>} $text "\b" text

		# Write XHTML tags to plain text
		set i 0
		foreach range $ranges {

			# Conditional abort
			if {$getDataAsXHTML_abort} {
				set getDataAsXHTML_abort 0
				return {}
			}

			# Update progress bar
			if {![expr {$i % 50}]} {
				incr ::X::compilation_progress
				update
			}

			# Local variables
			set idx [split [lindex $range 0] {.}]	;# Text index
			set row [lindex $idx 0]			;# Line number
			set col [lindex $idx 1]			;# Column number

			# Determinate scalar text index
			set idx [expr {$line($row) + $col}]
			# Skip unused tags
			if {$idx < 0} {set idx 0}

			# Deterinate string to insert
			if {[lindex $range 2]} {
				set tag "span class='[lindex $range 1]'"
			} {
				set tag {/span}
			}

			# Insert XHTML tag into the text
			set char [string index $text $idx]
			set text [string replace $text $idx $idx "<$tag>$char"]

			incr i
		}

		# Translate '&' -> &amp;
		regsub -all "&" $text {\&amp;} text
		# Traslate '\a', '\b' -> '&lt;', '&gt;'
		regsub -all "\a" $text {\&lt;} text
		regsub -all "\b" $text {\&gt;} text

		# Create final XHTML document
		append html "\t<body>\n\t\t<pre>\n"
		append html "\t\t<!-- CODE BLOCK - begin -->\n"
		append html $text
		append html "\t\t<!-- CODE BLOCK - end -->\n"
		append html "\t\t</pre>\n\t</body>\n</html>"

		# Return result
		return $html
	}

	# -------------------------------------------------------------------
	# KEY EVENT HANDLERS
	# -------------------------------------------------------------------

	## Determinate new cursor position when moving by one line up or down
	 # @return TextIndex - New cursor position
	private method get_up_down_idx {up__down} {
		# Local variables
		set insertIndex [$editor index insert]	;# Insert index
		set lineNum [expr {int($insertIndex)}]	;# Line number

		# Line start
		if {
			!$lastUpDownIndex && $insertIndex == [$editor index {insert linestart}]
		} then {
			if {$up__down} {
				return [$editor index {insert-1l linestart}]
			} {
				return [$editor index {insert+1l linestart}]
			}

		# Somewhere else
		} else {
			# Determinate true column number
			set col [text_index_to_column $insertIndex]

			# Determinate target column number
			if {!$lastUpDownIndex} {
				set lastUpDownIndex $col
			} {
				set col $lastUpDownIndex
			}

			# Traslate column number to text index
			if {$up__down} {
				incr lineNum -1
			} {
				incr lineNum
			}
			return [$editor index $lineNum.[column_to_text_index $lineNum $col]]
		}
	}

	## Translate text index (e.g. 5.11) to column number
	 # @parm TextIndex insertIndex - Text index to translate
	 # @return Int - Resulting column
	private method text_index_to_column {insertIndex} {
		set col [lindex [split $insertIndex {.}] 1]
		set lineText [$editor get [list $insertIndex linestart] $insertIndex]

		if {[regexp {\t} $lineText]} {
			set idx -1
			set cor 0
			while 1 {
				set idx [string first "\t" $lineText [expr {$idx + 1}]]
				if {$idx == -1} {break}

				incr cor [expr {7 - (($idx + $cor) % 8)}]
			}
			incr col $cor
		}
		return $col
	}

	## Translate column number to text index
	 # @parm Int lineNum	- Line number
	 # @parm Int col	- Column number
	 # @return TextIndex - Resulting insertIndex
	private method column_to_text_index {lineNum col} {
		set lineText [$editor get $lineNum.0 [list $lineNum.0 lineend]]
		set col_x $col
		if {[regexp {\t} $lineText]} {
			set cor 0
			set spaces 0
			set idx -1
			while 1 {
				set idx [string first "\t" $lineText [expr {$idx + 1}]]
				if {($idx == -1) || (($idx + $cor) >= $col)} {break}
				set spaces [expr {7 - (($idx + $cor) % 8)}]
				incr cor $spaces
				incr col_x -$spaces
			}
		}
		return $col_x
	}

	## Adjust current selection (tag "sel") to block selection mode
	 # @return void
	private method adjust_selection_to_block {} {
		# Nothing selected -> abort
		if {![llength [$editor tag nextrange sel 1.0]]} {
			return
		}

		# Get current selection shape
		set sel_range_s [$editor tag nextrange sel 1.0]
		set sel_range_e [$editor tag prevrange sel end]
		scan [lindex $sel_range_s 0] %d.%d row_s0 col_s0
		scan [lindex $sel_range_s 1] %d.%d row_s1 col_s1
		scan [lindex $sel_range_e 0] %d.%d row_e0 col_e0
		scan [lindex $sel_range_e 1] %d.%d row_e1 col_e1

		# This is only a speed improvement (may cause unexected probles)
		if {$row_s0 == $row_s1 && $row_e0 == $row_e1 && $col_s0 == $col_e0 && $col_s1 == $col_e1} {
			return
		}

		# Translate column numbers to real column numbers
		set col_s0 [text_index_to_column $row_s0.$col_s0]
		set col_s1 [text_index_to_column $row_s1.$col_s1]
		set col_e0 [text_index_to_column $row_e0.$col_e0]
		set col_e1 [text_index_to_column $row_e1.$col_e1]

		# Adjust column numbers
		if {$col_s0 > $col_s1} {
			set tmp $col_s0
			set col_s0 $col_s1
			set col_s1 $tmp
		}
		if {$col_e0 > $col_e1} {
			set tmp $col_e0
			set col_e0 $col_e1
			set col_e1 $tmp
		}

		# Adjust row numbers
		set row_s1 $row_s0
		if {$row_s0 != $row_s1} {
			col_s0 $col_s1
		}
		set row_e0 $row_e1
		if {$row_e0 != $row_e1} {
			col_e1 $col_e0
		}

		# Determinate width of the selected block
		if {abs($col_s1 - $col_s0) < abs($col_e1 - $col_e0)} {
			set width [expr {abs($col_s1 - $col_s0)}]
		} {
			set width [expr {abs($col_e1 - $col_e0)}]
		}

		# Regerate selection tags
		$editor tag remove sel 0.0 end
		set col 0
		for {set row $row_s0} {$row <= $row_e1} {incr row} {
			set col [column_to_text_index $row $col_s0]
			if {[$editor compare $row.$col >= [list $row.0 lineend]]} {
				continue
			}
			if {[$editor compare $row.$col+${width}c > [list $row.0 lineend]]} {
				$editor tag add sel $row.$col [list $row.0 lineend]
			} {
				$editor tag add sel $row.$col $row.$col+${width}c
			}
		}
	}

	## Handles pseudo-event: "Selection"
	 # @return void
	public method editor_selection {} {
		if {$selection_in_progress} {return}
		set selection_in_progress 1

		switch -- $selection_mode {
			0 { ;# Normal selection mode
			}
			1 { ;# Block selection mode
				adjust_selection_to_block
			}
		}

		set selection_in_progress 0
	}

	## Handles event: "Control-Key-Up"
	 # @return void
	public method control_down {} {
		$editor yview scroll 1 units
	}

	## Handles event: "Control-Key-Up"
	 # @return void
	public method control_up {} {
		$editor yview scroll -1 units
	}

	## Handles event: "Shift-Key-Down"
	 # @return void
	public method shift_down {} {
		tk::TextKeySelect $editor [get_up_down_idx 0]

		# Adjust selection in list of bookmarks and list of breakpoints
		rightPanel_adjust [expr {int([$editor index insert])}]

		# Adjust status bar counters
		recalc_status_counter {}
		$editor see insert
	}

	## Handles event: "Shift-Key-Up"
	 # @return void
	public method shift_up {} {
		tk::TextKeySelect $editor [get_up_down_idx 1]

		# Adjust selection in list of bookmarks and list of breakpoints
		rightPanel_adjust [expr {int([$editor index insert])}]

		# Adjust status bar counters
		recalc_status_counter {}
		$editor see insert
	}

	## Handles event: "Key-Up"
	 # @return void
	public method up {} {
		# Move insertion cursor
		$editor mark set insert [get_up_down_idx 1]

		# Remove selection
		catch {
			$editor tag remove sel 1.0 end
		}

		# Adjust selection in list of bookmarks and list of breakpoints
		rightPanel_adjust [expr {int([$editor index insert])}]

		# Adjust status bar counters
		recalc_status_counter {} 1
		$editor see insert
	}

	## Handles event: "Key-Down"
	 # @return void
	public method down {} {
		# Focus completion popup window
		if {$completion_win_opened} {
			catch {
				focus -force $completion_listbox
				$completion_listbox selection set [$completion_listbox items 0]
			}
			return
		}

		# Move insertion cursor
		$editor mark set insert [get_up_down_idx 0]

		# Remove selection
		catch {
			$editor tag remove sel 1.0 end
		}

		# Adjust selection in list of bookmarks and list of breakpoints
		rightPanel_adjust [expr {int([$editor index insert])}]

		# Adjust status bar counters
		recalc_status_counter {}
		$editor see insert
	}

	## Handles event: "Key-Escape"
	 # @return void
	public method key_escape {} {
		if {$completion_win_opened} {
			catch {
				detete_text_in_editor sel.first sel.last
			}
		}
		catch {
			$editor tag remove sel 1.0 end
		}
	}

	## Handles event: "Shift-Key-Home"
	 # @return void
	public method shift_home {} {
		# Selection tag defined
		if {[llength [$editor tag nextrange sel 1.0]]} {
			set sel_f [$editor index sel.first]
			set sel_l [$editor index sel.last]
			set idx0 [$editor index insert]
			$editor tag remove sel 1.0 end
			home_press
			set idx1 [$editor index insert]

			if {[$editor compare $idx0 == $sel_f]} {
				$editor tag add sel $idx1 $sel_l
			} elseif {[$editor compare $idx0 == $sel_l]} {
				$editor tag add sel $sel_f $idx1
			}

		# Nothing selected
		} {
			set idx [$editor index insert]
			home_press
			catch {
				$editor tag remove sel 1.0 end
			}

			if {[$editor compare $idx < insert]} {
				$editor tag add sel $idx insert
			} {
				$editor tag add sel insert $idx
			}
		}
	}

	## Handles event: "Key-Home"
	 # @return void
	public method home_press {} {
		# Local variables
		set idx [$editor index insert]	;# Insert index
		set row [expr {int($idx)}]	;# Current row
		regexp {\d+$} $idx col_original	;# Current column

		# Determinate start line index (true line start)
		if {[regexp {^\s+} [$editor get $row.0 "$row.0 lineend"] space]} {
			set col [string length $space]
			if {$col_original == $col} {
				$editor mark set insert $row.0
			} {
				$editor mark set insert $row.$col
			}
		} {
			$editor mark set insert $row.0
		}

		# Unset selection
		catch {
			$editor tag remove sel 1.0 end
		}

		# Adjust status bar counters
		recalc_status_counter {} 0
		$editor see insert
	}

	## Handles event: "Key-Tab"
	 # @return void
	public method tab_press {} {
	
		if {$spaces_no_tabs} {
			set indent_char [string repeat { } $number_of_spaces]
		} {
			set indent_char "\t"
		}

		# Nothing selected or popup completion window is opened -> insert tab character
		if {$completion_win_opened || [$editor tag ranges sel] == {}} {
			Key $indent_char

		# Something selected -> indent
		} {
			# convert selection indexes to line numbers
			set start [expr {int([$editor index sel.first])}]
			set end_o [$editor index sel.last]
			set end [expr {int($end_o)}]
			if {$end == $end_o} {incr end -1}
			# perform indent on each line in the block
			for {set line $start} {$line <= $end} {incr line} {
				$editor insert $line.0 $indent_char
				rightPanel_changeLineContent $line
				restore_line_markers $line
			}
			$editor tag add sel $start.0 [expr {$end + 1}].0

			# Recalculate status bar
			recalc_status_counter {} 0

			$editor see insert
		}
	}

	## Convert selected text to lowercase
	 # @retrun void
	public method lowercase {} {
		# Nothing to do -> terminate
		if {![llength [$editor tag nextrange sel 1.0]]} {
			Sbar "Unable to execute: nothing selected"
			return 0
		}

		$editor configure -autoseparators 0
		set ranges [$editor tag ranges sel]
		set len [llength $ranges]
		for {set i 0; set j 1} {$i < $len} {incr i 2; incr j 2} {
			set first [lindex $ranges $i]
			set last [lindex $ranges $j]
			set data [$editor get $first $last]
			$editor tag remove $first $last
			detete_text_in_editor $first $last
			$editor insert $first [string tolower $data]
			$editor tag add sel $first $last

			set first [expr {int($first)}]
			set last [expr {int($last)}]
			set highlighted_lines [string replace $highlighted_lines	\
				$first $last [string repeat 0 [expr {$last - $first + 1}]]]
		}

		highlight_visible_area
		$editor edit separator
		$editor configure -autoseparators 1
		return 1
	}

	## Convert selected text to uppercase
	 # @retrun void
	public method uppercase {} {
		# Nothing to do -> terminate
		if {![llength [$editor tag nextrange sel 1.0]]} {
			Sbar "Unable to execute: nothing selected"
			return 0
		}

		$editor configure -autoseparators 0
		set ranges [$editor tag ranges sel]
		set len [llength $ranges]
		for {set i 0; set j 1} {$i < $len} {incr i 2; incr j 2} {
			set first [lindex $ranges $i]
			set last [lindex $ranges $j]
			set data [$editor get $first $last]
			$editor tag remove $first $last
			detete_text_in_editor $first $last
			$editor insert $first [string toupper $data]
			$editor tag add sel $first $last

			set first [expr {int($first)}]
			set last [expr {int($last)}]
			set highlighted_lines [string replace $highlighted_lines	\
				$first $last [string repeat 0 [expr {$last - $first + 1}]]]
		}

		highlight_visible_area
		$editor edit separator
		$editor configure -autoseparators 1
		return 1
	}

	## Convert the first character of selected text to uppercase
	 # @retrun void
	public method capitalize {} {
		# Nothing to do -> terminate
		if {![llength [$editor tag nextrange sel 1.0]]} {
			Sbar "Unable to execute: nothing selected"
			return 0
		}

		$editor configure -autoseparators 0
		set first [$editor index sel.first]
		set last [$editor index sel.last]
		set data [string toupper [$editor get $first $first+1c]]
		detete_text_in_editor $first $first+1c
		$editor insert $first $data
		$editor tag add sel $first $last
		parse [expr {int($first)}]
		$editor edit separator
		$editor configure -autoseparators 1
		return 1
	}

	## Handles event: "Shift-Key-Return", "Shift-Key-KP_Enter"
	 # Smart new line
	 # @return void
	public method shift_enter {} {
		if {$critical_edit_proc} {return}
		set critical_edit_proc 1

		deleteselection
		$editor insert insert "\n"

		set line [$editor get [list insert-1l linestart] [list insert-1l lineend]]
		if {![regexp {^\s*[^\w]+} $line line]} {
			set critical_edit_proc 0
			return
		}
		$editor insert insert $line

		# Recalcutlate Left frame, status bar and right panel
		$editor see insert
		update
		recalc_left_frame
		recalc_status_counter {}
		rightPanel_adjust [expr {int([$editor index insert])}]
		set critical_edit_proc 0

		# Reevaluate highlight on the next line if C language is used
		if {$prog_language == 1} {
			c_syntax_highlight [expr {int([$editor index insert])+1}]
		}
	}

	## Handles event: "Key-Return", "Key-KP_Enter"
	 # @return void
	public method enter {} {
		if {$critical_edit_proc} {return}
		set critical_edit_proc 1
		
		$editor configure -autoseparators 0

		set idx [$editor index insert]	;# Determinate insert index
		$editor insert $idx "\n"	;# Insert EOL
		resetUpDownIndex		;# Column changed

		set idx [expr {int($idx)}]
		incr idx

		# Keep indention of the previous line
		if {$intentation_mode == {normal}} {

			# Determinate indetication charactes
			set prev_line [$editor get \
				[$editor index {insert-1l linestart}]	\
				[$editor index {insert-1l lineend}]	\
			]
			if {[string length $prev_line]} {
				set indent_chars {}
				regexp {^\s+} $prev_line indent_chars

				# Insert indentication characers from the previous line
				if {$indent_chars != {}} {
					$editor insert $idx.0 $indent_chars
				}

				if {$prev_line == $indent_chars} {
					$editor delete {insert-1l linestart} {insert-1l lineend}
				}
			}
		}

		# Remove selected text
		deleteselection

		# Recalcutlate Left frame, status bar and right panel
		$editor see $idx.0
		update
		recalc_left_frame
		recalc_status_counter {}
		rightPanel_adjust $idx
		set critical_edit_proc 0

		# Reevaluate highlight on the next line if C language is used
		if {$prog_language == 1} {
			incr idx
			c_syntax_highlight $idx
		}

		$editor edit separator
		$editor configure -autoseparators 1
	}

	## Copy the selected text to the clipboard
	 # @return bool - 1: successful; 0: failed
	public method copy {} {
		# get selected text
		set data [getselection]

		# Nothing to do -> terminate
		if {$data == {}} {
			Sbar "Unable to execute: nothing selected"
			return 0
		# Adjust clipboard content
		} {
			clipboard clear
			clipboard append $data
			return 1
		}
	}

	## Paste clipboard content to the text at the cursor position
	 # @return bool - 1: successful; 0: failed
	public method paste {} {
		if {$critical_edit_proc} {return}
		set critical_edit_proc 1

		# Restore original cursor position in block selection mode
		if {$selection_mode} {
			set original_cur_pos [$editor index insert]
		}

		# Get clipboard content
		if {[catch {
			set data [regsub -all {[\u0000-\u0008\u000B-\u000C\u000E-\u001F\u007F-\u009F]} [clipboard get] {}]
		}]} then {
			# Clipboard empty -> abort
			Sbar "Unable to execute: nothing in the clipboard"
			set critical_edit_proc 0
			return 0
		}

		# delete selected block
		$editor configure -autoseparators 0
		deleteselection
		recalc_left_frame
		# insert data to the text, restore syntax hightlight and return 1
		$editor insert [$editor index insert] $data
		recalc_left_frame
		recalc_status_counter {}
		$editor see [$editor index insert]
		update
		set line [expr {int([$editor index insert])}]
		rightPanel_adjust $line
		parse $line
		highlight_visible_area
		set critical_edit_proc 0
		$editor edit separator
		$editor configure -autoseparators 1

		# Reevaluate highlight on the next line if C language is used
		if {$prog_language == 1} {
			incr line
			c_syntax_highlight $line
		}
		# Restore original cursor position in block selection mode
		if {$selection_mode} {
			$editor see $original_cur_pos
			$editor mark set insert $original_cur_pos
		}
		return 1
	}

	## Take back last editor operation
	 # @return void
	public method undo {} {
		if {$critical_edit_proc} {return}
		set critical_edit_proc 1

		if {![catch {$editor edit undo}]} {
			recalc_left_frame
			recalc_status_counter {}
			recalc_status_modified 1
			rightPanel_adjust [expr {int([$editor index insert])}]
			$editor see [$editor index insert]
			set highlighted_lines [string repeat 0 [string bytelength $highlighted_lines]]
			update
			highlight_visible_area
			catch {
				$editor tag remove sel 1.0 end
			}
		}
		set critical_edit_proc 0
	}

	## Take back last Undo operation
	 # @return void
	public method redo {} {
		if {$critical_edit_proc} {return}
		set critical_edit_proc 1
		if {![catch {$editor edit redo}]} {
			recalc_left_frame
			recalc_status_counter {}
			recalc_status_modified 1
			rightPanel_adjust [expr {int([$editor index insert])}]
			$editor see [$editor index insert]
			set highlighted_lines [string repeat 0 [string bytelength $highlighted_lines]]
			update
			highlight_visible_area
			catch {
				$editor tag remove sel 1.0 end
			}
		}
		set critical_edit_proc 0
	}

	## Handles event: 'Menu'
	 # @return void
	public method Key_Menu {} {
		# Close autocompletion popup window
		if {$completion_win_opened} {
			close_completion_popup_window
		}

		# Invoke popup menu
		$editor see insert
		set bbox [$editor bbox [$editor index insert]]
		tk_popup $menu	\
			[expr {[winfo rootx $editor] + [lindex $bbox 0] + 10}]	\
			[expr {[winfo rooty $editor] + [lindex $bbox 1] + 10}]
	}

	## Handles event: 'KeyRelease'
	 # @parm String key - Key name
	 # @return void
	public method KeyRelease {key} {
		if {[lsearch {ISO_Next_Group ISO_Prev_Group} $key] == -1} {
			if {$do_not_hide_comp_win} {
				set do_not_hide_comp_win 0
			} {
				close_completion_popup_window
			}
		}
	}

	## Handles event: 'Key'
	 # @return void
	public method Key {key} {
		# Skip values with no meaning
		if {![string is print -strict $key] && $key != "\t"} {
			return
		}

		if {$key_handler_in_progress} {
			if {[llength $key_handler_buffer] < 4} {
				lappend key_handler_buffer $key
			}
			return
		}
		set key_handler_in_progress 1

		autocompletion_maybe_important_change insert insert

		$editor configure -autoseparators 0

		if {
			$auto_brackets &&
			($key == {'} || $key == "\"" || $key == {(} || $key == "\[" || $key == "\{")
		} then {
			# Enclose selected text by the selected charactere
			if {[llength [$editor tag nextrange sel 1.0]]} {
				$editor insert sel.first $key
				switch -- $key {
					{(} {$editor insert sel.last {)}}
					{[} {$editor insert sel.last {]}}
					"\{" {$editor insert sel.last "\}"}
					default {
						$editor insert sel.last $key
					}
				}
				$editor mark set insert sel.last
				$editor tag remove sel 1.0 end

			# Insert the selected character twice
			} {
				set next_char [$editor get insert insert+1c]
				$editor insert insert $key
				switch -- $key {
					{(} {$editor insert insert {)}}
					{[} {$editor insert insert {]}}
					"\{" {$editor insert insert "\}"}
					{'} {
						if {$next_char != {'}} {
							$editor insert insert {'}
						}
					}
					"\"" {
						if {$next_char != "\""} {
							$editor insert insert "\""
						}
					}
					default {
						$editor insert insert $key
					}
				}
				$editor mark set insert {insert - 1c}
			}

		} else {
			# Delete selected text
			deleteselection

			# Mode overwrite
			if {!$ins_ovr_mode} {
				if {[$editor compare insert != {insert lineend}]} {
					detete_text_in_editor insert insert+1c
				}
			}

			# Insert the given character
			$editor insert insert $key
		}

		# Restore highlight on the current line
		parse [expr {int([$editor index insert])}]
		recalc_left_frame

		# Invoke popup completion menu
		if {$auto_completion} {
			aux_Key_autocompletion_0			\
				[$editor index {insert-1c wordstart}]	\
				[$editor index {insert-1c wordend}]
		}

		$editor edit separator
		$editor configure -autoseparators 1
		

		if {[llength $key_handler_buffer]} {
			set key [lindex $key_handler_buffer 0]
			set key_handler_buffer [lreplace $key_handler_buffer 0 0]
			update
			set key_handler_in_progress 0
			Key $key
		}
		set key_handler_in_progress 0
		update
	}

	## Auxiliary method for method "Key"
	 # Invokes autocompletion menu
	 # @parm TextIndex wordstart	- Index of {insert-1c wordstart}
	 # @parm TextIndex wordstart	- Index of {insert-1c wordend}
	 # @return void
	private method aux_Key_autocompletion_0 {wordstart wordend} {
		# Get range of tag MACRO (possibly incomplete instruction) on the curent line
		set mc_range [$editor tag nextrange tag_macro $wordstart $wordend]
		if {![llength $mc_range]} {
			set mc_range [$editor tag nextrange tag_directive $wordstart $wordend]
		}
		if {![llength $mc_range]} {
			set mc_range [$editor tag nextrange tag_instruction $wordstart $wordend]
		}

		# Open completion window
		if {[llength $mc_range] && [$editor compare insert == [lindex $mc_range 1]]} {
			invoke_completion_popup_window 0	\
				[lindex $mc_range 0] [lindex $mc_range 1]

		# Try comething else ...
		} {
			aux_Key_autocompletion_1 $wordstart $wordend
		}
	}
	
	## Auxiliary method for method "aux_Key_autocompletion_0"
	 # Invokes autocompletion menu
	 # @parm TextIndex wordstart	- Index of {insert-1c wordstart}
	 # @parm TextIndex wordstart	- Index of {insert-1c wordend}
	 # @return void
	private method aux_Key_autocompletion_1 {wordstart wordend} {
		# Get range of tag CONSTANT or SFR on the curent line
		set mc_range [$editor tag nextrange tag_constant $wordstart $wordend]
		if {![llength $mc_range]} {
			set mc_range [$editor tag nextrange tag_sfr $wordstart $wordend]
		}

		# Open completion window
		if {[llength $mc_range] && [$editor compare insert == [lindex $mc_range 1]]} {
			invoke_completion_popup_window 1		\
				[lindex $mc_range 0] [lindex $mc_range 1]

		# Try comething else ...
		} {
			aux_Key_autocompletion_2 $wordstart $wordend
		}
	}
	
	## Auxiliary method for method "aux_Key_autocompletion_1"
	 # Invokes autocompletion menu
	 # @parm TextIndex wordstart	- Index of {insert-1c wordstart}
	 # @parm TextIndex wordstart	- Index of {insert-1c wordend}
	 # @return void
	private method aux_Key_autocompletion_2 {wordstart wordend} {
		# Get range of tag indirect on the curent line
		set mc_range [$editor tag nextrange tag_indirect $wordstart-1c $wordend]

		# Open completion window
		if {[llength $mc_range] && [$editor compare insert == [lindex $mc_range 1]]} {
			invoke_completion_popup_window 3		\
				[lindex $mc_range 0] [lindex $mc_range 1]

		# Try comething else ...
		} {
			aux_Key_autocompletion_3 $wordstart $wordend
		}
	}
	
	## Auxiliary method for method "aux_Key_autocompletion_2"
	 # Invokes autocompletion menu
	 # @parm TextIndex wordstart	- Index of {insert-1c wordstart}
	 # @parm TextIndex wordstart	- Index of {insert-1c wordend}
	 # @return void
	private method aux_Key_autocompletion_3 {wordstart wordend} {
		# Get range of tag IMMEDIATE CONSTANT on the curent line
		set mc_range [$editor tag nextrange tag_imm_constant $wordstart-1c $wordend]

		# Open completion window
		if {[llength $mc_range] && [$editor compare insert == [lindex $mc_range 1]]} {
			invoke_completion_popup_window 1				\
				[$editor index [list [lindex $mc_range 0] + 1c]]	\
				[lindex $mc_range 1]

		# Try comething else ...
		} {
			aux_Key_autocompletion_4 $wordstart $wordend
		}
	}
	
	## Auxiliary method for method "aux_Key_autocompletion_3"
	 # Invokes autocompletion menu
	 # @parm TextIndex wordstart	- Index of {insert-1c wordstart}
	 # @parm TextIndex wordstart	- Index of {insert-1c wordend}
	 # @return void
	private method aux_Key_autocompletion_4 {wordstart wordend} {
		# Get range of tag DOXYGEN TAG on the curent line
		set mc_range [$editor tag nextrange tag_c_dox_tag $wordstart-1c $wordend]

		# Open completion window
		if {
			$prog_language == 1
				&&
			[llength $mc_range]
				&&
			[$editor compare insert == [lindex $mc_range 1]]
		} then {
			invoke_completion_popup_window 4	\
				[lindex $mc_range 0] [lindex $mc_range 1]

		# Try comething else ...
		} {
			aux_Key_autocompletion_5 $wordstart $wordend
		}
	}
	
	## Auxiliary method for method "aux_Key_autocompletion_4"
	 # Invokes autocompletion menu
	 # @parm TextIndex wordstart	- Index of {insert-1c wordstart}
	 # @parm TextIndex wordstart	- Index of {insert-1c wordend}
	 # @return void
	private method aux_Key_autocompletion_5 {wordstart wordend} {
		# Find word with no tags
		if {
			$prog_language == 1
				&&
			[$editor compare insert == $wordend]
		} then {
			set tags [$editor tag names insert-1c]

			# Remove unimportant tags
			if {[llength $tags]} {
				foreach lm [concat $line_markers {tag_current_line}] {
					set idx [lsearch -ascii -exact $tags $lm]
					if {$idx != -1} {
						set tags [lreplace $tags $idx $idx]
					}
				}
			}

			# Open completion window
			if {![llength $tags]} {
				invoke_completion_popup_window 2 $wordstart $wordend
			}

		# Close completion window
		} {
			close_completion_popup_window
		}
	}
	
	## Handles event: 'Key-Delete'
	 # @return void
	public method key_delete {} {
		if {![$this deleteselection]} {
			if {[$editor compare {insert linestart} != {insert+1c linestart}]} {
				set remove_trailing_space 1
			} {
				set remove_trailing_space 0
			}

			$this detete_text_in_editor insert insert+1c

			if {$remove_trailing_space && [regexp {\s+$} [$editor get {insert linestart} {insert lineend}] space]} {
				set line_end [$editor index {insert lineend}]
				$editor delete $line_end-[string length $space]c {insert lineend}
			}
		}

		$this resetUpDownIndex
		$this recalc_left_frame
		$this parse [expr {int([$editor index insert])}]
		update
	}

	## Handles event: 'Key-Backspace'
	 # @return void
	public method key_backspace {} {
		if {$auto_brackets} {
			set char0 [$editor get insert-1c insert]
			set char1 [$editor get insert insert+1c]
			if {
				($char0 == "\{" && $char1 == "\}") ||
				($char0 == {(} && $char1 == {)}) ||
				($char0 == {[} && $char1 == {]})
			} then {
				$this detete_text_in_editor insert insert+1c
			} elseif {$char0 == $char1 && ($char0 == "\"" || $char1 == {'})} {
				$this detete_text_in_editor insert insert+1c
			}
		}
		if {![$this deleteselection]} {
			$this detete_text_in_editor insert-1c insert
		}
		$this resetUpDownIndex
		$this recalc_left_frame
		$this parse [expr {int([$editor index insert])}]
		update
	}

	## Delete current line
	 # @return void
	public method delete_current_line {} {
		if {
			[$editor compare {insert linestart} == {insert lineend}]
				&&
			[$editor compare {insert linestart} == {end-1l}]
		} then {
			return
		}
		detete_text_in_editor {insert linestart} {insert linestart + 1l}
		$this resetUpDownIndex
		$this recalc_left_frame
		$this parse [expr {int([$editor index insert])}]
		update
		break
	}

	## Cut the selected text and put it into the clipboard
	 # @return bool - 1: successful; 0: failed
	public method cut {} {
		if {$critical_edit_proc} {return}
		set critical_edit_proc 1
		# get selected text
		set data [getselection]

		# Nothing to do -> terminate
		if {$data == {}} {
			Sbar "Unable to execute: nothing selected"
			set critical_edit_proc 0
			return 0
		# Cut
		} {
			# Adjust clipboard content
			deleteselection
			clipboard clear
			clipboard append $data
			$editor see insert
			set line [expr {int([$editor index insert])}]
			parse $line
			update
			recalc_left_frame
			rightPanel_adjust $line
			set critical_edit_proc 0
			return 1
		}
	}

	## Insure that command line is focused
	 # @retrun void
	public method cmd_line_force_on {} {
		if {$editor_to_use} {return}
		if {![winfo viewable $cmd_line]} {
			pack $cmd_line -side top -fill x
		}
		update
	}

	## Insure that command line is NOT focused
	 # @retrun void
	public method cmd_line_force_off {} {
		if {$editor_to_use} {return}

		if {!${::APPLICATION_LOADED}} {return}
		if {[winfo viewable $cmd_line]} {
			pack forget $cmd_line
			focus $editor
		}
		update
	}

	## Define highlighting tags in editor text widget and command line text widget
	 # @retrun void
	public method create_highlighting_tags {} {
		if {$editor_to_use} {return}

		if {$prog_language == 1} {
			CsyntaxHighlight::create_tags $editor $fontSize $fontFamily
		} elseif {$prog_language == 2} {
			LSTsyntaxHighlight::create_tags $editor $fontSize $fontFamily
		}
		ASMsyntaxHighlight::create_tags $editor $fontSize $fontFamily
		ASMsyntaxHighlight::create_tags $cmd_line $cmd_line_fontSize	\
			$cmd_line_fontFamily $cmd_line_highlighting

		refresh_highlighting_for_autocompletion
	}

	## Determinate color for instructions, directives, etc.
	 # @return void
	private method refresh_highlighting_for_autocompletion {} {
		foreach key ${::ASMsyntaxHighlight::hightlight_tags} {
			if {[lindex $key 0] == {tag_instruction}} {
				set ins_fg [lindex $key 1]
				set ins_font [$editor tag cget tag_instruction -font]

			} elseif {[lindex $key 0] == {tag_directive}} {
				set dir_fg [lindex $key 1]
				set dir_font [$editor tag cget tag_directive -font]

			} elseif {[lindex $key 0] == {tag_constant}} {
				set const_fg [lindex $key 1]
				set const_font [$editor tag cget tag_constant -font]

			} elseif {[lindex $key 0] == {tag_macro}} {
				set macro_fg [lindex $key 1]
				set macro_font [$editor tag cget tag_macro -font]

			} elseif {[lindex $key 0] == {tag_label}} {
				set label_fg [lindex $key 1]
				set label_font [$editor tag cget tag_label -font]

			} elseif {[lindex $key 0] == {tag_sfr}} {
				set sfr_fg [lindex $key 1]
				set sfr_font [$editor tag cget tag_sfr -font]

			} elseif {[lindex $key 0] == {tag_symbol}} {
				set symbol_fg [lindex $key 1]
				set symbol_font [$editor tag cget tag_symbol -font]

			} elseif {[lindex $key 0] == {tag_indirect}} {
				set indirect_fg [lindex $key 1]
				set indirect_font [$editor tag cget tag_indirect -font]
			}
		}

		if {$prog_language == 1} {
			foreach key ${::CsyntaxHighlight::hightlight_tags} {
				if {[lindex $key 0] == {tag_c_dox_tag}} {
					set doxytag_fg [lindex $key 1]
					set doxytag_font [$editor tag cget tag_c_dox_tag -font]
				}
			}
		}
	}

	## Kill child processes
	 # @return void
	public method kill_childern {} {
		if {$editor_to_use} {
			bind $top_frame <Destroy> {}
			catch {
				exec -- kill $pid
			}
		}
	}

	## Create terminal emulator with external editor embedded into editor frame
	 # IMPORTANT: This is only an auxiliary function for "recreate_terminal"
	 # @parm String filename - Name of file to open with the external editor
	 # @return void
	public method create_terminal {filename} {
		if {$terminal_created} {return}
		set terminal_created 1

		if {$filename == {untitled}} {
			set filename {}
		}

		# Determinate editor command
		set opt {}
		switch -- $editor_to_use {
			1 {set cmd {vim}}
			2 {
				set cmd {emacs}
				set opt {-nw}
			}
			3 {set cmd {nano}}
			4 {set cmd {dav}}
			5 {set cmd {le}}
			default {
				error "Unknown internal error in ::Editor::create_terminal($filename)"
			}
		}

		# Change directory
		set cur_dir [pwd]
		
		if {[catch {
			if {$filename == {}} {
				cd [$parentObject cget -projectPath]
			} {
				cd [file dirname $filename]
			}
		}]} then {
			cd ~
		}

		# Run embedded editor
		if {[catch {
			if {$opt == {}} {
				if {$filename == {}} {
					set pid [exec -- urxvt -embed [expr [winfo id $top_frame]]	\
						+sb -bg "$normal_text_bg" -b 0 -w 0 -sl 0	\
						-fn "xft:$fontFamily:pixelsize=$fontSize"	\
						-e $cmd &]
				} {
					set pid [exec -- urxvt -embed [expr [winfo id $top_frame]]	\
						+sb -bg "$normal_text_bg" -b 0 -w 0 -sl 0	\
						-fn "xft:$fontFamily:pixelsize=$fontSize"	\
						-e $cmd "$filename" &]
				}
			} else {
				if {$filename == {}} {
					set pid [exec -- urxvt -embed [expr [winfo id $top_frame]]	\
						+sb -bg "$normal_text_bg" -b 0 -w 0 -sl 0	\
						-fn "xft:$fontFamily:pixelsize=$fontSize"	\
						-e $cmd $opt &]
				} {
					set pid [exec -- urxvt -embed [expr [winfo id $top_frame]]	\
						+sb -bg "$normal_text_bg" -b 0 -w 0 -sl 0	\
						-fn "xft:$fontFamily:pixelsize=$fontSize"	\
						-e $cmd $opt "$filename" &]
				}
			}
		} result]} then {
			puts stderr $result
			tk_messageBox		\
				-parent .	\
				-icon warning	\
				-type ok	\
				-title "FATAL ERROR"	\
				-message "Unable to start embedded editor due to an unknow error. This error did not occured in MCU 8051 IDE code but somewhere else. Please try to restart MCU 8051 IDE."
		}

		# Return to previous directory
		cd $cur_dir
	}

	## Create terminal emulator with external editor embedded into editor frame
	 # @parm String filename - Name of file to open with the external editor
	 # @return void
	public method recreate_terminal {filename} {
		update idle
		if {![winfo exists $ed_sc_frame]} {return}
		set top_frame [frame $ed_sc_frame.top_frame_$top_frame_idx -container 1]
		pack $top_frame -expand 1 -fill both
		bind $top_frame <Visibility> "$this create_terminal {$filename}"
		bind $top_frame <Destroy> "$this recreate_terminal {$filename}"
		set terminal_created 0
		incr top_frame_idx
	}

	## Determinate file type acording to its name externsion
	 # @parm Bool reset - Reset syntax highlight
	 # @return void
	private method determinate_prog_lang {reset} {
		# Determinate file type
		set ext [string replace [file extension $filename] 0 0]
		set prog_language_old $prog_language
		 # - C language
		if {$ext == {c} || $ext == {h} || $ext == {cxx} || $ext == {cpp} || $ext == {cc}} {
			set prog_language 1
		 # - Code listing
		} elseif {$ext == {lst}} {
			set prog_language 2
		 # - Unknown -> Assembly language
		} else {
			set prog_language 0
		}

		# Reset highlight
		if {$reset && ($prog_language_old != $prog_language)} {
			prog_lang_changed
		}
	}

	## Get ID of file type (programing language used)
	 # @return Int - 0 == Assembly language; 1 == C language
	public method get_language {} {
		if {$prog_language == -1} {
			set ext [string trimleft [file extension $filename] {.}]
			if {$ext == {c} || $ext == {h} || $ext == {cxx} || $ext == {cpp} || $ext == {cc}} {
				return 1
			} elseif {$ext == {lst}} {
				return 2
			} else {
				return 0
			}
		} {
			return $prog_language
		}
	}

	## Set file type (programing language used)
	 # @parm Int lang -  -1 == unknown; 0 == Assembly language; 1 == C language
	 # @return void
	public method force_language {lang} {
		set prog_language_old $prog_language
		set prog_language $lang

		if {$prog_language_old != $prog_language} {
			prog_lang_changed
		}
	}

	## This function shoul be called after each change of file type
	 # Reset syntax highlight and adjust editor status bar
	 # @return void
	private method prog_lang_changed {} {
		# Clear current highlighting tags
		$editor tag remove tag_error 0.0 end
		$editor tag remove tag_error_line 0.0 end
		foreach tag [concat						\
				${::CsyntaxHighlight::hightlight_tags}		\
				${::ASMsyntaxHighlight::hightlight_tags}	\
				${LSTsyntaxHighlight::hightlight_tags}		\
			] {
				$editor tag remove [lindex $tag 0] 0.0 end
		}

		# Create C highlighting tags
		if {$object_initialized && $prog_language == 1 && !$c_hg_tags_created} {
			set c_hg_tags_created 1
			CsyntaxHighlight::create_tags $editor $fontSize $fontFamily
			$parentObject rightPanel_bm_bp_create_c_hg_tags
			refresh_highlighting_for_autocompletion

		# Create LST highlighting tags
		} elseif {$object_initialized && $prog_language == 2 && !$lst_hg_tags_created} {
			set lst_hg_tags_created 1
			::LSTsyntaxHighlight::create_tags $editor $fontSize $fontFamily
			$parentObject rightPanel_bm_bp_create_lst_hg_tags
		}

		# Create new highlight
		parseAll

		# Adjust status bar
		adjust_sbar_to_prog_lang

		# Adjust main menu and main toolbar
		if {$prog_language == 1} {
			set uses_c 1
		} {
			set uses_c 0
		}
		::X::adjust_mainmenu_and_toolbar_to_editor {} $uses_c
		$parentObject filelist_editor_sh_changed $this $prog_language
	}

	## Adjust editor status bar the language used (file type)
	 # @return void
	private method adjust_sbar_to_prog_lang {} {
		if {$prog_language == -1} {
			$Sbar_prog_lang configure -text {}
		} elseif {$prog_language == 1} {
			$Sbar_prog_lang configure -fg {#AA8800} -text "C/H"
		} elseif {$prog_language == 2} {
			$Sbar_prog_lang configure -fg {#00DDEE} -text "LST"
		} elseif {$prog_language == 3} {
			$Sbar_prog_lang configure -fg {#0000DD} -text "ASX"
		} {
			$Sbar_prog_lang configure -fg {#00CC00} -text "ASM"
		}
	}

	## Document current function
	 # @return void
	public method document_current_func {} {
		# Critical procedure
		if {$critical_edit_proc} {return}
		set critical_edit_proc 1

		# Check if this procedure can be done
		if {$editor_to_use || $prog_language != 1} {return}

		# Determinate line content
		set line_number [expr {int([$editor index insert])}]
		set line_content [$editor get {insert linestart} {insert lineend}]
		set line_n $line_number
		for {set i 0} {$i < 50} {incr i} {
			if {[string first {)} $line_content] != -1} {
				break
			}
			incr line_n
			append line_content { } [$editor get [list $line_n.0 linestart] [list $line_n.0 lineend]]
		}

		# Check if line content is valid function declaration
		if {![regexp {^\s*(\w+\s+)+\w+\(.*\)} $line_content]} {
			Sbar "No function to document"
			set critical_edit_proc 0
			return
		}

		# Determinate leading space to keep indention level
		if {![regexp {^\s+} $line_content space]} {
			set space {}
		}

		# Insert '/**'
		$editor insert $line_number.0 "${space}/**\n"
		incr line_number

		# Document arguments
		set args {}
		if {[regexp {\(.*\)} $line_content args]} {
			set args [string range $args 1 end-1]
			foreach word [split $args {,}] {
				set word [split [string trim $word "\t  "]]
				if {[llength $word] < 2} {
					continue
				}
				set word [lindex $word end]
				regsub {\[.*\]$} $word {} word
				regsub {^(\&|\*\*?)} $word {} word
				$editor insert $line_number.0 "${space} * @param ${word}\n"
				incr line_number
			}
		}

		# Document return value
		if {![regexp {^\s*void\s+} $line_content]} {
			$editor insert $line_number.0 "${space} * @return\n"
			incr line_number
		}
		$editor insert $line_number.0 "${space} */\n"
		incr line_number

		# Highlight
		recalc_left_frame
		recalc_status_counter {}
		$editor see $line_number.0
		update
		rightPanel_adjust $line_number
		parse $line_number
		highlight_visible_area
		incr line_number
		c_syntax_highlight $line_number
		set critical_edit_proc 0
	}

	## Get file statistics
	 # @return List - List of integers in this format:
	 #	Index	Meaning
	 #	0	Words and numbers	(Characters)
	 #	1	Comments		(Characters)
	 #	2	Others			(Characters)
	 #	3	Total characters	(Characters)
	 #	4	Words			(Strings)
	 #	5	Keywords		(Strings)
	 #	6	Comments		(Strings)
	 #	7	Total strings		(Strings)
	 #	8	Empty lines		(Lines)
	 #	9	Commented lines		(Lines)
	 #	10	Normal lines		(Lines)
	 #	11	Total lines		(Lines)
	public method getFileStatistics {} {
		if {$editor_to_use} {return {0 0 0 0 0 0 0 0}}
		set last_line [expr {int([$editor index end])}]

		set words_and_numbers	0
		set chars_comments	0
		set others		0
		set words		0
		set keywords		0
		set comments		0
		set empty_lines		0
		set commented_lines	0
		set normal_lines	0

		# List of highlighting tags related to comments
		set comment_tags {
			tag_comment	tag_c_comment	tag_c_dox_comment
			tag_c_dox_tag	tag_c_dox_word	tag_c_dox_name
			tag_c_dox_html
		}
		# List of highlighting tags related to keywords
		set keyword_tags {
			tag_directive	tag_instruction	tag_control
			tag_c_keyword	tag_c_directive
		}

		# Iterate over lines in editor
		for {set line_num 1} {$line_num < $last_line} {incr line_num} {
			# Get line length and content
			set line [string trimright [$editor get $line_num.0 [list $line_num.0 lineend]]]
			set len [string length $line]

			# Handle empty lines
			if {!$len} {
				incr empty_lines
				continue
			}
			# Save some values for purpose of section "Lines"
			set last_words		$words
			set last_keywords	$keywords
			set last_comments	$comments

			# Iterate over characters on the line
			set char {}
			set found 0
			set last_wordstart -1
			for {set i 0} {$i < $len} {incr i} {
				# Determinate word type
				set found 0
				set wordstart [$editor index [list $line_num.$i wordstart]]
				if {$wordstart != $last_wordstart} {
					set wordend [$editor index [list $line_num.$i wordend]]
					set last_wordstart $wordstart
					set tag_names [$editor tag names $line_num.$i]
					foreach tags [list $comment_tags $keyword_tags]	\
						var {comments keywords}			\
					{
						foreach tag $tag_names {
							if {[lsearch $tags $tag] != -1} {
								incr $var
								set found 1
								break
							}
						}
						if {$found} {break}
					}

					if {!$found} {
						incr words
					}
				}

				# Determinate character type
				set found 0
				set char [string index $line $i]
				foreach tag [$editor tag names $line_num.$i] {
					if {[lsearch $comment_tags $tag] != -1} {
						incr chars_comments
						set found 1
						break
					}
				}

				if {$found} {continue}
				if {[string is wordchar -strict $char]} {
					incr words_and_numbers
				} {
					incr others
				}
			}

			# Determinate line type excluding empty lines
			if {$last_words == $words && $last_keywords == $keywords && $last_comments < $comments} {
				incr commented_lines
			} {
				incr normal_lines
			}
		}

		# Composite and return results
		return [list								\
			$words_and_numbers	$chars_comments		$others		\
			[expr {$words_and_numbers + $chars_comments + $others}]		\
											\
			$words			$keywords		$comments	\
			[expr {$words + $keywords + $comments}]				\
											\
			$empty_lines		$commented_lines	$normal_lines	\
			[expr {$empty_lines + $commented_lines + $normal_lines}]	\
		]
	}

	## Set read only mode
	 # @parm Bool mode_frag - 1 == Read only; 0 = Read and write
	 # @return void
	public method change_RO_MODE {mode_frag} {
		set ro_mode $mode_frag

		# Set to read only
		if {$ro_mode} {
			$editor configure -state disabled
			$ins_mode_lbl configure		\
				-bg {#DD0000}		\
				-fg {#FFFFFF}		\
				-text " READ ONLY "	\
				-cursor left_ptr
			setStatusTip -widget $ins_mode_lbl \
				-text "This editor is only for reading, to change that press alt+v and o"
			bind $ins_mode_lbl <Button-1> {}
			set state {disabled}

		# Set to normal mode
		} {
			$editor configure -state normal
			$ins_mode_lbl configure	\
				-bg {#DDDDDD}	\
				-cursor hand1
			setStatusTip -widget $ins_mode_lbl \
				-text "Insertion mode -- OVR == overwrite; INS == insert"
			bind $ins_mode_lbl <Button-1> "$this switch_ins_ovr"
			adjust_INS_OVR_label
			set state {normal}
		}

		# Adjust menus and main toolbar
		foreach entry $read_na_only_menu_items {
			$menu entryconfigure $entry -state $state
		}
		::X::adjust_mainmenu_and_toolbar_to_editor $ro_mode {}
	}
	
	## Perform program jump
	 # @return void
	public method ljmp_this_line {} {
		# Determinate target address
		set address [$parentObject simulator_line2address	\
			[expr {int([$editor index insert])}]		\
			[$parentObject simulator_get_filenumber $fullFileName]	\
		]
		if {$address == {}} {
			return
		}

		# Perform program jump
		$parentObject setPC $address
		set lineNum [$parentObject simulator_getCurrentLine]
		if {$lineNum != {}} {
			$parentObject move_simulator_line $lineNum
		} {
			$parentObject editor_procedure {} unset_simulator_line {}
		}
		$parentObject Simulator_sync_PC_etc
	}

	## Perform subprogram call
	 # @return void
	public method lcall_this_line {} {
		# Determinate target address
		set address [$parentObject simulator_line2address	\
			[expr {int([$editor index insert])}]		\
			[$parentObject simulator_get_filenumber $fullFileName]	\
		]
		if {$address == {}} {
			return
		}
		
		# Perform subprogram call
		$parentObject simulator_subprog_call $address
		set lineNum [$parentObject simulator_getCurrentLine]
		if {$lineNum != {}} {
			$parentObject move_simulator_line $lineNum
		} {
			$parentObject editor_procedure {} unset_simulator_line {}
		}
		$parentObject Simulator_sync_PC_etc
	}
}
