#!/usr/bin/tclsh
# Part of MCU 8051 IDE ( http://mcu8051ide.sf.net )

############################################################################
#    Copyright (C) 2007-2009 by Martin Ošmera                              #
#    martin.osmera@gmail.com                                               #
#                                                                          #
#    This program is free software; you can redistribute it and#or modify  #
#    it under the terms of the GNU General Public License as published by  #
#    the Free Software Foundation; either version 2 of the License, or     #
#    (at your option) any later version.                                   #
#                                                                          #
#    This program is distributed in the hope that it will be useful,       #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of        #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
#    GNU General Public License for more details.                          #
#                                                                          #
#    You should have received a copy of the GNU General Public License     #
#    along with this program; if not, write to the                         #
#    Free Software Foundation, Inc.,                                       #
#    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             #
############################################################################

# --------------------------------------------------------------------------
# DESCRIPTION
# Implements virtual 8051 processor. This class is part of class Simulator.
#
# Class consist of the groups of procedures:
#	- Initialization & cleanup related procedures
#	- Control procedures
#	- External interface management procedures
#	- Virtual HW controller procedures
#	- Instruction procedures
#	- Opcode procedures
#	- Mcu configuration related procedures
#	- Auxiliary alo procedures
#	- Memory management related procedures
#	- Reverse run related procedures
# --------------------------------------------------------------------------

# Load hibernation facility
source "${::LIB_DIRNAME}/hibernate.tcl"

class Simulator_ENGINE {

	inherit Hibernate	;# Import hibernation facility

	## COMMON
	common symbol		;# Array of SFR symbolic names (eg. $symbol(P0) == "80")
	common DEBUG	1	;# Turn on debugging
	common PIN		;# Array describing pins with some special function
	common PORT_LATCHES	;# List: Port latch registers

	# Default values for SFR (values to set after reset)
	common reset_reg_values {
		{A	0}	{B	0}	{DP0L	0}	{DP0H	0}
		{IE	0}	{IP	0}	{PSW	0}	{TCON	0}
		{TMOD	0}	{TH0	0}	{TH1	0}	{TL0	0}
		{TL1	0}	{PCON	0}	{SP	7}
	}
	# Default values for special (uC dependend) SFR (values to set after reset)
	common reset_reg_values_1 {
		{T2CON	0}	{T2MOD	0}	{RCAP2L	0}	{RCAP2H	0}
		{TL2	0}	{TH2	0}	{AUXR1	0}	{ACSR	0}
		{AUXR	0}	{P0	255}	{P1	255}	{P2	255}
		{P3	255}	{P4	255}	{SCON	0}	{DP1L	0}
		{DP1H	0}	{SADEN	0}	{SADDR	0}	{IPH	0}
		{CLKREG	0}	{WDTCON	0}	{EECON	3}	{SPCR	4}
		{SPSR	0}
	}

	## PUBLIC
	public variable programming_language	0	;# Int: ID of used programing language (0 == Assembler; 1 == C language)

	## PRIVATE
	private variable breakpoints			;# Array of Lists of breakpoints (by line numbers) -- (eg '{1 45 399}')

	private variable ram				;# Array of internal RAM;	addr: 0..255;	val: 0..255
	private variable eram				;# Array of expanded RAM;	addr: 0..4096;	val: 0..255
	private variable sfr				;# Array of SFR;		addr: 128..255;	val: 0..255
	private variable xram				;# Array of external RAM;	addr: 0..65535;	val: 0..255
	private variable code				;# Array of program memory;	addr: 0..65535;	val: 0..255
	private variable eeprom 			;# Array of data EEPROM;	addr: 0..65535;	val: 0..255
	private variable iram_size			;# Capacity of internal data memory
	private variable eram_size			;# Capacity of expanded data memory
	private variable xram_size			;# Capacity of external data memory
	private variable code_size			;# Capacity of program memory
	private variable eeprom_size			;# Capacity of data EEPROM

	private variable eeprom_prev		{}	;# List: Previous values of EEPROM registers before write cycle ({{addr val} ...})
	private variable eeprom_WR_ofs		{}	;# Int: EEPROM write buffer offset (for EEPROM WB window only)
	private variable eeprom_WR_buff			;# List: Data EEPROM write buffer for page mode; addr: 0..31; val: 0..255
	private variable eeprom_WR		0	;# Bool: Data EEPROM write cycle in progress
	private variable eeprom_WR_time		0	;# Float: Time of EEPROM write cycle (micro-seconds)

	private variable Line				;# $Line($PC) == {line in source code} {filenumber} {level} {block}
	private variable list_of_filenames		;# List of filenames for [lindex $Line($pc) 1]
	private variable line2PC			;# $line2PC($line) == PC
	private variable bank			0	;# Current register bank (0..3)
	private variable pc			0	;# Program counter
	private variable clock_kHz		0	;# MCU clock in kHz
	private variable time			0	;# Number of instruction cycles consumed by current instruction
	private variable sync_ena		0	;# Bool: Enabled synchronization with an external interface
	private variable address_error		0	;# Bool: Addressing error occured

	private variable break			0	;# Bool: Immediately terminate the loaded program
	private variable simulation_in_progress 0	;# Bool: Engine is running
	private variable run_in_progress	0	;# Bool: Mode "Run" engaged
	private variable animation_in_progress	0	;# Bool: Mode "Animation" engaged
	private variable stepover_in_progress	0	;# Bool: Mode "Stepover" engaged
	private variable ports_previous_state	{}	;# List: {P0_hex P1_hex P2_hex P3_hex P4_hex}
	private variable rmw_instruction	0	;# Bool: This instruction is one of READ-MODIFY-WRITE ones

	private variable avaliable_sfr		{}	;# List: Addresses of implemented SFR
	private variable feature_avaliable		;# Array: Avaliable features
	private variable restricted_bits	{}	;# List: Decimal addresses of unimplemented bits
	private variable write_only_regs	{}	;# List: Decimal addresses of write only registers
	private variable incomplite_regs	{}	;# List: Decimal addresses of not fully implemented registers
	private variable incomplite_regs_mask		;# Array: key == dec. addr.; val == mask of implemented bits

	private variable DPL {DP0L}			;# Address of current DPL register (DTPR)
	private variable DPH {DP0H}			;# Address of current DPH register (DTPR)
	private variable hidden_DPTR0		{0 0}	;# Value of DPTR0 (if dual DPTR is hidden)
	private variable hidden_DPTR1		{0 0}	;# Value of DPTR1 (if dual DPTR is hidden)
	
	private variable watchdog_value		0	;# Int: Current value of watchdog timer (if avaliable)
	private variable wdtrst_prev_val	0	;# Int: Previous value of register WDTRST
	private variable wdt_prescaler_val	0	;# Int: Watchdog prescaler value (content)

	private variable stepback_spec		{}	;# List: List of special values (for function stepback)
	private variable stepback_local		{}	;# List: The same as stepback_normal but only for 1 instruction
	private variable stepback_local_regs	{}	;# List: Register addresses recorded in stepback_local
	private variable stepback_ena		0	;# Bool: Enable stepback
	private variable stepback_length	0	;# Int: Length of stepback stack
	## List: List of changed memory registers (for function stepback)
	 # Format: {{{MEM_TYPE ADDRESS VALUE} ...} ...}
	 # MEM_TYPE	== E (Eram), X (XRAM), I (IRAM), S (SFR)
	 # ADDRESS	== Decimal address
	 # VALUE	== Decimal value
	private variable stepback_normal	{}

	private variable interrupts_in_progress	{}	;# List: Priority flags of interrupts which are in progress
	private variable inter_in_p_flags	{}	;# List: Interrupt flags of interrupts which are in progress
	private variable interrupt_on_next	0	;# Bool: Engage interrupt routine on the next instruction cycle
	private variable skip_interrupt		0	;# Bool: Last instruction was RETI or any access to the IE or IP
	private variable interrupt_pri_flg	{}	;# List: Interrupt flags in order of their priorities
	private variable interrupt_pri_num	{}	;# List: Interrup priorities levels in decremental order (by intr flags)

	private variable timer_0_running	0	;# Bool: Timer/Counter 0 engaged
	private variable timer_1_running	0	;# Bool: Timer/Counter 1 engaged
	private variable timer_2_running	0	;# Bool: Timer/Counter 2 engaged
	private variable pwm_running		0	;# Bool: PWM controller engaged (uses Timer/Counter 0 & 1)
	private variable pwm_OCR		0	;# Int: Content of OCR (8-bit data register)

	private variable anlcmp_running		0	;# Bool: Analog comparator is engaged
	private variable anlcmp_output		0	;# Bool: Output from Analog comparator
	private variable anlcpm_db_timer	0	;# Int: Analog comparator debounce timer

	private variable uart_clock_prescaler	0	;# Int: UART clock prescaler
	private variable timer1_overflow	0	;# Bool: Timer 1 overflow detected
	private variable timer2_overflow	0	;# Bool: Timer 2 overflow detected
	private variable uart_RX_clock		0	;# Int: UART 16-bit RX clock prescaler
	private variable uart_TX_clock		0	;# Int: UART 16-bit TX clock prescaler
	private variable uart_RX_in_progress	0	;# Bool: UART reception in progress
	private variable uart_TX_in_progress	0	;# Bool: UART transmission in progress
	private variable uart_RX_shift_reg	0	;# Int: UART reception receive register
	private variable uart_TX_shift_reg	0	;# Int: UART transmission receive register

	private variable controllers_conf		;# Array of various internal configuration flags
	private variable overall_time		0	;# Overall program time in 1/CLOCK seconds
	private variable overall_instructions	0	;# Counter of instruction cycles

	## Array of Int: Program run statistics
	 # IDX	- Meaning
	 # ---------------------------------
	 # 0	- Nano-seconds
	 # 1	- Clock cycles
	 # 2	- Instruction cycles
	 # 3	- Instructions passed
	 # 4	- Program bytes passed
	 # 5	- Interrupts invoked
	 # 6	- Subprogram calls
	 # 7	- Returns from subprogram
	 # 8	- Returns from interrupt
	 # 9	- Breakpoints reached
	private variable run_statistics

	# ----------------------------------------------------------------
	# INITIALIZATION & CLEANUP RELATED PROCEDURES
	# ----------------------------------------------------------------

	## Object constructor
	constructor {} {
		# Initialize array of program run statistics
		for {set i 0} {$i < 10} {incr i} {
			set run_statistics($i) 0
		}
		
		# Initialize program memory debugging map
		for {set i 0} {$i <= 0xFFFF} {incr i} {
			set Line($i) {}
		}
	}

	## Object destructor
	destructor {
		set break 1
	}

	## This procedure sets these SFR which are not affected by reset
	 # This function is called after each simulator start
	 # @return void
	private method simulator_system_power_on {} {
		if {$feature_avaliable(wtd)} {
			set sfr($symbol(WDTRST)) [undefined_octet]
			set wdtrst_prev_val $sfr($symbol(WDTRST))
			if {$sync_ena} {
				catch {$this Simulator_GUI_sync S 166}
			}
		}

		if {$feature_avaliable(uart)} {
			set sfr($symbol(SBUFR)) [undefined_octet]
			set sfr($symbol(SBUFT)) [undefined_octet]
			set controllers_conf(UART_M) 0
			if {$sync_ena} {
				catch {$this Simulator_GUI_sync S 153}
				catch {$this Simulator_GUI_sync S 409}
			}
		}

		if {$feature_avaliable(pof)} {
			set controllers_conf(POF) 1
			set sfr($symbol(PCON)) 16
			if {$sync_ena} {
				catch {$this Simulator_GUI_sync S 135}
			}
		}

		if {$feature_avaliable(spi)} {
			set sfr($symbol(SPDR)) 0
			if {$sync_ena} {
				catch {$this Simulator_GUI_sync S $symbol(SPDR)}
			}
		}
	}

	## Initialize memory (this function must be called before using this class)
	 # This function is called after constructor
	 # @return void
	public method simulator_initialize_mcu {} {
		# Get processor definition
		set proc_data [$this cget -procData]

		# Determinate capacities of uC memories
		set iram_size [lindex $proc_data 3]
		set eram_size [lindex $proc_data 8]
		set eeprom_size [lindex $proc_data 32]
		set xram_size [expr {int([$this cget -P_option_mcu_xdata])}]
		set code_size [expr {
			int(([lindex $proc_data 2] * 1024)
				+
			[$this cget -P_option_mcu_xcode])}]

		# Parse processor definition and set array feature_avaliable
		foreach index	{
				5	6	7	9	10	11	17
				20	21	22	23	24	25	26
				27	28	29	30	31	33	34
				35	36	37		38	39	40
				41		42		0	1
			} name	{
				uart	t2	wtd	ddp	auxr	t2mod	pof
				gf1	gf0	pd	idl	smod0	iph	acomparator
				euart	clkreg	pwdex	spi	wdtcon	intelpe	pwm
				x2reset	ckcon	auxr1gf3	ao	wdtprg	hddptr
				auxrwdidle	auxrdisrto	xram	xcode
		} {
			if {[lindex $proc_data $index] == {yes}} {
				set feature_avaliable($name) 1
			} {
				set feature_avaliable($name) 0
			}
		}
		for {set i 12; set j 0} {$i < 17} {incr i; incr j} {
			set port_mask [lindex $proc_data $i]
			if {$port_mask != {} && $port_mask != {00000000}} {
				set feature_avaliable(p$j) 1
				set feature_avaliable(port$j) $port_mask
			} {
				set feature_avaliable(p$j) 0
				set feature_avaliable(port$j) {00000000}
			}
		}

		# Set incomplite_regs_mask, restricted_bits and incomplite_regs
		array unset incomplite_regs_mask
		set restricted_bits {}
		foreach reg_mask [lindex $proc_data 18] {
			set addr [string range $reg_mask 0 1]
			set mask [string range $reg_mask 2 3]
			set addr [expr "0x$addr"]
			set mask [expr "0x$mask"]
			
			set incomplite_regs_mask($addr) $mask

			if {$addr > 127 && !($addr % 8)} {
				for {set i 1} {$i <= 128} {set i [expr {$i * 2}]; incr addr} {
					if {![expr $mask & $i]} {
						lappend restricted_bits $addr
					}
				}
			}
		}
		set incomplite_regs [array names incomplite_regs_mask]

		# Determiate list of write-only registers
		set write_only_regs {}
		foreach reg [lindex $proc_data 19] {
			lappend write_only_regs [expr "0x$reg"]
		}

		# (Re)initialize uC memories and configuration
		array unset sfr
		array unset ram
		array unset eram
		array unset xram
		array unset code
		array unset eeprom
		array unset eeprom_WR_buff
		array unset controllers_conf

		# Set critical MCU configuration
		set controllers_conf(WatchDogPrescaler)	0
		set controllers_conf(RDYBSY)	1
		set controllers_conf(WRTINH)	1
		foreach key {
			X2 HWDT WDTEN WSWRST IE0 TF0 IE1 TF1 TI RI SPIF TF2
			EXF2 CF DPS DCEN T2OE T0_MOD T1_MOD UART_M SMOD0 FE
			CEN INT0 INT1 TCLK RCLK SMOD
		} {
			set controllers_conf($key) 0
		}

		# Power on virtual uC and derminate list of implemented SFR
		simulator_system_power_on
		master_reset 0
		set avaliable_sfr [array names sfr]

		# Initialize/Clear code memory and data EEPROM
		simulator_clear_memory code
		simulator_clear_memory eeprom
	}
	
	## Clear memory content
	 # @parm String mem_type - Type of memory to clear
	 #	code	- Program memory
	 #	xdata	- External data memory
	 #	eram	- Expanded data memory
	 #	eeprom	- Data EEPROM
	 # @return void
	public method simulator_clear_memory {mem_type} {
		switch -- $mem_type {
			{code} {	;# Program memory
				for {set i 0} {$i < $code_size} {incr i} {
					set code($i) {}
				}
			}
			{xdata} {	;# External data memory
				for {set i 0} {$i < $xdata_size} {incr i} {
					set xdata($i) {}
				}
			}
			{eram} {	;# Expanded data memory
				for {set i 0} {$i < $eram_size} {incr i} {
					set eram($i) {}
				}
			}
			{eeprom} {	;# Data EEPROM
				for {set i 0} {$i < $eeprom_size} {incr i} {
					set eeprom($i) 0
				}
				if {$eeprom_size} {
					for {set i 0} {$i < 32} {incr i} {
						set eeprom_WR_buff($i) {}
					}
				}
			}
		}
	}

	## Inicliaze array 'symbol' -- this function must be called after definition of this class
	 # @return void
	proc InitializeNS {} {
		variable symbol	;# Array of SFR symbolic names (eg. $symbol(P0) == "80")

		foreach symb_name {
			{P0	80}	{SP	81}	{DP0L	82}	{DP0H	83}
			{DP1L	84}	{DP1H	85}	{IPH	B7}	{SADDR	A9}
			{PCON	87}	{TCON	88}	{TMOD	89}	{TL0	8A}
			{TL1	8B}	{TH0	8C}	{TH1	8D}	{AUXR	8E}
			{P1	90}	{SCON	98}	{SBUFR	99}	{P2	A0}
			{AUXR1	A2}	{WDTRST	A6}	{IE	A8}	{P3	B0}
			{IP	B8}	{P4	C0}	{T2CON	C8}	{T2MOD	C9}
			{RCAP2L	CA}	{RCAP2H	CB}	{TL2	CC}	{TH2	CD}
			{PSW	D0}	{A	E0}	{B	F0}	{SPDR	86}
			{WDTCON	A7}	{EECON	96}	{CLKREG	8F}	{ACSR	97}
			{SADEN	B9}	{SPCR	D5}	{SPSR	AA}	{SBUFT	199}

			{IT0	88}	{IE0	89}	{IT1	8A}	{IE1	8B}
			{TR0	8C}	{TF0	8D}	{TR1	8E}	{TF1	8F}

			{RI	98}	{TI	99}	{RB8	9A}	{TB8	9B}
			{REN	9C}	{SM2	9D}	{SM1	9E}	{SM0	9F}

			{EX0	A8}	{ET0	A9}	{EX1	AA}	{ET1	AB}
			{ES	AC}	{ET2	AD}	{EC	AE}	{EA	AF}

			{RXD	B0}	{TXD	B1}	{INT0	B2}	{INT1	B3}
			{T0	B4}	{T1	B5}	{WR	B6}	{RD	B7}

			{PX0	B8}	{PT0	B9}	{PX1	BA}	{PT1	BB}
			{PS	BC}	{PT2	BD}	{PC	BE}

			{TF2	CF}	{EXF2	CE}	{RCLK	CD}	{TCLK	CC}
			{EXEN2	CB}	{TR2	CA}	{CT2	C9}	{CPRL2	C8}

			{P	D0}			{OV	D2}	{RS0	D3}	
			{RS1	D4}	{F0	D5}	{AC	D6}	{C	D7}
		} {
			set symbol([lindex $symb_name 0]) [expr "0x[lindex $symb_name 1]"]
		}
		
		array set ::Simulator_ENGINE::PIN {
			RXD	{3 0}
			TXD	{3 1}
			INT0	{3 2}
			INT1	{3 3}
			T0	{3 4}
			T1	{3 5}
			WR	{3 6}
			RD	{3 7}
			
			ANL0	{1 1}
			ANL1	{1 0}
		}

		set PORT_LATCHES [list $symbol(P0) $symbol(P1) $symbol(P2) $symbol(P3) $symbol(P4)]
	}

	## Shutdown simulator engine
	 # @return void
	private method internal_shutdown {} {
		set break 1
		$this Simulator_sync_clock
	}

	## Determinate if the specified feature is avaliable on this MCU
	 # @parm String key - feature name (e.g. 'p0')
	 # @return Bool - result (1 == yes; 0 == no)
	public method get_feature_avaliable {key} {
		return $feature_avaliable($key)
	}

	## Get number of implemented ports and list of port indexes
	 # @return List - {number_of_ports {idx0 idx1...}} (e.g. {4 {0 1 2 3}})
	public method get_ports_info {} {
		set sum 0
		for {set i 0} {$i < 5} {incr i} {
			if {$feature_avaliable(p$i)} {
				incr sum
				lappend lst $i
			}
		}
		return [list $sum $lst]
	}


	# ----------------------------------------------------------------
	# CONTROL PROCEDURES
	# ----------------------------------------------------------------

	## Force return from subprogram or interrupt handler
	 # @parm Bool intr__sub - 1 == Interrupt; 0 == subprogram
	 # @retunr void
	public method simulator_return_from_SP {intr__sub} {
		if {$intr__sub} {
			simulator_cancel_interrupt [lindex $inter_in_p_flags end]
		} {
			ins_ret
			$this move_simulator_line $Line($pc)
		}
	}

	## Resize external data memory
	 # Warning: This functon is unsafe
	 # @parm Int new_size - New memory size
	 # @return void
	public method simulator_resize_xdata_memory {new_size} {
		# Shrink memory
		if {$xram_size > $new_size} {
			if {!$new_size} {
				array unset xram
			}

		# Expand memory
		} elseif {$xram_size < $new_size} {
			for {set i $xram_size} {$i < $new_size} {incr i} {
				set xram($i) 0
			}
		}

		set xram_size $new_size
	}

	## Resize program memory
	 # Warning: This functon is unsafe
	 # @parm Int new_size - New memory size
	 # @return void
	public method simulator_resize_code_memory {new_size} {
		set new_size [expr {int($new_size)}]

		# Shrink memory
		if {$code_size > $new_size} {
			if {!$new_size} {
				array unset code
			}

		# Expand memory
		} elseif {$code_size < $new_size} {
			for {set i $code_size} {$i < $new_size} {incr i} {
				set code($i) 0
			}
		}

		set code_size $new_size
	}

	## Get list of decimal address of implemented SFR
	 # @return List - Implemented special function registers
	public method simulator_get_avaliable_sfr {} {
		return $avaliable_sfr
	}

	## Determinate whether the specified SFR is avaliable on the target MCU or not
	 # @parm Int sfr_addr - Address of SFR register
	 # @return Bool - 1 == Avaliable; 0 == Not avaliable
	public method simulator_is_sfr_avaliable {sfr_addr} {
		if {[lsearch -ascii -exact $avaliable_sfr $sfr_addr] == -1} {
			return 0
		} {
			return 1
		}
	}

	## Reset counter of overall program time
	 # @return void
	public method simulator_clear_overall_time {} {
		set overall_time 0
		set overall_instructions 0
	}
	
	## Reset vitrual processor
	 # @parm String mode - Reset mode:
	 #		'-' == no change	(IRAM, ERAM and XRAM)
	 #		'0' == all zeroes	(IRAM, ERAM and XRAM)
	 #		'1' == all ones		(IRAM, ERAM and XRAM)
	 #		'r' == random values	(IRAM, ERAM and XRAM)
	 # @return void
	public method master_reset {mode} {
		set break	1	;# Terminate running program
		set pc		0	;# Reset program counter
		set bank	0	;# Reset active register bank

		# Reset controllers configurations
		set controllers_conf(WatchDogTimer)	0	;# Stop Watchdog Timer
		set controllers_conf(SM0)		0	;# UART mode bit 0
		set controllers_conf(FE)		0	;# UART frame error flag bit
		if {$eeprom_size} {
			simulator_cancel_write_to_eeprom
			catch {
				$this sim_GUI_bit_set_clear 1 EECON RDYBSY
			}
		}

		# Reset engine configurations
		set DPL			{DP0L}	;# Select DTPR0
		set DPH			{DP0H}	;# Select DTPR0
		set hidden_DPTR0	{0 0}	;# Value of DPTR0 (if dual DPTR is hidden)
		set hidden_DPTR1	{0 0}	;# Value of DPTR1 (if dual DPTR is hidden)
		set idle_mode		0	;# Normal mode (not IDLE)
		set timer_0_running	0	;# Bool: Timer/Counter 0 engaged
		set timer_1_running	0	;# Bool: Timer/Counter 1 engaged
		set timer_2_running	0	;# Bool: Timer/Counter 2 engaged
		set pwm_running		0	;# Bool: PWM controller engaged (uses Timer/Counter 0 & 1)
		set wdt_prescaler_val	0	;# Int: Value of Watchdog prescaler
		set watchdog_value	0	;# Int: Value of watchdog timer
		set eeprom_WR		0	;# Bool: Data EEPROM write cycle in progress

		simulator_Sbar {} 0 $this		;# Clear simulator status bar
		set interrupt_on_next		0	;# Bool: Engage interrupt routine on the next instruction cycle
		set interrupts_in_progress	{}	;# Priority flags of interrupts which are in progress
		set inter_in_p_flags		{}	;# Interrupt flags of interrupts which are in progress

		switch -- $mode {
			- {	;# no changes
			}
			0 {	;# all zeroes
				for {set i 0} {$i < $iram_size} {incr i} {
					set ram($i) 0
				}
				for {set i 0} {$i < $eram_size} {incr i} {
					set eram($i) 0
				}
				update
				for {set i 0} {$i < $xram_size} {incr i} {
					set xram($i) 0
				}
			}
			1 {	;# all ones
				for {set i 0} {$i < $iram_size} {incr i} {
					set ram($i) 255
				}
				for {set i 0} {$i < $eram_size} {incr i} {
					set eram($i) 255
				}
				update
				for {set i 0} {$i < $xram_size} {incr i} {
					set xram($i) 255
				}
			}
			r {	;# random values
				for {set i 0} {$i < $iram_size} {incr i} {
					set ram($i) [expr {int(rand() * 256)}]
				}
				for {set i 0} {$i < $eram_size} {incr i} {
					set eram($i) [expr {int(rand() * 256)}]
				}
				update
				for {set i 0} {$i < $xram_size} {incr i} {
					set xram($i) [expr {int(rand() * 256)}]
				}
			}
		}

		# Make backup copy of all SFR for purpose of stepback function
		foreach addr [array names sfr] {
			stepback_reg_change S $addr
		}

		# Set port states
		set ports_previous_state {255 255 255 255 255}

		# Set SFR to defaults
		foreach reg $reset_reg_values {
			set sfr($symbol([lindex $reg 0])) [lindex $reg 1]
		}
		foreach item $reset_reg_values_1 {
			set reg [lindex $item 0]
			switch -- $reg {
				{T2CON}		{if {!$feature_avaliable(t2)}		{continue}}
				{RCAP2L}	{if {!$feature_avaliable(t2)}		{continue}}
				{RCAP2H}	{if {!$feature_avaliable(t2)}		{continue}}
				{TL2}		{if {!$feature_avaliable(t2)}		{continue}}
				{TH2}		{if {!$feature_avaliable(t2)}		{continue}}
				{T2MOD}		{if {!$feature_avaliable(t2mod)}	{continue}}
				{AUXR}		{if {!$feature_avaliable(auxr)}		{continue}}
				{SCON}		{if {!$feature_avaliable(uart)}		{continue}}
				{P0}		{if {!$feature_avaliable(p0)}		{continue}}
				{P1}		{if {!$feature_avaliable(p1)}		{continue}}
				{P2}		{if {!$feature_avaliable(p2)}		{continue}}
				{P3}		{if {!$feature_avaliable(p3)}		{continue}}
				{P4}		{if {!$feature_avaliable(p4)}		{continue}}
				{ACSR}		{if {!$feature_avaliable(acomparator)}	{continue}}
				{SADEN}		{if {!$feature_avaliable(euart)}	{continue}}
				{SADDR}		{if {!$feature_avaliable(euart)}	{continue}}
				{IPH}		{if {!$feature_avaliable(iph)}		{continue}}
				{WDTCON}	{if {!$feature_avaliable(wdtcon)}	{continue}}
				{EECON}		{if {!$eeprom_size}			{continue}}
				{SPCR}		{if {!$feature_avaliable(spi)}		{continue}}
				{SPSR}		{if {!$feature_avaliable(spi)}		{continue}}
				{DP1H}		{
					if {!$feature_avaliable(ddp) || $feature_avaliable(hddptr)} {
						continue
					}
				}
				{DP1L}		{
					if {!$feature_avaliable(ddp) || $feature_avaliable(hddptr)} {
						continue
					}
				}
				{CLKREG}	{
					if {!$feature_avaliable(clkreg) && !$feature_avaliable(ckcon)} {
						continue
					}
				}
				{AUXR1}		{
					if {!$feature_avaliable(ddp) || $feature_avaliable(wdtcon)} {
						continue
					}
				}

				default		{continue}
			}
			set sfr($symbol($reg)) [lindex $item 1]
		}

		# Restore bits which are not affected by reset
		if {$feature_avaliable(pof) && $controllers_conf(POF)} {
			set sfr($symbol(PCON)) [expr {$sfr($symbol(PCON)) | 16}]
		}
		if {$feature_avaliable(x2reset) && $controllers_conf(X2)} {
			set sfr($symbol(CLKREG)) [expr {$sfr($symbol(CLKREG)) | 1}]
		}

		# Reevaluate internal configuration flags
		set sync_ena 0
		foreach addr [array names sfr] {
			evaluate_sfr $addr
		}
		
		# Reset program run statistics
		for {set i 0} {$i < 10} {incr i} {
			set run_statistics($i) 0
		}

		# Synchronize with special GUI controls
		$this simulator_GUI_cancel_write_to_eeprom	;# Abort data EEPROM write cycle
		$this interrupt_monitor_reset			;# Reset interrupt monitor
		$this subprograms_clear				;# Clear list subprograms
		$this stopwatch_refresh				;# Stopwatch
		
		$this pale_reset				;#
		for {set i 0} {$i < 5} {incr i} {
			set j 0
			foreach bit [split $feature_avaliable(port$i) {}] {
				if {$bit == 0} {
					$this pale_SLSF [list $i $j] 6
				}
				incr j
			}
		}

		# Allow engagement
		set break 0
	}

	## Step program back
	 # @return Bool - false if no more backward steps can be done
	public method stepback {} {
		if {!${::Simulator::reverse_run_steps} || !$stepback_length} {
			return 0
		}
		set sync_ena 1
		incr stepback_length -1

		set lst [lindex $stepback_normal $stepback_length]
		set max [llength $lst]
		incr max -1

		for {set i $max} {$i >= 0} {incr i -1} {
			set item	[lindex $lst $i]
			set addr	[lindex $item 1]
			set val		[lindex $item 2]

			switch -- [lindex $item 0] {
				{I} {
					set ram($addr) $val
					$this Simulator_sync_reg $addr
				}
				{E} {
					set eram($addr) $val
					$this Simulator_XDATA_sync $addr
				}
				{P} {
					set eeprom($addr) $val
					::X::sync_eeprom_mem_window [format %X $addr] 0 $this
				}
				{X} {
					set xram($addr) $val
					$this Simulator_XDATA_sync $addr
				}
				{S} {
					set sfr($addr) $val
					evaluate_sfr $addr
				}
			}
		}

		set overall_instructions_org $overall_instructions
		set overall_time_org $overall_time
		simulator_set_special [lindex $stepback_spec $stepback_length]

		set opcode [getCode $pc]
		if {[lsearch ${CompilerConsts::defined_OPCODE} $opcode] == -1} {
			incr run_statistics(4) -1
		} {
			incr run_statistics(4) -[lindex $CompilerConsts::Opcode($opcode) 2]
		}

		incr run_statistics(0) [expr {int(($overall_time - $overall_time_org) * (12000000.0 / $clock_kHz))}]
		incr run_statistics(1) [expr {int($overall_time - $overall_time_org) * 12}]
		incr run_statistics(2) [expr {int($overall_instructions - $overall_instructions_org)}]
		incr run_statistics(3) -1

		switch -- [lindex $stepback_spec [list $stepback_length 0]] {
			4 { ;# Invocaton of subprogram
				incr run_statistics(7) -1
				$this subprograms_call 3 -1 -1
			}
			3 { ;# Invocaton from interrupt routine
				incr run_statistics(8) -1
				$this subprograms_call 2 -1 -1
			}
			2 { ;# Return from subprogram
				incr run_statistics(6) -1
				$this subprograms_return 0
			}
			1 { ;# Return from an interrupt
				incr run_statistics(5) -1
				$this subprograms_return 1
			}
		}
		if {[llength $interrupts_in_progress]} {
			simulator_Sbar "Interrupt at vector 0x[format %X [intr2vector [lindex $interrupts_in_progress end]]]  " 1 $this
		} {
			simulator_Sbar {} 0 $this
		}
		$this graph_stepback [expr {int(($overall_time_org - $overall_time) * 2)}]
		$this interrupt_monitor_reevaluate
		$this stopwatch_refresh
		if {$eeprom_size} {
			for {set i 0} {$i < 32} {incr i} {
				::X::sync_eeprom_write_buffer $i $this
			}
			::X::eeprom_write_buffer_set_offset $eeprom_WR_ofs $this
		}

		if {$eeprom_WR} {
			eeprom_controller [expr {int(($overall_time_org - $overall_time) * (-2))}]
		} {
			foreach reg $eeprom_prev {
				::X::sync_eeprom_clear_bg_hg [lindex $reg 0] $this
			}
			$this simulator_GUI_cancel_write_to_eeprom
		}

		$this Simulator_sync_PC_etc

		set stepback_spec	[lreplace $stepback_spec end end]
		set stepback_normal	[lreplace $stepback_normal end end]

		$this Simulator_GUI_sync S 224
		if {!$stepback_length} {
			return 0
		} {
			return 1
		}
	}

	## Engage mode "Step"
	 # @return Int - line in source code
	public method step {} {
		set address_error 0

		# Valid OP code
		if {[check_address_validity C $pc]} {return {}}
		if {$code($pc) != {}} {
			set sync_ena 1		;# Enable synchronization
			instruction_cycle	;# Execute instruction

			# Synchronize
			$this Simulator_GUI_sync S 208
			# Return line number
			update
			return $Line($pc)

		# Invalid OP code
		} {
			bell
			$this sim_txt_output "No instruction found at 0x[NumSystem::dec2hex $pc]"
			incr_pc 1
			return {}
		}
	}

	## Engage/Disengage mode "Step over"
	 # @return Int - line in source code
	public method sim_stepover {} {
		set address_error 0

		# Disengage
		if {$simulation_in_progress} {
			set break 1
			set simulation_in_progress 0
			set stepover_in_progress 0

		# Engage
		} {
			# Local variables
			set current_line		0 ;# Current line in source code
			set stepover_in_progress	1 ;# Bool: "Step over" mode flag
			set simulation_in_progress	1 ;# Bool: Simulator engaged
			set ::X::critical_procedure_in_progress 0

			# Valid OP code
			set tmp_pc 0
			if {[check_address_validity C $pc]} {return {}}
			if {$code($pc) != {}} {
				set sync_ena 1	;# Enable synchronization

				while 1 {
					# Conditionaly abort simulation
					if {$break} {
						set break 0
						set ::X::critical_procedure_in_progress 0
						set simulation_in_progress 0
						set stepover_in_progress 0
						break
					}

					# Abort simulation on invalid OP code
					if {[check_address_validity C $pc]} {return {}}
					if {$code($pc) == {}} {
						incr_pc 1
						bell
						$this sim_txt_output "No instruction found at 0x[NumSystem::dec2hex $pc]"
						set simulation_in_progress 0
						set stepover_in_progress 0
						return {}
					}

					# Execute instruction
					set current_line [lindex $Line($pc) 0]
					instruction_cycle
					if {[lindex $Line($pc) 0] != {} && [lindex $Line($pc) 0] != $current_line} {
						break
					}

					# Synchronize and update GUI
					$this Simulator_sync_PC_etc
					update
				}

				if {$break} {
					set break 0
					set ::X::critical_procedure_in_progress 0
				}

				# Reset flags
				set simulation_in_progress	0 ;# Simulator engaged
				set stepover_in_progress	0 ;# Mode "Step over" engaged

				# Return line
				return $Line($pc)

			# No OP code
			} {
				incr_pc 1
				bell
				$this sim_txt_output "No instruction found at 0x[NumSystem::dec2hex $pc]"
				set simulation_in_progress 0
				set stepover_in_progress 0
				return {}
			}
		}
	}

	## Engage/Disengage mode "Run"
	 # @return Int - line in source code
	public method sim_run {} {
		set address_error 0
		
		# Local variables
		set last_line [lindex $Line($pc) 0]	;# Line of the last instruction (line in source code)

		# Disengage
		if {$simulation_in_progress} {
			set break			1 ;# Terminate running program
			set simulation_in_progress	0 ;# Bool: Simulator engaged
			set run_in_progress		0 ;# Bool: "Run" mode flag

		# Engage
		} {
			set sync_ena	0		;# Disabled synchronizations

			# Local variables
			set run_in_progress		1 ;# Bool: "Run" mode flag
			set simulation_in_progress	1 ;# Bool: Simulator engaged
			set idx				0 ;# Instruction index (GUI is updated after each 1000)
			set ::X::critical_procedure_in_progress 0

			# Infinitely execute program instructions until break
			while 1 {
				incr idx
				# Conditionaly abort simulation
				if {$break} {
					set simulation_in_progress 0
					set run_in_progress 0
					set break 0
					set ::X::critical_procedure_in_progress 0
					break
				}

				# Empty OP code -> abort simulation
				if {[check_address_validity C $pc]} {return {}}
				if {$code($pc) == {}} {
					set simulation_in_progress 0
					set run_in_progress 0
					bell
					$this sim_txt_output "No instruction found at 0x[NumSystem::dec2hex $pc]"
					break
				}

				# Execute instruction
				instruction_cycle

				# Run update command after each 25 instructions
				if {!($idx % 25)} {
					update
				}
				# Update GUI after each 100 instructions
				if {$idx >= 100} {
					set idx 0
					$this Simulator_sync_PC_etc
					update
				}

				# Handle breakpoints
				if {[lindex $Line($pc) 0] != {}} {
					set last_line [lindex $Line($pc) 0]
					if {[lsearch $breakpoints([lindex $Line($pc) 1]) [lindex $last_line 0]] != -1} {
						incr run_statistics(9)
						if {$::CONFIG(BREAKPOINTS_ALLOWED)} {
							set simulation_in_progress 0
							set run_in_progress 0
							Sbar "Breakpoint reached at 0x[NumSystem::dec2hex $pc]"
							break
						}
					}
				}
			}

			if {$break} {
				set break 0
				set ::X::critical_procedure_in_progress 0
			}

			# Return last line (line in source code)
			return $Line($pc)
		}
	}

	## Engage/Disengage mode "Animation"
	 # @return Int - line in source code
	public method sim_animate {} {
		set address_error 0
		
		# Disengage
		if {$simulation_in_progress} {
			set break			1 ;# Terminate running program
			set simulation_in_progress	0 ;# Bool: Simulator engaged
			set animation_in_progress	0 ;# Bool: "Animation" mode flag

		# Engage
		} {
			set animation_in_progress	1 ;# Bool: "Animation" mode flag
			set simulation_in_progress	1 ;# Bool: Simulator engaged
			set ::X::critical_procedure_in_progress 0

			# Infinitely execute program instructions until break
			while 1 {
				# Conditionaly abort simulation
				if {$break} {
					set simulation_in_progress 0
					set animation_in_progress 0
					set break 0
					set ::X::critical_procedure_in_progress 0
					break
				}

				# Perform program step
				set lineNum [step]

				if {$lineNum == {}} {
					set simulation_in_progress 0
					set animation_in_progress 0
					break
				}

				# Move simulator line in the editor
				$this move_simulator_line $lineNum
				$this Simulator_sync_PC_etc

				# Handle breakpoints
				if {[string length [lindex $lineNum 0]] && [string length [lindex $lineNum 1]]} {
					if {[lsearch $breakpoints([lindex $lineNum 1]) [lindex $lineNum 0]] != -1} {
						incr run_statistics(9)
						if {$::CONFIG(BREAKPOINTS_ALLOWED)} {
							set simulation_in_progress 0
							set animation_in_progress 0
							Sbar "Breakpoint reached at 0x[NumSystem::dec2hex $pc]"
							break
						}
					}
				}

				# Update GUI
				update
			}

			if {$break} {
				set break 0
				set ::X::critical_procedure_in_progress 0
			}
		}
	}

	## Return true if the engine is engaged
	 # @return Bool - engaged flag
	public method sim_is_busy {} {
		return $simulation_in_progress
	}

	## Return true if the engine is in mode "Step over"
	 # @return Bool - "Step over" flag
	public method sim_stepover_in_progress {} {
		return $stepover_in_progress
	}

	## Return true if the engine is in mode "Run"
	 # @return Bool - "Run" flag
	public method sim_run_in_progress {} {
		return $run_in_progress
	}

	## Return true if the engine is in mode "Animation"
	 # @return Bool - "Animation" flag
	public method sim_anim_in_progress {} {
		return $animation_in_progress
	}

	## Disengage simulator (Power off the MCU)
	 # @return void
	public method Simulator_shutdown {} {
		# Clear content of IDATA memory
		for {set i 0} {$i < $iram_size} {incr i} {
			set ram($i) 0
		}

		# Cancel EEPROM write process
		if {$eeprom_WR} {
			simulator_cancel_write_to_eeprom
		}

		# Discard stepback stack
		stepback_discard_stack

		set break 1
	}

	## Engage simulator  (Power on the MCU)
	 # @return void
	public method Simulator_initiate {} {
		set sync_ena 1
		simulator_system_power_on
		master_reset -

		# Reset watchdog
		set watchdog_value	0
		set wdt_prescaler_val	0
		
		set break 0
	}


	# ----------------------------------------------------------------
	# EXTERNAL INTERFACE MANAGEMENT PROCEDURES
	# ----------------------------------------------------------------

	## Get value of bit SMOD0 in PCON if avaliable
	 # @return Bool - SMOD0 value (if unavaliable then 0)
	public method get_SMOD0 {} {
		return $controllers_conf(SMOD0)
	}

	## Set value of bit FE in SCON if avaliable
	 # @param Bool value - New value for SCON.FE (or SCON.SM0 if FE is not avaliable)
	 # @return void
	public method sim_engine_set_FE {value} {
		if {$feature_avaliable(smod0)} {
			set controllers_conf(FE) $value
		} {
			sim_engine_set_SM0 $value
		}
	}
	
	## Set value of bit SM0 in SCON
	 # @param Bool value - New value for PCON.SM0
	 # @return void
	public method sim_engine_set_SM0 {value} {
		set controllers_conf(SM0) $value
	}
	
	## Get value of bit FE in SCON if avaliable
	 # @return Bool - FE value (if unavaliable then {})
	public method sim_engine_get_FE {} {
		if {!$feature_avaliable(smod0)} {
			return {}
		} {
			return $controllers_conf(FE)
		}
	}
	
	## Get value of bit SM0 in SCON
	 # @return Bool - SM0 value
	public method sim_engine_get_SM0 {} {
		return $controllers_conf(SM0)
	}
	
	## Get program run statistics
	 # @return List - Array run_statistics converted to list
	public method get_run_statistics {} {
		return [array get run_statistics]
	}

	## Retrieve filename from list of files from which this program has been compiled
	 # @return String - Resulting relative filename or {}
	public method simulator_get_filename {filenumber} {
		return [lindex $list_of_filenames $filenumber]
	}
	
	## Get list of files defined in debug file
	 # @return List - List of files
	public method simulator_get_list_of_filenames {} {
		return $list_of_filenames
	}

	## Retrieve file number from list of files from which this program has been compiled
	 # @return Int - Resulting file number or -1
	public method simulator_get_filenumber {filename} {
		return [lsearch -exact -ascii $list_of_filenames $filename]
	}
	
	## Get maximum valid interrupt priority level
	 # @return Int - 0..3
	public method simulator_get_max_intr_priority {} {
		if {$feature_avaliable(iph)} {
			return 3
		} {
			return 1
		}
	}

	## Get current interrupt priority for certain interrupt
	 # @parm String flag - Interrupt flag (e.g. TF2)
	 # @return Int - 0..3
	public method simulator_get_interrupt_priority {flag} {
		return [lindex $interrupt_pri_num [lsearch $interrupt_pri_flg $flag]]
	}

	## Invoke certain interrupt (from user interface)
	 # @parm String flag - Interrupt flag (e.g. CF)
	 # @return void
	public method simulator_invoke_interrupt {flag} {
		if {[lsearch $inter_in_p_flags $flag] != -1} {
			return
		}

		switch -- $flag {
			{IE0}	{set name PX0}
			{TF0}	{set name PT0}
			{IE1}	{set name PX1}
			{TF1}	{set name PT1}
			{TF2}	{set name PT2}
			{EXF2}	{set name PT2}
			{SPIF}	{set name PS}
			{TI}	{set name PS}
			{RI}	{set name PS}
			{CF}	{set name PC}
		}
		interrupt_handler $name [intr2vector $name] $flag 1
		$this interrupt_monitor_intr $flag
	}

	## Increment interrupt priority for certain interrupt
	 # @parm String flag - Interrupt flag (e.g. CF)
	 # @return void
	public method simulator_incr_intr_priority {flag} {
		set level [simulator_get_interrupt_priority $flag]
		if {[simulator_get_max_intr_priority] <= $level} {
			return
		}
		incr level
		set_interrupt_priority_flag $flag $level
	}

	## Decrement interrupt priority for certain interrupt
	 # @parm String flag - Interrupt flag (e.g. IE0)
	 # @return void
	public method simulator_decr_intr_priority {flag} {
		set level [simulator_get_interrupt_priority $flag]
		if {!$level} {
			return
		}
		incr level -1
		set_interrupt_priority_flag $flag $level
	}

	## Set interrupt priority for certain interrupt
	 # Auxiliary procedure for procedures:
	 #	"simulator_incr_intr_priority"
	 #	"simulator_decr_intr_priority"
	 # @parm String flag	- Interrupt flag (e.g. RI)
	 # @parm Int level	- New priority level (0..3)
	 # @return void
	private method set_interrupt_priority_flag {flag level} {
		# IP: - PC PT2 PS | PT1 PX1 PT0 PX0
		switch -- $flag {
			IE0	{set bit_mask 1}
			TF0	{set bit_mask 2}
			IE1	{set bit_mask 4}
			TF1	{set bit_mask 8}
			TI	{set bit_mask 16}
			RI	{set bit_mask 16}
			SPIF	{set bit_mask 16}
			TF2	{set bit_mask 32}
			EXF2	{set bit_mask 32}
			CF	{set bit_mask 64}
			default {
				return
			}
		}

		# Adjust register IPH
		if {$feature_avaliable(iph)} {
			if {${::Simulator::reverse_run_steps}} {
				stepback_reg_change S $symbol(IPH)
			}
			
			# Set priority bit
			if {$level / 2} {
				set sfr($symbol(IPH)) [expr $sfr($symbol(IPH)) | $bit_mask]
			# Clear priority bit
			} {
				set sfr($symbol(IPH)) [expr $sfr($symbol(IPH)) & ($bit_mask ^ 255)]
			}

			# Adjust internal engine configuration
			evaluate_sfr $symbol(IPH)
		}

		## Adjust register IP
		if {${::Simulator::reverse_run_steps}} {
			stepback_reg_change S $symbol(IP)
		}
		 # Set priority bit
		if {$level % 2} {
			set sfr($symbol(IP)) [expr $sfr($symbol(IP)) | $bit_mask]
		 # Clear priority bit
		} {
			set sfr($symbol(IP)) [expr $sfr($symbol(IP)) & ($bit_mask ^ 255)]
		}
		 # Adjust internal engine configuration
		evaluate_sfr $symbol(IP)
	}

	## Clear certain interrupt flag
	 # @parm String flag - Interrupt flag (e.g. RI)
	 # @return void
	public method simulator_clear_intr_flag {flag} {
		# Determinate register address and mask
		switch -- $flag {
			IE0 {
				set addr 0x88	;# TCON
				set mask 0xFD	;# 0x02
			}
			IE1 {
				set addr 0x88	;# TCON
				set mask 0xF7	;# 0x20
			}
			TF0 {
				set addr 0x88	;# TCON
				set mask 0xDF	;# 0x08
			}
			TF1 {
				set addr 0x88	;# TCON
				set mask 0x7F	;# 0x80
			}
			TI {
				set addr 0x98	;# SCON
				set mask 0xFD	;# 0x02
			}
			RI {
				set addr 0x98	;# SCON
				set mask 0xFE	;# 0x01
			}
			SPIF {
				set addr 0xAA	;# SPCR
				set mask 0x7F	;# 0x80
			}
			TF2 {
				set addr 0xC8	;# T2CON
				set mask 0x7F	;# 0x8F
			}
			EXF2 {
				set addr 0xC8	;# T2CON
				set mask 0xBF	;# 0x40
			}
			CF {
				set addr 0x97	;# ACSR
				set mask 0xEF	;# 0x10
			}
			default {
				return
			}
		}

		# Adjust register which contains the specified flag
		set addr [expr "$addr"]
		if {${::Simulator::reverse_run_steps}} {
			stepback_reg_change S $addr
		}
		set sfr($addr) [expr {$sfr($addr) & $mask}]

		# Adjust internal engine configuration
		evaluate_sfr $addr
		$this interrupt_monitor_intr_flags [simulator_get_active_intr_flags]
	}

	## Force return from certain interrupt
	 # @parm String flag - Interrupt flag (e.g. RI)
	 # @return void
	public method simulator_cancel_interrupt {flag} {
		set index [lsearch $inter_in_p_flags $flag]
		if {$index == -1} {
			return
		}
		set last [llength $inter_in_p_flags]
		incr last -1

		$this interrupt_monitor_reti	[lindex $inter_in_p_flags $index]
		set interrupts_in_progress	[lreplace $interrupts_in_progress $index $index]
		set inter_in_p_flags		[lreplace $inter_in_p_flags $index $index]

		if {$last != $index} {return}

		incr run_statistics(8)
		$this subprograms_return 1
		if {[llength $interrupts_in_progress]} {
			set vector [format %X [intr2vector [lindex $interrupts_in_progress end]]]
			simulator_Sbar "Interrupt at vector 0x$vector  " 1 $this
		} {
			simulator_Sbar {} 0 $this
		}

		set pch [stack_pop]
		set pcl [stack_pop]

		set pc [expr {($pch << 8) + $pcl}]
		$this move_simulator_line $Line($pc)
	}

	## Get list of interrupt flags which are set
	 # @return List - List of active interrupt flags (e.g. {TF0 CF})
	public method simulator_get_active_intr_flags {} {
		set result {}
		foreach flag {IE0 TF0 IE1 TF1 TI RI SPIF TF2 EXF2 CF} {
			if {$controllers_conf($flag)} {
				lappend result $flag
			}
		}
		return $result
	}

	## Get arguments for function "interrupt_monitor_intr_prior"
	 # Get list of possible interrupt flags in order of their priorities
	 # @return List - Interrupt flags in order of their priorities (decremental)
	public method simulator_get_intr_flags_with_priorities {} {
		return $interrupt_pri_flg
	}

	## Get list of interrupt flags of interrupts which are in progress
	 # @return List - Interrupt flags
	public method simulator_get_interrupts_in_progress {} {
		return $inter_in_p_flags
	}

	## Return list of interrupt priority bits of these intrerrupts which are currently in progress
	 # @return List - Priority bits
	public method simulator_get_interrupts_in_progress_pb {} {
		return $interrupts_in_progress
	}

	## Get list of possible interrupt flags on this MCU
	 # @return List - Something like {IE0 TF0 IE1 TF1 RI TI CF}
	public method simulator_get_intr_flags {} {
		set result {IE0 TF0 IE1 TF1}
		if {$feature_avaliable(uart)}		{lappend result RI TI}
		if {$feature_avaliable(spi)}		{lappend result SPIF}
		if {$feature_avaliable(t2)}		{lappend result TF2 EXF2}
		if {$feature_avaliable(acomparator)}	{lappend result CF}
		return $result
	}

	## Get list of addresses in data EEPROM which are beeing written
	 # @return List - {dec_addr0 dec_addr1 ...}
	public method simulator_get_eeprom_beeing_written {} {
		set result {}
		foreach reg $eeprom_prev {
			lappend result [lindex $reg 0]
		}
		return $result
	}

	## Cancel data EEPROM write cycle
	 # @return void
	public method simulator_cancel_write_to_eeprom {} {
		if {!$eeprom_size || !$eeprom_WR} {return}

		foreach reg $eeprom_prev {
			set addr [lindex $reg 0]
			stepback_reg_change P $addr
			set eeprom($addr) [lindex $reg 1]
			::X::sync_eeprom_mem_window [format %X $addr] 0 $this
		}
		simulator_finalize_write_to_eeprom
	}

	## Finalize EEPROM write cycle
	 # @return void
	public method simulator_finalize_write_to_eeprom {} {
		if {!$eeprom_size || !$eeprom_WR} {return}

		# Clear background highlight in EEPROM hexeditor
		foreach reg $eeprom_prev {
			::X::sync_eeprom_clear_bg_hg [lindex $reg 0] $this
		}

		# Adjust engine configuration
		$this simulator_GUI_cancel_write_to_eeprom
		set eeprom_WR		0
		set eeprom_WR_time	0
		set eeprom_WR_ofs	{}
		set eeprom_prev		{}

		# Set flag EECON.RDYBSY (EEPROM is ready)
		$this sim_GUI_bit_set_clear 1 EECON RDYBSY
		if {${::Simulator::reverse_run_steps}} {
			stepback_reg_change S 150
		}
		set sfr(150) [expr {$sfr(150) | 2}]
		set controllers_conf(RDYBSY) 1
		if {$sync_ena} {
			$this Simulator_GUI_sync S 150
		}
	}

	## Try to translate line number to address in CODE
	 # @parm Int line - Line number (in source code)
	 # @parm Int file - File number
	 # @return Int	- Address in program memory or {} on fail
	public method simulator_line2address {line file} {
		if {$line == {}} {
			return {}
		}
		set line [expr $line]
		if {[llength [array names line2PC -exact "$line,$file"]]} {
			return $line2PC($line,$file)
		} {
			return {}
		}
	}

	## Set watchdog timer value
	 # This procedure does nothing on MCUs without watchdog timer
	 # @parm Int value - new value (0..8192)
	 # @return void
	public method simulator_setWatchDogTimer {value} {
		if {!$feature_avaliable(wtd)} {return}
		set watchdog_value $value
	}

	## Get value of watchdog timer
	 # @return Int - Current value of watchdog timer 0..8192
	public method simulator_getWatchDogTimerValue {} {
		return $watchdog_value
	}

 	## Get size of watch dog prescaler (0 - 128)
 	 # @return Int - Maximum value - 1
	public method simulator_getWatchDogPrescalerSize {} {
		return $controllers_conf(WatchDogPrescaler)
	}

	## Get current value of watchdog prescaler
	 # @return Int - Prescaler content
	public method simulator_getWatchDogPrescalerValue {} {
		return $wdt_prescaler_val
	}

	## Set value of watchdog prescaler
	 # @parm Int value - New prescaler value
	 # @return void
	public method simulator_setWatchDogPrescalerValue {value} {
		set wdt_prescaler_val $value
	}

	## Start/Stop watchdog timer
	 # This procedure does nothing on MCUs without watchdog timer
	 # @parm Bool bool - 0 == STOP; 1 == START
	 # @return void
	public method simulator_startStopWatchDogTimer {bool} {
		if {!$feature_avaliable(wtd)} {return}
		set controllers_conf(WatchDogTimer) $bool
	}

	## Determinate wheather watchdog timer is running or not
	 # @return Bool - 1 == RUNNING; 0 == STOPPED
	public method simulator_isWatchDogTimerRuning {} {
		return $controllers_conf(WatchDogTimer)
	}

	## Perform subprogram call
	 # @parm Int value - Subprogram vector
	 # @return void
	public method simulator_subprog_call {value} {
		stack_push [expr {$value & 0xFF}]
		stack_push [expr {($value & 0xFF00) >> 8}]
		incr run_statistics(6)
		$this subprograms_call 3 $pc $value
		$this stopwatch_refresh
		set pc $value
	}

	## Set value of Program Couter (PC)
	 # @parm Int value - new value
	 # @return void
	public method setPC {value} {
		set pc $value
	}

	## Get current value of Program Couter (PC)
	 # @return Int - PC value
	public method getPC {} {
		return $pc
	}

	## Get information about current line
	 # @return List - {line_number file_number level block}
	public method simulator_getCurrentLine {} {
		return $Line($pc)
	}

	## Get current line number only
	 # @return Int - Line number
	public method simulator_get_line_number {} {
		return [lindex $Line($pc) 0]
	}

	## Translate adress in program memory to line info
	 # @parm Int addr - Address to translate
	 # @return List - Line information list
	public method simulator_address2line {addr} {
		return $Line($addr)
	}

	## Change content of internal data memory
	 # @parm Int addr	- Register address (decimal)
	 # @parm String val	- New value (hexadecimal) or {} (means '0')
	 # @return void
	public method setData {addr val} {
		if {$val == {}} {
			set val 0
		}
		set ram($addr) [expr "0x$val"]
	}

	## Change content of internal data memory
	 # @parm Int addr	- Register address (decimal)
	 # @parm String val	- New value (decimal) or {} (means '0')
	 # @return void
	public method setDataDEC {addr val} {
		if {$val == {}} {
			set val 0
		}
		set ram($addr) $val
	}

	## Change content of register in SFR area
	 # @parm Int addr	- Register address (decimal)
	 # @parm String val	- New value (hexadecimal) or {} (means '0')
	 # @return void
	public method setSfr {addr val} {
		# Empty value == zero
		if {$val == {}} {
			set val 0
		}

		# Set value
		set foo $stepback_ena
		set stepback_ena 0
		switch -- $addr {
			153 {	;# SBUF R
				set sfr(153) [expr "0x$val"]
			}
			default {
				write_sfr $addr [expr "0x$val"]
			}
		}

		# Take care of read-only bits
		switch -- $addr {
			{150} {	;# EECON
				if {$sfr(150) & 1} {
					set controllers_conf(WRTINH) 1
				} {
					set controllers_conf(WRTINH) 0
				}
			}
		}

		# Adjust internal engine configuration
		evaluate_sfr $addr 0
		set stepback_ena $foo

		# If adress points to Primary Accumulator (Acc) -> reevaluate PSW
		if {$addr == 224} {
			$this Simulator_GUI_sync S 208
		}
	}

	## Change content of register in SFR area directly
	 # @parm Int addr	- Register address (decimal)
	 # @parm String val	- New value (decimal)
	 # @return void
	public method setSfr_directly {addr val} {
		set sfr($addr) $val
	}

	## Change content of external data memory
	 # @parm Int addr	- Register address (decimal)
	 # @parm String val	- New value (hexadecimal) or {} (means '0')
	 # @return void
	public method setXdata {addr val} {
		if {$val == {}} {
			set val 0
		}
		set xram($addr) [expr "0x$val"]
	}
	
	## Change content of expanded data memory
	 # @parm Int addr	- Register address (decimal)
	 # @parm String val	- New value (hexadecimal) or {} (means '0')
	 # @return void
	public method setEram {addr val} {
		if {$val == {}} {
			set val 0
		}
		set eram($addr) [expr "0x$val"]
	}

	## Change content of external data memory
	 # @parm Int addr	- Register address (decimal)
	 # @parm String val	- New value (decimal) or {} (means '0')
	 # @return void
	public method setXdataDEC {addr val} {
		if {$val == {}} {
			set val 0
		}
		set xram($addr) $val
	}
	
	## Change content of expanded data memory
	 # @parm Int addr	- Register address (decimal)
	 # @parm String val	- New value (decimal) or {} (means '0')
	 # @return void
	public method setEramDEC {addr val} {
		if {$val == {}} {
			set val 0
		}
		set eram($addr) $val
	}

	## Change content of the program memory
	 # @parm Int addr	- Register address (decimal)
	 # @parm String val	- New value (hexadecimal) or {}
	 # @return void
	public method setCode {addr val} {
		set code($addr) [expr "0x$val"]
	}

	## Change content of the program memory
	 # @parm Int addr	- Register address (decimal)
	 # @parm String val	- New value (decimal) or {}
	 # @return void
	public method setCodeDEC {addr val} {
		set code($addr) $val
	}
	
	## Get value (DEC) of some register in expanded data memory
	 # @parm Int addr - register address
	 # @return Int - register value
	public method getEramDEC {addr} {
		return $eram($addr)
	}

	## Get value (HEX) of some register in expanded data memory
	 # @parm Int addr - register address
	 # @return String - register value (2 hexadecimal digits)
	public method getEram {addr} {
		# Get hexadecimal value
		set result [format "%X" $eram($addr)]
		# Adjust the value
		if {[string length $result] == 1} {
			set result "0$result"
		}
		# Return the value
		return $result
	}
	
	## Get value (HEX) of some register in internal data memory
	 # @parm Int addr - register address
	 # @return String - register value (2 hexadecimal digits)
	public method getData {addr} {
		# Get hexadecimal value
		set result [format "%X" $ram($addr)]
		# Adjust the value
		if {[string length $result] == 1} {
			set result "0$result"
		}
		# Return the value
		return $result
	}

	## Get value (HEX) of some register in SFR area
	 # @parm Int addr - register address
	 # @return String - register value (2 hexadecimal digits)
	public method getSfr {addr} {
		# Get hexadecimal value
		set result [format "%X" $sfr($addr)]
		# Adjust the value
		if {[string length $result] == 1} {
			set result "0$result"
		}
		# Return the value
		return $result
	}
	
	## Get value (DEC) of some register in SFR area
	 # @parm Int addr - register address
	 # @return Int - register value
	public method getSfrDEC {addr} {
		return $sfr($addr)
	}

	## Get value (DEC) of some register in SFR area
	 # @parm Int addr - register address
	 # @return Int - register value
	public method getDataDEC {addr} {
		return $ram($addr)
	}

	## Get value (DEC) of some register in data EEPROM
	 # @parm Int addr - register address
	 # @return Int - register value
	public method getEepromDEC {addr} {
		return $eeprom($addr)
	}
	
	## Get value (HEX) of some register in data EEPROM
	 # @parm Int addr - register address
	 # @return String - register value (2 hexadecimal digits)
	public method getEeprom {addr} {
		# Get hexadecimal value
		set result [format "%X" $eeprom($addr)]
		# Adjust the value
		if {[string length $result] == 1} {
			set result "0$result"
		}
		# Return the value
		return $result
	}

	## Change content of external data memory
	 # @parm Int addr	- Register address (decimal)
	 # @parm String val	- New value (decimal) or {} (means '0')
	 # @return void
	public method setEepromDEC {addr val} {
		if {$val == {}} {
			set val 0
		}
		set eeprom($addr) $val
	}

	## Get value (HEX) of some register in external data memory
	 # @parm Int addr - register address
	 # @return String - register value (2 hexadecimal digits)
	public method getXdata {addr} {
		# Get hexadecimal value
		set result [format "%X" $xram($addr)]
		# Adjust the value
		if {[string length $result] == 1} {
			set result "0$result"
		}
		# Return the value
		return $result
	}

	## Get value (DEC) of some register in external data memory
	 # @parm Int addr - register address
	 # @return Int - register value
	public method getXdataDEC {addr} {
		return $xram($addr)
	}

	## Get value (HEX) of some register in the program memory
	 # @parm Int addr - register address
	 # @return String - register value (2 hexadecimal digits) or {}
	public method getCode {addr} {
		# Get value (and return {} if it's {})
		if {![simulator_address_range C $addr] || $code($addr) == {}} {return {}}
		set result [format "%X" $code($addr)]
		# Normalize the value
		if {[string length $result] == 1} {
			set result "0$result"
		}
		# Return the value
		return $result
	}

	## Get value (DEC) of certain register in program memory
	 # @parm Int addr - Register address
	 # @return Int - Register value
	public method getCodeDEC {addr} {
		if {![simulator_address_range C $addr]} {
			return {}
		}
		return $code($addr)
	}

	## Get value (DEC) of certain cell in data EEPROM write buffer
	 # @parm Int addr - Cell address
	 # @return Int - Register value
	public method getEepromWrBufDEC {addr} {
		return $eeprom_WR_buff($addr)
	}

	## Set value (DEC) of certain cell in data EEPROM write buffer
	 # @parm Int addr	- Cell address
	 # @parm Int val	- New cell value
	 # @return void
	public method setEepromWrBufDEC {addr val} {
		set eeprom_WR_buff($addr) $val
	}

	## Set value (DEC) of EEPROM write buffer offset
	 # @return Int - Offset (0..65535)
	public method getEepromWrOffsetDEC {} {
		return $eeprom_WR_ofs
	}

	## Load program to virtual processor from ADF file (Assembler Debug File)
	 # @parm File adf_file - ADF file ID (content of *.adf file)
	 # @return void
	public method load_program_from_adf {adf_file} {
		unload_program	;# Clear current content of the program memory

		set programming_language 0
		
		set firts_line 1
		set list_of_filenames [list]

		# Parse the program data
		while {![eof $adf_file]} {
			set line [gets $adf_file]

			# Skip empty lines and comments
			if {$line == {} || [regexp {^\s*#} $line]} {continue}

			# Take first line
			if {$firts_line} {
				set firts_line 0
				set project_dir [$this cget -projectPath]
				set len [llength $line]
				for {set i 1} {$i < $len} {incr i 2} {
					lappend list_of_filenames	\
						[file join $project_dir [lindex $line $i]]
				}
				continue
			}

			# Local variables
			set fileNum	[lindex $line 0]	;# File number
			set lineNum	[lindex $line 1]	;# Number of line in source code
			set addr	[lindex $line 2]	;# Code address
			set line	[lreplace $line 0 2]	;# List of codes (DEC)

			# Set address for translating from line number
			set line2PC($lineNum,$fileNum) $addr

			# Iterate over codes and save them
			foreach num $line {
				# Check for allowed address range
				if {$addr >= $code_size} {
					tk_messageBox		\
						-parent .	\
						-icon warning	\
						-type ok	\
						-title "Out of memory"	\
						-message "[$this cget -P_option_mcu_type] has not enought program memory to load this program. Simulator will work but the loaded code is incomplite"
					return 0
				}
				set Line($addr) [list $lineNum $fileNum 0 0]	;# Set line number
				set code($addr) $num				;# Set program code
				incr addr					;# Increment address
			}
		}
	}

	## Load program to virtual processor from CDB file
	 # @parm File filename - Full name of source file from which SIM file was generaded
	 # @parm File cdb_file - CDB file ID (content of *.cdb file)
	 # @parm File ihx_file - HEX file ID (content of *.ihx or *.hex file)
	 # @return void
	public method load_program_from_cdb {filename cdb_file ihx_file} {
		unload_program	;# Clear current content of the program memory

		set programming_language 1

		set lineNum		0
		set highest_addr	0
		set eof			0

		# Iterate over HEX records
		while {![eof $ihx_file]} {
			set line [gets $ihx_file]
			incr lineNum	;# Increment line number

			# Skip comments
			if {[string index $line 0] != {:}} {continue}

			# Check for valid charters
			if {![regexp {^:[0-9A-Fa-f]+$} $line]} {
				return 0
			}
			# Check for odd lenght
			set len [string length $line]
			if {[expr {$len % 2}] != 1} {
				return 0
			}

			# Analize HEX record
			set len		[ string range $line 1		2	] ;# Lenght field
			set addr	[ string range $line 3		6	] ;# Address field
			set type	[ string range $line 7		8	] ;# Type field
			set data	[ string range $line 9		{end-2}	] ;# Data field
			set check	[ string range $line {end-1}	end	] ;# Checksum field
			set line	[ string range $line 1		{end-2}	] ;# Record without ':' and checksum

			# Handle record type (01 == EOF; 00 == normal record)
			if {$type == {01}} {
				set eof 1
				break
			} elseif {$type != {00}} {
				return 0
			}

			# Check for valid checksum
			set new_check [::IHexTools::getCheckSum $line]
			if {$new_check != $check} {
				return 0
			}

			# Check for correct value of the length field
			set len [expr "0x$len"]
			if {([string length $data] / 2) != $len} {
				return 0
			}

			# Parse and load data field
			set addr [expr "0x$addr"]
			for {set i 0; set j 1} {$i < ($len * 2)} {incr i 2; incr j 2} {
				set code($addr) [expr "0x[string range $data $i $j]"]
				incr addr
			}

			# Store highest address
			if {$addr > $highest_addr} {
				set highest_addr $addr
			}
		}

		# If there is no EOF then report that as an error
		if {!$eof} {return 0}

		# Parse CDB file
		set list_of_filenames [list $filename]
		set filenumber		0
		set filename		{}
		set level		{}
		set block		{}
		while {![eof $cdb_file]} {
			set line [split [gets $cdb_file] {:$}]

			if {[lindex $line 0] != {L}} {continue}
			if {[lindex $line 1] != {C}} {continue}

			set filename	[lindex $line 2]
			set linenumber	[lindex $line 3]
			set level	[lindex $line 4]
			set block	[lindex $line 5]
			scan		[lindex $line 6] {%x} address

			set filename [file normalize [file join [$this cget -projectPath] $filename]]
			set filenumber [lsearch -exact -ascii $list_of_filenames $filename]
			if {$filenumber == -1} {
				set filenumber [llength $list_of_filenames]
				lappend list_of_filenames $filename
			}
			set Line($address) [list $linenumber $filenumber $level $block]
			set line2PC($linenumber,$filenumber) $address
		}
		return 1
	}
	
	## Get name of loaded CDB file (C language DeBug file) generated by SDCC
	 # @return String - full filename
	public method simulator_get_cdb_filename {} {
		set filename [lindex $list_of_filenames 0]
		set filename [file rootname $filename]
		return $filename.cdb
	}

	## Clear current content of the program memory
	 # @return void
	public method unload_program {} {
		array unset line2PC
		array unset breakpoints
		
		for {set i 0} {$i < $code_size} {incr i} {
			set code($i) {}
			set Line($i) {{} {} {} {}}
		}
		for {set i $code_size} {$i <= 0xFFFF} {incr i} {
			set Line($i) {}
		}
	}

	## Import list of breakpoints (e.g. '{0 0 0 1 0 0 1 1 0}')
	 # @parm String full_filename	- Name of source code file
	 # @parm List breakpoints_list	- list of breakpoints
	 # @return void
	public method Simulator_import_breakpoints {full_filename breakpoints_list} {
		set file_number [lsearch -exact -ascii $list_of_filenames $full_filename]
		set breakpoints($file_number) {}
		set line 0
		foreach bool $breakpoints_list {
			if {$bool == 1} {
				lappend breakpoints($file_number) $line
			}
			incr line
		}
	}

	## Set MCU clock
	 # @parm Int clockkHz - clock frequency in kHz
	 # @return void
	public method setEngineClock {clockkHz} {
		set clock_kHz $clockkHz
	}

	## Get program uptime as human readable string
	 # @return String - the time (eg. '2 s  42 ms 987 us')
	public method getTime {} {
		# Initial computations
		if {!$clock_kHz} {
			set s	0
			set ms	0
			set ns	0
		} {
			set s	[expr {int($overall_time * (0.012 / $clock_kHz))}]
			set ms	[expr {int($overall_time * (12000.0 / $clock_kHz)) % 1000000}]
			set ns	[expr {int($overall_time * (12000000.0 / $clock_kHz)) % 1000}]
		}

		# Local variables
		set us [expr {($ms % 1000)}]	;# Number of micro seconds
		set ms [expr {$ms / 1000}]	;# Number of mili seconds
		set s [expr {int($s)}]		;# Number of seconds
		set h [expr {$s / 3600}]	;# Number of hours
		set s [expr {$s % 3600}]
		set m [expr {$s / 60}]		;# Number of minutes
		set s [expr {$s % 60}]

		# Adjust length of nano-seconds string
		set len [string length $ns]
		if {$len < 3} {
			set ns_s "[string repeat { } [expr {3 - $len}]]$ns"
		} {
			set ns_s $ns
		}

		# Adjust length of micro-seconds string
		set len [string length $us]
		if {$len < 3} {
			set us_s "[string repeat { } [expr {3 - $len}]]$us"
		} {
			set us_s $us
		}

		# Adjust length of mili-seconds string
		set len [string length $ms]
		if {$len < 3} {
			set ms_s "[string repeat { } [expr {3 - $len}]]$ms"
		} {
			set ms_s $ms
		}

		# Adjust seconds and minutes strings
		if {[string length $s] == 1} {
			set s_s " $s"
		} {
			set s_s $s
		}
		if {[string length $m] == 1} {
			set m_s " $m"
		} {
			set m_s $m
		}

		# Initialize resulting string
		set result {}

		# Append hours
		if {$h > 0} {
			append result
		}
		# Append minutes
		if {$m > 0 || $result != {}} {
			append result " ${m_s}m"
		}
		# Append seconds
		if {$s > 0 || $result != {}} {
			append result " ${s_s}s"
		}
		# Append mili-seconds
		if {$ms > 0 || $result != {}} {
			append result " ${ms_s}ms"
		}
		# Append micro-seconds
		if {$us > 0 || $result != {}} {
			append result " ${us_s}us"
		}
		# Append nano-seconds
		if {$ns > 0 || $result != {}} {
			append result " ${ns_s}ns"
		}

		# Done ...
		return [string trim $result]
	}


	# ----------------------------------------------------------------
	# VIRTUAL HW CONTROLLER PROCEDURES
	# ----------------------------------------------------------------

	## Perform one instruction cycle
	 # @return void
	private method instruction_cycle {} {
		set rmw_instruction 0
		set stepback_ena 1
		
		stepback_save_spec

		# Execute instruction
		if {$DEBUG} {
			$code($pc)
		} {
			if {[catch {
				$code($pc)
			} result]} then {
				if {!${::Simulator::ignore_invalid_ins}} {
					$this simulator_invalid_instruction $pc $Line($pc)
					internal_shutdown
					incr_pc 1
				}
				puts stderr $result
			}
		}
		
		# Adjust simulator action history
		stepback_save_spec_time

		# Increment program time
		set wtd_rst_flag [increment_program_time $time]

		# Interrupts control
		if {$controllers_conf(EA)} {
			# Determinate minimum interrupt priority
			set min_priority -1
			foreach int $interrupts_in_progress {
				set p [interrupt_priority $int]
				if {$p > $min_priority} {
					set min_priority $p
				}
			}

			# Iterate over list of interrupt priorities
			foreach IntName $controllers_conf(IP) {

				# Skip requests with insufficient priority
				if {[interrupt_priority $IntName] <= $min_priority} {
					continue
				}

				# Test iterrupt flag
				set vector__flag [isInterruptActive $IntName]
				# Handle the interrupt
				if {$vector__flag != 0} {
					interrupt_handler $IntName [lindex $vector__flag 0] [lindex $vector__flag 1] 0
					break
				}
			}
		}

		# Send port states to PALE (Peripheral Abstraction Layer Engine)
		set max $time
		if {!$controllers_conf(X2)} {
			set max [expr {$max * 2}]
		}
		for {set i 1} {$i < $max} {incr i} {
			$this pale_simulation_cycle $ports_previous_state
		}
		set rmw_instruction 1
		set ports_previous_state [list]
		for {set i 0; set addr 128} {$i < 5} {incr i; incr addr 16} {
			if {$feature_avaliable(p$i)} {
				lappend ports_previous_state $sfr($addr)
			} {
				lappend ports_previous_state 0
			}
		}
		$this pale_simulation_cycle $ports_previous_state

		# Analog comparator controller
		if {[$this pale_is_enabled]} {
			if {$controllers_conf(CEN)} {
				anlcmp_controller
			} {
				set anlcmp_running 0
				$this pale_SLSF $PIN(ANL0) 0
				$this pale_SLSF $PIN(ANL1) 0
			}
		}

		# UART controller
		if {$feature_avaliable(uart)} {
			if {($uart_RX_in_progress || $uart_TX_in_progress) && ${::Simulator::reverse_run_steps}} {
				stepback_reg_change S $symbol(SBUFR)
				stepback_reg_change S $symbol(SBUFT)
				stepback_reg_change S $symbol(SCON)
			}
			uart_controller $time
		}

		# Handle exteranal interrupts
		foreach int_src {0 1} {
			set ext_int 0	;# Local external interrupt flag

			# Anylize port state
			for {set i -$time} {$i < 0} {incr i} {
				# Determinate value of input pin INTX
				set intx [$this pale_RRPPV $PIN(INT${int_src}) $i]

				# Detect falling edge on INTx
				if {$controllers_conf(IT${int_src})} {
					set intx_prev $controllers_conf(INT${int_src})
					set controllers_conf(INT${int_src}) $intx

					if {$intx_prev && !$intx} {
						set ext_int 1
					}
				# Just copy inverted value from INTx to IEx
				} {
					set ext_int [expr {!$intx}]
				}
			}

			# Invoke external interrupt
			if {$ext_int} {
				if {${::Simulator::reverse_run_steps}} {
					stepback_reg_change S $symbol(TCON)
				}
				setBit $symbol(IE${int_src}) 1
			}
		}
	
		## Timers control
		 # Timers 0 and 1 are engaged by PWM
		if {$controllers_conf(PWMEN)} {
			if {!$pwm_running} {
				set pwm_running 1
				$this pale_SLSF $PIN(T1) 3
			} {
				# Make backup for affected SFR
				if {${::Simulator::reverse_run_steps}} {
					stepback_reg_change S $symbol(TL0)
					stepback_reg_change S $symbol(TH0)
					stepback_reg_change S $symbol(TL1)
				}
				
				# Timer 0 is forced into mode 2 (8-bit autoreload)
				# TL1 is incremented after each T0 overflow
				incr sfr($symbol(TL0)) $time
				while 1 {
					if {$sfr($symbol(TL0)) > 255} {
						set sfr($symbol(TL0)) [expr {$sfr($symbol(TL0)) - 256 + $sfr($symbol(TH0))}]
						# Increment TL1
						incr sfr($symbol(TL1))

						# TL1 overflow -> High PWM pulse
						if {$sfr($symbol(TL1)) > 255} {
							set sfr($symbol(TL1)) 0		;# Clear TL1
							set pwm_OCR $sfr($symbol(TH1))	;# Load OCR
							pale_WPBBL $PIN(T2) 1 -$sfr($symbol(TL0))
						}

						# OCR and TL1 are equal -> Low PWM pulse
						if {$pwm_OCR == $sfr($symbol(TL1))} {
							set time_back [expr {256 - $sfr($symbol(TL0))}]
							if {$time_back >= 0} {
								$this pale_WPBBL $PIN(T1) 0
							} {
								$this pale_WPBBL $PIN(T1) $time_back
							}
						}
					} {
						break
					}
				}

				# Synchronize affected SFR
				if {$sync_ena} {
					$this Simulator_GUI_sync S $symbol(TL0)
					$this Simulator_GUI_sync S $symbol(TH0)
					$this Simulator_GUI_sync S $symbol(TL1)
				}
			}
		
		 # Timers 0 and 1 are free
		} else {
			# Shutdown PWM controller
			if {$pwm_running} {
				set pwm_running 0
				$this pale_SLSF $PIN(T1) 0
			}

			# Make backup for TCON register
			if {($controllers_conf(TR0) || $controllers_conf(TR1)) && ${::Simulator::reverse_run_steps}} {
				stepback_reg_change S $symbol(TCON)
			}
	
			# Increment timer 0
			if {$controllers_conf(TR0)} {
				if {${::Simulator::reverse_run_steps}} {
					stepback_reg_change S $symbol(TL0)
					stepback_reg_change S $symbol(TH0)
				}
				timer_controller_01 0
				if {$sync_ena} {
					$this Simulator_GUI_sync S $symbol(TL0)
					$this Simulator_GUI_sync S $symbol(TH0)
				}
			# Shutdown timer 0
			} {
				set timer_0_running 0
			}

			# Increment timer 1
			if {$controllers_conf(TR1)} {
				if {${::Simulator::reverse_run_steps}} {
					stepback_reg_change S $symbol(TL1)
					stepback_reg_change S $symbol(TH1)
					if {$controllers_conf(T0_MOD) == 3} {
						stepback_reg_change S $symbol(TH0)
					}
				}
				timer_controller_01 1
				if {$sync_ena} {
					$this Simulator_GUI_sync S $symbol(TL1)
					$this Simulator_GUI_sync S $symbol(TH1)
					if {$controllers_conf(T0_MOD) == 3} {
						$this Simulator_GUI_sync S $symbol(TH0)
					}
				}
			# Shutdown timer 1
			} {
				set timer_1_running 0
			}

			# Keep timer 1 running while timer 0 is in mode 3
			if {$controllers_conf(T0_MOD) == 3} {
				if {${::Simulator::reverse_run_steps}} {
					stepback_reg_change S $symbol(TL1)
					stepback_reg_change S $symbol(TH1)
				}
				special_timer1_controller_T0_MOD_3
				if {$sync_ena} {
					$this Simulator_GUI_sync S $symbol(TL1)
					$this Simulator_GUI_sync S $symbol(TH1)
				}
			}
		}

		# Increment timer 2
		if {$feature_avaliable(t2)} {
			if {$controllers_conf(TR2)} {
				if {${::Simulator::reverse_run_steps}} {
					stepback_reg_change S $symbol(TL2)
					stepback_reg_change S $symbol(TH2)
					stepback_reg_change S $symbol(RCAP2L)
					stepback_reg_change S $symbol(RCAP2H)
					stepback_reg_change S $symbol(T2CON)
				}
				timer_controller_2
				if {$sync_ena} {
					$this Simulator_GUI_sync S $symbol(TL2)
					$this Simulator_GUI_sync S $symbol(TH2)
					$this Simulator_GUI_sync S $symbol(RCAP2L)
					$this Simulator_GUI_sync S $symbol(RCAP2H)
				}
			# Shutdown timer 2
			} {
				set timer_2_running 0
			}
		}

		# Manage interrupt monitor
		$this interrupt_monitor_intr_flags [simulator_get_active_intr_flags]

		# Adjust stopwatch timer
		incr run_statistics(3)
		$this stopwatch_refresh
		
		# Manage stack for stepback stack
		stepback_save_norm
		set stepback_ena 0

		# Manage PALE
		$this pale_finish_simulation_cycle
	}

	## Analog comparator controller
	 # It's safe to call this function even if there is no analog comparator implemented on the MCU
	 # @return void
	private method anlcmp_controller {} {
		set sample 0

		# Start analog comparator
		if {!$anlcmp_running} {
			$this pale_SLSF $PIN(ANL0) 4
			$this pale_SLSF $PIN(ANL1) 4

			set anlcmp_running 1
			return
		}

		## Sample inputs
		 # Debounce mode
		if {[lsearch {2 3 6} $controllers_conf(AC_MOD)] != -1} {
			incr anlcpm_db_timer $timer1_overflow
			if {$anlcpm_db_timer >= 2} {
				incr anlcpm_db_timer -$anlcpm_db_timer
				set sample 1
			}
		 # Normal mode (sample every S4)
		} {
			set sample 1
		}
		 # Sample port pins ANL0 and ANL1
		if {!$sample} {
			return
		}
		set anlcmp_output_prev $anlcmp_output
		set anlcmp_output [expr {([$this pale_RRPPV $PIN(ANL0)] - [$this pale_RRPPV $PIN(ANL1)]) > 0 ? 1 : 0}]

		# Conditionaly ionvoke analog comparator interrupt
		switch -- $controllers_conf(AC_MOD) {
			0 {	;# Negative (Low) level
				if {!$anlcmp_output} {
					write_sfr $symbol(ACSR) [expr {$sfr($symbol(ACSR)) | 0x10}]
				}
			}
			1 {	;# Positive edge
				if {!$anlcmp_output_prev && $anlcmp_output} {
					write_sfr $symbol(ACSR) [expr {$sfr($symbol(ACSR)) | 0x10}]
				}
			}
			2 {	;# Toggle with debounce
				if {$anlcmp_output_prev != $anlcmp_output} {
					write_sfr $symbol(ACSR) [expr {$sfr($symbol(ACSR)) | 0x10}]
				}
			}
			3 {	;# Positive edge with debounce
				if {!$anlcmp_output_prev && $anlcmp_output} {
					write_sfr $symbol(ACSR) [expr {$sfr($symbol(ACSR)) | 0x10}]
				}
			}
			4 {	;# Negative edge
				if {$anlcmp_output_prev && !$anlcmp_output} {
					write_sfr $symbol(ACSR) [expr {$sfr($symbol(ACSR)) | 0x10}]
				}
			}
			5 {	;# Toggle
				if {$anlcmp_output_prev != $anlcmp_output} {
					write_sfr $symbol(ACSR) [expr {$sfr($symbol(ACSR)) | 0x10}]
				}
			}
			6 {	;# Negative edge with debounce
				if {$anlcmp_output_prev && !$anlcmp_output} {
					write_sfr $symbol(ACSR) [expr {$sfr($symbol(ACSR)) | 0x10}]
				}
			}
			7 {	;# Positive (High) level
				if {$anlcmp_output} {
					write_sfr $symbol(ACSR) [expr {$sfr($symbol(ACSR)) | 0x10}]
				}
			}
		}
	}

	## Timer controller for timer 2
	 # @retrun void
	private method timer_controller_2 {} {
		set timer2_overflow 0
		set increment 0

		## Determinate counter increment
		 # Counter 16-bit
		if {$controllers_conf(CT2)} {
			# Detect 1-to-0 transition on external input
			for {set i -$time} {$i < 0} {incr i} {
				set counter_input_prev $controllers_conf(T2)
				set controllers_conf(T2) [$this pale_RRPPV $PIN(T2) $i]

				if {$counter_input_prev && !$controllers_conf(T2)} {
					incr increment
				}
			}

		 # Timer 16-bit
		} {
			set increment $time
		}

		# Programmable Clock-output
		if {$controllers_conf(T2OE)} {
			# 16-bit timer
			if {$timer_2_running && !$controllers_conf(CT2)} {
				set increment [expr {$time * 6}]
			} {
				set increment 0
			}
			if {[increment_timer2 $increment 1]} {
				while 1 {
					set sfr($symbol(TH2)) $sfr($symbol(RCAP2H))
					if {![increment_timer2 $sfr($symbol(RCAP2L)) 1]} {
						break
					}
					incr timer2_overflow
					
					# Detect 1-to-0 transition on external input
					set controllers_conf(T2) [expr {!$controllers_conf(T2)}]
					$this pale_WPBBL $PIN(T2) $controllers_conf(T2) C
				}
			}

			# External interrupt
			for {set i -$time} {$i < 0} {incr i} {
				# Detect 1-to-0 transition on external input
				set t2ex_prev $controllers_conf(T2EX)
				set controllers_conf(T2EX) [$this pale_RRPPV $PIN(T2EX) $i]

				# Invoke external interrupt
				if {$t2ex_prev && !$controllers_conf(T2EX) && $controllers_conf(EXEN2)} {
					setBit $symbol(EXF2)
				}
			}
		

		# Baud Rate Generator
		} elseif {$controllers_conf(RCLK) || $controllers_conf(TCLK)} {
			if {$controllers_conf(CT2)} {
				set increment [expr {$increment * 6}]
			}
			if {[increment_timer2 $increment 1]} {
				while 1 {
					set sfr($symbol(TH2)) $sfr($symbol(RCAP2H))
					if {![increment_timer2 $sfr($symbol(RCAP2L)) 1]} {
						break
					}
					incr timer2_overflow
				}
			}
				
			# External interrupt
			for {set i -$time} {$i < 0} {incr i} {
				# Detect 1-to-0 transition on external input
				set t2ex_prev $controllers_conf(T2EX)
				set controllers_conf(T2EX) [$this pale_RRPPV $PIN(T2EX) $i]

				# Invoke external interrupt
				if {$t2ex_prev && !$controllers_conf(T2EX) && $controllers_conf(EXEN2)} {
					setBit $symbol(EXF2)
				}
			}
			
		# 16-bit Capture
		} elseif {$controllers_conf(CPRL2) && $timer_2_running} {
			set capture_flag 0

			# Increment timer registers
			if {[increment_timer2 $increment 1]} {
				setBit $symbol(TF2) 1
			}

			# Detect 1-to-0 transition on external input
			for {set i -$time} {$i < 0} {incr i} {
				set t2ex_prev $controllers_conf(T2EX)
				set controllers_conf(T2EX) [$this pale_RRPPV $PIN(T2EX) $i]

				# Capture TL2 and TH2 to RCAP2L and RCAP2H
				if {$t2ex_prev && !$controllers_conf(T2EX) && $controllers_conf(EXEN2)} {
					setBit $symbol(EXF2)

					set sfr($symbol(RCAP2L)) $sfr($symbol(TL2))
					set sfr($symbol(RCAP2H)) $sfr($symbol(TH2))
					
				}
			}

		# 16-bit Auto-reload (DCEN == 1)
		} elseif {$controllers_conf(DCEN) && $timer_2_running} {
			set updown	[$this pale_RRPPV $PIN(T2EX) $i]

			# Overflow
			if {[increment_timer2 $increment $updown]} {
				while 1 {
					set sfr($symbol(TH2)) $sfr($symbol(RCAP2H))
					if {![increment_timer2 $sfr($symbol(RCAP2L)) $updown]} {
						break
					}
				}
				
				setBit $symbol(TF2)	1
				setBit $symbol(EXF2)	1
			}
			
			# Underflow
			if {!$updown} {
				set cur_val [expr {$sfr($symbol(TL2)) + $sfr($symbol(TH2)) << 8}]
				set min_val [expr {$sfr($symbol(RCAP2L)) + $sfr($symbol(RCAP2H)) << 8}]
				set diff [expr {$increment - ($min_val - $cur_val}]

				if {$diff > 0} {
					while 1 {
						set sfr($symbol(TL2)) $sfr($symbol(RCAP2L))
						set sfr($symbol(TH2)) $sfr($symbol(RCAP2H))

						if {![increment_timer2 $diff 0]} {
							break
						}
					}
					
					setBit $symbol(TF2)	1
					setBit $symbol(EXF2)	1
				}
			}

		# 16-bit Auto-reload (DCEN == 0)
		} elseif {$timer_2_running} {
			if {[increment_timer2 $increment 1]} {
				while 1 {
					set sfr($symbol(TH2)) $sfr($symbol(RCAP2H))
					if {![increment_timer2 $sfr($symbol(RCAP2L)) 1]} {
						break
					}
				}
				setBit $symbol(TF2)	1
			}
		}
		
		# Start the timer if it is not already started
		if {!$timer_2_running} {
			set timer_2_running 1
			return
		}
	}

	## Increment timer 2
	 # @parm Int increment_by	- value to increment by
	 # @parm Bool updown		- 1 == count up; 0 == count down
	 # @retrun Int - 1 == Owerflow; -1 == Underflow; 0 == normal
	private method increment_timer2 {increment_by updown} {
		if {$updown} {
			incr sfr($symbol(TL2)) $increment_by
		} {
			incr sfr($symbol(TL2)) -$increment_by
		}

		# Low-order byte overflow
		if {$sfr($symbol(TL2)) > 255} {
			incr sfr($symbol(TH2))
			incr sfr($symbol(TL2)) -256
			if {$sfr($symbol(TH2)) > 255} {
				set sfr($symbol(TH2)) 0
				return 1
			}
		# Low-order byte underflow
		} elseif {$sfr($symbol(TL2)) < 0} {
			incr sfr($symbol(TH2)) -1
			incr sfr($symbol(TL2)) 256
			if {$sfr($symbol(TH2)) < 0} {
				set sfr($symbol(TH2)) 255
				return -1
			}
		}

		# Normal operation
		return 0
		
	}

	## Order UART to initialize transmission cycle
	 # @return void
	private method uart_start_transmission {} {
		# Begin transmission with two instruction cycles delay
		set uart_TX_in_progress -1

		# Initialize internall transmission buffer (shift register)
		switch $controllers_conf(UART_M) {
			0 {
				set uart_TX_shift_reg [expr {$sfr($symbol(SBUFT)) + 0x100}]
			}
			1 {
				set uart_TX_shift_reg [expr {($sfr($symbol(SBUFT)) << 1) + 0x600}]
			}
			2 {
				set uart_TX_shift_reg [expr {($sfr($symbol(SBUFT)) << 1) + 0xC00 + ([getBit $symbol(TB8)] << 9)}]
			}
			3 {
				set uart_TX_shift_reg [expr {($sfr($symbol(SBUFT)) << 1) + 0xC00 + ([getBit $symbol(TB8)] << 9)}]
			}
		}

		# Set line special funtion to data transmission
		if {$controllers_conf(UART_M) == 0} {
			$this pale_SLSF $PIN(TXD) 2
			$this pale_SLSF $PIN(RXD) 1
		} {
			$this pale_SLSF $PIN(TXD) 1
		}
	}

	## UART (Universal Asynchronous Receiver Transmitter) controller
	 # @parm Int num - Number of machine cycles performed by last set of instructions
	 # @return void
	private method uart_controller {num} {
		set send_tx_shift_clock_sequence 0

		# Manage UART clock prescaler
		if {$uart_RX_in_progress || $uart_TX_in_progress} {
			if {$controllers_conf(UART_M) == 1 || $controllers_conf(UART_M) == 3} {
				incr uart_clock_prescaler $timer1_overflow
			} elseif {$controllers_conf(UART_M) == 2} {
				incr uart_clock_prescaler $num
			}
		}

		# ----------------------------------------------------------------------
		# RECEPTION PROCEDURE
		# ----------------------------------------------------------------------

		# Reception is already in progress
		if {$uart_RX_in_progress} {
			# Make backup for SBUF-R
			if {${::Simulator::reverse_run_steps}} {
				stepback_reg_change S $symbol(SBUFR)
			}

			# Mode 0
			if {!$controllers_conf(UART_M)} {
				set send_tx_shift_clock_sequence $num
				for {set i -$num} {$i <= 0} {incr i} {
					set uart_RX_shift_reg [expr {$uart_RX_shift_reg << 1}]
					incr uart_RX_shift_reg [$this pale_RRPPV $PIN(RXD) $i]

					if {!($uart_RX_shift_reg & 0x100)} {
						# Stop reception
						set uart_RX_in_progress 0
						$this pale_SLSF $PIN(RXD) 0
						$this pale_SLSF $PIN(TXD) 0

						# Set RI and SBUF
						setBit $symbol(RI) 1
						set sfr($symbol(SBUFR)) [expr {$uart_RX_shift_reg & 0x0FF}]

						break
					}
				}
			# Mode 1, 2 or 3
			} else {
				## Select RX clock source
				if {$controllers_conf(UART_M) == 2} {
					if {$controllers_conf(SMOD)} {
						incr uart_RX_clock [expr {$uart_clock_prescaler / 2}]
					} {
						incr uart_RX_clock [expr {$uart_clock_prescaler / 4}]
					}
				} {
					# Timer 2 overflow
					if {$controllers_conf(RCLK)} {
						incr uart_RX_clock $timer2_overflow
					# Timer 1 overflow
					} {
						if {$controllers_conf(SMOD)} {
							incr uart_RX_clock $timer1_overflow
						} {
							incr uart_RX_clock [expr {$uart_clock_prescaler / 2}]
						}
					}
				}

				# Prescaler overflew -> Commence 1b reception
				if {$uart_RX_clock >= 16} {
					incr uart_RX_clock -16
					set uart_RX_shift_reg [expr {$uart_RX_shift_reg << 1}]
					incr uart_RX_shift_reg [$this pale_RRPPV $PIN(RXD)]

					# Mode 1
					if {$controllers_conf(UART_M) == 1} {
						if {!($uart_RX_shift_reg & 0x200)} {
							# Stop reception
							set uart_RX_in_progress 0
							$this pale_SLSF $PIN(RXD) 0

							# Set RI and SBUF
							setBit $symbol(RB8) [expr {$uart_RX_shift_reg & 1}]
							set sfr($symbol(SBUFR)) [expr {($uart_RX_shift_reg & 0x1FE) >> 1}]

							if {
								(!$controllers_conf(RI) &&
									(!$controllers_conf(SM2) || ($uart_RX_shift_reg & 1))
								)
							} then {
								setBit $symbol(RI) 1
							# Frame error
							} else {
								$this simulator_uart_invalid_stop_bit $pc $Line($pc)
								internal_shutdown
							}

							# Frame error detection
							if {$feature_avaliable(smod0) && !($uart_RX_shift_reg & 1)} {
								set controllers_conf(FE) 1
								if {$sync_ena} {
									$this Simulator_GUI_sync S $symbol(SCON)
								}
							}
						}
					# Mode 2 or 3
					} elseif {!($uart_RX_shift_reg & 0x400)} {
						# Stop reception
						set uart_RX_in_progress 0
						$this pale_SLSF $PIN(RXD) 0

						# Set RI and SBUF
						setBit $symbol(RB8) [expr {$uart_RX_shift_reg & 2}]
						set sfr($symbol(SBUFR)) [expr {($uart_RX_shift_reg & 0x3FC) >> 2}]
						if {$controllers_conf(SM2)} {
							if {$feature_avaliable(euart)} {
								# Check for broadcast address
								if {$sfr($symbol(SBUFR)) == ($sfr($symbol(SADEN)) | $sfr($symbol(SADDR)))} {
									setBit $symbol(RI) $controllers_conf(RB8)
								# Check for the given address
								} elseif {($sfr($symbol(SBUFR)) & $sfr($symbol(SADEN))) == ($sfr($symbol(SADDR)) & $sfr($symbol(SADEN)))} {
									setBit $symbol(RI) $controllers_conf(RB8)
								}
							} {
								setBit $symbol(RI) $controllers_conf(RB8)
							}
						} {
							setBit $symbol(RI) 1
						}

						# Frame error
						if {$feature_avaliable(smod0) && !($uart_RX_shift_reg & 1)} {
							set controllers_conf(FE) 1
							if {$sync_ena} {
								$this Simulator_GUI_sync S $symbol(SCON)
							}
						}
					}
				}
			}

			# Synchronize SBUF-R
			if {$sync_ena} {
				$this Simulator_GUI_sync S $symbol(SBUFR)
			}

		# Reception may begin now ...
		} else {
			# Mode 0 - REN starts reception
			if {!$controllers_conf(UART_M)} {
				if {$controllers_conf(REN) && !$controllers_conf(RI)} {
					set uart_RX_in_progress 1
					$this pale_SLSF $PIN(TXD) 2
					$this pale_SLSF $PIN(RXD) 1

					set uart_RX_shift_reg 0x1FE
				}
			# Mode 1, 2, 3 - Start bit starts reception
			} {
				if {![$this pale_RRPPV $PIN(RXD)]} {
					set uart_RX_clock 0
					set uart_RX_in_progress 1
					$this pale_SLSF $PIN(RXD) 1

					set uart_RX_shift_reg 0x3FE
				}
			}
		}

		# ----------------------------------------------------------------------
		# TRASMISSION PROCEDURE
		# ----------------------------------------------------------------------

		# Trasmission just began, but with one machine cycle delay
		if {$uart_TX_in_progress == -2} {
			set uart_TX_in_progress 1
			incr num -1

		# Begin transmission on next instruction
		} elseif {$uart_TX_in_progress == -1} {
			set uart_TX_in_progress -2
		
		# Transmission
		} elseif {$uart_TX_in_progress == 1} {
			# Make bakup for SBUF-T
			if {${::Simulator::reverse_run_steps}} {
				stepback_reg_change S $symbol(SBUFT)
			}

			# Mode 0
			if {!$controllers_conf(UART_M)} {
				set start_pos [expr {$num * (-2)}]

				for {set i $start_pos} {$i < 0} {} {
					incr i
					set bit_to_send [expr {$uart_TX_shift_reg & 1}]
					if {!$controllers_conf(X2)} {
						$this pale_WPBBL $PIN(RXD) $bit_to_send $i
						$this pale_WPBBL $PIN(TXD) 0 $i
						incr i
						$this pale_WPBBL $PIN(RXD) $bit_to_send $i
						$this pale_WPBBL $PIN(TXD) 1 $i
					} {
						incr i
						$this pale_WPBBL $PIN(TXD) {|} [expr {int($i / 2)}]
						$this pale_WPBBL $PIN(RXD) $bit_to_send [expr {int($i / 2)}]
					}

					set uart_TX_shift_reg [expr {$uart_TX_shift_reg >> 1}]
					set sfr($symbol(SBUFT)) $uart_TX_shift_reg

					if {!($uart_TX_shift_reg & 0x1FE)} {

						# Stop transmission
						set uart_TX_in_progress 0
						$this pale_SLSF $PIN(RXD) 0
						$this pale_SLSF $PIN(TXD) 0

						# Set TI
						setBit $symbol(TI) 1

						break
					}
				}
			# Mode 1, 2, 3
			} else {
				
				## Select TX clock source
				if {$controllers_conf(UART_M) == 2} {
					if {$controllers_conf(SMOD)} {
						incr uart_TX_clock [expr {$uart_clock_prescaler / 2}]
					} {
						incr uart_TX_clock [expr {$uart_clock_prescaler / 4}]
					}
				} {
					# Timer 2 overflow
					if {$controllers_conf(TCLK)} {
						incr uart_TX_clock $timer2_overflow
					# Timer 1 overflow
					} {
						if {$controllers_conf(SMOD)} {
							incr uart_TX_clock $timer1_overflow
						} {
							incr uart_TX_clock [expr {$uart_clock_prescaler / 2}]
						}
					}
				}

				# Prescaler overflew -> Commence 1b transmission
				if {$uart_TX_clock >= 16} {
					incr uart_TX_clock -16

					$this pale_WPBBL $PIN(TXD) [expr {$uart_TX_shift_reg & 1}]
					set uart_TX_shift_reg [expr {$uart_TX_shift_reg >> 1}]
					set sfr($symbol(SBUFT)) $uart_TX_shift_reg

					if {!($uart_TX_shift_reg & 0x7FE)} {
						# Stop transmission
						set uart_TX_in_progress 0
						$this pale_SLSF $PIN(TXD) 0

						# Set TI
						setBit $symbol(TI) 1
					}
				}
			}

			# Synchronize SBUF-T
			if {$sync_ena} {
				$this Simulator_GUI_sync S $symbol(SBUFT)
			}
		}

		# Again manage UART clock prescaler
		if {$uart_clock_prescaler && $controllers_conf(UART_M)} {
			if {$controllers_conf(SMOD)} {
				set uart_clock_prescaler [expr {$uart_clock_prescaler % 2}]
			} {
				set uart_clock_prescaler [expr {$uart_clock_prescaler % 4}]
			}
		}
	}

	## Timer 1 controller which operate while timer 0 is engaged in mode 3
	 # @return void
	private method special_timer1_controller_T0_MOD_3 {} {
		set timer1_overflow 0

		set TL1 $symbol(TL1)
		set TH1 $symbol(TH1)

		# Timer 1 mode
		switch -- $controllers_conf(T1_MOD) {
			0 {	;# Mode 0 - 13 bit counter
				incr sfr($TL1) $time
				if {$sfr($TL1) > 31} {
					incr sfr($TH1)
					incr sfr($TL1) -32
					if {$sfr($TH1) > 255} {
						set sfr($TH1) 0
						set timer1_overflow 1
					}
				}
			}
			1 {	;# Mode 1 - 16 bit counter
				incr sfr($TL1) $time
				if {$sfr($TL1) > 255} {
					incr sfr($TH1)
					incr sfr($TL1) -256
					if {$sfr($TH1) > 255} {
						set sfr($TH1) 0
						set timer1_overflow 1
					}
				}
			}
			2 {	;# Mode 2 - 8 bit auto-reload counter
				incr sfr($TL1) $time
				while 1 {
					if {$sfr($TL1) > 255} {
						set sfr($TL1) [expr {$sfr($TL1) - 256 + $sfr($TH1)}]
						incr timer1_overflow
					} {
						break
					}
				}
			}
			3 {	;# Timer halted
			}
		}
	}

	## Timers controller for timers 0 and 1
	 # -- should be called after each instruction cycle
	 # @parm Int timer_num - Number of timer to hadnle ('0' or '1')
	 # @return void
	private method timer_controller_01 {timer_num} {
		set timer1_overflow	0
		set increment		0

		# Start the timer if it is not already started
		if {$timer_num == 1} {
			if {!$timer_1_running} {
				set timer_1_running 1
				return
			}
		} {
			if {!$timer_0_running} {
				set timer_0_running 1
				return
			}
		}

		# Determinate counter increment
		if {$controllers_conf(CT${timer_num})} {
			# Detect 1-to-0 transition on external input
			for {set i -$time} {$i < 0} {incr i} {
				set counter_input_prev $controllers_conf(T${timer_num})
				set controllers_conf(T${timer_num}) [$this pale_RRPPV $PIN(T${timer_num}) $i]

				if {$counter_input_prev && !$controllers_conf(T${timer_num})} {
					incr increment
				}
			}

			# Trigered counter
			if {$controllers_conf(GATE${timer_num})} {
				# Read Intx and allow increment only if INTx is 1
				if {![$this pale_RRPPV $PIN(INT${timer_num})]} {
					set increment 0
				}
			}
		} {
			set increment $time
				
			# Trigered timer
			if {$controllers_conf(GATE${timer_num})} {
				# Read Intx and allow increment only if INTx is 1
				if {![$this pale_RRPPV $PIN(INT${timer_num})]} {
					set increment 0
				}
			}
		}

		# Inrement timer 0 in mode 3 (Dual timer/counter)
		if {$controllers_conf(T0_MOD) == 3} {
			# Increment TH0
			if {$timer_num == 1} {
				incr sfr($symbol(TH0)) $time
				if {$sfr($symbol(TH0)) > 255} {
					set sfr($symbol(TH0)) [expr {$sfr($symbol(TH0)) - 256}]
					setBit $symbol(TF1) 1
				}
			# Increment TL0
			} {
				incr sfr($symbol(TL0)) $increment
				if {$sfr($symbol(TL0)) > 255} {
					set sfr($symbol(TL0)) [expr {$sfr($symbol(TL0)) - 256}]
					setBit $symbol(TF0) 1
				}
			}
			
			return
		}

		# Determinate TLx and THx addresses
		set TL_addr $symbol(TL${timer_num})
		set TH_addr $symbol(TH${timer_num})

		# Increment the timer in mode 0, 1 or 2
		switch -- $controllers_conf(T${timer_num}_MOD) {
			0 {	;# Mode 0 - 13 bit counter
			
				set TL_upper_3_bits [expr {$sfr($TL_addr) & 0xE0}]
				set sfr($TL_addr) [expr {$sfr($TL_addr) & 0x1F}]

				incr sfr($TL_addr) $increment
				if {$sfr($TL_addr) > 31} {
					incr sfr($TH_addr)
					incr sfr($TL_addr) -32
					if {$sfr($TH_addr) > 255} {
						set sfr($TH_addr) 0
						setBit $symbol(TF${timer_num}) 1
						if {$timer_num} {
							set timer1_overflow 1
						}
					}
				}
				set sfr($TL_addr) [expr {$sfr($TL_addr) | $TL_upper_3_bits}]
			}
			1 {	;# Mode 1 - 16 bit counter
				incr sfr($TL_addr) $increment
				if {$sfr($TL_addr) > 255} {
					incr sfr($TH_addr)
					incr sfr($TL_addr) -256
					if {$sfr($TH_addr) > 255} {
						set sfr($TH_addr) 0
						setBit $symbol(TF${timer_num}) 1
						if {$timer_num} {
							set timer1_overflow 1
						}
					}
				}
			}
			2 {	;# Mode 2 - 8 bit auto-reload counter
				incr sfr($TL_addr) $increment
				while 1 {
					if {$sfr($TL_addr) > 255} {
						set sfr($TL_addr) [expr {$sfr($TL_addr) - 256 + $sfr($TH_addr)}]
						setBit $symbol(TF${timer_num}) 1
						if {$timer_num} {
							incr timer1_overflow
						}
					} {
						break
					}
				}
			}
			3 {	;# Timer halted (timer 1)
			}
		}
	}

	## Data EEPROM controller
	 # @parm Int num - Number of clock cycles preformed divided by 6
	 # @return void
	private method eeprom_controller {num} {
		if {!$eeprom_size || !$eeprom_WR} {return}

		# Conditionaly abort write cycle
		if {!$controllers_conf(WRTINH) || !$controllers_conf(EEMWE)} {
			if {!${::Simulator::ignore_EEPROM_WR_abort}} {
				$this simulator_EEPROM_WR_abort $pc $Line($pc)
				internal_shutdown
			}

			# Fill incomplite bytes with random values
			set eeprom_prev_new [list]
			foreach reg $eeprom_prev {
				lappend eeprom_prev_new [lindex $reg 0] [undefined_octet]
			}
			set eeprom_prev $eeprom_prev_new
			
			simulator_cancel_write_to_eeprom
			return
		}

		set eeprom_WR_time_org [expr {int($eeprom_WR_time)}]
		set eeprom_WR_time [expr {$eeprom_WR_time + $num * (300.0 / $clock_kHz)}]

		# Write cycle complete
		if {$eeprom_WR_time > 100.0} {
			simulator_finalize_write_to_eeprom

		# Write still in progress
		} elseif {$eeprom_WR_time_org != int($eeprom_WR_time)} {
			$this simulator_WTE_prg_set [expr {int($eeprom_WR_time)}]
		}
		
	}

	## Watchdog controller
	 # @return Bool - true == MCU reseted; false == all in normal
	private method watchdog_controller {} {

		# Watchdog timer software controll
		if {!$controllers_conf(HWDT)} {

			# Reset
			if {$controllers_conf(WSWRST)} {
				set watchdog_value -$time
				set controllers_conf(WSWRST) 0
				if {${::Simulator::reverse_run_steps}} {
					stepback_reg_change S 167
				}
				set sfr(167) [expr {$sfr(167) - 2}]
				if {$sync_ena} {
					$this Simulator_GUI_sync S 167
				}
			}

			# Enable / Disable
			if {$controllers_conf(WDTEN) && !$controllers_conf(WatchDogTimer)} {
				incr watchdog_value -$time
				set controllers_conf(WatchDogTimer) 1
			}

		# Hardware control -- WDTEN is read-only (1 == running; 0 == stopped)
		} elseif {$controllers_conf(WatchDogTimer) != $controllers_conf(WDTEN)} {
			set controllers_conf(WDTEN) $controllers_conf(WatchDogTimer)
			if {${::Simulator::reverse_run_steps}} {
				stepback_reg_change S 167
			}
			if {$controllers_conf(WDTEN)} {
				set sfr(167) [expr {$sfr(167) | 1}]
			} {
				set sfr(167) [expr {$sfr(167) - 1}]
			}
			if {$sync_ena} {
				$this Simulator_GUI_sync S 167
			}
		}
	
		# Check if watchdog is enabled
		if {!$controllers_conf(WatchDogTimer)} {
			return 0
		}

		# Increment watchdog prescaler first
		set increment 0
		if {$controllers_conf(WatchDogPrescaler)} {
			incr wdt_prescaler_val $time
			while 1 {
				if {$wdt_prescaler_val >= $controllers_conf(WatchDogPrescaler)} {
					incr wdt_prescaler_val -$controllers_conf(WatchDogPrescaler)
					incr increment
				} {
					break
				}
			}
		} {
			set increment $time
		}

		# Increment watchdog time
		if {!$increment} {return 0}
		incr watchdog_value $increment
		if {$watchdog_value < 8192} {
			return 0
		}

		# Handle watchdog overflow
		incr watchdog_value -8192
		incr time -$watchdog_value
		incr time 8
		set watchdog_value 0
		master_reset -
		$this Simulator_sync_PC_etc

		# Shutdown simulator and inform user about the situation
		if {!${::Simulator::ignore_watchdog_reset}} {
			internal_shutdown
			$this simulator_watchdog_reset $pc $Line($pc)
		}

		return 1
	}

	## Resolve interrupt priority
	 # @parm String flag - Interrupt name
	 # @return Int - 0..3
	private method interrupt_priority {IntName} {
		return [lindex $controllers_conf(IP_level)		\
			[lsearch $controllers_conf(IP) $IntName]	\
		]
	}

	## Translate interrupt name to interrupt vector
	 # @parm String IntName		- Interrupt name (one of {PX0 PT0 PX1 PT1 PS PT2 PC})
	 # @return Int - Interrupt vector
	public method intr2vector {intname} {
		switch -- $intname {
			{PX0}	{return 3}
			{PT0}	{return 11}
			{PX1}	{return 19}
			{PT1}	{return 27}
			{PS}	{return 35}
			{PT2}	{return 43}
			{PC}	{return 51}
		}
	}

	## Interrupt controller -- handles interrupt requests
	 # @parm String IntName		- Interrupt name (one of {PX0 PT0 PX1 PT1 PS PT2 PC})
	 # @parm Int vector		- Interrupt vector (eg. '27' (T1 on 0x1B))
	 # @parm String flag_bit	- Name of interrupt flag
	 # @parm Bool immediately	- Invoked by user
	 # @return Bool - 0 == interrupt denied; 1 == interrupt accepted
	private method interrupt_handler {IntName vector flag_bit immediately} {

		if {!$immediately} {
			# Set interrupt_on_next if the time is too low
			if {$time == 1 && !$interrupt_on_next} {
				set interrupt_on_next 1
				return 0
			}

			# If the last instruction was RETI or any access to the IE, IP or IPH registers -> SKIP
			if {$skip_interrupt} {
				set skip_interrupt 0
				return 0
			}
		}
		
		# Adjust program run statistics
		incr run_statistics(5)

		# Set interrupt related variables
		set interrupt_on_next		0		;# Bool: Invoke interrupt on the next instruction
		lappend interrupts_in_progress	$IntName	;# List: Priority flags of interrupts which are in progress
		lappend inter_in_p_flags	$flag_bit	;# List: Interrupt flags of interrupts which are in progress

		# Adjust status bar
		simulator_Sbar "Interrupt  PC: 0x[format %X $pc]; line: [lindex $Line($pc) 0]; vector 0x[format %X $vector]  " 1 $this
		$this pale_interrupt $vector

		# Invoke LCALL to interrupt vector
		incr time 2
		if {!$immediately} {
			stepback_save_spec_subprog 1
		}
		uart_controller 2
		if {[increment_program_time 2]} {return}
		stack_push [expr {$pc & 255}]
		stack_push [expr {($pc & 65280) >> 8}]
		$this subprograms_call 2 $pc $vector
		incr run_statistics(3)

		# Unset interrupt flag
		switch -- $IntName {
			{PX0}	{	;# External 0
				if {!$controllers_conf(IT0)} {
					setBit $symbol(IE0) 0
				}
			}
			{PT0}	{	;# Timer 0
				setBit $symbol(TF0) 0
			}
			{PX1}	{	;# External 1
				if {!$controllers_conf(IT1)} {
					setBit $symbol(IE1) 0
				}
			}
			{PT1}	{	;# Timer 1
				setBit $symbol(TF1) 0
			}
			{PT2}	{	;# Timer 2
			}
			{PS}	{	;# UART
			}
			{PC}	{	;# Analog comparator
			}
		}

		# Report interrupt to interrupt monitor
		set flag {}
		switch $IntName {
			{PX0}	{set flag IE0}
			{PT0}	{set flag TF0}
			{PX1}	{set flag IE1}
			{PT1}	{set flag TF1}
			{PT2}	{
				foreach flag {TF2 EXF2} {
					if {$controllers_conf($flag)} {break}
				}
			}
			{PS}	{
				foreach flag {SPIF TI RI} {
					if {$controllers_conf($flag)} {break}
				}
			}
			{PC}	{set flag CF}
		}
		if {!$immediately} {
			$this interrupt_monitor_intr $flag
		}

		# Done ...
		set pc $vector
		return 1
	}

	## Test if the given interrupt is active (routine engaged)
	 # @parm String IntName - Interrupt name
	 # @return List - Interrupt vector or 0 in there is no interrupt active & Flag bit
	private method isInterruptActive {IntName} {
		switch -- $IntName {
			{PX0}	{	;# External 0
				if {!$controllers_conf(EX0)}	{return 0}
				if {$controllers_conf(IE0)}	{return {3 IE0}}
			}
			{PT0}	{	;# Timer 0
				if {!$controllers_conf(ET0)}	{return 0}
				if {$controllers_conf(TF0)}	{return {11 TF0}}
			}
			{PX1}	{	;# External 1
				if {!$controllers_conf(EX1)}	{return 0}
				if {$controllers_conf(IE1)}	{return {19 IE1}}
			}
			{PT1}	{	;# Timer 1
				if {!$controllers_conf(ET1)}	{return 0}
				if {$controllers_conf(TF1)}	{return {27 TF1}}
			}
			{PS}	{	;# UART & SPI
				if {!$controllers_conf(ES)}	{return 0}
				if {$feature_avaliable(uart)} {
					if {$controllers_conf(TI)} {
						return {35 TI}
					} elseif {$controllers_conf(RI)} {
						return {35 RI}
					}
				}
				if {$feature_avaliable(spi) && $controllers_conf(SPIE) && $controllers_conf(SPIF)} {
					return {35 SPIE}
				}
			}
			{PT2}	{	;# Timer 2
				if {!$controllers_conf(ET2)}	{return 0}
				if {$controllers_conf(TF2)} {
					return {43 TF2}
				} elseif {$controllers_conf(EXF2)} {
					return {43 EXF2}
				}
			}
			{PC}	{	;# Analog comparator
				if {!$controllers_conf(EC)}	{return 0}
				if {$controllers_conf(CF)}	{return {51 CF}}
			}
		}
		return 0
	}

	# ----------------------------------------------------------------
	# INSTRUCTION PROCEDURES
	# ----------------------------------------------------------------

	## Instruction: ACALL
	 # @parm Int haddr	- High part of the target address
	 # @parm Int laddr	- Low part of the target address
	 # @return void
	private method ins_acall {haddr laddr} {
		set time 2
		stepback_save_spec_subprog 2

		stack_push [expr {($pc & 255)}]
		stack_push [expr {($pc & 0xFF00) >> 8}]

		incr laddr [expr {$haddr << 8}]
		incr run_statistics(6)
		$this subprograms_call 1 $pc $laddr
		set pc $laddr
	}

	## Instruction: ADD
	 # @parm Int val	- Value to add to Acc
	 # @return void
	private method ins_add {val} {
		set time 1

		alo_add $val
		incr_pc 1

		evaluate_sfr 224
	}

	## Instruction: ADD A, addr
	 # @parm Int addr	- Direct address
	 # @return void
	private method ins_add_D {addr} {
		if {[check_address_validity D $addr]} {
			ins_add [undefined_octet]
			return
		}
		if {$addr < 128} {
			ins_add $ram($addr)
		} {
			ins_add [read_sfr $addr]
		}
	}

	## Instruction: ADD A, @Ri
	 # @parm Int addr	- Indirect address
	 # @return void
	private method ins_add_ID {addr} {
		set time 1
		incr_pc 1
		if {[check_address_validity I $addr]} {
			alo_add [undefined_octet]
		} {
			alo_add $ram($addr)
		}
		evaluate_sfr 224
	}

	## Instruction: ADDC
	 # @parm Int val	- Value to add to Acc
	 # @return void
	private method ins_addc {val} {
		set time 1

		alo_addc $val
		incr_pc 1

		evaluate_sfr 224
	}
	
	## Instruction: ADDC A, addr
	 # @parm Int addr	- Value to add to Acc
	 # @return void
	private method ins_addc_D {addr} {
		if {[check_address_validity D $addr]} {
			ins_addc [undefined_octet]
			return
		}
		if {$addr < 128} {
			ins_addc $ram($addr)
		} {
			ins_addc [read_sfr $addr]
		}
	}

	## Instruction: ADDC A, @Ri
	 # @parm Int addr	- Indirect address
	 # @return void
	private method ins_addc_ID {addr} {
		set time 1
		incr_pc 1
		if {[check_address_validity I $addr]} {
			alo_addc [undefined_octet]
			return
		} {
			alo_addc $ram($addr)
		}
	}

	## Instruction: AJMP
	 # @parm Int haddr	- High part of the target address
	 # @parm Int laddr	- Low part of the target address
	 # @return void
	private method ins_ajmp {haddr laddr} {
		set time 2
		incr laddr [expr {$haddr << 8}]
		set pc $laddr
	}

	## Instruction: ANL
	 # @parm Int addr	- Register address
	 # @parm Int val	- Operation arument
	 # @return void
	private method ins_anl {addr val} {
		set rmw_instruction 1
		set time 2
		if {[check_address_validity D $addr]} {return}
		if {$addr < 128} {
			if {${::Simulator::reverse_run_steps}} {
				stepback_reg_change I $addr
			}
			set ram($addr) [expr {$ram($addr) & $val}]
			if {$sync_ena} {
				$this Simulator_sync_reg $addr
			}
		} {
			write_sfr $addr [expr {[read_sfr $addr] & $val}]
			evaluate_sfr $addr
		}
	}

	## Instruction: ANL A
	 # @parm Int val	- Operation arument
	 # @return void
	private method ins_anl_A {val} {
		set rmw_instruction 1
		set time 1
		if {${::Simulator::reverse_run_steps}} {
			stepback_reg_change S 224
		}
		set sfr(224) [expr {$sfr(224) & $val}]
		incr_pc 1
		evaluate_sfr 224
	}

	## Instruction: ANL A, addr
	 # @parm Int addr	- Address of register containing value to add
	 # @return void
	private method ins_anl_A_D {addr} {
		set rmw_instruction 1
		if {[check_address_validity D $addr]} {return}
		if {$addr < 128} {
			ins_anl_A $ram($addr)
			if {$sync_ena} {
				$this Simulator_sync_reg $addr
			}
		} {
			ins_anl_A [read_sfr $addr]
			evaluate_sfr $addr
		}
	}

	## Instruction: ANL A, @Ri
	 # @parm Int addr	- Indirect address
	 # @return void
	private method ins_anl_A_ID {addr} {
		set rmw_instruction 1
		set time 1
		incr_pc 1
		if {[check_address_validity I $addr]} {return}
		if {${::Simulator::reverse_run_steps}} {
			stepback_reg_change S 224
		}
		set sfr(224) [expr {$sfr(224) & $ram($addr)}]
		evaluate_sfr 224
	}

	## Instruction: ANL C, bit
	 # @parm Int addr	- Bit address
	 # @return void
	private method ins_anl_C {addr} {
		set rmw_instruction 1
		set time 2
		if {[check_address_validity B $addr]} {return}
		if {![getBit $addr]} {setBit $symbol(C) 0}
		evaluate_bit $symbol(C)
	}

	## Instruction: ANL C, /bit
	 # @parm Int addr	- Bit address
	 # @return void
	private method ins_anl_C_N {addr} {
		set rmw_instruction 1
		set time 2
		if {[check_address_validity B $addr]} {
			setBit $symbol(C) [expr {rand() > 0.5}]
		} {
			if {[getBit $addr]} {setBit $symbol(C) 0}
		}
		evaluate_bit $symbol(C)
	}
	
	## Instruction: CJNE A, addr ...
	 # @parm Int addr	- 2nd value to compare
	 # @parm Int roff	- Relative offset for jump
	 # @return void
	private method ins_cjne_AD {addr roff} {
		if {[check_address_validity D $addr]} {
			set val [undefined_octet]
		} {
			if {$addr < 128} {
				set val $ram($addr)
			} {
				set val $sfr($addr)
			}
		}
		ins_cjne $sfr(224) $val $roff
	}

	## Instruction: CJNE
	 # @parm Int val0	- 1st value to compare
	 # @parm Int val1	- 2nd value to compare
	 # @parm Int roff	- Relative offset for jump
	 # @return void
	private method ins_cjne {val0 val1 roff} {
		set time 2

		if {$val0 != $val1} {
			if {$val0 < $val1} {
				setBit $symbol(C) 1
			} {
				setBit $symbol(C) 0
			}

			if {$roff > 127} {incr roff -256}
			incr_pc $roff
		}
	}

	## Instruction: CJNE @Ri, ...
	 # @parm Int addr	- Indirect address
	 # @parm Int val1	- 2nd value to compare
	 # @parm Int roff	- Relative offset for jump
	 # @return void
	private method ins_cjne_ID {addr val1 roff} {
		set time 2
		
		if {[check_address_validity I $addr]} {
			set val0 [undefined_octet]
		} {
			set val0 $ram($addr)
		}
		if {$val0 != $val1} {
			if {$val0 < $val1} {
				setBit $symbol(C) 1
			} {
				setBit $symbol(C) 0
			}

			if {$roff > 127} {incr roff -256}
			incr_pc $roff
		}
	}

	## Instruction: CLR
	 # @parm String opr	- Register name or bit address
	 # @return void
	private method ins_clr {opr} {
		set rmw_instruction 1
		set time 1
		incr_pc 1

		# Primary accumulator (Acc)
		if {$opr == {A}} {
			if {${::Simulator::reverse_run_steps}} {
				stepback_reg_change S 224
			}
			set sfr(224) 0
			evaluate_sfr 224
		# Bit PSW.C
		} elseif {$opr == {C}} {
			setBit $symbol(C) 0
			evaluate_bit $symbol(C)
		# Some bit
		} {
			if {[check_address_validity B $opr]} {return}
			setBit $opr 0
			evaluate_bit $opr
		}
	}

	## Instruction: CPL
	 # @parm String opr	- Register name or bit address
	 # @return void
	private method ins_cpl {opr} {
		set rmw_instruction 1
		set time 1
		incr_pc 1

		# Primary accumulator (Acc)
		if {$opr == {A}} {
			if {${::Simulator::reverse_run_steps}} {
				stepback_reg_change S 224
			}
			set sfr(224) [expr {$sfr(224) ^ 255}]
			evaluate_sfr 224
		# Bit PSW.C
		} elseif {$opr == {C}} {
			if {[getBit $symbol(C)]} {
				setBit $symbol(C) 0
			} {
				setBit $symbol(C) 1
			}
			evaluate_bit $symbol(C)
		# Some bit
		} {
			if {[check_address_validity B $opr]} {return}
			if {[getBit $opr]} {
				setBit $opr 0
			} {
				setBit $opr 1
			}
			evaluate_bit $opr
		}
	}

	## Instruction: DA
	 # @return void
	private method ins_da {} {
		set time 1
		incr_pc 1

		set hi [expr {($sfr(224) & 240) >> 4}]
		set lo [expr {$sfr(224) & 15}]

		if {($lo > 9) || [getBit $symbol(AC)]} {
			incr lo -10
			incr hi
			setBit $symbol(AC) 1
		} {
			setBit $symbol(AC) 0
		}

		if {($hi > 9) || [getBit $symbol(C)]} {
			incr hi -10
			setBit $symbol(C) 1
		} {
			setBit $symbol(C) 0
		}

		if {${::Simulator::reverse_run_steps}} {
			stepback_reg_change S 224
		}
		set sfr(224) [expr {($hi << 4) + $lo}]
		evaluate_sfr 224
	}
	## Instruction: DEC
	 # @parm Int addr	- Register to decrement
	 # @return void
	private method ins_dec {addr} {
		set rmw_instruction 1
		set time 1
		incr_pc 1
		incr_8b D $addr -1
	}

	## Instruction: DEC @Ri
	 # @parm Int addr	- Register to decrement (indirect address)
	 # @return void
	private method ins_dec_ID {addr} {
		set rmw_instruction 1
		set time 1
		incr_pc 1
		incr_8b I $addr -1
	}

	## Instruction: DIV
	 # @return void
	private method ins_div {} {
		set time 4
		incr_pc 1

		if {${::Simulator::reverse_run_steps}} {
			stepback_reg_change S $symbol(A)
			stepback_reg_change S $symbol(B)
		}

		setBit $symbol(C) 0
		if {$sfr($symbol(B)) == 0} {
			setBit $symbol(OV) 1
			set sfr(224) 0
			set sfr($symbol(B)) 0
		} {
			setBit $symbol(OV) 0
			set A $sfr(224)
			set sfr(224) [expr {$A / $sfr($symbol(B))}]
			set sfr($symbol(B)) [expr {$A % $sfr($symbol(B))}]
		}

		evaluate_sfr 224
		evaluate_sfr 240
	}

	## Instruction: DJNZ
	 # @parm Int addr	- Register to decrement
	 # @parm Int roff	- Relative offset for jump
	 # @return void
	private method ins_djnz {addr roff} {
		set rmw_instruction 1
		set time 2

		if {[incr_8b D $addr -1]} {return}
		if {$addr > 127} {
			if {[read_sfr $addr] != 0} {
				if {$roff > 127} {incr roff -256}
				incr_pc $roff
			}
			evaluate_sfr $addr
		} {
			if {$ram($addr) != 0} {
				if {$roff > 127} {incr roff -256}
				incr_pc $roff
			}
			if {$sync_ena} {
				$this Simulator_sync_reg $addr
			}
		}
	}

	## Instruction: INC
	 # @parm Int addr	- Register to increment
	 # @return void
	private method ins_inc {addr} {
		set rmw_instruction 1
		set time 1
		incr_pc 1
		if {[incr_8b D $addr 1]} {return}
		if {$addr > 127} {
			evaluate_sfr $addr
		} elseif {$sync_ena} {
			$this Simulator_sync_reg $addr
		}
	}
	
	## Instruction: INC @Ri
	 # @parm Int addr	- Register to increment (indirect address)
	 # @return void
	private method ins_inc_ID {addr} {
		set rmw_instruction 1
		set time 1
		incr_pc 1
		incr_8b I $addr 1
	}

	## Instruction: INC DPTR
	 # @return void
	private method ins_inc_DPTR {} {
		set rmw_instruction 1
		set time 2
		incr_pc 1

		if {${::Simulator::reverse_run_steps}} {
			stepback_reg_change S $symbol($DPL)
			stepback_reg_change S $symbol($DPH)
		}

		incr sfr($symbol($DPL))
		if {$sfr($symbol($DPL)) > 255} {
			set sfr($symbol($DPL)) 0
			incr sfr($symbol($DPH))
		}

		if {$sfr($symbol($DPH)) > 255} {
			set sfr($symbol($DPH)) 0
		}

		evaluate_sfr $symbol($DPH)
		evaluate_sfr $symbol($DPL)
	}

	## Instruction: JB
	 # @parm Int addr	- Bit to test
	 # @parm Int roff	- Relative offset for jump
	 # @return void
	private method ins_jb {addr roff} {
		set time 2
		if {[check_address_validity B $addr]} {
			set val [expr {rand() > 0.5}]
		} {
			set val [getBit $addr]
		}
		if {$val} {
			if {$roff > 127} {incr roff -256}
			incr_pc $roff
		}
	}

	## Instruction: JNB
	 # @parm Int addr	- Bit to test
	 # @parm Int roff	- Relative offset for jump
	 # @return void
	private method ins_jnb {addr roff} {
		set time 2
		if {[check_address_validity B $addr]} {
			set val [expr {rand() > 0.5}]
		} {
			set val [getBit $addr]
		}
		if {!$val} {
			if {$roff > 127} {incr roff -256}
			incr_pc $roff
		}
	}

	## Instruction: JBC
	 # @parm Int addr	-
	 # @parm Int roff	- Relative offset for jump
	 # @return void
	private method ins_jbc {addr roff} {
		set rmw_instruction 1
		set time 2
		if {[check_address_validity B $addr]} {
			set val [expr {rand() > 0.5}]
		} {
			set val [getBit $addr]
		}
		if {$val} {
			setBit $addr 0
			if {$roff > 127} {incr roff -256}
			incr_pc $roff
		}
	}

	## Instruction: JC
	 # @parm Int roff	- Relative offset for jump
	 # @return void
	private method ins_jc {roff} {
		ins_jb $symbol(C) $roff
	}

	## Instruction: JNC
	 # @parm Int roff	- Relative offset for jump
	 # @return void
	private method ins_jnc {roff} {
		ins_jnb $symbol(C) $roff
	}

	## Instruction: JZ
	 # @parm Int roff	- Relative offset for jump
	 # @return void
	private method ins_jz {roff} {
		set time 2
		if {$sfr(224) == 0} {
			if {$roff > 127} {incr roff -256}
			incr_pc $roff
		}
	}

	## Instruction: JNZ
	 # @parm Int roff	- Relative offset for jump
	 # @return void
	private method ins_jnz {roff} {
		set time 2

		if {$sfr(224) != 0} {
			if {$roff > 127} {incr roff -256}
			incr_pc $roff
		}
	}

	## Instruction: JMP
	 # @return void
	private method ins_jmp {} {
		set time 2
		set pc [expr {($sfr(224) + $sfr($symbol($DPL))) + ($sfr($symbol($DPH)) << 8)}]
		if {$pc > 65535} {
			incr pc -65536
		}
	}

	## Instruction: LCALL
	 # @parm Int haddr	- High part of the target address
	 # @parm Int laddr	- Low part of the target address
	 # @return void
	private method ins_lcall {haddr laddr} {
		set time 2
		stepback_save_spec_subprog 2

		stack_push [expr {$pc & 255}]
		stack_push [expr {($pc & 0xFF00) >> 8}]

		set target [expr {($haddr << 8) + $laddr}]
		incr run_statistics(6)
		$this subprograms_call 0 $pc $target
		set pc $target
	}

	## Instruction: LJMP
	 # @parm Int haddr	- High part of the target address
	 # @parm Int laddr	- Low part of the target address
	 # @return void
	private method ins_ljmp {haddr laddr} {
		set time 2
		set pc [expr {($haddr << 8) + $laddr}]
	}

	## Instruction: MOV
	 # @parm Int addr	- Register to set
	 # @parm Int val	- New value
	 # @return void
	private method ins_mov {addr val} {
		set time 1
		incr_pc 1
		
		if {[check_address_validity D $addr]} {return}
		if {$addr < 128} {
			if {${::Simulator::reverse_run_steps}} {
				stepback_reg_change I $addr
			}
			set ram($addr) $val
			if {$sync_ena} {
				$this Simulator_sync_reg $addr
			}
		} {
			write_sfr $addr $val
			evaluate_sfr $addr
		}
	}
	
	## Instruction: MOV
	 # @parm Int addr1	- Source register
	 # @parm Int addr0	- Target register
	 # @return void
	private method ins_mov_D {addr1 addr0} {
		set time 1
		incr_pc 1
		
		if {[check_address_validity D $addr0]} {return}
		if {[check_address_validity D $addr1]} {
			set val [undefined_octet]
		} {
			if {$addr1 < 128} {
				set val $ram($addr1)
			}  {
				set val [read_sfr $addr1]
			}
		}

		if {$addr0 < 128} {
			if {${::Simulator::reverse_run_steps}} {
				stepback_reg_change I $addr0
			}
			set ram($addr0) $val
			if {$sync_ena} {
				$this Simulator_sync_reg $addr0
			}
		} {
			write_sfr $addr0 $val
			evaluate_sfr $addr0
		}
	}

	## Instruction: MOV @Ri, addr
	 # @parm Int addr0	- Register to set (indirect addresing)
	 # @parm Int addr1	- Source register
	 # @return void
	private method ins_mov_ID2 {addr0 addr1} {
		if {[check_address_validity D $addr1]} {
			ins_mov_ID0 $addr0 [undefined_octet]
		} {
			if {$addr1 < 128} {
				ins_mov_ID0 $addr0 $ram($addr1)
			} {
				ins_mov_ID0 $addr0 [read_sfr $addr1]
			}
		}
		set time 2
	}
	
	## Instruction: MOV
	 # @parm Int addr	- Register to set
	 # @parm Int addr_id	- Address of new value (indirect addresing)
	 # @return void
	private method ins_mov_ID1 {addr addr_id} {
		set time 1
		incr_pc 1

		if {[check_address_validity D $addr]} {return}
		if {[check_address_validity I $addr_id]} {
			set val [undefined_octet]
		} {
			set val $ram($addr_id)
		}
		if {$addr < 128} {
			if {${::Simulator::reverse_run_steps}} {
				stepback_reg_change I $addr
			}
			set ram($addr) $val
			if {$sync_ena} {
				$this Simulator_sync_reg $addr
			}
		} {
			write_sfr $addr $val
			evaluate_sfr $addr
		}
	}

	## Instruction: MOV @Ri, ..
	 # @parm Int addr	- Register to set (indirect addresing)
	 # @parm Int val	- New value
	 # @return void
	private method ins_mov_ID0 {addr val} {
		set time 1
		incr_pc 1

		if {[check_address_validity I $addr]} {return}
		if {${::Simulator::reverse_run_steps}} {
			stepback_reg_change I $addr
		}
		set ram($addr) $val
		if {$sync_ena} {
			$this Simulator_sync_reg $addr
		}
	}

	## Instruction: MOV DPTR
	 # @parm Int haddr	- High part of the new value
	 # @parm Int laddr	- Low part of the new value
	 # @return void
	private method ins_mov_DPTR {hval lval} {
		set time 2

		if {${::Simulator::reverse_run_steps}} {
			stepback_reg_change S $symbol($DPL)
			stepback_reg_change S $symbol($DPH)
		}
		set sfr($symbol($DPL)) $lval
		set sfr($symbol($DPH)) $hval

		evaluate_sfr $symbol($DPH)
		evaluate_sfr $symbol($DPL)
	}

	## Instruction: MOV Rx
	 # @parm Int idx	- Register index (0..7) (target)
	 # @parm Int addr	- Register address (source)
	 # @return void
	private method ins_mov_Rx_ADDR {idx addr} {
		set time 2

		set t_addr [R $idx]
		if {[check_address_validity D $addr]} {return}
		
		if {${::Simulator::reverse_run_steps}} {
			stepback_reg_change I $t_addr
		}
		if {$addr < 128} {
			set ram($t_addr) $ram($addr)
		} {
			set ram($t_addr) [read_sfr $addr]
		}

		if {$sync_ena} {
			$this Simulator_sync_reg $t_addr
		}
	}

	## Instruction: MOV bit
	 # @parm String dest	- Destination bit or 'C'
	 # @parm String source	- Source bit or 'C'
	 # @return void
	private method ins_mov_bit {dest source} {
		set rmw_instruction 1
		set time 1
		if {$dest == {C}} {
			if {[check_address_validity B $source]} {
				set val [expr {rand() < 0.5}]
			} {
				set val [getBit $source]
			}
			if {$val} {
				setBit $symbol(C) 1
			} {
				setBit $symbol(C) 0
			}
		} {
			incr time
			if {[check_address_validity B $dest]} {return}
			if {[getBit $symbol(C)]} {
				setBit $dest 1
			} {
				setBit $dest 0
			}
		}
	}

	## Instruction: MOVC
	 # @parm String arg	- Offset register (one of {DPTR PC})
	 # @return void
	private method ins_movc {arg} {
		set time 2
		incr_pc 1

		# MOVC A, @A+DPTR
		if {$arg == {DPTR}} {
			set addr [expr {($sfr(224) + $sfr($symbol($DPL))) + ($sfr($symbol($DPH)) << 8)}]
		# MOVC A, @A+PC
		} {
			set addr $pc
			incr addr $sfr(224)
		}

		if {$addr > 65535} {
			incr addr -65356
		}

		if {${::Simulator::reverse_run_steps}} {
			stepback_reg_change S 224
		}
		if {[check_address_validity C $addr]} {
			set sfr(224) [undefined_octet]
		} elseif {$code($addr) != {}} {
			set sfr(224) $code($addr)
		} {
			set sfr(224) [undefined_octet]
		}

		evaluate_sfr 224
	}

	## Instruction: MOVX
	 # @parm String opr0	- Register name (one of {A R0 R1 DPTR})
	 # @parm String opr1	- Register name (one of {A R0 R1 DPTR})
	 # @return void
	private method ins_movx {opr0 opr1} {
		set time 2
		incr_pc 1

		if {$opr1 == {R0}} {
			set Saddr $ram([R 0])
		} elseif {$opr1 == {R1}} {
			set Saddr $ram([R 1])
		} elseif {$opr1 == {DPTR}} {
			set Saddr [expr {($sfr($symbol($DPH)) << 8) + $sfr($symbol($DPL))}]
		}

		if {$opr0 == {A}} {
			if {${::Simulator::reverse_run_steps}} {
				stepback_reg_change S 224
			}

			# Read from expanded data memory
			if {$Saddr < $eram_size && !$controllers_conf(EXTRAM)} {
				if {[check_address_validity E $Saddr]} {
					set sfr(224) [undefined_octet]
				} {
					set sfr(224) $eram($Saddr)
				}
			# Read from data EEPROM
			} elseif {$Saddr < $eeprom_size && $controllers_conf(EEMEN)} {
				if {[check_address_validity P $Saddr]} {
					set sfr(224) [undefined_octet]
				} {
					set complement_MSB 0
					foreach reg $eeprom_prev {
						if {$Saddr == [lindex $reg 0]} {
							set complement_MSB 1
							break
						}
					}
					if {$complement_MSB} {
						set sfr(224) [expr {$eeprom($Saddr) ^ 0x80}]
					} {
						set sfr(224) $eeprom($Saddr)
					}
				}
			# Read from external data memory
			} else {
				if {$feature_avaliable(xram) && [$this pale_is_enabled]} {
					for {set i -3} {$i < 0} {incr i} {
						if {!$controllers_conf(X2)} {
							$this pale_WPBBL $PIN(RD) {X} $i
							$this pale_WPBL 0 X $i
							$this pale_WPBL 2 X $i
							incr i
							$this pale_WPBBL $PIN(RD) {X} $i
							$this pale_WPBL 0 X $i
							$this pale_WPBL 2 X $i
						} {
							incr i
							$this pale_WPBL 0 X [expr {int($i / 2)}]
							$this pale_WPBL 2 X [expr {int($i / 2)}]
							$this pale_WPBBL $PIN(RD) {X} [expr {int($i / 2)}]
						}
					}
				}
				if {[check_address_validity X $Saddr]} {
					set sfr(224) [undefined_octet]
				} {
					set sfr(224) $xram($Saddr)
				}
			}

			evaluate_sfr 224
			return
		} elseif {$opr0 == {R0}} {
			set Daddr $ram([R 0])
		} elseif {$opr0 == {R1}} {
			set Daddr $ram([R 1])
		} elseif {$opr0 == {DPTR}} {
			set Daddr [expr {($sfr($symbol($DPH)) << 8) + $sfr($symbol($DPL))}]
		}

		# Write to expanded data memory
		if {$Daddr < $eram_size && !$controllers_conf(EXTRAM)} {
			if {[check_address_validity E $Daddr]} {return}
			set eram($Daddr) $sfr(224)

		# Write to data EEPROM
		} elseif {$Daddr < $eeprom_size && $controllers_conf(EEMEN)} {
			# Check if this operation is valid
			if {[check_address_validity P $Daddr]} {return}
			if {
				!$controllers_conf(EEMWE)	||
				!$controllers_conf(RDYBSY)	||
				!$controllers_conf(WRTINH)
			} then {
				if {!${::Simulator::ignore_EEPROM_WR_fail}} {
					$this simulator_EEPROM_WR_fail $pc $Line($pc)
					internal_shutdown
				}
				return
			}

			# Append value to the buffer
			set low_addr [expr {$Daddr & 0x1F}]
			set eeprom_WR_buff($low_addr) $sfr(224)

			# Synchronize with write buffer window
			::X::sync_eeprom_write_buffer $low_addr $this
			set offset [format %X [expr {$Daddr & 0xFFE0}]]
			set len [string length $offset]
			if {$len < 4} {
				set offset "[string repeat 0 [expr {4 - $len}]]$offset"
			}
			set eeprom_WR_ofs "0x$offset"
			::X::eeprom_write_buffer_set_offset $eeprom_WR_ofs $this

			# Start EEPROM programing cycle
			if {!$controllers_conf(EELD)} {
				# Write data to data EEPROM
				set eeprom_prev {}
				set addr [expr {$Daddr & 0xFFE0}]
				for {set i 0} {$i < 32} {incr i; incr addr} {
					if {$eeprom_WR_buff($i) != {}} {
						lappend eeprom_prev [list $addr $eeprom($addr)]
						stepback_reg_change P $addr
						set eeprom($addr) $eeprom_WR_buff($i)
						::X::sync_eeprom_mem_window [format %X $addr] 1 $this
					}
					set eeprom_WR_buff($i) {}
				}

				# Clear write buffer hexeditor
				::X::eeprom_write_buffer_set_offset {} $this
				::X::clear_eeprom_write_buffer $this

				# Clear flag EECON.RDYBSY (EEPROM is busy)
				$this sim_GUI_bit_set_clear 0 EECON RDYBSY
				set sfr(150) [expr {$sfr(150) & 0xFD}]
				set controllers_conf(RDYBSY) 0
				if {${::Simulator::reverse_run_steps}} {
					stepback_reg_change S 150
				}
				if {$sync_ena} {
					$this Simulator_GUI_sync S 150
				}

				# Adjust engine configuration
				set eeprom_WR_time 1
				set eeprom_WR 1
				$this simulator_GUI_invoke_write_to_eeprom
			}
			return

# $feature_avaliable(xcode)
		# Write to external data memory
		} else {
			if {$feature_avaliable(xram) && [$this pale_is_enabled]} {
				for {set i -3} {$i < 0} {incr i} {
					if {!$controllers_conf(X2)} {
						$this pale_WPBBL $PIN(WR) {X} $i
						$this pale_WPBL 0 X $i
						$this pale_WPBL 2 X $i
						incr i
						$this pale_WPBBL $PIN(WR) {X} $i
						$this pale_WPBL 0 X $i
						$this pale_WPBL 2 X $i
					} {
						incr i
						$this pale_WPBL 0 X [expr {int($i / 2)}]
						$this pale_WPBL 2 X [expr {int($i / 2)}]
						$this pale_WPBBL $PIN(WR) {X} [expr {int($i / 2)}]
					}
				}
			}

			if {[check_address_validity X $Daddr]} {return}
			set xram($Daddr) $sfr(224)
		}

		if {$sync_ena} {
			$this Simulator_XDATA_sync $Daddr
		}
	}

	## Instruction: MUL
	 # @return void
	private method ins_mul {} {
		set time 4
		incr_pc 1

		setBit $symbol(C) 0

		if {${::Simulator::reverse_run_steps}} {
			stepback_reg_change S 224
			stepback_reg_change S 240
		}

		set result [expr {$sfr(224) * $sfr(240)}]
		if {$result > 255} {
			set sfr(240) [expr {($result & 0xFF00) >> 8}]
			setBit $symbol(OV) 1
		} {
			set sfr(240) 0
			setBit $symbol(OV) 0
		}
		set sfr(224) [expr {$result & 255}]

		evaluate_sfr 224
		evaluate_sfr 240
	}

	## Instruction: NOP
	 # @return void
	private method ins_nop {} {
		set time 1
		incr_pc 1
	}

	## Instruction: ORL
	 # @parm Int addr	- Register addres
	 # @parm Int val	- Operation argument
	 # @return void
	private method ins_orl {addr val} {
		set rmw_instruction 1
		set time 1
		incr_pc 1

		if {[check_address_validity D $addr]} {return}
		if {$addr < 128} {
			if {${::Simulator::reverse_run_steps}} {
				stepback_reg_change I $addr
			}
			set ram($addr) [expr {$ram($addr) | $val}]
			if {$sync_ena} {
				$this Simulator_sync_reg $addr
			}
		} {
			write_sfr $addr [expr {[read_sfr $addr] | $val}]
			evaluate_sfr $addr
		}
	}

	## Instruction: ORL Addr0, Addr1
	 # 1st operand must be 224 !
	 # @parm Int addr0	- Register addres
	 # @parm Int addr1	- Operation argument
	 # @return void
	private method ins_orl_D {addr0 addr1} {
		set rmw_instruction 1
		if {[check_address_validity D $addr1]} {
			ins_orl $addr0 [undefined_octet]
		} elseif {$addr1 < 128} {
			ins_orl $addr0 $ram($addr1)
		} {
			ins_orl $addr0 [read_sfr $addr1]
		}
	}

	## Instruction: ORL .., @Ri
	 # @parm Int addr	- Target addres
	 # @parm Int addr_id	- Source address (indirect)
	 # @return void
	private method ins_orl_ID {addr addr_id} {
		set rmw_instruction 1
		set time 1
		incr_pc 1

		if {[check_address_validity D $addr]} {return}
		if {[check_address_validity I $addr_id]} {
			set val [undefined_octet]
		} {
			set val $ram($addr_id)
		}
		if {$addr < 128} {
			if {${::Simulator::reverse_run_steps}} {
				stepback_reg_change I $addr
			}
			set ram($addr) [expr {$ram($addr) | $val}]
			if {$sync_ena} {
				$this Simulator_sync_reg $addr
			}
		} {
			write_sfr $addr [expr {[read_sfr $addr] | $val}]
			evaluate_sfr $addr
		}
	}

	## Instruction: ORL C, /bit
	 # @parm Int addr	- Bit address
	 # @return void
	private method ins_orl_not_bit {addr} {
		set rmw_instruction 1
		set time 2

		if {[check_address_validity B $addr]} {
			setBit $symbol(C) [expr {rand() < 0.5}]
		} elseif {[getBit $symbol(C)] || ![getBit $addr]} {
			setBit $symbol(C) 1
		} {
			setBit $symbol(C) 0
		}
	}

	## Instruction: ORL C, bit
	 # @parm Int addr	- Bit address
	 # @return void
	private method ins_orl_bit {addr} {
		set time 2

		if {[check_address_validity B $addr]} {
			setBit $symbol(C) [expr {rand() < 0.5}]
		} elseif {[getBit $symbol(C)] || [getBit $addr]} {
			setBit $symbol(C) 1
		} {
			setBit $symbol(C) 0
		}
	}

	## Instruction: POP
	 # @parm Int addr	- Register address (target)
	 # @return void
	private method ins_pop {addr} {
		set time 2
		if {[check_address_validity D $addr]} {return}
		if {$addr < 128} {
			if {${::Simulator::reverse_run_steps}} {
				stepback_reg_change I $addr
			}
			set ram($addr) [stack_pop]
			if {$sync_ena} {
				$this Simulator_sync_reg $addr
			}
		} {
			write_sfr $addr [stack_pop]
			evaluate_sfr $addr
		}
	}

	## Instruction: PUSH
	 # @parm Int addr	- Register address (source)
	 # @return void
	private method ins_push {addr} {
		set time 2
		if {[check_address_validity D $addr]} {
			stack_push [undefined_octet]
		} elseif {$addr < 128} {
			stack_push $ram($addr)
		} elseif {$sync_ena} {
			stack_push $sfr($addr)
		}
	}

	## Instruction: RET
	 # @return void
	private method ins_ret {} {
		set time 2
		stepback_save_spec_subprog 4

		set pch [stack_pop]
		set pcl [stack_pop]

		set pc [expr {($pch << 8) + $pcl}]
		incr run_statistics(7)
		$this subprograms_return 0
	}

	## Instruction: RETI
	 # @return void
	private method ins_reti {} {
		set time 2

		if {[llength $inter_in_p_flags]} {
			stepback_save_spec_subprog 3
			set skip_interrupt	1
			set interrupt_on_next	0
			$this interrupt_monitor_reti	[lindex $inter_in_p_flags end]
			set interrupts_in_progress	[lreplace $interrupts_in_progress end end]
			set inter_in_p_flags		[lreplace $inter_in_p_flags end end]
			if {[llength $interrupts_in_progress]} {
				set vector [format %X [intr2vector [lindex $interrupts_in_progress end]]]
				simulator_Sbar "Interrupt at vector 0x$vector  " 1 $this
			} {
				simulator_Sbar {} 0 $this
			}
		} {
			$this simulator_invalid_reti_dlg $pc $Line($pc)
		}

		set pch [stack_pop]
		set pcl [stack_pop]

		set pc [expr {($pch << 8) + $pcl}]
		incr run_statistics(8)
		$this subprograms_return 1
	}

	## Instruction: RL
	 # @return void
	private method ins_rl {} {
		set time 1
		incr_pc 1

		if {${::Simulator::reverse_run_steps}} {
			stepback_reg_change S 224
		}
		set sfr(224) [expr {$sfr(224) << 1}]
		if {$sfr(224) > 255} {
			incr sfr(224) -255
		}

		evaluate_sfr 224
	}

	## Instruction: RLC
	 # @return void
	private method ins_rlc {} {
		set time 1
		incr_pc 1

		if {${::Simulator::reverse_run_steps}} {
			stepback_reg_change S 224
		}
		set sfr(224) [expr {$sfr(224) << 1}]

		if {[getBit $symbol(C)]} {
			incr sfr(224)
		}
		if {$sfr(224) > 255} {
			incr sfr(224) -256
			setBit $symbol(C) 1
		} {
			setBit $symbol(C) 0
		}

		evaluate_sfr 224
	}

	## Instruction: RR
	 # @return void
	private method ins_rr {} {
		set time 1
		incr_pc 1

		if {${::Simulator::reverse_run_steps}} {
			stepback_reg_change S 224
		}
		if {[expr {$sfr(224) % 2}]} {
			set C 1
		} {
			set C 0
		}

		set sfr(224) [expr {$sfr(224) / 2}]

		if {$C} {incr sfr(224) 128}

		evaluate_sfr 224
	}

	## Instruction: RRC
	 # @return void
	private method ins_rrc {} {
		set time 1
		incr_pc 1

		if {${::Simulator::reverse_run_steps}} {
			stepback_reg_change S 224
		}
		if {[expr {$sfr(224) % 2}]} {
			set C 1
		} {
			set C 0
		}

		set sfr(224) [expr {$sfr(224) / 2}]

		if {[getBit $symbol(C)]} {
			incr sfr(224) 128
		}

		if {$C} {
			setBit $symbol(C) 1
		} {
			setBit $symbol(C) 0
		}

		evaluate_sfr 224
	}

	## Instruction: SETB
	 # @parm String opr	- Bit address or 'C'
	 # @return void
	private method ins_setb {opr} {
		set rmw_instruction 1
		set time 1
		incr_pc 1

		if {$opr == {C}} {
			setBit $symbol(C) 1
		} {
			if {[check_address_validity B $opr]} {return}
			setBit $opr 1
		}
	}

	## Instruction: SJMP
	 # @parm Int roff	- Relative offset for jump
	 # @return void
	private method ins_sjmp {roff} {
		set time 2

		if {$roff > 127} {incr roff -256}
		incr_pc $roff
	}

	## Instruction: SUBB A, ...
	 # @parm Int val	- Value
	 # @return void
	private method ins_subb {val} {
		set time 1
		incr_pc 1
		alo_subb $val

		evaluate_sfr 224
	}
	
	## Instruction: SUBB A, Addr
	 # @parm Int addr	- Value
	 # @return void
	private method ins_subb_D {addr} {
		if {[check_address_validity D $addr]} {
			ins_subb [undefined_octet]
		} elseif {$addr < 128} {
			ins_subb $ram($addr)
		} {
			ins_subb $sfr($addr)
		}
	}
	
	## Instruction: SUBB A, @Ri
	 # @parm Int addr	- Indirect address
	 # @return void
	private method ins_subb_ID {addr} {
		set time 1
		incr_pc 1
		if {[check_address_validity I $addr]} {
			ins_subb [undefined_octet]
		} {
			alo_subb $ram($addr)
		}
		evaluate_sfr 224
	}

	## Instruction: SWAP
	 # @return void
	private method ins_swap {} {
		set time 1
		incr_pc 1

		set lo [expr {$sfr(224) & 15}]
		set hi [expr {($sfr(224) & 240) >> 4}]

		if {${::Simulator::reverse_run_steps}} {
			stepback_reg_change S 224
		}
		set sfr(224) [expr {($lo << 4) + $hi}]

		evaluate_sfr 224
	}

	## Instruction: XCH
	 # @parm Int addr	- Register address
	 # @return void
	private method ins_xch {addr} {
		set time 1
		incr_pc 1

		if {[check_address_validity D $addr]} {return}
		if {${::Simulator::reverse_run_steps}} {
			stepback_reg_change S 224
		}

		set A $sfr(224)
		if {$addr < 128} {
			if {${::Simulator::reverse_run_steps}} {
				stepback_reg_change I $addr
			}
			set sfr(224) $ram($addr)
			set ram($addr) $A
			if {$sync_ena} {
				$this Simulator_sync_reg $addr
			}
		} {
			set sfr(224) [read_sfr $addr]
			write_sfr $addr $A
			evaluate_sfr $addr
		}
		evaluate_sfr 224
	}

	## Instruction: XCH @Ri
	 # @parm Int addr	- Register address (indirect addressing)
	 # @return void
	private method ins_xch_ID {addr} {
		set time 1
		incr_pc 1

		if {[check_address_validity I $addr]} {return}
		if {${::Simulator::reverse_run_steps}} {
			stepback_reg_change S 224
			stepback_reg_change I $addr
		}

		set A $sfr(224)
		set sfr(224) $ram($addr)
		set ram($addr) $A

		evaluate_sfr 224
		if {$sync_ena} {
			$this Simulator_sync_reg $addr
		}
	}

	## Instruction: XCHD
	 # @parm Int addr	- Register address
	 # @return void
	private method ins_xchd {addr} {
		set time 1
		incr_pc 1

		if {[check_address_validity I $addr]} {
			set val [undefined_octet]
		} elseif {$addr < 128} {
			set val $ram($addr)
		} {
			set val $sfr($addr)
		}
		if {${::Simulator::reverse_run_steps}} {
			stepback_reg_change S 224
			if {$addr < 128} {
				stepback_reg_change I $addr
			} {
				stepback_reg_change S $addr
			}
		}

		set nibble0 [expr {$sfr(224) & 15}]
		set nibble1 [expr {$val & 15}]

		set sfr(224) [expr {($sfr(224) & 240) + $nibble1}]
		set val [expr {($val & 240) + $nibble0}]
		if {$addr < 128} {
			set ram($addr) $val
		} {
			set sfr($addr) $val
		}

		evaluate_sfr 224
		if {$sync_ena} {
			$this Simulator_sync_reg $addr
		}
	}

	## Instruction: XRL
	 # @parm Int addr	- Register address
	 # @parm Int val	- Operation argument
	 # @return void
	private method ins_xrl {addr val} {
		set rmw_instruction 1
		set time 1
		incr_pc 1

		if {[check_address_validity D $addr]} {return}
		if {$addr < 128} {
			if {${::Simulator::reverse_run_steps}} {
				stepback_reg_change I $addr
			}
			set ram($addr) [expr {$ram($addr) ^ $val}]
			if {$sync_ena} {
				$this Simulator_sync_reg $addr
			}
		} {
			write_sfr $addr [expr {[read_sfr $addr] ^ $val}]
			evaluate_sfr $addr
		}
	}

	## Instruction: XRL ..., addr
	 # @parm Int addr0	- Target register
	 # @parm Int addr1	- Source register
	 # @return void
	private method ins_xrl_D {addr0 addr1} {
		set rmw_instruction 1
		if {[check_address_validity D $addr1]} {
			ins_xrl $addr0 [undefined_octet]
		} elseif {$addr1 < 128} {
			ins_xrl $addr0 $ram($addr1)
		} {
			ins_xrl $addr0 [read_sfr $addr1]
		}
	}

	## Instruction: XRL .., @Ri
	 # @parm Int addr	- Register address
	 # @parm Int addr_id	- Indirect address
	 # @return void
	private method ins_xrl_ID {addr addr_id} {
		set rmw_instruction 1
		set time 1
		incr_pc 1

		if {[check_address_validity I $addr_id]} {
			set val [undefined_octet]
		} {
			set val $ram($addr_id)
		}
		if {$addr < 128} {
			if {${::Simulator::reverse_run_steps}} {
				stepback_reg_change I $addr
			}
			set ram($addr) [expr {$ram($addr) ^ $val}]
			if {$sync_ena} {
				$this Simulator_sync_reg $addr
			}
		} {
			write_sfr $addr [expr {[read_sfr $addr] ^ $val}]
			evaluate_sfr $addr
		}
	}

	# ----------------------------------------------------------------
	# OPCODE PROCEDURES
	# ----------------------------------------------------------------

	## ACALL
	private method 17	{} {ins_acall 0 [getLastOperand]}	;# 0x11 :: acall 0x0__
	private method 49	{} {ins_acall 1 [getLastOperand]}	;# 0x31 :: acall 0x1__
	private method 81	{} {ins_acall 2 [getLastOperand]}	;# 0x51 :: acall 0x2__
	private method 113	{} {ins_acall 3 [getLastOperand]}	;# 0x71 :: acall 0x3__
	private method 145	{} {ins_acall 4 [getLastOperand]}	;# 0x91 :: acall 0x4__
	private method 177	{} {ins_acall 5 [getLastOperand]}	;# 0xB1 :: acall 0x5__
	private method 209	{} {ins_acall 6 [getLastOperand]}	;# 0xD1 :: acall 0x6__
	private method 241	{} {ins_acall 7 [getLastOperand]}	;# 0xF1 :: acall 0x7__

	## ADD
	private method 36	{} {ins_add [getNextOperand]}		;# 0x24 :: add  A, #imm8
	private method 37	{} {ins_add_D [getNextOperand]}		;# 0x25 :: add  A, addr
	private method 38	{} {ins_add_ID $ram([R 0])}		;# 0x26 :: add  A, @R0
	private method 39	{} {ins_add_ID $ram([R 1])}		;# 0x27 :: add  A, @R1
	private method 40	{} {ins_add $ram([R 0])}		;# 0x28 :: add  A, R0
	private method 41	{} {ins_add $ram([R 1])}		;# 0x29 :: add  A, R1
	private method 42	{} {ins_add $ram([R 2])}		;# 0x2A :: add  A, R2
	private method 43	{} {ins_add $ram([R 3])}		;# 0x2B :: add  A, R3
	private method 44	{} {ins_add $ram([R 4])}		;# 0x2C :: add  A, R4
	private method 45	{} {ins_add $ram([R 5])}		;# 0x2D :: add  A, R5
	private method 46	{} {ins_add $ram([R 6])}		;# 0x2E :: add  A, R6
	private method 47	{} {ins_add $ram([R 7])}		;# 0x2F :: add  A, R7

	## ADDC
	private method 52	{} {ins_addc [getNextOperand]}		;# 0x34 :: addc  A, #imm8
	private method 53	{} {ins_addc_D [getNextOperand]}	;# 0x35 :: addc  A, addr
	private method 54	{} {ins_addc_ID $ram([R 0])}		;# 0x36 :: addc  A, @R0
	private method 55	{} {ins_addc_ID $ram([R 1])}		;# 0x37 :: addc  A, @R1
	private method 56	{} {ins_addc $ram([R 0])}		;# 0x38 :: addc  A, R0
	private method 57	{} {ins_addc $ram([R 1])}		;# 0x39 :: addc  A, R1
	private method 58	{} {ins_addc $ram([R 2])}		;# 0x3A :: addc  A, R2
	private method 59	{} {ins_addc $ram([R 3])}		;# 0x3B :: addc  A, R3
	private method 60	{} {ins_addc $ram([R 4])}		;# 0x3C :: addc  A, R4
	private method 61	{} {ins_addc $ram([R 5])}		;# 0x3D :: addc  A, R5
	private method 62	{} {ins_addc $ram([R 6])}		;# 0x3E :: addc  A, R6
	private method 63	{} {ins_addc $ram([R 7])}		;# 0x3F :: addc  A, R7

	## AJMP
	private method 1	{} {ins_ajmp 0 [getNextOperand]}	;# 0x01 :: ajmp 0x0__
	private method 33	{} {ins_ajmp 1 [getNextOperand]}	;# 0x21 :: ajmp 0x1__
	private method 65	{} {ins_ajmp 2 [getNextOperand]}	;# 0x41 :: ajmp 0x2__
	private method 97	{} {ins_ajmp 3 [getNextOperand]}	;# 0x61 :: ajmp 0x3__
	private method 129	{} {ins_ajmp 4 [getNextOperand]}	;# 0x81 :: ajmp 0x4__
	private method 161	{} {ins_ajmp 5 [getNextOperand]}	;# 0xA1 :: ajmp 0x5__
	private method 193	{} {ins_ajmp 6 [getNextOperand]}	;# 0xC1 :: ajmp 0x6__
	private method 225	{} {ins_ajmp 7 [getNextOperand]}	;# 0xE1 :: ajmp 0x7__

	## ANL
	private method 82	{} {ins_anl [getLastOperand] $sfr(224); incr time -1}	;# 0x52 :: anl  addr, A
	private method 83	{} {ins_anl [getNextOperand] [getLastOperand]}		;# 0x53 :: anl  addr, #imm8
	private method 84	{} {ins_anl_A [getNextOperand]}				;# 0x54 :: anl  A, #imm8
	private method 85	{} {ins_anl_A_D [getNextOperand]}			;# 0x55 :: anl  A, addr
	private method 86	{} {ins_anl_A_ID $ram([R 0])}				;# 0x56 :: anl  A, @R0
	private method 87	{} {ins_anl_A_ID $ram([R 1])}				;# 0x57 :: anl  A, @R1
	private method 88	{} {ins_anl_A $ram([R 0])}				;# 0x58 :: anl  A, R0
	private method 89	{} {ins_anl_A $ram([R 1])}				;# 0x59 :: anl  A, R1
	private method 90	{} {ins_anl_A $ram([R 2])}				;# 0x5A :: anl  A, R2
	private method 91	{} {ins_anl_A $ram([R 3])}				;# 0x5B :: anl  A, R3
	private method 92	{} {ins_anl_A $ram([R 4])}				;# 0x5C :: anl  A, R4
	private method 93	{} {ins_anl_A $ram([R 5])}				;# 0x5D :: anl  A, R5
	private method 94	{} {ins_anl_A $ram([R 6])}				;# 0x5E :: anl  A, R6
	private method 95	{} {ins_anl_A $ram([R 7])}				;# 0x5F :: anl  A, R7
	private method 130	{} {ins_anl_C [getLastOperand]}				;# 0x82 :: anl  C, Baddr
	private method 176	{} {ins_anl_C_N [getLastOperand]}			;# 0xB0 :: anl  C, /Baddr


	## CJNE
	private method 180	{} {						;# 0xB4 :: cjne  A, #imm8, Roff
		ins_cjne $sfr(224) [getNextOperand] [getLastOperand]
	}
	private method 181	{} {						;# 0xB5 :: cjne  A, addr, Roff
		ins_cjne_AD [getNextOperand] [getLastOperand]
	}
	private method 182	{} {						;# 0xB6 :: cjne  @R0, #imm8, Roff
		ins_cjne_ID $ram([R 0]) [getNextOperand] [getLastOperand]
	}
	private method 183	{} {						;# 0xB7 :: cjne  @R1, #imm8, Roff
		ins_cjne_ID $ram([R 1]) [getNextOperand] [getLastOperand]
	}
	private method 184	{} {						;# 0xB8 :: cjne  R0, #imm8, Roff
		ins_cjne $ram([R 0]) [getNextOperand] [getLastOperand]
	}
	private method 185	{} {						;# 0xB9 :: cjne  R1, #imm8, Roff
		ins_cjne $ram([R 1]) [getNextOperand] [getLastOperand]
	}
	private method 186	{} {						;# 0xBA :: cjne  R2, #imm8, Roff
		ins_cjne $ram([R 2]) [getNextOperand] [getLastOperand]
	}
	private method 187	{} {						;# 0xBB :: cjne  R3, #imm8, Roff
		ins_cjne $ram([R 3]) [getNextOperand] [getLastOperand]
	}
	private method 188	{} {						;# 0xBC :: cjne  R4, #imm8, Roff
		ins_cjne $ram([R 4]) [getNextOperand] [getLastOperand]
	}
	private method 189	{} {						;# 0xBD :: cjne  R5, #imm8, Roff
		ins_cjne $ram([R 5]) [getNextOperand] [getLastOperand]
	}
	private method 190	{} {						;# 0xBE :: cjne  R6, #imm8, Roff
		ins_cjne $ram([R 6]) [getNextOperand] [getLastOperand]
	}
	private method 191	{} {						;# 0xBF :: cjne  R7, #imm8, Roff
		ins_cjne $ram([R 7]) [getNextOperand] [getLastOperand]
	}

	## CLR
	private method 228	{} {ins_clr A}			;# 0xE4 :: clr A
	private method 195	{} {ins_clr C}			;# 0xC3 :: clr C
	private method 194	{} {ins_clr [getNextOperand]}	;# 0xC2 :: clr Baddr

	## CPL
	private method 244	{} {ins_cpl A}			;# 0xF4 :: cpl A
	private method 179	{} {ins_cpl C}			;# 0xC3 :: cpl C
	private method 178	{} {ins_cpl [getNextOperand]}	;# 0xC2 :: cpl Baddr

	## DA
	private method 212	{} {ins_da}	;# 0xD4 :: da A

	## DEC
	private method 20	{} {ins_dec 224}			;# 0x14 :: dec A
	private method 21	{} {ins_dec [getNextOperand]}		;# 0x15 :: dec addr
	private method 22	{} {ins_dec_ID $ram([R 0])}		;# 0x16 :: dec @R0
	private method 23	{} {ins_dec_ID $ram([R 1])}		;# 0x17 :: dec @R1
	private method 24	{} {ins_dec [R 0]}			;# 0x18 :: dec R0
	private method 25	{} {ins_dec [R 1]}			;# 0x19 :: dec R1
	private method 26	{} {ins_dec [R 2]}			;# 0x1A :: dec R2
	private method 27	{} {ins_dec [R 3]}			;# 0x1B :: dec R3
	private method 28	{} {ins_dec [R 4]}			;# 0x1C :: dec R4
	private method 29	{} {ins_dec [R 5]}			;# 0x1D :: dec R5
	private method 30	{} {ins_dec [R 6]}			;# 0x1E :: dec R6
	private method 31	{} {ins_dec [R 7]}			;# 0x1F :: dec R7

	## DIV
	private method 132	{} {ins_div}	;# 0x84 :: div AB

	## DJNZ
	private method 213	{} {ins_djnz [getNextOperand] [getLastOperand]}	;# 0xD5 :: djnz  addr, Roff
	private method 216	{} {ins_djnz [R 0] [getLastOperand]}		;# 0xD8 :: djnz  R0, Roff
	private method 217	{} {ins_djnz [R 1] [getLastOperand]}		;# 0xD9 :: djnz  R1, Roff
	private method 218	{} {ins_djnz [R 2] [getLastOperand]}		;# 0xDA :: djnz  R2, Roff
	private method 219	{} {ins_djnz [R 3] [getLastOperand]}		;# 0xDB :: djnz  R3, Roff
	private method 220	{} {ins_djnz [R 4] [getLastOperand]}		;# 0xDC :: djnz  R4, Roff
	private method 221	{} {ins_djnz [R 5] [getLastOperand]}		;# 0xDD :: djnz  R5, Roff
	private method 222	{} {ins_djnz [R 6] [getLastOperand]}		;# 0xDE :: djnz  R6, Roff
	private method 223	{} {ins_djnz [R 7] [getLastOperand]}		;# 0xDF :: djnz  R7, Roff

	## INC
	private method 4	{} {ins_inc 224}		;# 0x04 :: inc A
	private method 5	{} {ins_inc [getNextOperand]}	;# 0x05 :: inc addr
	private method 6	{} {ins_inc_ID $ram([R 0])}	;# 0x06 :: inc @R0
	private method 7	{} {ins_inc_ID $ram([R 1])}	;# 0x07 :: inc @R1
	private method 8	{} {ins_inc [R 0]}		;# 0x08 :: inc R0
	private method 9	{} {ins_inc [R 1]}		;# 0x09 :: inc R1
	private method 10	{} {ins_inc [R 2]}		;# 0x0A :: inc R2
	private method 11	{} {ins_inc [R 3]}		;# 0x0B :: inc R3
	private method 12	{} {ins_inc [R 4]}		;# 0x0C :: inc R4
	private method 13	{} {ins_inc [R 5]}		;# 0x0D :: inc R5
	private method 14	{} {ins_inc [R 6]}		;# 0x0E :: inc R6
	private method 15	{} {ins_inc [R 7]}		;# 0x0F :: inc R7
	private method 163	{} {ins_inc_DPTR}		;# 0xA3 :: inc DPTR

	## JB
	private method 32	{} {ins_jb [getNextOperand] [getLastOperand]}	;# 0x20 :: jb  Baddr, Roff

	## JNB
	private method 48	{} {ins_jnb [getNextOperand] [getLastOperand]}	;# 0x30 :: jnb  Baddr, Roff

	## JBC
	private method 16	{} {ins_jbc [getNextOperand] [getLastOperand]}	;# 0x10 :: jbc  Baddr, Roff

	## JC
	private method 64	{} {ins_jc [getLastOperand]}	;# 0x40 :: jc Roff

	## JNC
	private method 80	{} {ins_jnc [getLastOperand]}	;# 0x50 :: jnc Roff

	## JZ
	private method 96	{} {ins_jz [getLastOperand]}	;# 0x60 :: jz Roff

	## JNZ
	private method 112	{} {ins_jnz [getLastOperand]}	;# 0x70 :: jnz Roof

	## JMP
	private method 115	{} {ins_jmp}			;# 0x79 :: jmp @A+DPTR

	## LCALL
	private method 18	{} {ins_lcall [getNextOperand] [getLastOperand]}	;# 0x12 :: lcall Paddr16

	## LJMP
	private method 2	{} {ins_ljmp [getNextOperand] [getNextOperand]}		;# 0x02 :: ljmp Paddr16

	## MOV
	private method 116	{} {ins_mov 224 [getNextOperand]}		;# 0x74 :: mov  A, #imm8
	private method 229	{} {ins_mov_D [getNextOperand] 224}		;# 0xE5 :: mov  A, addr
	private method 230	{} {ins_mov_ID1 224 $ram([R 0])}		;# 0xE6 :: mov  A, @R0
	private method 231	{} {ins_mov_ID1 224 $ram([R 1])}		;# 0xE7 :: mov  A, @R1
	private method 232	{} {ins_mov 224 $ram([R 0])}			;# 0xE8 :: mov  A, R0
	private method 233	{} {ins_mov 224 $ram([R 1])}			;# 0xE9 :: mov  A, R1
	private method 234	{} {ins_mov 224 $ram([R 2])}			;# 0xEA :: mov  A, R2
	private method 235	{} {ins_mov 224 $ram([R 3])}			;# 0xEB :: mov  A, R3
	private method 236	{} {ins_mov 224 $ram([R 4])}			;# 0xEC :: mov  A, R4
	private method 237	{} {ins_mov 224 $ram([R 5])}			;# 0xED :: mov  A, R5
	private method 238	{} {ins_mov 224 $ram([R 6])}			;# 0xEE :: mov  A, R6
	private method 239	{} {ins_mov 224 $ram([R 7])}			;# 0xEF :: mov  A, R7
	private method 245	{} {ins_mov [getNextOperand] $sfr(224)}		;# 0xF5 :: mov  addr, A
	private method 117	{} {						;# 0x75 :: mov  addr, #imm8
		ins_mov [getNextOperand] [getNextOperand]
		incr time
	}
	private method 133	{} {						;# 0x85 :: mov  addr, addr
		ins_mov_D [getNextOperand] [getNextOperand]
		incr time
	}
	private method 134	{} {						;# 0x86 :: mov  addr, @R0
		ins_mov_ID1 [getNextOperand] $ram([R 0])
		incr time
	}
	private method 135	{} {						;# 0x87 :: mov  addr, @R1
		ins_mov_ID1 [getNextOperand] $ram([R 1])
		incr time
	}
	private method 136	{} {						;# 0x88 :: mov  addr, R0
		ins_mov [getNextOperand] $ram([R 0])
		incr time
	}
	private method 137	{} {						;# 0x89 :: mov  addr, R1
		ins_mov [getNextOperand] $ram([R 1])
		incr time
	}
	private method 138	{} {						;# 0x8A :: mov  addr, R2
		ins_mov [getNextOperand] $ram([R 2])
		incr time
	}
	private method 139	{} {						;# 0x8B :: mov  addr, R3
		ins_mov [getNextOperand] $ram([R 3])
		incr time
	}
	private method 140	{} {						;# 0x8C :: mov  addr, R4
		ins_mov [getNextOperand] $ram([R 4])
		incr time
	}
	private method 141	{} {						;# 0x8D :: mov  addr, R5
		ins_mov [getNextOperand] $ram([R 5])
		incr time
	}
	private method 142	{} {						;# 0x8E :: mov  addr, R6
		ins_mov [getNextOperand] $ram([R 6])
		incr time
	}
	private method 143	{} {						;# 0x8F :: mov  addr, R7
		ins_mov [getNextOperand] $ram([R 7])
		incr time
	}
	private method 246	{} {ins_mov_ID0 $ram([R 0]) $sfr(224)}		;# 0xF6 :: mov  @R0, A
	private method 247	{} {ins_mov_ID0 $ram([R 1]) $sfr(224)}		;# 0xF7 :: mov  @R1, A
	private method 118	{} {ins_mov_ID0 $ram([R 0]) [getNextOperand]}	;# 0x76 :: mov  @R0, #imm8
	private method 119	{} {ins_mov_ID0 $ram([R 1]) [getNextOperand]}	;# 0x77 :: mov  @R1, #imm8
	private method 166	{} {ins_mov_ID2 $ram([R 0]) [getNextOperand]}	;# 0xA6 :: mov  @R0, addr
	private method 167	{} {ins_mov_ID2 $ram([R 1]) [getNextOperand]}	;# 0xA7 :: mov  @R1, addr
	private method 248	{} {ins_mov [R 0] $sfr(224)}			;# 0xF8 :: mov  R0, A
	private method 249	{} {ins_mov [R 1] $sfr(224)}			;# 0xF9 :: mov  R1, A
	private method 250	{} {ins_mov [R 2] $sfr(224)}			;# 0xFA :: mov  R2, A
	private method 251	{} {ins_mov [R 3] $sfr(224)}			;# 0xFB :: mov  R3, A
	private method 252	{} {ins_mov [R 4] $sfr(224)}			;# 0xFC :: mov  R4, A
	private method 253	{} {ins_mov [R 5] $sfr(224)}			;# 0xFD :: mov  R5, A
	private method 254	{} {ins_mov [R 6] $sfr(224)}			;# 0xFE :: mov  R6, A
	private method 255	{} {ins_mov [R 7] $sfr(224)}			;# 0xFF :: mov  R7, A
	private method 120	{} {ins_mov [R 0] [getNextOperand]}		;# 0x78 :: mov  R0, #imm8
	private method 121	{} {ins_mov [R 1] [getNextOperand]}		;# 0x79 :: mov  R1, #imm8
	private method 122	{} {ins_mov [R 2] [getNextOperand]}		;# 0x7A :: mov  R2, #imm8
	private method 123	{} {ins_mov [R 3] [getNextOperand]}		;# 0x7B :: mov  R3, #imm8
	private method 124	{} {ins_mov [R 4] [getNextOperand]}		;# 0x7C :: mov  R4, #imm8
	private method 125	{} {ins_mov [R 5] [getNextOperand]}		;# 0x7D :: mov  R5, #imm8
	private method 126	{} {ins_mov [R 6] [getNextOperand]}		;# 0x7E :: mov  R6, #imm8
	private method 127	{} {ins_mov [R 7] [getNextOperand]}		;# 0x7F :: mov  R7, #imm8
	private method 168	{} {ins_mov_Rx_ADDR 0 [getLastOperand]}		;# 0xA8 :: mov  R0, addr
	private method 169	{} {ins_mov_Rx_ADDR 1 [getLastOperand]}		;# 0xA9 :: mov  R1, addr
	private method 170	{} {ins_mov_Rx_ADDR 2 [getLastOperand]}		;# 0xAA :: mov  R2, addr
	private method 171	{} {ins_mov_Rx_ADDR 3 [getLastOperand]}		;# 0xAB :: mov  R3, addr
	private method 172	{} {ins_mov_Rx_ADDR 4 [getLastOperand]}		;# 0xAC :: mov  R4, addr
	private method 173	{} {ins_mov_Rx_ADDR 5 [getLastOperand]}		;# 0xAD :: mov  R5, addr
	private method 174	{} {ins_mov_Rx_ADDR 6 [getLastOperand]}		;# 0xAE :: mov  R6, addr
	private method 175	{} {ins_mov_Rx_ADDR 7 [getLastOperand]}		;# 0xAF :: mov  R7, addr
	private method 144	{} {ins_mov_DPTR [getNextOperand] [getLastOperand]}	;# 0x90 :: mov  DPTR, #imm16
	private method 146	{} {ins_mov_bit [getLastOperand] {C}}		;# 0x92 :: mov  Baddr, C
	private method 162	{} {ins_mov_bit {C} [getLastOperand]}		;# 0xA2 :: mov  C, Baddr

	## MOVC
	private method 147	{} {ins_movc {DPTR}}		;# 0x93 :: movc  A, @A+DPTR
	private method 131	{} {ins_movc {PC}}		;# 0x93 :: movc  A, @A+PC

	## MOVX
	private method 226	{} {ins_movx {A} {R0}	}	;# 0xE2 :: movx  A, @R0
	private method 227	{} {ins_movx {A} {R1}	}	;# 0xE3 :: movx  A, @R1
	private method 224	{} {ins_movx {A} {DPTR}	}	;# 0xE0 :: movx  A, @DPTR
	private method 242	{} {ins_movx {R0} {A}	}	;# 0xF2 :: movx  @R0, A
	private method 243	{} {ins_movx {R1} {A}	}	;# 0xF3 :: movx  @R1, A
	private method 240	{} {ins_movx {DPTR} {A}	}	;# 0xF0 :: movx  @DPTR, A

	# MUL
	private method 164	{} {ins_mul}			;# 0xA4 :: mul AB

	## NOP
	private method 0	{} {ins_nop}			;# 0x00 :: nop

	## ORL
	private method 66	{} {ins_orl [getNextOperand] $sfr(224)}	;# 0x42 :: orl  addr, A
	private method 67	{} {ins_orl [getNextOperand] [getNextOperand]; incr time}	;# 0x43 :: orl  addr, #imm8
	private method 68	{} {ins_orl 224 [getNextOperand]}	;# 0x44 :: orl  A, #imm8
	private method 69	{} {ins_orl_D 224 [getNextOperand]}	;# 0x45 :: orl  A, addr
	private method 70	{} {ins_orl_ID 224 $ram([R 0])}		;# 0x46 :: orl  A, @R0
	private method 71	{} {ins_orl_ID 224 $ram([R 1])}		;# 0x47 :: orl  A, @R1
	private method 72	{} {ins_orl 224 $ram([R 0])}		;# 0x48 :: orl  A, R0
	private method 73	{} {ins_orl 224 $ram([R 1])}		;# 0x49 :: orl  A, R1
	private method 74	{} {ins_orl 224 $ram([R 2])}		;# 0x4A :: orl  A, R2
	private method 75	{} {ins_orl 224 $ram([R 3])}		;# 0x4B :: orl  A, R3
	private method 76	{} {ins_orl 224 $ram([R 4])}		;# 0x4C :: orl  A, R4
	private method 77	{} {ins_orl 224 $ram([R 5])}		;# 0x4D :: orl  A, R5
	private method 78	{} {ins_orl 224 $ram([R 6])}		;# 0x4E :: orl  A, R6
	private method 79	{} {ins_orl 224 $ram([R 7])}		;# 0x4F :: orl  A, R7
	private method 114	{} {ins_orl_bit [getLastOperand]}	;# 0x72 :: orl  C, Baddr
	private method 160	{} {ins_orl_not_bit [getLastOperand]}	;# 0xA0 :: orl  C, /Baddr

	## POP
	private method 208	{} {ins_pop [getLastOperand]}	;# 0xD0 :: pop addr

	## PUSH
	private method 192	{} {ins_push [getLastOperand]}	;# 0xC0 :: push addr

	## RET
	private method 34	{} {ins_ret}			;# 0x22 :: ret

	## RETI
	private method 50	{} {ins_reti}			;# 0x32 :: reti

	## RL
	private method 35	{} {ins_rl}			;# 0x23 :: rl A

	## RR
	private method 3	{} {ins_rr}			;# 0x03 :: rr A

	## RLC
	private method 51	{} {ins_rlc}			;# 0x33 :: rlc A

	## RRC
	private method 19	{} {ins_rrc}			;# 0x13 :: rrc A

	## SETB
	private method 211	{} {ins_setb {C}}		;# 0xD3 :: setb C
	private method 210	{} {ins_setb [getNextOperand]}	;# 0xD2 :: setb Baddr

	## SJMP
	private method 128	{} {ins_sjmp [getLastOperand]}	;# 0x80 :: sjmp Roff

	## SUBB
	private method 148	{} {ins_subb [getNextOperand]}		;# 0x94 :: subb  A, #imm8
	private method 149	{} {ins_subb_D [getNextOperand]}	;# 0x95 :: subb  A, addr
	private method 150	{} {ins_subb_ID $ram([R 0])}		;# 0x96 :: subb  A, @R0
	private method 151	{} {ins_subb_ID $ram([R 1])}		;# 0x97 :: subb  A, @R1
	private method 152	{} {ins_subb $ram([R 0])}		;# 0x98 :: subb  A, R0
	private method 153	{} {ins_subb $ram([R 1])}		;# 0x99 :: subb  A, R1
	private method 154	{} {ins_subb $ram([R 2])}		;# 0x9A :: subb  A, R2
	private method 155	{} {ins_subb $ram([R 3])}		;# 0x9B :: subb  A, R3
	private method 156	{} {ins_subb $ram([R 4])}		;# 0x9C :: subb  A, R4
	private method 157	{} {ins_subb $ram([R 5])}		;# 0x9D :: subb  A, R5
	private method 158	{} {ins_subb $ram([R 6])}		;# 0x9E :: subb  A, R6
	private method 159	{} {ins_subb $ram([R 7])}		;# 0x9F :: subb  A, R7

	## SWAP
	private method 196	{} {ins_swap}			;# 0xC4 :: swap A

	## XCH
	private method 197	{} {ins_xch [getNextOperand]}	;# 0xC5 :: xch  A, addr
	private method 198	{} {ins_xch_ID $ram([R 0])}	;# 0xC6 :: xch  A, @R0
	private method 199	{} {ins_xch_ID $ram([R 1])}	;# 0xC7 :: xch  A, @R1
	private method 200	{} {ins_xch [R 0]}		;# 0xC8 :: xch  A, R0
	private method 201	{} {ins_xch [R 1]}		;# 0xC9 :: xch  A, R1
	private method 202	{} {ins_xch [R 2]}		;# 0xCA :: xch  A, R2
	private method 203	{} {ins_xch [R 3]}		;# 0xCB :: xch  A, R3
	private method 204	{} {ins_xch [R 4]}		;# 0xCC :: xch  A, R4
	private method 205	{} {ins_xch [R 5]}		;# 0xCD :: xch  A, R5
	private method 206	{} {ins_xch [R 6]}		;# 0xCE :: xch  A, R6
	private method 207	{} {ins_xch [R 7]}		;# 0xCF :: xch  A, R7

	## XCHD
	private method 214	{} {ins_xchd $ram([R 0])}	;# 0xD6 :: xchd  A, @R0
	private method 215	{} {ins_xchd $ram([R 1])}	;# 0xD6 :: xchd  A, @R1

	## XRL
	private method 98	{} {ins_xrl [getNextOperand] $sfr(224)}		;# 0x62 :: xrl  addr, A
	private method 99	{} {ins_xrl [getNextOperand] [getNextOperand];incr time};# 0x63 :: xrl  addr, #imm8
	private method 100	{} {ins_xrl 224 [getNextOperand]}		;# 0x64 :: xrl  A, #imm8
	private method 101	{} {ins_xrl_D 224 [getNextOperand]}		;# 0x64 :: xrl  A, addr
	private method 102	{} {ins_xrl_ID 224 $ram([R 0])}			;# 0x66 :: xrl  A, @R1
	private method 103	{} {ins_xrl_ID 224 $ram([R 1])}			;# 0x67 :: xrl  A, @R1
	private method 104	{} {ins_xrl 224 $ram([R 0])}			;# 0x68 :: xrl  A, R0
	private method 105	{} {ins_xrl 224 $ram([R 1])}			;# 0x69 :: xrl  A, R1
	private method 106	{} {ins_xrl 224 $ram([R 2])}			;# 0x6A :: xrl  A, R2
	private method 107	{} {ins_xrl 224 $ram([R 3])}			;# 0x6B :: xrl  A, R3
	private method 108	{} {ins_xrl 224 $ram([R 4])}			;# 0x6C :: xrl  A, R4
	private method 109	{} {ins_xrl 224 $ram([R 5])}			;# 0x6D :: xrl  A, R5
	private method 110	{} {ins_xrl 224 $ram([R 6])}			;# 0x6E :: xrl  A, R6
	private method 111	{} {ins_xrl 224 $ram([R 7])}			;# 0x6F :: xrl  A, R7


	# ----------------------------------------------------------------
	# MCU CONFIGURATION RELATED PROCEDURES
	# ----------------------------------------------------------------

	## Increment program overall time
	 # This function also manages watchdog and data EEPROM
	 # @parm Int num - Number of instruction cycles
	 # @return Bool - 1 == device reseted; 0 == normal
	private method increment_program_time {num} {
		# Increment counter of instruction cycles
		incr overall_instructions $num
		incr run_statistics(2) $num

		# Increment overall time
		if {$controllers_conf(X2)} {
			set num [expr {$num / 2.0}]
		}
		set overall_time [expr {$overall_time + $num}]
		incr run_statistics(1) [expr {int($num * 12)}]
		incr run_statistics(0) [expr {int($num * (12000000.0 / $clock_kHz))}]
		
		eeprom_controller $num
		return [watchdog_controller]
	}
		
	## Adjust configuration acording to new value of the given bit
	 # @parm Int addr - Bit address
	 # @return void
	private method evaluate_bit {addr} {
		if {$addr < 128} {
			if {$sync_ena} {
				$this Simulator_sync_reg [getRegOfBit $addr]
			}
		} {
			evaluate_sfr [getRegOfBit $addr]
		}
	}

	## Evaluate list of interrupt priorities acording to values of IP and IPH
	 # @return void
	private method evaluate_interrupt_priorities {} {
		# Determinate value of Interrupt Priority High register
		if {$feature_avaliable(iph)} {
			set iph $sfr(183)
		} {
			set iph 0
		}

		# Lists of priority flags
		set ip_0 {}
		set ip_1 {}
		set ip_2 {}
		set ip_3 {}

		# Lists of priority levels
		set pl_0 {}
		set pl_1 {}
		set pl_2 {}
		set pl_3 {}
		set ip 0

		# Determinate list of priority flags and priority levels in decremental order
		foreach key {PX0 PT0 PX1 PT1 PS PT2 PC} mask {1 2 4 8 16 32 64} {
			set ip 0
			if {[expr {$sfr(184) & $mask}]} {
				incr ip
			}
			if {[expr {$iph & $mask}]} {
				incr ip 2
			}
			switch -- $ip {
				0 {lappend ip_0 $key}
				1 {lappend ip_1 $key}
				2 {lappend ip_2 $key}
				3 {lappend ip_3 $key}
			}
			lappend pl_${ip} $ip
		}
		set controllers_conf(IP)	[concat $ip_3 $ip_2 $ip_1 $ip_0]
		set controllers_conf(IP_level)	[concat $pl_3 $pl_2 $pl_1 $pl_0]

		# Determinate list of interrupts flags and priority levels in decremental order
		set interrupt_pri_flg {}
		set interrupt_pri_num {}
		foreach flag $controllers_conf(IP) ip $controllers_conf(IP_level) {
			switch -- $flag {
				PS {
					if {$feature_avaliable(uart)} {
						lappend interrupt_pri_flg RI TI
						lappend interrupt_pri_num $ip $ip
					}
					if {$feature_avaliable(spi)} {
						lappend interrupt_pri_flg SPIF
						lappend interrupt_pri_num $ip
					}
				}
				PT2 {
					if {!$feature_avaliable(t2)} {continue}
					lappend interrupt_pri_flg EXF2 TF2
					lappend interrupt_pri_num $ip $ip
				}
				PX0 {
					lappend interrupt_pri_flg IE0
					lappend interrupt_pri_num $ip
				}
				PT0 {
					lappend interrupt_pri_flg TF0
					lappend interrupt_pri_num $ip
				}
				PX1 {
					lappend interrupt_pri_flg IE1
					lappend interrupt_pri_num $ip
				}
				PT1 {
					lappend interrupt_pri_flg TF1
					lappend interrupt_pri_num $ip
				}
				PC {
					if {!$feature_avaliable(acomparator)} {continue}
					lappend interrupt_pri_flg CF
					lappend interrupt_pri_num $ip
				}
			}
		}

		# Adjust interrup monitor
		$this interrupt_monitor_intr_prior $interrupt_pri_flg
	}

	## Adjust configuration acording to new value of the given SFR
	 # @parm Int		- Register address
	 # @parm Bool = 1	- Synchronize this SFR with external interface
	 # @return void
	private method evaluate_sfr args {
		set addr [lindex $args 0]
		set sync [lindex $args 1]

		switch -- $addr {
			135	{	;# PCON		0x87
				set SMOD0_prev $controllers_conf(SMOD0)
				
				write_conf 135 {SMOD1 SMOD0 PWMEN POF GF1 GF0 PD IDL}

				if {$SMOD0_prev != $controllers_conf(SMOD0)} {
					$this simulator_gui_SMOD0_changed
				}
			}
			168	{	;# IE		0xA8
				write_conf 168 {EA EC ET2 ES ET1 EX1 ET0 EX0}

				# Inform interrupt monitor
				$this interrupt_monitor_intr_ena_dis
			}
			184	{	;# IP		0xB8
				evaluate_interrupt_priorities
			}
			152	{	;# SCON		0x98
				write_conf 152 {- SM1 SM2 REN TB8 RB8 TI RI}

				# Determinate SM0 and FE
				if {$controllers_conf(SMOD0)} {
					set controllers_conf(FE) [expr {($sfr(152) & 0x80) ? 1 : 0}]
				} {
					set controllers_conf(SM0) [expr {($sfr(152) & 0x80) ? 1 : 0}]
				}

				# Determinate UART operating mode
				set UART_M_prev $controllers_conf(UART_M)
				set controllers_conf(UART_M) [expr {$controllers_conf(SM0) * 2 + $controllers_conf(SM1)}]
				if {$timer_0_running && $UART_M_prev != $controllers_conf(UART_M)} {
					$this simulator_invalid_uart_mode_change $pc $Line($pc)
					internal_shutdown
				}

				# Inform interrupt monitor
				$this interrupt_monitor_intr_flags [simulator_get_active_intr_flags]
			}
			136	{	;# TCON		0x88
				write_conf 136 {TF1 TR1 TF0 TR0 IE1 IT1 IE0 IT0}

				# Inform interrupt monitor
				$this interrupt_monitor_intr_flags [simulator_get_active_intr_flags]
			}
			137	{	;# TMOD		0x89
				write_conf 137 {GATE1 CT1 M11 M01 GATE0 CT0 M10 M00}

				set T0_MOD_prev $controllers_conf(T0_MOD)
				set T1_MOD_prev $controllers_conf(T1_MOD)

				set controllers_conf(T0_MOD) [expr {$controllers_conf(M10) * 2 + $controllers_conf(M00)}]
				set controllers_conf(T1_MOD) [expr {$controllers_conf(M11) * 2 + $controllers_conf(M01)}]

				# Manual: It is important to stop timer/counter before changing modes
				if {$timer_0_running && $T0_MOD_prev != $controllers_conf(T0_MOD)} {
					$this simulator_invalid_timer_mode_change 0 $pc $Line($pc)
					internal_shutdown
				}
				if {$timer_1_running && $T1_MOD_prev != $controllers_conf(T1_MOD)} {
					$this simulator_invalid_timer_mode_change 1 $pc $Line($pc)
					internal_shutdown
				}
			}
			208	{	;# PSW		0xD0
				set bank [expr {($sfr(208) & 24) >> 3}]
			}
			224	{	;# A		0xE0
				set A 0
				set count 0
				set mask 1
				for {set i 0} {$i < 8} {incr i} {
					if {[expr {$sfr(224) & $mask}] > 0} {
						incr count
					}
					set mask [expr {$mask << 1}]
				}

				if {[expr {$count % 2}] == 1} {
					setBit $symbol(P) 1
				} {
					setBit $symbol(P) 0
				}
			}

			162	{	;# AUXR1	0xA2
				set DPS_org $controllers_conf(DPS)
				write_conf 162 {- - - - - - - DPS}

				# Switch visible dual DPTR
				if {!$feature_avaliable(hddptr)} {
					if {$controllers_conf(DPS)} {
						set DPL {DP1L}
						set DPH {DP1H}
					} {
						set DPL {DP0L}
						set DPH {DP0H}
					}

				# Switch hidden dual DPTR
				} elseif {$DPS_org != $controllers_conf(DPS)} {
					if {$DPS_org} {
						set hidden_DPTR1 [list $sfr($symbol(DP0L)) $sfr($symbol(DP0H))]
					} {
						set hidden_DPTR0 [list $sfr($symbol(DP0L)) $sfr($symbol(DP0H))]
					}
					if {$controllers_conf(DPS)} {
						set sfr($symbol(DP0L)) [lindex $hidden_DPTR1 0]
						set sfr($symbol(DP0H)) [lindex $hidden_DPTR1 1]
					} {
						set sfr($symbol(DP0L)) [lindex $hidden_DPTR0 0]
						set sfr($symbol(DP0H)) [lindex $hidden_DPTR0 1]
					}
					if {$sync_ena} {
						$this Simulator_GUI_sync S $symbol(DP0L)
						$this Simulator_GUI_sync S $symbol(DP0H)
					}
					if {${::Simulator::reverse_run_steps}} {
						stepback_reg_change S $symbol(DP0L)
						stepback_reg_change S $symbol(DP0H)
					}
				}
			}
			142	{	;# AUXR		0x8E
				if {$feature_avaliable(wdtcon)} {
					if {$feature_avaliable(intelpe)} {
						write_conf 142 {- - - - - - IPE DISALE}
					} {
						write_conf 142 {- - - - - - EXTRAM DISALE}
					}
				} {
					write_conf 142 {- - - WDIDLE DISRTO - EXTRAM DISALE}
				}
			}
			166	{	;# WDTRST	0xA6
				if {$controllers_conf(HWDT)} {
					if {$sfr(166) == 225 && $wdtrst_prev_val == 30} {
						set controllers_conf(WatchDogTimer) 1
						set watchdog_value -$time

						if {$feature_avaliable(wdtcon)} {
							set controllers_conf(WDTEN) 1
							set sfr(167) [expr {$sfr(167) | 1}]
							if {${::Simulator::reverse_run_steps}} {
								stepback_reg_change S 167
							}
							if {$sync_ena} {
								$this Simulator_GUI_sync S 167
							}
						}
					}
					set wdtrst_prev_val $sfr(166)
				}
			}
			200	{	;# T2CON	0xC8
				write_conf 200 {TF2 EXF2 RCLK TCLK EXEN2 TR2 CT2 CPRL2}

				# Inform interrupt monitor
				$this interrupt_monitor_intr_flags [simulator_get_active_intr_flags]
			} 
			201	{	;# T2MOD	0xC9
				write_conf 201 {- - - - - - T2OE DCEN}
			}
			143	{	;# CLKREG/CKCON	0x8F
				write_conf 143 {- - - - - - PWDEX X2}
			}
			151	{	;# ACSR		0x97
				write_conf 151 {- - - CF CEN CM2 CM1 CM0}

				set controllers_conf(AC_MOD) [expr {
					$controllers_conf(CM0) * 1 +
					$controllers_conf(CM1) * 2 +
					$controllers_conf(CM2) * 4
				}]

				# Inform interrupt monitor
				$this interrupt_monitor_intr_flags [simulator_get_active_intr_flags]
			}
			183	{	;# IPH		0xB7
				evaluate_interrupt_priorities
			}
			213	{	;# SPCR		0xD5
				write_conf 213 {SPIE SPE DORD MSTR CPOL CPHA SPR1 SPR0}
			}
			170	{	;# SPSR		0xAA
				write_conf 170 {SPIF WCOL LDEN - - - DISSO ENH}
				
				# Inform interrupt monitor
				$this interrupt_monitor_intr_flags [simulator_get_active_intr_flags]
			}
			167	{	;# WDTCON/WDTPRG 0xA7
				if {$feature_avaliable(wdtprg)} {
					write_conf 167 {- - - - - PS2 PS1 PS0}
				} {
					write_conf 167 {PS2 PS1 PS0 WDIDLE DISRTO HWDT WSWRST WDTEN}
				}
				set controllers_conf(WatchDogPrescaler) 0
				if {$controllers_conf(PS2)} {
					incr controllers_conf(WatchDogPrescaler) 4
				}
				if {$controllers_conf(PS1)} {
					incr controllers_conf(WatchDogPrescaler) 2
				}
				if {$controllers_conf(PS0)} {
					incr controllers_conf(WatchDogPrescaler) 1
				}
				set controllers_conf(WatchDogPrescaler)		\
					[expr {int(pow(2,$controllers_conf(WatchDogPrescaler)))}]
			}
			150	{	;# EECON	0x96
				set bit_RDYBSY $controllers_conf(RDYBSY)
				set bit_WRTINH $controllers_conf(WRTINH)

				write_conf 150 {- - EELD EEMWE EEMEN DPS RDYBSY WRTINH}

				# Bits RDYBSY and WRTINH are READ-ONLY
				if {
					$controllers_conf(RDYBSY) != $bit_RDYBSY
						||
					$controllers_conf(WRTINH) != $bit_WRTINH
				} then {
					set sfr(150) [expr {(($sfr(150) & 0xFC) | $bit_RDYBSY * 2) | $bit_WRTINH}]
				}
				set controllers_conf(RDYBSY) $bit_RDYBSY
				set controllers_conf(WRTINH) $bit_WRTINH

				if {$controllers_conf(DPS)} {
					set DPL {DP1L}
					set DPH {DP1H}
				} {
					set DPL {DP0L}
					set DPH {DP0H}
				}
			}
		}

		# Synchronize with an external interface
		if {$sync_ena && $sync != {0}} {
			$this Simulator_GUI_sync S $addr
		}
	}

	## Modify configuration
	 # @parm Int addr	- Source register
	 # @parm List key_list	- List of keys for array controllers_conf
	 # @return void
	private method write_conf {addr key_list} {
		set mask 256
		foreach key $key_list {

			set mask [expr {$mask >> 1}]
			if {$key == {-}} {continue}

			if {[expr {$sfr($addr) & $mask}] == 0} {
				set controllers_conf($key) 0
			} {
				set controllers_conf($key) 1
			}
		}
	}

	## Increment program counter
	 # @parm Int val - Value to increment by
	 # @return void
	private method incr_pc {val} {
		set pc [incr_16b $pc $val]
	}

	## Increment 16 bit value
	 # @parm Int val	- Value to increment
	 # @parm Int byVal	- Value to increment by
	 # @return Int - 16 bit result
	private method incr_16b {val byVal} {
		incr val $byVal
		if {$val > 65535} {
			incr val -65536
		} elseif {$val < 0} {
			incr val 65536
		}
		return $val
	}

	## Increment 8 bit value
	 # @parm Char type	- D == Direct addressing; I == Indirect addressing
	 # @parm Int addr	- Register to increment
	 # @parm Int val	- Value to increment by
	 # @return Bool - 0 == successful; 1 == failed
	private method incr_8b {type addr val} {
		if {[check_address_validity $type $addr]} {return 1}

		# Indirect addressing
		if {$type == {I} || $addr < 128} {
			if {${::Simulator::reverse_run_steps}} {
				stepback_reg_change I $addr
			}
			
			incr ram($addr) $val
			if {$ram($addr) > 255} {
				incr ram($addr) -256
			} elseif {$ram($addr) < 0} {
				incr ram($addr) 256
			}
			if {$sync_ena} {
				$this Simulator_sync_reg $addr
			}

		# Direct addressing
		} else {
			if {${::Simulator::reverse_run_steps}} {
				stepback_reg_change S $addr
			}

			incr sfr($addr) $val
			set val [read_sfr $addr]
			if {$val > 255} {
				incr sfr($addr) -256
			} elseif {$val < 0} {
				incr sfr($addr) 256
			}
			if {$sync_ena} {
				$this Simulator_GUI_sync S $addr
			}
		}
		return 0
	}

	## Pop value from stack
	 # @return Int - result
	private method stack_pop {} {
		if {[check_address_validity I $sfr(129)]} {
			set result [undefined_octet]
		} {
			if {${::Simulator::reverse_run_steps}} {
				stepback_reg_change I $sfr(129)
			}
			set result $ram($sfr(129))	;# 129d == 0x81 == SP
		}
		if {${::Simulator::reverse_run_steps}} {
			stepback_reg_change S 129
		}
		incr sfr(129) -1
		if {$sfr(129) < 0} {
			set sfr(129) 255
			if {!${::Simulator::ignore_stack_underflow}} {
				$this simulator_stack_warning under $pc [lindex $Line($pc) 0]
				internal_shutdown
			}
		}
		evaluate_sfr 129
		return $result
	}

	## Push value onto stack
	 # @parm Int - Value to push onto stack
	 # @return void
	private method stack_push {val} {
		if {${::Simulator::reverse_run_steps}} {
			stepback_reg_change S 129
		}
		incr sfr(129)			;# 129d == 0x81 == SP
		if {$sfr(129) > 255} {
			set sfr(129) 0
			if {!${::Simulator::ignore_stack_overflow}} {
				$this simulator_stack_warning over $pc [lindex $Line($pc) 0]
				internal_shutdown
			}
		}
		if {[check_address_validity I $sfr(129)]} {
			return
		} {
			if {${::Simulator::reverse_run_steps}} {
				stepback_reg_change I $sfr(129)
			}
			set ram($sfr(129)) $val
			if {$sync_ena} {
				$this Simulator_sync_reg $sfr(129)
			}
		}
		evaluate_sfr 129
	}

	# ----------------------------------------------------------------
	# AUXILIARY ALO PROCEDURES
	# ----------------------------------------------------------------

	## Generate random octet
	 # @return Int - random value in range 0..255
	private method undefined_octet {} {
		switch -- ${::Simulator::undefined_value} {
			0 {	;# Return 0
				return 0
			}
			1 {	;# Return 255
				return 255
			}
			2 {	;# Return random value
				set result [expr {int(rand() * 256)}]
				if {$result == 256} {set result 255}
			}
		}
		return $result
	}

	## Add value to accumulator and affect PSW flags
	 # @parm Int val - value to add
	 # @return void
	private method alo_add {val} {

		# Adjust stepback stack
		if {${::Simulator::reverse_run_steps}} {
			stepback_reg_change S 224
		}

		# Local variables
		set A_h [expr {($sfr(224) & 240) >> 4}]	;# High-order nibble of Acc
		set A_l [expr {$sfr(224) & 15}]		;# Low-order  nibble of Acc
		set val_h [expr {($val & 240) >> 4}]	;# High-order nibble of val
		set val_l [expr {$val & 15}]		;# Low-order  nibble of val

		# Compute low-order nibble of result
		set result [expr {$val_l + $A_l}]

		# Flag AC
		if {$result > 15} {
			incr val_h
			incr result -16
			setBit $symbol(AC) 1
		} {
			setBit $symbol(AC) 0
		}

		# Compute high-order nibble of result
		incr result [expr {($val_h + $A_h) << 4}]

		# Flag C
		if {$result > 255} {
			incr result -256
			setBit $symbol(C) 1
		} {
			setBit $symbol(C) 0
		}

		# Flag OV
		if {($val < 128) && ($sfr(224) < 128) && ($result > 127)} {
			setBit $symbol(OV) 1
		} elseif {($val > 127) && ($sfr(224) > 127) && ($result < 128)} {
			setBit $symbol(OV) 1
		} {
			setBit $symbol(OV) 0
		}

		# Set Acc
		set sfr(224) $result
		evaluate_sfr 224
	}

	## Add value to accumulator with carry and affect PSW flags
	 # @parm Int val - value to add
	 # @return void
	private method alo_addc {val} {
		if {[getBit $symbol(C)]} {
			incr val
		}
		alo_add $val
	}

	## Subtract tegister from ACC with borrow and affect PSW flags
	 # @parm Int val - value to subtract
	 # @return void
	private method alo_subb {val} {

		# Adjust stepback stack
		if {${::Simulator::reverse_run_steps}} {
			stepback_reg_change S 224
		}

		# Flag PSW.C
		if {[getBit $symbol(C)]} {
			incr val
		}

		# Local variables
		set A_h [expr {($sfr(224) & 240) >> 4}]	;# High-order nibble of Acc
		set A_l [expr {$sfr(224) & 15}]		;# Low-order  nibble of Acc
		set val_h [expr {($val & 240) >> 4}]	;# High-order nibble of val
		set val_l [expr {$val & 15}]		;# Low-order  nibble of val

		# Compute low-order nibble of result
		set result_l [expr {$A_l - $val_l}]

		# Flag AC
		if {$result_l < 0} {
			incr result_l 16
			incr val_h
			setBit $symbol(AC) 1
		} {
			setBit $symbol(AC) 0
		}

		# Compute high-order nibble of result
		set result_h [expr {$A_h - $val_h}]

		# Flag C
		if {$result_h < 0} {
			incr result_h 16
			setBit $symbol(C) 1
		} {
			setBit $symbol(C) 0
		}

		# Compute high-order nibble of result
		set result [expr {($result_h << 4) + $result_l}]

		# Flag OV
		if {($val > 127) && ($sfr(224) < 128) && ($result > 127)} {
			setBit $symbol(OV) 1
		} elseif {($val < 128) && ($sfr(224) > 127) && ($result < 128)} {
			setBit $symbol(OV) 1
		} {
			setBit $symbol(OV) 0
		}

		# Set Acc
		set sfr(224) $result
	}


	# ----------------------------------------------------------------
	# MEMORY MANAGEMENT RELATED PROCEDURES
	# ----------------------------------------------------------------

	## Get value of the next operand
	 # @return Int - OP code
	private method getNextOperand {} {
		incr_pc 1
		incr run_statistics(4)
		if {[check_address_validity C $pc]} {
			set result [undefined_octet]

			bell
			$this sim_txt_output "Incomplete istruction OP code at 0x[NumSystem::dec2hex $pc]"
			return [undefined_octet]
		} {
			return $code($pc)
		}
	}

	## Get value of the last operand
	 # @return Int - OP code
	private method getLastOperand {} {
		incr_pc 1
		incr run_statistics(4)
		if {[check_address_validity C $pc]} {
			bell
			$this sim_txt_output "Incomplete istruction OP code at 0x[NumSystem::dec2hex $pc]"
			set result [undefined_octet]
		} {
			set result $code($pc)
		}
		
		incr_pc 1
		return $result
	}

	## Get address of Rx register of current bank
	 # @parm int idx - number of register [0;7]
	 # @return int - address (decimal)
	private method R {idx} {
		incr idx [expr {$bank * 8}]
		return $idx
	}

	## Set bit at address $addr to value of $value
	 # @parm int addr	- bit address (decimal)
	 # @parm bool value	- bit value
	 # @return bool - 1: bit value changed; 0: nothing happened
	public method setBit {addr value} {

		set regAddr [getRegOfBit $addr]
		set bitNumber [expr {$addr % 8}]
		
		if {${::Simulator::reverse_run_steps}} {
			if {$regAddr < 128} {
				stepback_reg_change I $regAddr
			} {
				stepback_reg_change S $regAddr
			}
		}

		switch -- $bitNumber {
			7 {set mask 128}
			6 {set mask 64}
			5 {set mask 32}
			4 {set mask 16}
			3 {set mask 8}
			2 {set mask 4}
			1 {set mask 2}
			0 {set mask 1}
		}

		if {$regAddr < 0x80} {
			if {([expr {$ram($regAddr) & $mask}] > 0) && !$value} {
				set ram($regAddr) [expr {$ram($regAddr) ^ $mask}]
			} elseif {([expr {$ram($regAddr) & $mask}] == 0) && $value} {
				set ram($regAddr) [expr {$ram($regAddr) ^ $mask}]
			}
		} {
			if {([expr {$sfr($regAddr) & $mask}] > 0) && !$value} {
				write_sfr $regAddr [expr {$sfr($regAddr) ^ $mask}]
			} elseif {([expr {$sfr($regAddr) & $mask}] == 0) && $value} {
				write_sfr $regAddr [expr {$sfr($regAddr) ^ $mask}]
			}
		}

		if {$regAddr > 127} {
			evaluate_sfr $regAddr
		} {
			if {$sync_ena} {
				$this Simulator_sync_reg $regAddr
			}
		}
	}

	## Get bit value by bit address
	 # @parm int addr - bit address (decimal)
	 # @return bool
	public method getBit {addr} {
		set regAddr [getRegOfBit $addr]
		set bitNumber [expr {$addr % 8}]

		return [getBitByReg $regAddr $bitNumber]
	}

	## Get bit value by register address and bit number
	 # @attribite int regAddr	- register address (decimal)
	 # @attrinte int bitNumber	- bit number (eg. 5)
	 # @return bool
	public method getBitByReg {regAddr bitNumber} {
		switch -- $bitNumber {
			7 {set mask 128}
			6 {set mask 64}
			5 {set mask 32}
			4 {set mask 16}
			3 {set mask 8}
			2 {set mask 4}
			1 {set mask 2}
			0 {set mask 1}
		}

		if {$regAddr < 0x80} {
			if {[expr {$ram($regAddr) & $mask}] == 0} {
				return 0
			} {
				return 1
			}
		} {
			if {[lsearch -ascii -exact $PORT_LATCHES $regAddr] == -1} {
				set byte [read_sfr $regAddr]
			} {
				set byte $sfr($regAddr)
			}

			if {[expr {$byte & $mask}] == 0} {
				return 0
			} {
				return 1
			}
		}
	}

	## Get address of register containing bit specified by argument
	 # @parm int addr - bit address (decimal)
	 # @return int - register address
	public method getRegOfBit {addr} {
		set reg [expr {$addr / 8}]

		if {$addr > 127} {
			set reg [expr {$reg * 8}]
		} {
			incr reg 32
		}

		return $reg
	}

	## Check if the specified address at the given location is implemented in this MCU
	 # If check fail, this procedure will invoke error message and stop simulator
	 # @parm Char location	- Memory type
	 #	D == IDATA direct addressing
	 #	I == IDATA indirect addressing (or operations on stack)
	 #	X == XDATA
	 #	B == Bit area
	 #	C == CODE
	 # @parm Int address	- Memory address (0..65536)
	 # @return Bool - result (false == memory implemented; true == invalid access)
	private method check_address_validity {location address} {
		if {$address_error} {return 1}
		
		if {[simulator_address_range $location $address]} {
			return 0
		}

		switch -- $location {
			{D} {	;# IDATA direct addressing
				if {${::Simulator::ignore_invalid_IDATA}} {
					return 1
				}
			}
			{I} {	;# IDATA indirect addressing (or operations with stack)
				if {${::Simulator::ignore_invalid_IDATA}} {
					return 1
				}
			}
			{X} {	;# XDATA
				if {${::Simulator::ignore_invalid_XDATA}} {
					return 1
				}
			}
			{B} {	;# Bit area
				if {${::Simulator::ignore_invalid_BIT}} {
					return 1
				}
			}
			{C} {	;# CODE
				if {${::Simulator::ignore_invalid_CODE}} {
					return 1
				}
			}
		}

		internal_shutdown
		$this invalid_addressing_dialog $location $address
		set address_error 1
		return 1
	}
	
	## Check if the specified address at the given location is implemented in this MCU
	 # @parm Char location	- Memory type
	 #	D == IDATA direct addressing or SFR
	 #	I == IDATA indirect addressing (or operations on stack)
	 #	B == Bit area
	 #	X == XDATA
	 #	C == CODE
	 #	E == ERAM
	 #	P == Data EEPROM
	 # @parm Int address	- Memory address (0..65536)
	 # @return Bool - result (true == memory implemented; false == invalid access)
	public method simulator_address_range {location address} {
		switch -- $location {
			{D} {	;# IDATA direct addressing or SFR
				if {$address < 128 && $address < $iram_size} {
					return 1
				}
				if {[lsearch $avaliable_sfr $address] != -1} {
					return 1
				}
			}
			{I} {	;# IDATA indirect addressing (or operations with stack)
				if {$address < $iram_size} {
					return 1
				}
			}
			{B} {	;# Bit area
				if {[lsearch $restricted_bits $address] != -1} {
					return 0
				}
				set reg_addr [getRegOfBit $address]
				if {$reg_addr < 128 && $reg_addr < $iram_size} {
					return 1
				}
				if {[lsearch $avaliable_sfr $reg_addr] != -1} {
					return 1
				}
			}
			{X} {	;# XDATA
				if {$address < $xram_size} {
					return 1
				}
			}
			{C} {	;# CODE
				if {$address < $code_size} {
					return 1
				}
			}
			{P} {	;# Data EEPROM
				if {$address < $eeprom_size} {
					return 1
				}
			}
			{E} {	;# ERAM
				if {$address < $eram_size} {
					return 1
				}
			}
		}
		return 0
	}

	## Write value to SFR
	 # - It does not check address validity !
	 # - Purpose is to write zero to unimplemented bits
	 # @parm Int addr	- Target address (128..255)
	 # @parm Int value	- New value (0.255)
	 # @return void
	private method write_sfr {addr value} {
		# Write to SBUF -- Set SBUF-T and begin UART transmission
		if {$addr == $symbol(SBUFR)} {
			if {${::Simulator::reverse_run_steps}} {
				stepback_reg_change S $symbol(SBUFT)
			}
			set sfr($symbol(SBUFT)) $value

			uart_start_transmission

			return
		}

		# Make backup for register value
		if {${::Simulator::reverse_run_steps}} {
			stepback_reg_change S $addr
		}

		# Interrupts configuration related SFR -- skip next interrupt
		if {$addr == $symbol(IP) || $addr == $symbol(IE) || $addr == $symbol(IPH)} {
			set skip_interrupt 1
		} {
			set skip_interrupt 0
		}

		# Write specified value into the SFR
		if {[lsearch $incomplite_regs $addr] == -1} {
			set sfr($addr) $value
		} {
			set sfr($addr) [expr {$value & $incomplite_regs_mask($addr)}]
		}
	}

	## Read value from SFR
	 # This function does not check for valid register address !
	 # - Unimplemented bits are set to random values
	 # @parm Int addr - Source address (128..255)
	 # @return Int - Register value
	private method read_sfr {addr} {
		# Port latch
		set port_number [lsearch -ascii -exact $PORT_LATCHES $addr]
		if {!$rmw_instruction && $port_number != -1} {
			set result [$this pale_RRPV $port_number]

		# Write only register
		} elseif {[lsearch $write_only_regs $addr] != -1} {
			if {!${::Simulator::ignore_read_from_wr_only}} {
				$this simulator_reading_wr_only $addr $pc [lindex $Line($pc) 0]
				internal_shutdown
			}
			return [undefined_octet]

		# Fully implemeneted register
		} elseif {[lsearch $incomplite_regs $addr] == -1} {
			return $sfr($addr)

		# Partialy implemented register
		} {
			return [expr {
				($incomplite_regs_mask($addr) & $sfr($addr))
					+
				(($incomplite_regs_mask($addr) ^ 0x0FF) & [undefined_octet])
			}]
		}
	}


	# ----------------------------------------------------------------
	# REVERSE RUN RELATED PROCEDURES
	# ----------------------------------------------------------------

	## Save current value of the given register for purpose of stepback operation
	 # This function does not check for address validity !
	 # @parm Char mem	- Memory type (one of {I E X S P})
	 # @parm Int addr	- Register address
	 # @return void
	private method stepback_reg_change {mem addr} {
		if {!$stepback_ena} {return}
		if {[lsearch $stepback_local_regs $addr] != -1} {
			return
		}
		lappend stepback_local_regs $addr

		switch -- $mem {
			{I} {set val $ram($addr)	}
			{E} {set val $eram($addr)	}
			{X} {set val $xram($addr)	}
			{S} {set val $sfr($addr)	}
			{P} {set val $eeprom($addr)	}
		}
		lappend stepback_local [list $mem $addr $val]
	}

	## Save local stepback stack to global stepback stack
	 # This function should be called after each instrucion
	 # @return void
	private method stepback_save_norm {} {
		if {!$stepback_ena} {return}
		lappend stepback_normal $stepback_local
		set stepback_local_regs {}
		set stepback_local {}
	}

	## Save special engine configuration variables for purpose of stepback operation
	 # This function should be called before each instrucion
	 # @return void
	private method stepback_save_spec {} {
		if {!$stepback_ena} {return}
		incr stepback_length
		set discard [expr {$stepback_length - ${::Simulator::reverse_run_steps}}]
		if {$discard > 0} {
			incr stepback_length -$discard
			set stepback_spec	[lreplace $stepback_spec 0 0]
			set stepback_normal	[lreplace $stepback_normal 0 0]
		}

		lappend stepback_spec [simulator_get_special]
	}

	## Set last list element in stepback tack to current time
	 # @return void
	private method stepback_save_spec_time {} {
		lset stepback_spec {end end} $time
	}

	## Set value for stepback function for list of subprograms current instruction performed
	 # @parm Int action -
	 #	4 - Return from subprogram
	 #	3 - Return from interrupt routine
	 #	2 - Invocaton of subprogram
	 #	1 - Invocaton of interrupt
	 #	0 - Nothing of thing mentioned above
	 # @return void
	private method stepback_save_spec_subprog {action} {
		lset stepback_spec {end 0} $action
	}

	## Discard stack for stepback functions
	 # @return void
	public method stepback_discard_stack {} {
		set stepback_local_regs	{}
		set stepback_normal	{}
		set stepback_spec	{}
		set stepback_local	{}
		set stepback_length	0
	}

	## Get stepback stack length
	 # @return Int - stack length
	public method simulator_get_SBS_len {} {
		if {${::Simulator::reverse_run_steps}} {
			return $stepback_length
		} {
			return 0
		}
	}

	## Set stepback stack length
	 # @parm Int value - stack length
	 # @return void
	public method simulator_set_SBS_len {value} {
		set stepback_length $value
	}

	# ----------------------------------------------------------------
	# HIBERNATION RELATED PROCEDURES
	# ----------------------------------------------------------------

	## Get special engine configuration list (for stepback and hibernation)
	 # @return List - Special configuration list
	public method simulator_get_special {} {
		set result 0
		lappend result								\
			$ports_previous_state	$pc			$bank		\
			$interrupts_in_progress	$interrupt_on_next	$skip_interrupt	\
			$timer_0_running	$timer_1_running	$overall_time	\
			$overall_instructions	$inter_in_p_flags	$timer1_overflow

		if {$feature_avaliable(t2)} {
			lappend result $timer_2_running $timer2_overflow
		}
		if {$feature_avaliable(wtd)} {
			lappend result $watchdog_value $wdtrst_prev_val
		}
		if {$feature_avaliable(wdtcon)} {
			lappend result $wdt_prescaler_val $controllers_conf(WatchDogPrescaler)
		}
		if {$feature_avaliable(pwm)} {
			lappend result $pwm_running $pwm_OCR
		}
		if {$eeprom_size} {
			lappend result		\
				$eeprom_WR_time	\
				$eeprom_WR	\
				$eeprom_prev	\
				$eeprom_WR_ofs	\
				$controllers_conf(RDYBSY)	\
				$controllers_conf(WRTINH)
				
			for {set i 0} {$i < 32} {incr i} {
				lappend result $eeprom_WR_buff($i)
			}
		}
		if {$feature_avaliable(hddptr)} {
			lappend result $hidden_DPTR0 $hidden_DPTR1
		}
		if {$feature_avaliable(acomparator)} {
			lappend result $anlcmp_running $anlcmp_output $anlcpm_db_timer
		}
		if {$feature_avaliable(uart)} {
			lappend result			\
				$uart_clock_prescaler	\
				$uart_RX_clock		\
				$uart_TX_clock		\
				$uart_RX_in_progress	\
				$uart_TX_in_progress	\
				$uart_RX_shift_reg	\
				$uart_TX_shift_reg
		}
		
		lappend result $time
		return $result
	}
	
	## Set special engine configuration list (for stepback and hibernation)
	 # @parm List list - list to set
	 # @return void
	public method simulator_set_special {list} {
		set i 1
		foreach var {
			ports_previous_state	pc			bank
			interrupts_in_progress	interrupt_on_next	skip_interrupt
			timer_0_running		timer_1_running		overall_time
			overall_instructions	inter_in_p_flags	timer1_overflow
		} {
			set $var [lindex $list $i]
			incr i
		}

		if {$feature_avaliable(t2)} {
			set timer_2_running	[lindex $list $i]
			incr i
			set timer2_overflow	[lindex $list $i]
			incr i
		}
		if {$feature_avaliable(wtd)} {
			set watchdog_value	[lindex $list $i]
			incr i
			set wdtrst_prev_val	[lindex $list $i]
			incr i
		}
		if {$feature_avaliable(wdtcon)} {
			set wdt_prescaler_val	[lindex $list $i]
			incr i
			set controllers_conf(WatchDogPrescaler)	[lindex $list $i]
			incr i
		}
		if {$feature_avaliable(pwm)} {
			set pwm_running		[lindex $list $i]
			incr i
			set pwm_OCR		[lindex $list $i]
			incr i
		}
		if {$eeprom_size} {
			set eeprom_WR_time	[lindex $list $i]
			incr i
			set eeprom_WR		[lindex $list $i]
			incr i
			set eeprom_prev		[lindex $list $i]
			incr i
			set eeprom_WR_ofs	[lindex $list $i]
			incr i
			set controllers_conf(RDYBSY)	[lindex $list $i]	;# Read only bit
			incr i
			set controllers_conf(WRTINH)	[lindex $list $i]	;# Read only bit
			incr i
			
			for {set j 0} {$j < 32} {incr j; incr i} {
				set eeprom_WR_buff($j)	[lindex $list $i]
			}
		}
		if {$feature_avaliable(hddptr)} {
			set hidden_DPTR0	[lindex $list $i]
			incr i
			set hidden_DPTR1	[lindex $list $i]
			incr i
		}
		if {$feature_avaliable(acomparator)} {
			set anlcmp_running	[lindex $list $i]
			incr i
			set anlcmp_output	[lindex $list $i]
			incr i
			set anlcpm_db_timer	[lindex $list $i]
			incr i
		}
		if {$feature_avaliable(uart)} {
			set uart_clock_prescaler [lindex $list $i]
			incr i
			set uart_RX_clock	[lindex $list $i]
			incr i
			set uart_TX_clock	[lindex $list $i]
			incr i
			set uart_RX_in_progress	[lindex $list $i]
			incr i
			set uart_TX_in_progress	[lindex $list $i]
			incr i
			set uart_RX_shift_reg	[lindex $list $i]
			incr i
			set uart_TX_shift_reg	[lindex $list $i]
			incr i
		}

		set time [lindex $list $i]
	}

	## Get special engine configuration list from stepback stack
	 # @parm Int i - Depth
	 # @return List - Config list
	public method simulator_hib_get_SB_spec {i} {
		return [lindex $stepback_spec $i]
	}

	## Get stepback stack for ordinary registes
	 # @parm Int i - Depth
	 # @return List - Part of stepback stack
	public method simulator_hib_get_SB_norm {i} {
		return [lindex $stepback_normal $i]
	}

	## Append special engine configuration list onto stepback stack
	 # @parm List list - Config list
	 # @return void
	public method simulator_hib_append_SB_spec {list} {
		lappend stepback_spec $list
	}

	## Append special engine configuration list onto stepback stack
	 # @parm List list - List of registers and their values
	 # @return void
	public method simulator_hib_append_SB_norm {list} {
		lappend stepback_normal $list
	}
}

# Initialize NS variables
Simulator_ENGINE::InitializeNS
